import { Component, OnInit, HostBinding } from '@angular/core';
import {TranslateService} from '@ngx-translate/core';
import { Observable } from 'rxjs/Observable';


import { select } from '@angular-redux/store';

import { NotifyService } from './partials/components/pnotify/pnotify.service';
import { AppState } from './configs/themeSettings';
import { CookieService } from 'ngx-cookie-service';

// Config Service
import { ConfigDataService } from './configs/services/data.service';
import { ConfigSettingsService } from './configs/services/settings.service';

// Auth Service
import { UserService } from './pages/users/services/auth.service';

import { AppConfig } from './configs/app.config';

@Component({
    selector: 'app-root',
    templateUrl: './app.component.html',
    providers: [UserService, ConfigDataService, ConfigSettingsService],
})
export class AppComponent implements OnInit {

  title = 'Dashboard App';
  message = '';

  constructor(
    private translate: TranslateService,
    private pnotify: NotifyService,
    private cookieService: CookieService,
    private userService: UserService,
    private config: AppConfig,
    private configdata: ConfigDataService
  ) {

     const _value = this.cookieService.get('_LANG');
      if ( _value === undefined || _value === '') {
         this.cookieService.set('_LANG', AppState.DEFAULT_LANG);
         // console.log(' again value is ' + this.cookieService.get('_LANG'));
      } else {
         AppState.DEFAULT_LANG =  _value;
      }
      console.log('selected language ' + AppState.DEFAULT_LANG);
      translate.addLangs(AppState.SUPPORTED_LANGS);
      translate.setDefaultLang(AppState.DEFAULT_LANG);
      translate.use(AppState.DEFAULT_LANG);
  }

  @select(['core', 'notify'])
  readonly notify$: Observable<any>;

  @select(['users', 'auth'])
  readonly auth$: Observable<any>;

  @select(['configuration', 'configs'])
  readonly configs$: Observable<any>;

  @select(['configuration', 'loading'])
  readonly settingLoading$: Observable<any>;

  ngOnInit() {
     this.notify$.subscribe(notify => setTimeout(() => {
        if (notify.title !== '') {
            this.pnotify.render(notify.title, notify.text, notify.css);
        }
     }, 0));

     if (this.config.getGlobalVar('userid') === 0) {
        this.message = 'Error occured while loading application.';
    }

    // load application settings
    this.configdata.LoadRecords({});
    // load & authorize user info
    this.userService.AuthorizeUser(this.config.getGlobalVar('userid') );
     // this.userService.CheckAuthentication();
  }

}
