import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { NgModule, APP_INITIALIZER } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';
import { HttpClientModule, HttpClient } from '@angular/common/http';
import {TranslateModule, TranslateLoader } from '@ngx-translate/core';

import { ToastrModule } from 'ngx-toastr';

import { JMediaLazyGuard } from './j-media-guard.guard';
import { AppConfig } from './configs/app.config';
/* REDUX */
import { NgReduxModule } from '@angular-redux/store';
import { NgReduxRouterModule } from '@angular-redux/router';

/* bootstrap module */
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { appRoutes } from './routes';
import { AppComponent } from './app.component';

// Module Component
import { PartialModule } from './partials/shared.module';
import { StoreModule } from './store/module';


import { NavigationComponent } from './shared/header-navigation/navigation.component';
import { SidebarComponent } from './shared/sidebar/sidebar.component';
import { BreadcrumbComponent } from './shared/breadcrumb/breadcrumb.component';

import { CookieService } from 'ngx-cookie-service';

// authentication
import { UserService } from './pages/users/services/auth.service';
import { SettingsService } from './pages/users/services/settings.service';
import { UserAPIActions } from './pages/users/reducer/actions';
import { PermissionService } from './pages/users/services/permission.service';
// configuration
import { ConfigAPIActions } from './configs/reducer/actions';

import {TranslateHttpLoader} from '@ngx-translate/http-loader';

// ../assets/admin/i18n/ for application integration, ./assets/i18n/ for development
export function createTranslateLoader(http: HttpClient) {
  return new TranslateHttpLoader(http, '../assets/admin/i18n/', '.json');
}

@NgModule({
  declarations: [
    AppComponent,
    NavigationComponent,
    BreadcrumbComponent,
    SidebarComponent
  ],
  imports: [
    RouterModule.forRoot(appRoutes),
    TranslateModule.forRoot({
      loader: {
          provide: TranslateLoader,
          useFactory: (createTranslateLoader),
          deps: [HttpClient]
      }
  }),
    ToastrModule.forRoot(),
    NgbModule,
    BrowserModule,
    BrowserAnimationsModule,
    FormsModule,
    HttpClientModule,
    NgReduxModule,
    NgReduxRouterModule,
    StoreModule,
    PartialModule
  ],
  providers: [CookieService,
    JMediaLazyGuard,
    UserService,
    SettingsService,
    UserAPIActions,
    ConfigAPIActions,
    AppConfig,
    PermissionService,
    { provide: APP_INITIALIZER, useFactory: (config: AppConfig) => () => config.load(), deps: [AppConfig], multi: true }],
  bootstrap: [AppComponent]
})
export class AppModule { }
