import { Component, Input, ViewEncapsulation, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { select } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';
import { FormService } from '../../pages/users/services/form.service';
import { DataService } from '../../pages/users/services/data.service';
import { SettingsService } from '../../pages/users/services/settings.service';
// shared services
import { CoreService } from '../../pages/core/coreService';
import { CoreAPIActions } from '../../pages/core/reducer/actions';

// reducer actions
import { UserAPIActions } from '../../pages/users/reducer/actions';
import { CookieService } from 'ngx-cookie-service';
import { PermissionService } from '../../pages/users/services/permission.service';

@Component({
    selector: 'app-dashboard',
    templateUrl: './dashboard.html',
    encapsulation: ViewEncapsulation.None,
    providers: [DataService, SettingsService, UserAPIActions, FormService, CookieService]
})
export class DashboardComponent implements OnInit {

  controls: any;
  showLoader = false;
  submitText = 'Login';
  submitCss = 'btn btn-info btn-lg btn-block text-uppercase waves-effect waves-light';

  constructor(
    private dataService: DataService,
    private coreService: CoreService,
    private coreActions: CoreAPIActions,
    public permission: PermissionService,
    private actions: UserAPIActions,
    private formService: FormService,
    private router: Router,
    private cookieService: CookieService
    ) {}

    @select(['users', 'auth'])
    readonly auth$: Observable<any>;

    // permission logic
    isAccessGranted = false; // Granc access on resource that can be full access or read only access with no action rights
    isActionGranded = false; // Grand action on resources like add / edit /delete

    ngOnInit() {
        this.controls = this.formService.getLoginControls({
        username: '',
        password: '',
        rememberme: false
        });
    }

  SubmitForm(payload) {
    this.showLoader = true;
    console.log(payload);

    this.dataService.Authenticate(payload).subscribe((data: any) => {

        if (data.status === 'error') {
          this.coreActions.Notify({
              title: data.message,
              text: '',
              css: 'bg-error' });
        } else {

              if (data.user === undefined) {
                  this.coreActions.Notify({
                      title: 'User Object Missing',
                      text: '',
                      css: 'bg-error' });
              }

              this.coreActions.Notify({
                  title: 'Authenticated...',
                  text: '',
                  css: 'bg-success' });

              const Auth_Obj = {
                  userid: data.user.userid,
                  username: data.user.username,
                  email: data.user.email,
                  picturename: data.user.img_url,
                  firstname: data.user.firstname,
                  lastname: data.user.lastname
              };

              if (payload.rememberme) {
                  // save user id in cookie
                  this.cookieService.set('_AUTH', JSON.stringify(Auth_Obj) );
              }
              const obj = {
                  isAuthenticated: true,
                  User: Auth_Obj
              };

              this.actions.Authenticate(obj);
              // redirect
              this.router.navigate(['dashboard']);
              // document.location.href = '/';
        }
        this.showLoader = false;
    }, err => {
        this.showLoader = false;
        this.coreActions.Notify({ title: 'Error Occured', text: '', css: 'bg-danger' });
    });
 }

}

