import { Component, ViewEncapsulation, OnInit } from '@angular/core';
import { select, select$ } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';
/* modal popup */
import {NgbModal, NgbModalOptions} from '@ng-bootstrap/ng-bootstrap';

// services
import { SettingsService } from './services/settings.service';
import { DataService } from './services/data.service';

// modal popup
import { ViewComponent } from './partials/modal.component';

// shared services
import { CoreService } from '../../core/coreService';
import { CoreAPIActions } from '../../core/reducer/actions';

// reducer actions
import { BlockIPAPIActions } from './reducer/actions';
import { fadeInAnimation } from '../../../animations/core';

import { PermissionService } from '../../../pages/users/services/permission.service';

@Component({
  templateUrl: './blockip.html',
  encapsulation: ViewEncapsulation.None,
  animations: [ fadeInAnimation ],
  host: { '[@fadeInAnimation]': '' }
})
export class BlockIPComponent implements OnInit {

  constructor(
    private settingService: SettingsService,
    private dataService: DataService,
    private modalService: NgbModal,
    private coreService: CoreService,
    public permission: PermissionService,
    private coreActions: CoreAPIActions,
    private actions: BlockIPAPIActions
    ) {}

   @select(['blockip', 'filteroptions'])
   readonly filteroptions$: Observable<any>;

   @select(['blockip', 'itemsselected'])
   readonly isItemSelected$: Observable<any>;

   @select(['blockip', 'isloaded'])
   readonly isloaded$: Observable<any>;

   @select(['users', 'auth'])
   readonly auth$: Observable<any>;

   // permission logic
   isAccessGranted = false; // Granc access on resource that can be full access or read only access with no action rights
   isActionGranded = false; // Grand action on resources like add / edit /delete

   heading = 'Block IP';
   subheading = 'Management';
   SearchOptions: any;
   ToolbarOptions: any;

   SelectedItems: any; // selected items in list by check / uncheck options
   isItemsSelected = false; // check the status of items there or not
   FilterOptions: any; // local copy of observable query filters
   ngOnInit() {

        // user authentication & access right management
        // full resource access key and readonly key can be generated via roles management
        this.auth$.subscribe((auth: any) => {
            const FullAccessID = '1539515910482';
            const ReadOnlyAccessID = '1539515881791';
            if (this.permission.GrandResourceAccess(false, FullAccessID, ReadOnlyAccessID, auth.Role)) {
                this.isAccessGranted = true;
                if (this.permission.GrandResourceAction(FullAccessID, auth.Role)) {
                    this.isActionGranded = true;
                }
            }
        });

        this.SearchOptions = this.settingService.getSearchOptions();
        this.ToolbarOptions = this.settingService.getToolbarOptions();

        this.filteroptions$.subscribe(options => {
            this.FilterOptions = options;
            if (options.track_filter) {
                this.dataService.LoadRecords(options);
                // reset track filter to false again
                options.track_filter = false;
                this.actions.updateFilterOptions(options);
            }
        });
        this.isloaded$.subscribe((loaded: boolean) => {
            if (! loaded) {
                this.dataService.LoadRecords(this.FilterOptions);
            }
        });
        this.isItemSelected$.subscribe((selectedItems: boolean) => {
            this.isItemsSelected = selectedItems;
        });
  }

  /* toolbar actions */
  toolbaraction(selection: any) {
    if (!this.isActionGranded) {
        this.coreActions.Notify({
            title: 'Permission Denied',
            text: '',
            css: 'bg-danger' });
        return;
    }
     switch (selection.action) {
          case 'add':
              this.AddRecord();
              return;
          case 'm_markas':
              this.ProcessActions(selection.value);
              return;
      }
  }

  /* find records event trigger */
  FindRecords(filters: any) {
    const _filterOptions = filters.filters;
    _filterOptions.pagenumber = 1;
    _filterOptions.track_filter = true; // to force triggering load event via obvervable subscription
    this.actions.updateFilterOptions(_filterOptions);
}

  /* Add Record */
  AddRecord() {
    const _options: NgbModalOptions = {
        backdrop: false
    };
    const modalRef = this.modalService.open(ViewComponent, _options);
    modalRef.componentInstance.Info = {
        title: 'Block IP Address',
        data: this.settingService.getInitObject()
    };
    modalRef.result.then((result) => {
      this.actions.addRecord({
         id: 0,
         ipaddress: result.data.ipaddress,
         date_added: new Date()
      });
      // this.closeResult = `Closed with: ${result}`;
    }, (dismissed) => {
       console.log('dismissed');
    });
  }

  getSelectedItems(arr: any) {
      this.SelectedItems = arr;
      let _selection = false;
      if (this.SelectedItems.length > 0) {
        _selection = true;
      }
      this.actions.updateItemsSelectionStatus(_selection);
      console.log('items selected');
      console.log(this.SelectedItems);
   }

  ProcessActions(selection: any) {
     if (this.SelectedItems.length > 0) {
        for (const item of this.SelectedItems) {
            item.actionstatus = selection.actionstatus;
        }
        this.dataService.ProcessActions(this.SelectedItems, selection);
     }
  }
}
