import { Component, OnInit, EventEmitter, Input, Output } from '@angular/core';
import { trigger, state, style, transition, animate, keyframes } from '@angular/animations';
import { Observable } from 'rxjs/Observable';
import { select } from '@angular-redux/store';
import { BlockIPAPIActions } from '../reducer/actions';
import { DataService } from '../services/data.service';
import { CoreAPIActions } from '../../../core/reducer/actions';

@Component({
  selector: 'app-list',
  templateUrl: './list.html',
  animations: [
    trigger('fadeIn', [
      transition(':enter', [
        style({ opacity: '0', background: '#f5fb98' }),
        animate('300ms ease-out', keyframes([
          style({opacity: 0, transform: 'translateY(-75%)', offset: 0}),
          style({opacity: 0.5, transform: 'translateY(35px)',  offset: 0.5}),
          style({opacity: 1, transform: 'translateY(0)',     offset: 1.0})
        ])),
      ]),
    ]),
  ]
})
export class ListComponent implements OnInit {
  @Input() isActionGranded = false;
  constructor(
    private actions: BlockIPAPIActions,
    private dataService: DataService,
    private coreActions: CoreAPIActions
  ) {}
  sortedFields: any = {
    ipaddress: {
       sort: 'ipaddress',
       direction: 'desc'
    },
    date_added: {
       sort: 'date_added',
       direction: 'desc'
    }
  };
  selectall = false;
  fieldstates = {
     ipaddress: false,
     date_added: false
  };

  @Output() SelectedItems = new EventEmitter<any>();

  @select(['blockip', 'posts'])
  readonly Data$: Observable<any>;

  @select(['blockip', 'loading'])
  readonly loading$: Observable<boolean>;

  @select(['blockip', 'pagination'])
  readonly pagination$: Observable<any>;

  @select(['blockip', 'selectall'])
  readonly selectAll$: Observable<any>;

  @Output() View = new EventEmitter<any>();

  ngOnInit() {
      this.selectAll$.subscribe((selectall: boolean) => {
        this.selectall = selectall;
        this.checkChange();
      });
  }

  Sort(field: string) {
     if (this.sortedFields[field] === undefined) {
        this.sortedFields[field] = {
           sort: field,
           direction: 'desc'
        };
     } else {
        if (this.sortedFields[field].direction === 'desc') {
          this.sortedFields[field].direction = 'asc';
        } else {
          this.sortedFields[field].direction = 'desc';
        }
     }

     for (const st in this.fieldstates) {
        if (st === field) {
             this.fieldstates[st] = true;
        } else {
           this.fieldstates[st] = false;
        }
     }
     console.log(this.fieldstates);
     // update filter
     this.actions.applyFilter({ attr: 'order', value: this.sortedFields[field].sort + ' ' + this.sortedFields[field].direction });
  }

  toggleEditView(item: any, event) {
      if (!item.editview) {
          item.editview = true;
      } else {
        item.editview = false;
      }
      event.stopPropagation();
  }

  _UpdateRecord(item: any) {
    if (!this.isActionGranded) {
      this.coreActions.Notify({
          title: 'Permission Denied',
          text: '',
          css: 'bg-danger' });
      return;
  }
      item.editview = false;
      /* this.coreActions.Notify({
        title: 'Record added successfully',
        text: '',
        css: 'bg-danger' }); */
      this.dataService.UpdateRecord(item);
  }

  delete(item: any, index: number, event) {
    if (!this.isActionGranded) {
      this.coreActions.Notify({
          title: 'Permission Denied',
          text: '',
          css: 'bg-danger' });
      return;
  }
    const r = confirm('Are you sure you want to delete selected record?');
    if (r === true) {
      this.dataService.DeleteRecord(item, index);
    }
  }
  // Since we're observing an array of items, we need to set up a 'trackBy'
  // parameter so Angular doesn't tear down and rebuild the list's DOM every
  // time there's an update.
  getKey(_, item: any) {
     return item.id;
  }

  processChange() {
     this.actions.selectAll(this.selectall);
  }

  checkChange() {
      this.Data$.subscribe(items => {
        const _items = []; 
        for (const item of items) {
          if (item.Selected) {
            _items.push(item);
          }
        }
        this.SelectedItems.emit(_items);
    });
  }

  /* pagination click event */
  PaginationChange(value: number) {
      // update filter option to query database
      this.actions.applyFilter({ attr: 'pagenumber', value: value });
      // update pagination current page (to hightlight selected page)
      this.actions.updatePaginationCurrentPage({ currentpage: value });
  }

}
