import { Injectable } from '@angular/core';

import * as Controls from '../../../../partials/components/dynamicform/model/elements';
import { FormBase } from '../../../../partials/components/dynamicform/model/base';
import * as OPTIONS from '../categories.model';
import { ContentTypes } from '../../../../configs/settings';
import { AppConfig } from '../../../../configs/app.config';

@Injectable()
export class FormService {
    constructor(private config: AppConfig) {
    }
  getControls(entity: OPTIONS.CategoriesEntity) {

    const controls: FormBase<any>[] = [];

        controls.push(new Controls.Textbox({
            key: 'title',
            label: 'Title',
            value: entity.title,
            required: true,
            minLength: 3,
            maxLength: 150,
            // pattern: '^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$',
            order: 1,
            // colsize: 'col-md-2',
            // disabled: _disabled,
            // helpblock: 'Unique key used to fetch mail template within site events. e.g REGUSER for user regiration'
        }));

        controls.push(new Controls.Textbox({
            key: 'term',
            label: 'Term',
            value: entity.term,
            minLength: 3,
            maxLength: 150,
            order: 2,
        }));

        const _category_types = [];
        for (const type of ContentTypes.CATEGORY_TYPES) {
            _category_types.push({
               key: type.value.toString(),
               value: type.title
            });
        }

        controls.push( new Controls.Dropdown({
            key: 'type',
            label: 'Select Type',
            required: true,
            value: entity.type.toString(),
            options: _category_types,
            order: 3
        }));

        controls.push( new Controls.Dropdown({
            key: 'parentid',
            label: 'Select Parent',
            required: true,
            value: entity.parentid.toString(),
            options: [],
            order: 4
        }));

        controls.push(new Controls.Textbox({
            key: 'priority',
            label: 'Priority',
            value: entity.priority,
            minLength: 3,
            maxLength: 150,
            order: 5,
        }));

        controls.push(new Controls.TinyMyceEditor({
            key: 'description',
            label: 'Description',
            value: entity.description,
            required: false,
            order: 6
        }));

        controls.push( new Controls.Dropdown({
            key: 'status',
            label: 'Select Status',
            required: true,
            value: entity.status.toString(),
            options: [{ key: '1', value: 'Public'}, {key: '0', value: 'Private'}],
            order: 7
        }));

        controls.push(new Controls.Textbox({
            key: 'icon',
            label: 'Icon',
            value: entity.icon,
            minLength: 1,
            maxLength: 100,
            order: 8,
            helpblock: 'Place font-awsome icon to display with selected category'
        }));

        const _files = [];
        if (entity.picturename !== '') {
           _files.push({
               fname: entity.picturename,
               filename: entity.picturename,
               filetype: '.jpg',
               img_url: entity.img_url
           });
        }
        controls.push( new Controls.Uploader({
          key: 'file',
          label: '',
          value: _files,
          required: false,
          helpblock: '',
          uploadoptions: {
              handlerpath: this.config.getConfig('host') + 'admin/categories/uploads',
              pickfilecaption: 'Select Image',
              uploadfilecaption: 'Start Uploading',
              pickbuttoncss: 'btn btn-danger waves-effect waves-light',
              maxfilesize: '8mb',
              chunksize: '8mb',
              headers: {},
              extensiontitle: 'Images Files',
              extensions: 'jpg,jpeg,png',
              filepath: '',
              username: 'administrator',
              removehandler: '',
              maxallowedfiles: 10
          },
          order: 9
        }));

    return controls.sort((a, b) => a.order - b.order);
  }
}
