import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { select } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';
import { ActivatedRoute } from '@angular/router';
// services
import { SettingsService } from './services/settings.service';
import { DataService } from './services/data.service';

// shared services
import { CoreService } from '../../core/coreService';
import { CoreAPIActions } from '../../core/reducer/actions';

// reducer actions
import { ConfigurationsAPIActions } from './reducer/actions';
import { fadeInAnimation } from '../../../animations/core';
import { FormService } from './services/form.service';

import { PermissionService } from '../../../pages/users/services/permission.service';

@Component({
  templateUrl: './configuration.html',
  // encapsulation: ViewEncapsulation.None,
})
export class ConfigurationComponent implements OnInit {

  constructor(
    private settingService: SettingsService,
    private dataService: DataService,
    private coreService: CoreService,
    private coreActions: CoreAPIActions,
    public permission: PermissionService,
    private actions: ConfigurationsAPIActions,
    private route: ActivatedRoute,
    private formService: FormService
    ) {}

   @select(['configurations', 'posts'])
   readonly Data$: Observable<any>;

   @select(['configurations', 'isloaded'])
   readonly isloaded$: Observable<any>;

   @select(['users', 'auth'])
   readonly auth$: Observable<any>;

   // permission logic
   isAccessGranted = false; // Granc access on resource that can be full access or read only access with no action rights
   isActionGranded = false; // Grand action on resources like add / edit /delete

   SearchOptions: any;
   IsLoaded = false;
   Configs = [];
   ConfigType = 0;
   controls: any = [];
   showProcessing = false;
   formHeading = 'Generate Settings';
   submitText = 'Save Changes';

   ngOnInit() {

    this.SearchOptions = this.settingService.getSearchOptions();

    // user authentication & access right management
    // full resource access key and readonly key can be generated via roles management
    this.auth$.subscribe((auth: any) => {
        const FullAccessID = '1539515300649';
        const ReadOnlyAccessID = '1539515273262';
        if (this.permission.GrandResourceAccess(false, FullAccessID, ReadOnlyAccessID, auth.Role)) {
            this.isAccessGranted = true;
            if (this.permission.GrandResourceAction(FullAccessID, auth.Role)) {
                this.isActionGranded = true;
            }
        }
    });

    this.route.params.subscribe(params => {
        this.ConfigType = Number.parseInt(params['type'], 10);
        if (isNaN(this.ConfigType)) {
             this.ConfigType = 0;
        }
        if (this.Configs.length > 0) {
            this.InitializeForm();
        }
    });
    this.isloaded$.subscribe((loaded: boolean) => {
        this.IsLoaded = loaded;
        if (! this.IsLoaded) {
            this.loadRecords();
        }
    });
    this.Data$.subscribe((data: any) => {
        this.Configs = data;
        if (this.Configs.length > 0) {
            this.InitializeForm();
        }
    });

  }

  loadRecords() {
    this.dataService.LoadRecords();
  }

  InitializeForm() {
     this.controls = this.formService.getControls(this.Configs, this.ConfigType);
     this.formHeading = this.settingService.getHeading(this.ConfigType);
  }

  SubmitForm(payload) {
    if (!this.isActionGranded) {
        this.coreActions.Notify({
            title: 'Permission Denied',
            text: '',
            css: 'bg-danger' });
        return;
    }
    this.showProcessing = true;
    this.dataService.Update(this.Configs, payload)
    .subscribe((data: any) => {
        if ( data.status === 'error') {
           this.coreActions.Notify({
             title: data.message,
             text: '',
             css: 'bg-success' });
        } else {
           this.coreActions.Notify({
             title: 'Settings Updated',
             text: '',
             css: 'bg-success' });
        }
        this.showProcessing = false;
    }, err => {
       this.coreActions.Notify({ title: 'Error Occured', text: '', css: 'bg-danger' });
       this.showProcessing = false;
    });
  }

}
