import { Injectable } from '@angular/core';
import { dispatch } from '@angular-redux/store';
import { FluxStandardAction } from 'flux-standard-action';
import { tassign } from 'tassign';

import { IConfigurationState } from './model';
// Flux-standard-action gives us stronger typing of our actions.
type Payload = any;
interface MetaData {};
export type ConfigurationsAPIAction = FluxStandardAction<Payload, MetaData>;

@Injectable()
export class ConfigurationsAPIActions {

  static readonly LOAD_STARTED = 'CONFIGURATIONS_LOAD_STARTED';
  static readonly LOAD_SUCCEEDED = 'CONFIGURATIONS_LOAD_SUCCEEDED';
  static readonly LOAD_FAILED = 'CONFIGURATIONS_LOAD_FAILED';

  static readonly UPDATE_CONFIGS = 'CONFIGURATIONS_UPDATE_CONFIGS';

  @dispatch()
  loadStarted = (): ConfigurationsAPIAction => ({
    type: ConfigurationsAPIActions.LOAD_STARTED,
    meta: { },
    payload: null,
  })

  @dispatch()
  loadSucceeded = (payload: Payload): ConfigurationsAPIAction => ({
    type: ConfigurationsAPIActions.LOAD_SUCCEEDED,
    meta: { },
    payload,
  })

  @dispatch()
  loadFailed = (error): ConfigurationsAPIAction => ({
    type: ConfigurationsAPIActions.LOAD_FAILED,
    meta: { },
    payload: null,
    error,
  })

  @dispatch()
  updateConfigs = (payload: Payload): ConfigurationsAPIAction => ({
    type: ConfigurationsAPIActions.UPDATE_CONFIGS,
    meta: { },
    payload
  })

}



export class ConfigurationsBLL {
  loadSucceeded(state: IConfigurationState, action: ConfigurationsAPIAction) {

    return tassign(state, {
        posts: action.payload.settings,
        loading: false
    });
  }
  updateConfigs(state: IConfigurationState, action: ConfigurationsAPIAction) {
      const posts = state.posts;
      for (const post of posts) {
         for (const updated_post of action.payload) {
             if (post.id === parseInt(updated_post.id, 10)) {
                post.value = updated_post.value;
             }
         }
      }
      return tassign(state, { posts: Object.assign([], state.posts, posts) });
    }
}
