import { Injectable } from '@angular/core';

import * as Controls from '../../../../partials/components/dynamicform/model/elements';
import { FormBase } from '../../../../partials/components/dynamicform/model/base';
import { ContentTypes } from '../../../../configs/settings';
// import { APIURL } from '../../../../configs/settings';

@Injectable()
export class FormService {
  getControls(configs: any, type: number) {
     switch (type) {
        case 0:
          // general settings
          return this.getGeneralSettings(configs);
        case 1:
          // feature settings
          return this.getFeatureSettings(configs);
        case 2:
          // media settings
          return this.getMediaSettings(configs);
        case 4:
          // social settings
          return this.getSocialSettings(configs);

        case 6:
          // player setttings
          return this.getPlayerSettings(configs);
        case 7:
          // multi login settings
          return this.getMultiLoginSettings(configs);
        case 8:
          // constact settings
          return this.getContactSettings(configs);
        case 9:
          // youtube api settings
          return this.getYoutubeSettings(configs);
        case 10:
          // listing settings
          return this.getListingSettings(configs);
        case 11:
          // movie settings
          return this.getMovieSettings(configs);

     }
  }

 
  getValue(configs, id) {
    for (const config of configs) {
       if (id === config.id) {
           return config.value;
       }
    }
    return '';
  }

  getYoutubeSettings(configs: any) {
        const controls: FormBase<any>[] = [];
        controls.push(new Controls.Textbox({
            key: 'config_104',
            label: 'Title',
            placeholder: 'API Key',
            value: this.getValue(configs, 104),
            required: true,
            order: 0
        }));
        return controls.sort((a, b) => a.order - b.order);
  }

  getListingSettings(configs: any) {
    const controls: FormBase<any>[] = [];
    controls.push(new Controls.Textbox({
        key: 'config_14',
        label: 'Video Page Size',
        value: this.getValue(configs, 14),
        required: true,
        order: 12,
        colsize: 'col-md-3',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_15',
        label: 'Photo Page Size',
        value: this.getValue(configs, 15),
        required: true,
        order: 13,
        colsize: 'col-md-3',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_16',
        label: 'Album Page Size',
        value: this.getValue(configs, 16),
        required: true,
        order: 14,
        colsize: 'col-md-3',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_17',
        label: 'Channel Page Size',
        value: this.getValue(configs, 17),
        required: true,
        order: 15,
        colsize: 'col-md-3',
        pattern: '[0-9]+'
    }));
    return controls.sort((a, b) => a.order - b.order);
  }
  getGeneralSettings(configs: any) {

    const controls: FormBase<any>[] = [];

          controls.push(new Controls.Textbox({
              key: 'config_1',
              label: 'Title',
              placeholder: 'Enter website title',
              value: this.getValue(configs, 1),
              order: 0,
              helpblock: 'Website title to represent website throught site in logo, footer and other areas.'
          }));

          controls.push(new Controls.TextArea({
              key: 'config_2',
              label: 'Description',
              value: this.getValue(configs, 2),
              order: 1,
          }));
          controls.push(new Controls.Textbox({
              key: 'config_247',
              label: 'Phone',
              value: this.getValue(configs, 247),
              order: 2,
              helpblock: 'Enter support phone number to display on site.'
          }));
          controls.push(new Controls.Textbox({
              key: 'config_3',
              label: 'Page Caption',
              value: this.getValue(configs, 3),
              order: 3,
              helpblock: 'Attach this with each and every static and dynamic page of website e.g My Home - Page Caption'
          }));
          controls.push(new Controls.Textbox({
              key: 'config_4',
              label: 'Admin Email',
              value: this.getValue(configs, 4),
              required: true,
              order: 4,
              helpblock: 'To be used on behalf of which mails to be sent to other users.'
          }));
          controls.push(new Controls.Textbox({
              key: 'config_5',
              label: 'Email Display Name',
              value: this.getValue(configs, 5),
              required: true,
              order: 5,
              helpblock: 'Display this in replace of email while sending mails'
          }));
          controls.push(new Controls.Textbox({
              key: 'config_208',
              label: 'Logo Url',
              value: this.getValue(configs, 208),
              required: true,
              order: 6,
              helpblock: 'Display this in replace of email while sending mails'
          }));
          controls.push(new Controls.Textbox({
              key: 'config_6',
              label: 'Pagination Links',
              value: this.getValue(configs, 6),
              required: true,
              order: 7,
              helpblock: 'Maximum pagination to display. 0 for unlimited'
          }));

          controls.push(new Controls.RadioButtonList({
              key: 'config_8',
              label: 'Content Screening',
              value: this.getValue(configs, 8),
              required: true,
              order: 8,
              options: [{
                key: '0', value: 'Screen'
              }, {
                key: '1', value: 'Screen & Replace'
              }],
              helpblock: 'Choose option to screen content based on dictionary words and take appropriate action. Screen option will only highlight word if matched, screen & replace will screen and encrypt word (e.g book to b**k if matched.'
           }));

          controls.push(new Controls.RadioButtonList({
            key: 'config_9',
            label: 'Review Type',
            value: this.getValue(configs, 9),
            required: true,
            order: 9,
            options: [{
              key: '1', value: 'Automatic'
            }, {
              key: '0', value: 'Manual'
            }],
            helpblock: 'If set automatic, it will automatically approve user posted contents, in case of manual, administrator must review and approve content manually.'
        }));
        controls.push(new Controls.Textbox({
            key: 'config_10',
            label: 'Spam Count',
            value: this.getValue(configs, 10),
            required: true,
            order: 10,
            colsize: 'col-md-3',
            helpblock: 'No of spam count should reach before automatically blocking content.',
            pattern: '[0-9]+'
        }));
        controls.push(new Controls.Textbox({
            key: 'config_11',
            label: 'Cache Duration',
            value: this.getValue(configs, 11),
            required: true,
            order: 11,
            colsize: 'col-md-3',
            helpblock: 'No of seconds to cache contents. 0 for unlimtied',
            pattern: '[0-9]+'
        }));

        controls.push(new Controls.Textbox({
            key: 'config_26',
            label: 'Url Length',
            value: this.getValue(configs, 26),
            required: true,
            order: 16,
            colsize: 'col-md-3',
            tooltip: 'Maxiumum chars of title or any data to display in dynamic url. 0 for unlimited.',
            pattern: '[0-9]+'
        }));
        controls.push(new Controls.TextArea({
            key: 'config_183',
            label: 'Tracking Script',
            value: this.getValue(configs, 183),
            order: 17,
        }));

        controls.push(new Controls.Textbox({
            key: 'config_287',
            label: 'Category Thumb Width',
            value: this.getValue(configs, 287),
            required: true,
            order: 18,
            colsize: 'col-md-6',
            tooltip: `Used for generating category cover / thumbnail via category management`,
            pattern: '[0-9]+'
        }));

        controls.push(new Controls.Textbox({
            key: 'config_288',
            label: 'Category Thumb Height',
            value: this.getValue(configs, 288),
            required: true,
            order: 19,
            tooltip: `Used for generating category cover / thumbnail via category management`,
            colsize: 'col-md-6',
            pattern: '[0-9]+'
        }));

       return controls.sort((a, b) => a.order - b.order);
  }
  getFeatureSettings(configs: any) {
      const controls: FormBase<any>[] = [];

      controls.push(new Controls.SectionHeader({
        key: 'config_section_02',
        label: 'Core Settings',
        order: 0
      }));

      controls.push(new Controls.RadioButtonList({
        key: 'config_70',
        label: 'Enable Videos',
        value: this.getValue(configs, 70),
        required: true,
        order: 1,
        options: [{
          key: '1', value: 'Yes'
        }, {
          key: '0', value: 'No'
        }],
      }));
      controls.push(new Controls.RadioButtonList({
        key: 'config_71',
        label: 'Enable Audio',
        value: this.getValue(configs, 71),
        required: true,
        order: 2,
        options: [{
          key: '1', value: 'Yes'
        }, {
          key: '0', value: 'No'
        }],
      }));

      controls.push(new Controls.RadioButtonList({
        key: 'config_74',
        label: 'Enable Channels',
        value: this.getValue(configs, 74),
        required: true,
        order: 5,
        options: [{
          key: '1', value: 'Yes'
        }, {
          key: '0', value: 'No'
        }],
      }));

      controls.push(new Controls.RadioButtonList({
        key: 'config_191',
        label: 'Enable Languages',
        value: this.getValue(configs, 191),
        required: true,
        order: 9,
        options: [{
          key: '1', value: 'Yes'
        }, {
          key: '0', value: 'No'
        }],
      }));
      controls.push(new Controls.RadioButtonList({
        key: 'config_81',
        label: 'Enable Media Uploads',
        value: this.getValue(configs, 81),
        required: true,
        order: 10,
        options: [{
          key: '1', value: 'Yes'
        }, {
          key: '0', value: 'No'
        }],
        helpblock: 'If enabled users can also upload videos, photos etc'
      }));
      controls.push(new Controls.RadioButtonList({
        key: 'config_182',
        label: 'Enable Wiki',
        value: this.getValue(configs, 182),
        required: true,
        order: 11,
        options: [{
          key: '1', value: 'Yes'
        }, {
          key: '0', value: 'No'
        }],
      }));
      controls.push(new Controls.RadioButtonList({
        key: 'config_189',
        label: 'Enable Packages',
        value: this.getValue(configs, 189),
        required: true,
        order: 12,
        options: [{
          key: '1', value: 'Yes'
        }, {
          key: '0', value: 'No'
        }],
      }));
      controls.push(new Controls.RadioButtonList({
        key: 'config_207',
        label: 'Enable Gamify',
        value: this.getValue(configs, 207),
        required: true,
        order: 13,
        options: [{
          key: '1', value: 'Yes'
        }, {
          key: '0', value: 'No'
        }],
      }));

      controls.push(new Controls.SectionHeader({
        key: 'config_section_01',
        label: 'Section Settings',
        order: 14
      }));

      controls.push(new Controls.RadioButtonList({
            key: 'config_82',
            label: 'Enable Archives',
            value: this.getValue(configs, 82),
            required: true,
            order: 15,
            options: [{
            key: '1', value: 'Yes'
            }, {
            key: '0', value: 'No'
            }],
      }));

      controls.push(new Controls.RadioButtonList({
            key: 'config_143',
            label: 'Enable Categories',
            value: this.getValue(configs, 143),
            required: true,
            order: 16,
            options: [{
            key: '1', value: 'Yes'
            }, {
            key: '0', value: 'No'
            }],
      }));
      controls.push(new Controls.RadioButtonList({
            key: 'config_83',
            label: 'Enable Tags',
            value: this.getValue(configs, 83),
            required: true,
            order: 17,
            options: [{
            key: '1', value: 'Yes'
            }, {
            key: '0', value: 'No'
            }],
       }));
       controls.push(new Controls.RadioButtonList({
                key: 'config_90',
                label: 'Enable Advertisement',
                value: this.getValue(configs, 90),
                required: true,
                order: 18,
                options: [{
                key: '1', value: 'Yes'
                }, {
                key: '0', value: 'No'
                }],
        }));
        controls.push(new Controls.RadioButtonList({
                key: 'config_91',
                label: 'Enable Email Processing',
                value: this.getValue(configs, 91),
                required: true,
                order: 19,
                options: [{
                key: '1', value: 'Yes'
                }, {
                key: '0', value: 'No'
                }],
        }));
        controls.push(new Controls.RadioButtonList({
                key: 'config_92',
                label: 'Enable Email Verification',
                value: this.getValue(configs, 92),
                required: true,
                order: 20,
                options: [{
                key: '1', value: 'Yes'
                }, {
                key: '0', value: 'No'
                }],
        }));
        controls.push(new Controls.RadioButtonList({
                key: 'config_94',
                label: 'Enable Adult Verification',
                value: this.getValue(configs, 94),
                required: true,
                order: 21,
                options: [{
                key: '1', value: 'Yes'
                }, {
                key: '0', value: 'No'
                }],
                helpblock: 'Display adult warning when user try to access content which has been marked as adult.'
        }));
        controls.push(new Controls.RadioButtonList({
                key: 'config_97',
                label: 'Enable Member Registration',
                value: this.getValue(configs, 97),
                required: true,
                order: 22,
                options: [{
                key: '1', value: 'Yes'
                }, {
                key: '0', value: 'No'
                }],
        }));
        controls.push(new Controls.RadioButtonList({
                key: 'config_12',
                label: 'Store User IP Addresses',
                value: this.getValue(configs, 12),
                required: true,
                order: 23,
                options: [{
                key: 'true', value: 'Yes'
                }, {
                key: 'false', value: 'No'
                }],
        }));
        controls.push(new Controls.RadioButtonList({
                key: 'config_25',
                label: 'Store User Searches',
                value: this.getValue(configs, 25),
                required: true,
                order: 24,
                options: [{
                key: 'true', value: 'Yes'
                }, {
                key: 'false', value: 'No'
                }],
        }));
        controls.push(new Controls.RadioButtonList({
                key: 'config_100',
                label: 'Display Ratings',
                value: this.getValue(configs, 100),
                required: true,
                order: 25,
                options: [{
                key: '1', value: 'Yes'
                }, {
                key: '0', value: 'No'
                }],
        }));
        controls.push(new Controls.RadioButtonList({
            key: 'config_95',
            label: 'Display Views',
            value: this.getValue(configs, 95),
            required: true,
            order: 26,
            options: [{
            key: '1', value: 'Yes'
            }, {
            key: '0', value: 'No'
            }],
      }));
      controls.push(new Controls.RadioButtonList({
                key: 'config_23',
                label: 'Display Channel Views',
                value: this.getValue(configs, 23),
                required: true,
                order: 27,
                options: [{
                key: '1', value: 'Yes'
                }, {
                key: '0', value: 'No'
                }],
        }));
        controls.push(new Controls.RadioButtonList({
                key: 'config_195',
                label: 'Display Downloads',
                value: this.getValue(configs, 23),
                required: true,
                order: 28,
                options: [{
                key: '1', value: 'Yes'
                }, {
                key: '0', value: 'No'
                }],
        }));
        controls.push(new Controls.RadioButtonList({
                key: 'config_96',
                label: 'Display Date',
                value: this.getValue(configs, 96),
                required: true,
                order: 29,
                options: [{
                key: '1', value: 'Yes'
                }, {
                key: '0', value: 'No'
                }],
        }));
        controls.push(new Controls.RadioButtonList({
                key: 'config_99',
                label: 'Display UserName',
                value: this.getValue(configs, 99),
                required: true,
                order: 30,
                options: [{
                key: '1', value: 'Yes'
                }, {
                key: '0', value: 'No'
                }],
       }));
      return controls.sort((a, b) => a.order - b.order);
  }
  getMediaSettings(configs: any) {
    const controls: FormBase<any>[] = [];
    controls.push(new Controls.RadioButtonList({
        key: 'config_27',
        label: 'Uploader Type',
        value: this.getValue(configs, 27),
        required: true,
        order: 1,
        options: [{
          key: '0', value: 'FFMPEG'
        }, {
          key: '1', value: 'Youtube'
        }, {
          key: '2', value: 'Direct'
        }, {
          key: '3', value: 'AWS'
        }],
    }));

    controls.push(new Controls.RadioButtonList({
        key: 'config_289',
        label: 'Thumbnail Rotation Option',
        value: this.getValue(configs, 289),
        required: true,
        order: 1,
        options: [{
          key: '1', value: 'No Rotator'
        }, {
          key: '0', value: 'Thumbnail'
        }, {
          key: '2', value: 'Movie Preview'
        }],
        helpblock: 'Rotate thumbnail when mouse hover on video'
    }));
    controls.push(new Controls.RadioButtonList({
        key: 'config_20',
        label: 'Media Play',
        value: this.getValue(configs, 20),
        required: true,
        order: 2,
        options: [{
          key: '0', value: 'Automatic'
        }, {
          key: '1', value: 'Manual'
        }],
    }));
    controls.push(new Controls.CheckBox({
        key: 'config_37',
        label: 'Delete Original Media',
        value: this.getValue(configs, 37),
        checked: (this.getValue(configs, 37) === 'true'),
        required: true,
        order: 3,
    }));

    controls.push(new Controls.SectionHeader({
        key: 'config_section_01',
        label: 'Video Settings',
        order: 4
    }));
    controls.push(new Controls.Textbox({
        key: 'config_34',
        label: 'Video Formats',
        value: this.getValue(configs, 34),
        required: true,
        order: 5,
        helpblock: 'Supported video formats for upload e.g .mp4,.wmv,.mpeg,.mpg,.flv.'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_36',
        label: 'Maximum Allowed Size',
        value: this.getValue(configs, 36),
        required: true,
        order: 6,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.RadioButtonList({
        key: 'config_30',
        label: 'Video Upload Options',
        value: this.getValue(configs, 30),
        required: true,
        order: 7,
        options: [{
          key: '2', value: 'Both Site & Embed Videos'
        }, {
          key: '0', value: 'Site Videos'
        }, {
            key: '1', value: 'Embed Videos'
        }],
        helpblock: 'Through embed videos you can allow user to post youtube or any other third party videos on website'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_200',
        label: 'MP4 Publihing Command',
        value: this.getValue(configs, 200),
        required: true,
        order: 8,
        helpblock: 'FFMPEG command for publishing mp4 videos'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_254',
        label: 'Publish Video Length',
        value: this.getValue(configs, 254),
        required: true,
        order: 9,
        colsize: 'col-md-2',
        pattern: '[0-9]+',
        helpblock: 'Enter length of video to publish, 0 for whole video'
    }));
    controls.push(new Controls.CheckBox({
        key: 'config_261',
        label: 'Enable Video Thumb Rotator',
        value: this.getValue(configs, 261),
        checked: (this.getValue(configs, 261) === 'true'),
        required: true,
        order: 10,
    }));
    controls.push(new Controls.SectionHeader({
        key: 'config_section_02',
        label: 'Thumb Settings',
        order: 11
    }));
    controls.push(new Controls.Textbox({
        key: 'config_185',
        label: 'Max Allowed Image Size',
        value: this.getValue(configs, 185),
        required: true,
        order: 12,
        colsize: 'col-md-2',
        pattern: '[0-9]+',
        helpblock: 'Max filesize restriction in MB'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_54',
        label: 'Thumb Width',
        value: this.getValue(configs, 54),
        required: true,
        order: 13,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_55',
        label: 'Thumb Height',
        value: this.getValue(configs, 55),
        required: true,
        order: 14,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_279',
        label: 'Thumb Width (Second Variation)',
        value: this.getValue(configs, 279),
        required: true,
        order: 15,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_280',
        label: 'Thumb Height (Second Variation)',
        value: this.getValue(configs, 280),
        required: true,
        order: 16,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_175',
        label: 'Medium Thumb Width',
        value: this.getValue(configs, 175),
        required: true,
        order: 17,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_246',
        label: 'Medium Thumb Height',
        value: this.getValue(configs, 246),
        required: true,
        order: 18,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_277',
        label: 'Medium Thumb Width (Second Variation)',
        value: this.getValue(configs, 277),
        required: true,
        order: 19,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_278',
        label: 'Medium Thumb Height (Second Variation)',
        value: this.getValue(configs, 278),
        required: true,
        order: 20,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_176',
        label: 'Thumb Quality',
        value: this.getValue(configs, 176),
        required: true,
        order: 21,
        colsize: 'col-md-2',
        pattern: '[0-9]+',
        helpblock: 'Thumb quality in %'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_177',
        label: 'Mid Thumb Quality (%)',
        value: this.getValue(configs, 177),
        required: true,
        order: 22,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_211',
        label: 'User Default Thumb Path',
        value: this.getValue(configs, 211),
        required: true,
        order: 23
    }));
    controls.push(new Controls.SectionHeader({
        key: 'config_section_03',
        label: 'Preview Video Settings',
        order: 24
    }));
    controls.push(new Controls.CheckBox({
        key: 'config_250',
        label: 'Enable Preview Url',
        value: this.getValue(configs, 250),
        checked: (this.getValue(configs, 250) === 'true'),
        order: 25,
    }));
    controls.push(new Controls.RadioButtonList({
        key: 'config_251',
        label: 'Preview Url Type',
        value: this.getValue(configs, 251),
        order: 26,
        options: [{
          key: '0', value: 'Clip View'
        }, {
          key: '1', value: 'Low Quality Video'
        }],
        helpblock: 'Through embed videos you can allow user to post youtube or any other third party videos on website'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_252',
        label: 'Clip Length',
        value: this.getValue(configs, 252),
        order: 27,
        colsize: 'col-md-2',
        pattern: '[0-9]+',
        helpblock: 'Enter clip length in seconds if preview url type \'Clip View\' selected'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_253',
        label: 'Preview Preset',
        value: this.getValue(configs, 253),
        order: 28,
        helpblock: 'Please enter preset used to publish video for normal users.'
    }));

    return controls.sort((a, b) => a.order - b.order);
  }

  getMovieSettings(configs) {
    const controls: FormBase<any>[] = [];
    controls.push(new Controls.CheckBox({
        key: 'config_258',
        label: 'Enable Movies',
        value: this.getValue(configs, 258),
        checked: (this.getValue(configs, 258) === 'true'),
        order: 34,
    }));
    controls.push(new Controls.Textbox({
        key: 'config_259',
        label: 'Movie Thumb Width',
        value: this.getValue(configs, 259),
        order: 35,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_260',
        label: 'Movie Thumb Height',
        value: this.getValue(configs, 260),
        order: 36,
        colsize: 'col-md-2',
        pattern: '[0-9]+'
    }));
    controls.push(new Controls.Textbox({
        key: 'config_263',
        label: 'Movie Bucket Name',
        value: this.getValue(configs, 263),
        helpblock: 'Amazon AWS Bucket Name where to host movie contents',
        order: 37,
    }));
    controls.push(new Controls.Textbox({
        key: 'config_264',
        label: 'Downloadable Movie Folder Path',
        value: this.getValue(configs, 264),
        helpblock: 'Folder path within AWS bucket if exist',
        order: 38,
    }));
    controls.push(new Controls.Textbox({
        key: 'config_265',
        label: 'Clip Movie Folder Path',
        value: this.getValue(configs, 265),
        helpblock: 'Folder path within AWS bucket if exist',
        order: 39,
    }));
    controls.push(new Controls.Textbox({
        key: 'config_266',
        label: 'Movie Cover Folder Path',
        value: this.getValue(configs, 266),
        helpblock: 'Folder path within AWS bucket if exist for saving movie covers',
        order: 40,
    }));
    controls.push(new Controls.Textbox({
        key: 'config_268',
        label: 'Private Movie Url',
        value: this.getValue(configs, 268),
        helpblock: 'AWS cloud front url for protected movies',
        order: 41,
    }));
    controls.push(new Controls.Textbox({
        key: 'config_267',
        label: 'Public Movie Url',
        value: this.getValue(configs, 267),
        helpblock: 'AWS cloud front url for public contents e.g clips / thumbs',

        order: 42,
    }));
    return controls.sort((a, b) => a.order - b.order);
  }

  getSocialSettings(configs: any) {
    const controls: FormBase<any>[] = [];
    controls.push(new Controls.Textbox({
        key: 'config_110',
        label: 'Facebook URL',
        value: this.getValue(configs, 110),

        order: 0
    }));
    controls.push(new Controls.Textbox({
        key: 'config_111',
        label: 'Twitter URL',
        value: this.getValue(configs, 111),
        order: 1
    }));
    controls.push(new Controls.Textbox({
        key: 'config_224',
        label: 'Linkdin URL',
        value: this.getValue(configs, 224),
        order: 2
    }));
    controls.push(new Controls.Textbox({
        key: 'config_224',
        label: 'Linkedin URL',
        value: this.getValue(configs, 224),
        order: 3
    }));
    controls.push(new Controls.Textbox({
        key: 'config_223',
        label: 'Flikr URL',
        value: this.getValue(configs, 223),
        order: 4
    }));
    controls.push(new Controls.Textbox({
        key: 'config_225',
        label: 'Thumblr URL',
        value: this.getValue(configs, 223),
        order: 5
    }));
    controls.push(new Controls.Textbox({
        key: 'config_243',
        label: 'Youtube URL',
        value: this.getValue(configs, 243),
        order: 6
    }));
    controls.push(new Controls.Textbox({
        key: 'config_244',
        label: 'Vimeo URL',
        value: this.getValue(configs, 244),
        order: 7
    }));
    controls.push(new Controls.Textbox({
        key: 'config_276',
        label: 'Google Pus Url',
        value: this.getValue(configs, 276),
        order: 8
    }));
    controls.push(new Controls.Textbox({
        key: 'config_275',
        label: 'Instagram Url',
        value: this.getValue(configs, 275),
        order: 9
    }));
    controls.push(new Controls.Textbox({
        key: 'config_112',
        label: 'Facebook App ID',
        value: this.getValue(configs, 112),
        order: 10
    }));
    controls.push(new Controls.Textbox({
        key: 'config_114',
        label: 'Facebook Secrete ID',
        value: this.getValue(configs, 114),
        order: 11
    }));
    controls.push(new Controls.Textbox({
        key: 'config_117',
        label: 'Twitter ID',
        value: this.getValue(configs, 117),
        order: 12
    }));
    return controls.sort((a, b) => a.order - b.order);
  }
 
  getPlayerSettings(configs: any) {
    const controls: FormBase<any>[] = [];
    controls.push(new Controls.CheckBox({
        key: 'config_229',
        label: 'Enable Video Ads',
        value: this.getValue(configs, 229),
        checked: (this.getValue(configs, 229) === 'true'),
     
        order: 34,
    }));
    return controls.sort((a, b) => a.order - b.order);
  }
  getMultiLoginSettings(configs: any) {
    const controls: FormBase<any>[] = [];
    controls.push(new Controls.CheckBox({
        key: 'config_231',
        label: 'Enable Google Plus',
        value: this.getValue(configs, 231),
        checked: (this.getValue(configs, 231) === 'true'),
    
        order: 0,
    }));
    controls.push(new Controls.Textbox({
        key: 'config_232',
        label: 'Google Client ID',
        value: this.getValue(configs, 232),
      
        order: 1
    }));
    controls.push(new Controls.Textbox({
        key: 'config_233',
        label: 'Google Secrete ID',
        value: this.getValue(configs, 233),
   
        order: 2
    }));

    controls.push(new Controls.CheckBox({
        key: 'config_234',
        label: 'Enable Facebook',
        value: this.getValue(configs, 234),
        checked: (this.getValue(configs, 234) === 'true'),
     
        order: 3,
    }));
    controls.push(new Controls.Textbox({
        key: 'config_235',
        label: 'Facebook Client ID',
        value: this.getValue(configs, 235),
    
        order: 4
    }));
    controls.push(new Controls.Textbox({
        key: 'config_236',
        label: 'Facebook Secrete ID',
        value: this.getValue(configs, 236),
 
        order: 5
    }));

    controls.push(new Controls.CheckBox({
        key: 'config_237',
        label: 'Enable Microsoft',
        value: this.getValue(configs, 237),
        checked: (this.getValue(configs, 237) === 'true'),

        order: 6,
    }));
    controls.push(new Controls.Textbox({
        key: 'config_238',
        label: 'Microsoft Client ID',
        value: this.getValue(configs, 238),

        order: 7
    }));
    controls.push(new Controls.Textbox({
        key: 'config_239',
        label: 'Microsoft Secrete ID',
        value: this.getValue(configs, 239),
   
        order: 8
    }));

    controls.push(new Controls.CheckBox({
        key: 'config_240',
        label: 'Enable Twitter',
        value: this.getValue(configs, 240),
        checked: (this.getValue(configs, 240) === 'true'),
        order: 9,
    }));
    controls.push(new Controls.Textbox({
        key: 'config_241',
        label: 'Twitter Client ID',
        value: this.getValue(configs, 241),
        order: 10
    }));
    controls.push(new Controls.Textbox({
        key: 'config_242',
        label: 'Twitter Secrete ID',
        value: this.getValue(configs, 242),
        order: 11
    }));
    return controls.sort((a, b) => a.order - b.order);
  }
  getContactSettings(configs: any) {
    const controls: FormBase<any>[] = [];
    controls.push(new Controls.TextArea({
        key: 'config_269',
        label: 'Contact Information',
        value: this.getValue(configs, 269),
        order: 0
    }));
    controls.push(new Controls.Textbox({
        key: 'config_270',
        label: 'Address',
        value: this.getValue(configs, 270),
        order: 1
    }));
    controls.push(new Controls.Textbox({
        key: 'config_271',
        label: 'Telephone No',
        value: this.getValue(configs, 271),
        order: 2
    }));
    controls.push(new Controls.Textbox({
        key: 'config_272',
        label: 'Telephone No 2',
        value: this.getValue(configs, 272),
        order: 3
    }));
    controls.push(new Controls.Textbox({
        key: 'config_273',
        label: 'Fax Number',
        value: this.getValue(configs, 273),
        order: 4
    }));
    controls.push(new Controls.Textbox({
        key: 'config_274',
        label: 'Email Address',
        value: this.getValue(configs, 274),
        order: 5
    }));
    return controls.sort((a, b) => a.order - b.order);
  }
}
