import { Injectable } from '@angular/core';

import * as OPTIONS from '../configuration.model';
import { iUploadOptions } from '../../../core/core.model';
import { AppConfig } from '../../../../configs/app.config';
import { CoreService } from '../../../core/coreService';
import { ThemeCSS } from '../../../../configs/themeSettings';

@Injectable()
export class SettingsService {

  // configurations
  private apiOptions: OPTIONS.IAPIOptions;
  private uploadOptions: iUploadOptions;
  private toolbarOptions: any;
  private searchOptions: any;

  constructor(private coreService: CoreService, private config: AppConfig) {
      const APIURL = config.getConfig('host');
      this.apiOptions = {
         load: APIURL + 'api/configuration/load',
         proc: APIURL + 'api/configuration/proc'
      };

      this.init_toolbar_options();

      this.init_search_options();
  }

  init_search_options() {
    this.searchOptions = {
        showpanel: true, // show, hide whole panel
        showSearchPanel: false,
        showAdvanceSearchLink: false,
        term: '',
        topselectioncheck: true,
        navList: this.coreService.getConfigsNavList(),
        filters: [],
        dropdownFilters: [],
        checkFilters: [],
        categories: [],
        selectedcategory: '',
        singleaction: false,
        actions: []
      };
  }

  init_toolbar_options() {
    this.toolbarOptions = {
        showtoolbar: false,
        showsecondarytoolbar: false,
        showcheckAll: true,
        navbarcss: ThemeCSS.NAVBAR_CSS,
        left_options: [],
        left_caption: 'Filter:',
        right_caption: '',
        right_options: [],
        actions: []
    };
  }

  getApiOptions() {
     return this.apiOptions;
  }

  getUploadOptions() {
      return this.uploadOptions;
  }

  getToolbarOptions() {
     return this.toolbarOptions;
  }

  getSearchOptions() {
     return this.searchOptions;
  }

  getHeading(configType: number) {
      switch (configType) {
          case 0:
            return 'Generate Settings';
            case 1:
            // feature settings
            return 'Feature Settings';
          case 2:
            // media settings
            return 'Media Settings';
          case 4:
            // social settings
            return 'Social Settings';

          case 6:
            // player setttings
            return 'Player Settings';
          case 7:
            // multi login settings
            return 'Multi Login Settings';
          case 8:
            // constact settings
            return 'Contact Settings';
          case 9:
            // youtube api settings
            return 'Youtube API Settings';
          case 10:
            // listing settings
            return 'Listing Settings';
          case 11:
            return 'Movie Settings';

      }
      return 'General Settings';
  }
}
