import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { select, select$ } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';
/* modal popup */
import {NgbModal} from '@ng-bootstrap/ng-bootstrap';

// services
import { SettingsService } from './services/settings.service';
import { DataService } from './services/data.service';


// shared services
import { CoreService } from '../../core/coreService';
import { CoreAPIActions } from '../../core/reducer/actions';

// reducer actions
import { LanguageAPIActions } from './reducer/actions';
import { fadeInAnimation } from '../../../animations/core';

import { PermissionService } from '../../../pages/users/services/permission.service';

@Component({
  templateUrl: './language.html',
  encapsulation: ViewEncapsulation.None,
  animations: [ fadeInAnimation ],
  host: { '[@fadeInAnimation]': '' }
})
export class LanguageComponent implements OnInit {

  constructor(
    private settingService: SettingsService,
    private dataService: DataService,
    private modalService: NgbModal,
    private coreService: CoreService,
    private coreActions: CoreAPIActions,
    public permission: PermissionService,
    private actions: LanguageAPIActions
    ) {}

   @select(['language', 'filteroptions'])
   readonly filteroptions$: Observable<any>;

   @select(['language', 'isloaded'])
   readonly isloaded$: Observable<any>;

   @select(['users', 'auth'])
   readonly auth$: Observable<any>;

   // permission logic
   isAccessGranted = false; // Granc access on resource that can be full access or read only access with no action rights
   isActionGranded = false; // Grand action on resources like add / edit /delete
   
   SearchOptions: any;
   ToolbarOptions: any;

   SelectedItems: any; // selected items in list by check / uncheck options
   isItemsSelected = false; // check the status of items there or not
   FilterOptions: any; // local copy of observable query filters
   ngOnInit() {
      // user authentication & access right management
      // full resource access key and readonly key can be generated via roles management
      this.auth$.subscribe((auth: any) => {
        const FullAccessID = '1539516010730';
        const ReadOnlyAccessID = '1539516042351';
        if (this.permission.GrandResourceAccess(false, FullAccessID, ReadOnlyAccessID, auth.Role)) {
            this.isAccessGranted = true;
            if (this.permission.GrandResourceAction(FullAccessID, auth.Role)) {
                this.isActionGranded = true;
            }
        }
    });
    this.SearchOptions = this.settingService.getSearchOptions();
    this.ToolbarOptions = this.settingService.getToolbarOptions();

    this.filteroptions$.subscribe(options => {
        this.FilterOptions = options;
        if (options.track_filter) {
            this.dataService.LoadRecords(options);
            // reset track filter to false again
            options.track_filter = false;
            this.actions.updateFilterOptions(options);
        }
    });
    this.isloaded$.subscribe((loaded: boolean) => {
        if (! loaded) {
            this.dataService.LoadRecords(this.FilterOptions);
        }
    });

  }

  /* find records event trigger */
  FindRecords(filters: any) {

    const _filterOptions = filters.filters;
    _filterOptions.pagenumber = 1;
    _filterOptions.track_filter = true; // to force triggering load event via obvervable subscription
    this.actions.updateFilterOptions(_filterOptions);
}

  ProcessActions(selection: any) {
    if (!this.isActionGranded) {
        this.coreActions.Notify({
            title: 'Permission Denied',
            text: '',
            css: 'bg-danger' });
        return;
    }
      selection.value.actionstatus = selection.action;
      const arr = [];
      arr.push(selection.value);
      this.dataService.ProcessActions(arr, selection);
  }

}
