import { Injectable } from '@angular/core';
import { LanguageAPIActions } from '../reducer/actions';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { select } from '@angular-redux/store';

import { SettingsService } from './settings.service';
import { CoreAPIActions } from '../../../core/reducer/actions';


@Injectable()
export class DataService {

  // headers  = new Headers({ 'Content-Type': 'application/json' }); // ... Set content type to JSON

  constructor(

    private settings: SettingsService,
    private http: HttpClient,
    private actions: LanguageAPIActions,
    private coreActions: CoreAPIActions) { }

  LoadRecords(FilterOptions) {
      const URL = this.settings.getApiOptions().load;
      this.actions.loadStarted();
      this.http.post(URL, JSON.stringify(FilterOptions))
        .subscribe((data: any) => {
          // update core data
          this.actions.loadSucceeded(data);

          // update list stats
          this.coreActions.refreshListStats(
              { totalrecords: data.records,
                pagesize:  FilterOptions.pagesize,
                pagenumber: FilterOptions.pagenumber
              });
      }, err => {
          this.actions.loadFailed(err);
      });

  }

  /* get single record */
  GetInfo(userid: number) {
      const URL = this.settings.getApiOptions().getinfo;
      return this.http.post(URL, JSON.stringify({userid}));
  }


    /* execute actions */
    ProcessActions(SelectedItems, status) {
    // apply changes directory instate
    this.actions.applyChanges({
        SelectedItems, status
    });

    // update database
    this.http.post(this.settings.getApiOptions().action, JSON.stringify(SelectedItems))
        .subscribe((data: any) => {

            this.coreActions.Notify({
            title: 'Operation Performed',
            text: '',
            css: 'bg-success' });
    }, err => {
        this.coreActions.Notify({ title: 'Error Occured', text: '', css: 'bg-danger' });
    });

    }
}
