import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { select } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';
/* modal popup */
import {NgbModal, NgbModalOptions} from '@ng-bootstrap/ng-bootstrap';

// services
import { SettingsService } from './services/settings.service';
import { DataService } from './services/data.service';

// modal popup
import { ViewComponent } from './partials/modal.component';

// shared services
import { CoreService } from '../../core/coreService';
import { CoreAPIActions } from '../../core/reducer/actions';

// reducer actions
import { LogAPIActions } from './reducer/actions';
import { fadeInAnimation } from '../../../animations/core';

import { PermissionService } from '../../../pages/users/services/permission.service';

@Component({
  templateUrl: './log.html',
  encapsulation: ViewEncapsulation.None,
  animations: [ fadeInAnimation ],
  host: { '[@fadeInAnimation]': '' }
})
export class LogComponent implements OnInit {

  constructor(
    private settingService: SettingsService,
    private dataService: DataService,
    private modalService: NgbModal,
    private coreService: CoreService,
    public permission: PermissionService,
    private coreActions: CoreAPIActions,
    private actions: LogAPIActions
    ) {}

   @select(['log', 'filteroptions'])
   readonly filteroptions$: Observable<any>;

   @select(['log', 'itemsselected'])
   readonly isItemSelected$: Observable<any>;

   @select(['log', 'isloaded'])
   readonly isloaded$: Observable<any>;

   @select(['users', 'auth'])
   readonly auth$: Observable<any>;

   // permission logic
   isAccessGranted = false; // Granc access on resource that can be full access or read only access with no action rights
   isActionGranded = false; // Grand action on resources like add / edit /delete

   heading = 'Log';
   subheading = 'Management';
   SearchOptions: any;
   ToolbarOptions: any;

   SelectedItems: any; // selected items in list by check / uncheck options
   isItemsSelected = false; // check the status of items there or not
   FilterOptions: any; // local copy of observable query filters
   ngOnInit() {

        // user authentication & access right management
        // full resource access key and readonly key can be generated via roles management
        this.auth$.subscribe((auth: any) => {
            const FullAccessID = '1539516107148';
            const ReadOnlyAccessID = '1539516085026';
            if (this.permission.GrandResourceAccess(false, FullAccessID, ReadOnlyAccessID, auth.Role)) {
                this.isAccessGranted = true;
                if (this.permission.GrandResourceAction(FullAccessID, auth.Role)) {
                    this.isActionGranded = true;
                }
            }
        });
        this.SearchOptions = this.settingService.getSearchOptions();
        this.ToolbarOptions = this.settingService.getToolbarOptions();

       this.filteroptions$.subscribe(options => {
            this.FilterOptions = options;
            if (options.track_filter) {
                this.dataService.LoadRecords(options);
                // reset track filter to false again
                options.track_filter = false;
                this.actions.updateFilterOptions(options);
            }
        });
        this.isloaded$.subscribe((loaded: boolean) => {
            if (! loaded) {
                this.dataService.LoadRecords(this.FilterOptions);
            }
        });
      this.isItemSelected$.subscribe((selectedItems: boolean) => {
        this.isItemsSelected = selectedItems;
      });
  }

  /* toolbar actions */
  toolbaraction(selection: any) {
    if (!this.isActionGranded) {
        this.coreActions.Notify({
            title: 'Permission Denied',
            text: '',
            css: 'bg-danger' });
        return;
    }
     switch (selection.action) {
          case 'view':
            console.log(selection.value);
            this.ViewRecord(selection.value);
            return;
          case 'deleteall':
              this.DeleteAll();
              return;
          case 'm_markas':
              this.ProcessActions(selection.value);
              return;
      }
  }

  /* find records event trigger */
  FindRecords(filters: any) {
    const _filterOptions = filters.filters;
    _filterOptions.pagenumber = 1;
    _filterOptions.track_filter = true; // to force triggering load event via obvervable subscription
    this.actions.updateFilterOptions(_filterOptions);
}

  /* Delete All Records */
  DeleteAll() {

     this.dataService.DeleteAllRecords();
  }

  ViewRecord(obj: any) {
        console.log('view record triggered');
        const _options: NgbModalOptions = {
            backdrop: false
        };
        const modalRef = this.modalService.open(ViewComponent, _options);
        modalRef.componentInstance.Info = {
            title: 'View Log Detail',
            data: obj
        };
        modalRef.result.then((result) => {
            // console.log('back result');
            // console.log(result);
            // this.closeResult = `Closed with: ${result}`;
        }, (dismissed) => {
        console.log('dismissed');
        });
  }


  getSelectedItems(arr: any) {
      this.SelectedItems = arr;
      if (this.SelectedItems.length > 0) {
         this.isItemsSelected = true;
      } else {
        this.isItemsSelected = false;
      }
   }

   ProcessActions(selection: any) {
    if (this.SelectedItems.length > 0) {
       for (const item of this.SelectedItems) {
           item.actionstatus = selection.actionstatus;
       }
       this.dataService.ProcessActions(this.SelectedItems, selection);
    }
 }
}
