import { LogAPIAction, LogAPIActions, LogBLL } from './actions';
import { ILogState, IPagination, IFilterOption } from './model';
import { tassign } from 'tassign';
import { Action } from 'redux';
import { CoreService } from '../../../../pages/core/coreService';

const INITIAL_STATE: ILogState = {
    posts: [],
    records: 0,
    loading: false,
    error: null,
    pagination: IPagination,
    filteroptions: IFilterOption,
    selectall: false,
    itemsselected: false,
    isloaded: false
};

export function createLogReducer() {
  return function logReducer(state: ILogState = INITIAL_STATE,
    a: Action): ILogState {

    const action = a as LogAPIAction;

    const bll = new LogBLL();
    if (!action.meta) {
      return state;
    }
    switch (action.type) {
          case LogAPIActions.IS_ITEM_SELECTED:
             return tassign(state, { itemsselected: action.payload });

          case LogAPIActions.SELECT_ALL:
             return bll.selectAll(state, action);

          case LogAPIActions.LOAD_STARTED:
            return tassign(state, { loading: true, error: null});

          case LogAPIActions.LOAD_SUCCEEDED:
            return bll.loadSucceeded(state, action);

          case LogAPIActions.LOAD_FAILED:
              return tassign(state, { loading: false, error: action.error});

          /* update wholefilter options */
          case LogAPIActions.UPDATE_FILTER_OPTIONS:
              return tassign(state, { filteroptions: Object.assign({}, state.filteroptions, action.payload) });

          /* update specific filter option */
          case LogAPIActions.APPLY_FILTER:
             return bll.applyFilterChanges(state, action);

          /* update pagination current page */
          case LogAPIActions.UPDATE_PAGINATION_CURRENTPAGE:
             return bll.updatePagination(state, action);

          /* add record */
          case LogAPIActions.ADD_RECORD:
             return bll.addRecord(state, action);

          /* update record state */
          case LogAPIActions.UPDATE_RECORD:
             return bll.updateRecord(state, action);

          /* apply changes (multiple selection items e.g delete selected records or enable selected records) */
          case LogAPIActions.APPLY_CHANGES:
             return bll.applyChanges(state, action);
          /* remove all records */
          case LogAPIActions.REMOVE_ALL_RECORDS:
            return tassign(state, { posts: [], records: 0 });

          // remove loader
          case LogAPIActions.LOAD_END:
          return tassign(state, { loading: false });

          case LogAPIActions.REFRESH_PAGINATION:
          return bll.refreshpagination(state, action);

          case LogAPIActions.REFRESH_DATA:
          const filterOptions = state.filteroptions;
          filterOptions.track_filter = true;
          return tassign(state, { filteroptions: Object.assign({}, state.filteroptions, filterOptions) });
      }

    return state;
  };
}
