import { Component, OnInit, EventEmitter, Input, Output } from '@angular/core';
import { trigger, state, style, transition, animate, keyframes } from '@angular/animations';
import { Observable } from 'rxjs/Observable';
import { select } from '@angular-redux/store';
import { PackagesAPIActions } from '../reducer/actions';
import { DataService } from '../services/data.service';
import { CoreAPIActions } from '../../../core/reducer/actions';

@Component({
  selector: 'app-list',
  templateUrl: './list.html',
  animations: [
    trigger('fadeIn', [
      transition(':enter', [
        style({ opacity: '0', background: '#f5fb98' }),
        animate('300ms ease-out', keyframes([
          style({opacity: 0, transform: 'translateY(-75%)', offset: 0}),
          style({opacity: 0.5, transform: 'translateY(35px)',  offset: 0.5}),
          style({opacity: 1, transform: 'translateY(0)',     offset: 1.0})
        ])),
      ]),
    ]),
  ]
})
export class ListComponent implements OnInit {

  constructor(
    private actions: PackagesAPIActions,
    private coreActions: CoreAPIActions,
    private dataService: DataService
  ) {}

  @Input() isActionGranded = false;
  @select(['packages', 'posts'])
  readonly Data$: Observable<any>;

  @select(['packages', 'loading'])
  readonly loading$: Observable<boolean>;

  @select(['packages', 'pagination'])
  readonly pagination$: Observable<any>;

  @select(['packages', 'selectall'])
  readonly selectAll$: Observable<any>;

  @Output() View = new EventEmitter<any>();
  @Output() SelectedItems = new EventEmitter<any>();

  sortedFields: any = {
    name: {
       sort: 'name',
       direction: 'desc'
    },
    price: {
       sort: 'price',
       direction: 'desc'
    },
    credits: {
      sort: 'credits',
      direction: 'desc'
   },
   type: {
        sort: 'type',
        direction: 'desc'
    },
    package_type: {
      sort: 'package_type',
      direction: 'desc'
  }
  };
  selectall = false;
  fieldstates = {
     name: false,
     price: false,
     credits: false,
     type: false,
     package_type: false
  };

  ngOnInit() {
    this.selectAll$.subscribe((selectall: boolean) => {
      this.selectall = selectall;
      this.checkChange();
    });
  }

  /* action trigger */
  Sort(field: string) {
    if (this.sortedFields[field] === undefined) {
       this.sortedFields[field] = {
          sort: field,
          direction: 'desc'
       };
    } else {
       if (this.sortedFields[field].direction === 'desc') {
         this.sortedFields[field].direction = 'asc';
       } else {
         this.sortedFields[field].direction = 'desc';
       }
    }

    for (const st in this.fieldstates) {
       if (st === field) {
            this.fieldstates[st] = true;
       } else {
          this.fieldstates[st] = false;
       }
    }
    console.log(this.fieldstates);
    // update filter
    this.actions.applyFilter({ attr: 'order', value: this.sortedFields[field].sort + ' ' + this.sortedFields[field].direction });
 }
  // Since we're observing an array of items, we need to set up a 'trackBy'
  // parameter so Angular doesn't tear down and rebuild the list's DOM every
  // time there's an update.
  getKey(_, item: any) {
     return item.packageid;
  }

  /* pagination click event */
  PaginationChange(value: number) {
      // update filter option to query database
      this.actions.applyFilter({ attr: 'pagenumber', value: value });
      // update pagination current page (to hightlight selected page)
      this.actions.updatePaginationCurrentPage({ currentpage: value });
  }
  delete(item: any, index: number, event) {
    if (!this.isActionGranded) {
      this.coreActions.Notify({
          title: 'Permission Denied',
          text: '',
          css: 'bg-danger' });
      return;
  }
    const r = confirm('Are you sure you want to delete selected record?');
    if (r === true) {
      this.dataService.DeleteRecord(item, index);
    }
  }
  processChange() {
    this.actions.selectAll(this.selectall);
}

checkChange() {
    this.Data$.subscribe(items => {
      const _items = [];
      for (const item of items) {
        if (item.Selected) {
          _items.push(item);
        }
      }
      this.SelectedItems.emit(_items);
  });
}
}
