import { Injectable } from '@angular/core';

import * as Controls from '../../../../partials/components/dynamicform/model/elements';
import { FormBase } from '../../../../partials/components/dynamicform/model/base';
import * as OPTIONS from '../package.model';
import { ContentTypes } from '../../../../configs/settings';

@Injectable()
export class FormService {

  getControls(entity: OPTIONS.PackagesEntity) {

    const controls: FormBase<any>[] = [];

        controls.push(new Controls.Textbox({
            key: 'name',
            label: 'Title',
            value: entity.name,
            required: true,
            minLength: 3,
            maxLength: 25,
            // pattern: '^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$',
            order: 1,
            // colsize: 'col-md-2',
            // disabled: _disabled,
        }));

        controls.push(new Controls.TinyMyceEditor({
            key: 'description',
            label: 'Description',
            value: entity.description,
            required: true,
            helpblock: 'A block of help text that breaks onto a new line and may extend beyond one line.',
            // pattern: '^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$',
            order: 2
       }));

       controls.push(new Controls.Textbox({
            key: 'price',
            label: 'Price',
            value: entity.price,
            required: true,
            minLength: 1,
            maxLength: 4,
            // pattern: '^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$',
            order: 3,
            colsize: 'col-md-2',
            // disabled: _disabled,
        }));

        controls.push(new Controls.Textbox({
          key: 'credits',
          label: 'Credits',
          value: entity.credits,
          required: true,
          minLength: 1,
          maxLength: 4,
          // pattern: '^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$',
          order: 4,
          colsize: 'col-md-2',
          // disabled: _disabled,
      }));

      controls.push( new Controls.Dropdown({
          key: 'type',
          label: 'Type',
          value: entity.type.toString(),
          options: [{key: 0, value: 'Virtual Amount'}, {key: 1, value: 'Actual Amount'}],
          order: 5
      }));

      controls.push( new Controls.Dropdown({
          key: 'package_type',
          label: 'Package Type',
          value: entity.type.toString(),
          required: true,
          options: [{key: '0', value: 'Free Package'}, {key: '1', value: 'Monthly Subscription'}, {key: '2', value: 'One Time'}],
          order: 6
      }));

      controls.push(new Controls.Textbox({
          key: 'discount',
          label: 'Discount',
          value: entity.discount,
          required: true,
          minLength: 1,
          maxLength: 4,
          // pattern: '^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$',
          order: 7,
          colsize: 'col-md-2',
          // disabled: _disabled,
      }));
        /* controls.push( new Controls.Uploader({
          key: 'file',
          label: '',
          value: '',
          required: true,
          helpblock: '',
          uploadoptions: {
              handlerpath: 'http://localhost:4147/api/user/uploads',
              pickfilecaption: 'Select Image',
              uploadfilecaption: 'Start Uploading',
              maxfilesize: '8mb',
              chunksize: '8mb',
              headers: {},
              extensiontitle: 'Images Files',
              extensions: 'jpg,jpeg,png',
              filepath: '',
              username: 'administrator',
              removehandler: '',
              maxallowedfiles: 10
          },
          order: 8
        })); */

        /* controls.push(new Controls.Select({
            key: 'test',
            label: 'Select Plugin',
            value: '',
            required: true,
            // pattern: '^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$',
            order: 9,
            options: [{ id: 'Amsterdam', text: 'Amsterdam' },
            { id: 'Banana', text: 'Banana' },
            { id: 'Apple', text: 'Apple' }]
        })); */

    return controls.sort((a, b) => a.order - b.order);
  }
}
