import { Injectable } from '@angular/core';

import * as OPTIONS from '../package.model';
import { iUploadOptions } from '../../../core/core.model';
import { AppConfig } from '../../../../configs/app.config';
import { CoreService } from '../../../core/coreService';
import { ButtonCSS, ICONCSS, ThemeCSS } from '../../../../configs/themeSettings';

@Injectable()
export class SettingsService {

  // configurations
  private apiOptions: OPTIONS.IAPIOptions;
  private uploadOptions: iUploadOptions;
  private toolbarOptions: any;
  private searchOptions: any;

  constructor(private coreService: CoreService, private config: AppConfig) {
    const APIURL = config.getConfig('host');
      this.apiOptions = {
        load: APIURL + 'api/packages/load',
        getinfo: APIURL + 'api/packages/getinfo',
        action: APIURL + 'api/packages/action',
        proc: APIURL + 'api/packages/proc'
      };

      this.init_toolbar_options();

      this.init_search_options();

  }

  init_search_options() {
    this.searchOptions = {
        showpanel: true, // show, hide whole panel
        showSearchPanel: true,
        showAdvanceSearchLink: false,
        term: '',
        topselectioncheck: true,
        navList: this.coreService.getSettingsNavList(),
        filters: [],
        dropdownFilters: [],
        checkFilters: [],
        categories: [],
        selectedcategory: '',
        singleaction: true,
        actions: this.initialize_actions()
      };
  }

  initialize_actions() {
      return [{
        id: 1,
        title: 'Add Package',
        tooltip: 'Add new package',
        row: 1,
        icon: 'icon-file-plus',
        options: {},
        css: 'btn waves-effect m-b-5 waves-light btn-block btn-success',
        event: 'add'
    }];
  }


  init_toolbar_options() {
    this.toolbarOptions = {
        showtoolbar: true,
        showsecondarytoolbar: true,
        showcheckAll: false,
        navbarcss: ThemeCSS.NAVBAR_CSS,
        left_options: [],
        left_caption: 'Filter:',
        right_caption: '',
        right_options: [],
        actions: []
    };
    this.toolbarOptions.left_options.push({
         title: 'By Type',
         ismultiple: true,
         icon: '', // icon-sort-amount-desc position-left
         Options: [{ id: '1', title: 'Show All', value: 0, isclick: true, clickevent: 'f_reset', tooltip: 'Show all items' },
         { id: '2', separator: true },
         { id: '2', title: 'Virtual Amount', value: 0, isclick: true, clickevent: 'f_type', tooltip: 'Load Wallpapers' },
         { id: '3', title: 'Editor Picked', value: 1, isclick: true, clickevent: 'f_type', tooltip: 'Load Editor Picked' }]
    });
    this.toolbarOptions.left_options.push({
        title: 'By Package',
        ismultiple: true,
        icon: '', // icon-sort-amount-desc position-left
        Options: [{ id: '1', title: 'Show All', value: 0, isclick: true, clickevent: 'f_reset', tooltip: 'Show all items' },
        { id: '2', separator: true },
        { id: '2', title: 'Free Packages', value: 0, isclick: true, clickevent: 'f_ptype', tooltip: 'Load Wallpapers' },
        { id: '3', title: 'Monthly Subscriptions', value: 1, isclick: true, clickevent: 'f_ptype', tooltip: 'Load Editor Picked' },
        { id: '3', title: 'One Time', value: 2, isclick: true, clickevent: 'f_ptype', tooltip: 'Load Editor Picked' }]
   });
   this.toolbarOptions.right_options.push(this.coreService.getPaginationSettings());
   this.toolbarOptions.actions.push({
    title: 'Mark As',
    ismultiple: true,
    icon: '',
    Options: [
        { id: '2', title: 'Delete', value: 0, actionstatus: 'delete',
            css: ButtonCSS.DANGER_BUTTON,
            attr: '', isclick: true, clickevent: 'm_markas', icon:  ICONCSS.DELETE_ICON, tooltip: 'Delete selected records' }]
    });

  }

  getApiOptions() {
     return this.apiOptions;
  }

  getUploadOptions() {
      return this.uploadOptions;
  }

  getToolbarOptions() {
     return this.toolbarOptions;
  }

  getSearchOptions() {
     return this.searchOptions;
  }

  getInitObject(): OPTIONS.PackagesEntity {
    return {
        id: 0,
        name: '',
        description: '',
        price: 0,
        credits: 0,
        type: 0,
        package_type: 0,
        discount: 0,
        discounttype: 0
    };
 }

}
