import { NgModule } from '@angular/core';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { Routes, RouterModule } from '@angular/router';

import { UsersComponent } from './index';
import { UserProfileComponent } from './profile/process.component';

import { UListComponent } from './partials/list.component';
import { ViewComponent } from './partials/modal';

/* services */
import { SettingsService } from './services/settings.service';
import { DataService } from './services/data.service';

/* actions */
import { UserAPIActions } from './reducer/actions';

/* modules */
import { UserProfileModule } from './profile/process.module';

import { PartialModule } from '../../partials/shared.module';

const routes: Routes = [{
    path: '',
    data: {
        title: 'Member Management',
        urls: [{title: 'Dashboard', url: '/'},
        {title: 'Users', url: '/users'},
        {title: 'Management'}]
      },
    component: UsersComponent
}, {
  path: 'profile/:id',
  data: {
      title: 'User Profile',
      urls: [{title: 'Dashboard', url: '/'},
      {title: 'Users', url: '/users'},
      {title: 'User profile'}]
    },
  component: UserProfileComponent
}];

@NgModule({
  imports: [
    CommonModule,
    FormsModule,
    PartialModule,
    UserProfileModule,
    NgbModule.forRoot(),
    RouterModule.forChild(routes)
  ],
  declarations: [
    UsersComponent,
    UListComponent,
    ViewComponent
  ],
  entryComponents: [
    ViewComponent
  ],
  exports: [
    UsersComponent
  ],
  providers: [
    SettingsService, DataService, UserAPIActions
  ]
})
export class UserModule { }
