import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { select } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';

// services
import { SettingsService } from '../services/settings.service';
import { DataService } from '../services/data.service';
import { FormService } from '../services/form.service';


// shared services
import { CoreService } from '../../core/coreService';
import { CoreAPIActions } from '../../core/reducer/actions';

// reducer actions
import { UserAPIActions } from '../reducer/actions';
import { fadeInAnimation } from '../../../animations/core';

/* modal popup */
import {NgbModal, NgbModalOptions} from '@ng-bootstrap/ng-bootstrap';
// modal popup
import { ViewComponent } from '../partials/modal';
import { PermissionService } from '../../../pages/users/services/permission.service';

@Component({
  templateUrl: './process.html',
  encapsulation: ViewEncapsulation.None,
  animations: [ fadeInAnimation ],
  host: { '[@fadeInAnimation]': '' }
})
export class UserProfileComponent implements OnInit {

   // load all available roles
   @select(['roles', 'roles'])
   readonly roles$: Observable<any>;

   @select(['roles', 'isroleloaded'])
   readonly isroleloaded$: Observable<any>;

  constructor(
    private settingService: SettingsService,
    private dataService: DataService,
    private coreService: CoreService,
    private coreActions: CoreAPIActions,
    private actions: UserAPIActions,
    private route: ActivatedRoute,
    private formService: FormService,
    public permission: PermissionService,
    private modalService: NgbModal,
    private router: Router
    ) {}

    ToolbarOptions: any;
    ToolbarLogOptions: any;
    isItemsSelected = true;
    RecordID = 0;
    SearchOptions: any;
    FilterOptions: any = {};
    controls: any = [];
    showLoader = false;
    formHeading = 'User Profile';
    submitText = 'Submit';
    Info: any = {};
    uploadedFiles = [];
    SelectedItems: any = [];
    FullName = '';
    UserLogView = false;
    ProfileView = true; // default option
    UserLog = [];

    // User Roles
    Roles: any = [];

    @select(['users', 'auth'])
    readonly auth$: Observable<any>;

    // permission logic
    isAccessGranted = false; // Granc access on resource that can be full access or read only access with no action rights
    isActionGranded = false; // Grand action on resources like add / edit /delete

    ngOnInit() {

        // user authentication & access right management
        // full resource access key and readonly key can be generated via roles management
        this.auth$.subscribe((auth: any) => {
            const FullAccessID = '1521143362403';
            const ReadOnlyAccessID = '1521143407965';
            if (this.permission.GrandResourceAccess(false, FullAccessID, ReadOnlyAccessID, auth.Role)) {
                this.isAccessGranted = true;
                if (this.permission.GrandResourceAction(FullAccessID, auth.Role)) {
                    this.isActionGranded = true;
                }
            }
       });

       this.SearchOptions = this.settingService.getSearchOptionsProfile();
       this.ToolbarOptions = this.settingService.getToolbarOptions();
       this.ToolbarOptions.showtoolbar = false;
       this.ToolbarOptions.showcheckAll = false;
       this.ToolbarOptions.showsecondarytoolbar = true;
       this.ToolbarOptions.ProfileView = true;

       this.ToolbarLogOptions = this.settingService.getUserLogToolbarOptions();
       this.ToolbarLogOptions.showcheckAll = false;

        // fetch param from url
       this.route.params.subscribe(params => {
           this.RecordID = Number.parseInt(params['id'], 10);
           if (isNaN(this.RecordID)) {
                this.RecordID = 0;
           }
           if (this.RecordID > 0) {
              this.LoadInfo();
           }
       });
   }

   LoadInfo() {
      this.showLoader = true;
      this.dataService.GetInfo(this.RecordID).subscribe( (data: any) => {
            if (data.status === 'error') {
                this.coreActions.Notify({
                    title: data.message,
                    text: '',
                    css: 'bg-success' });
                // redirect to user page
                this.router.navigate(['/users']);
            } else {
                this.Info = data.post;
                this.Info.created_at = new Date(parseInt(this.Info.created_at.substr(6), 10));
                this.Info.last_login = new Date(parseInt(this.Info.last_login.substr(6), 10));
                this.Info.birthdate = new Date(parseInt(this.Info.birthdate.substr(6), 10));
                // for actions like enable / disable / delete
                this.SelectedItems = []; // reset
                this.SelectedItems.push(this.Info);

                if (this.Info.firstname === null || this.Info.firstname === '') {
                    this.FullName = this.Info.username;
                } else {
                    this.FullName = this.Info.firstname + ' ' + this.Info.lastname;
                }
                const _files = [];
                if (this.Info.img_url !== '') {
                   _files.push({
                       fname: this.Info.img_url,
                       filename: this.Info.img_url,
                       filetype: '.jpg',
                       url: this.Info.img_url
                   });
                   this.uploadedFiles = _files;
                }
                this.AdjustNavigationLinks();
            }
            this.showLoader = false;
       });
   }

   AdjustNavigationLinks() {
        for (const nav of this.SearchOptions.navList) {
            if (!nav.clickevent) {
                nav.url = '/users/history/' + this.Info.userid;
            }
        }
   }

   OnUploadedImages(images: any) {
      if (!this.isActionGranded) {
            this.coreActions.Notify({
                title: 'Permission Denied',
                text: '',
                css: 'bg-danger' });
            return;
      }
      this.dataService.UpdateThumb(this.Info, images);
   }

   toolbaraction(selection: any) {
          switch (selection.action) {
              case 'm_markas':
                this.ProcessActions(selection.value);
              break;
              case 'm_log_markas':
                this.ProcessLogActions(selection.value);
              break;
              case 'add':
                  console.log('Add event triggered');
                  this.Trigger_Modal({
                        title: 'Create Account',
                        data: this.settingService.getInitObject(),
                        isActionGranded: this.isActionGranded,
                        viewType: 1
                  });
                  return;
              case 'edit_profile':
                  console.log('edit profile triggered');
                  this.Trigger_Modal({
                    title: 'Edit User Profile',
                    data: this.Info,
                    isActionGranded: this.isActionGranded,
                    viewType: 2
                });
                  return;
              case 'change_email':
                  console.log('change email triggered');
                  this.Trigger_Modal({
                    title: 'Change User Email',
                    data: this.Info,
                    isActionGranded: this.isActionGranded,
                    viewType: 3
                });
                  return;
              case 'change_password':
                  console.log('change password triggered');
                  this.Trigger_Modal({
                      title: 'Change User Password',
                      data: this.Info,
                      isActionGranded: this.isActionGranded,
                      viewType: 4
                  });
                  return;
              case 'change_usertype':
                  console.log('change usertype triggered');
                  this.Trigger_Modal({
                      title: 'Update User Type',
                      data: this.Info,
                      isActionGranded: this.isActionGranded,
                      viewType: 5
                  });
                  return;
               case 'toggle_userhistory':
                 this.ToggleUserHistory(0);
               break;
               case 'toggle_viewprofile':
                  this.ToggleUserHistory(1);
               break;
          }
      }

   ToggleUserHistory(index: number) {
     if (index === 0) {
            this.UserLogView = true;
            this.ProfileView = false;
            if (this.UserLog.length === 0) {
                this.loadUserLog();
            }
      } else {
            this.UserLogView = false;
            this.ProfileView = true;
      }
   }

   loadUserLog() {
        this.showLoader = true;
        this.dataService.GetUserLog(this.Info.username).subscribe( (data: any) => {
            if (data.status === 'error') {
                this.coreActions.Notify({
                    title: data.message,
                    text: '',
                    css: 'bg-success' });
            } else {
                this.UserLog = data.posts;
                // this.SelectedItems = []; // reset
                for (const log of this.UserLog) {
                    log.date_added = new Date(parseInt(log.date_added.substr(6), 10));
                    // for actions like enable / disable / delete
                    // this.SelectedItems.push(log);
                }
            }
            this.showLoader = false;
        });
   }
   /* Add Record */
   Trigger_Modal(InstanceInfo: any) {
        const _options: NgbModalOptions = {
            backdrop: false
        };
        const modalRef = this.modalService.open(ViewComponent, _options);
        modalRef.componentInstance.Info = InstanceInfo;
        modalRef.result.then((result) => {
        console.log('back result');
        console.log(result);
        // this.closeResult = `Closed with: ${result}`;
        }, (dismissed) => {
        console.log('dismissed');
        });
   }

   ProcessActions(selection: any) {
        if (!this.isActionGranded) {
                this.coreActions.Notify({
                    title: 'Permission Denied',
                    text: '',
                    css: 'bg-danger' });
                return;
        }
        if (this.SelectedItems.length > 0) {
        for (const item of this.SelectedItems) {
            item.actionstatus = selection.actionstatus;
        }
        this.dataService.ProcessActions(this.SelectedItems, selection);
        }
    }

    ProcessLogActions(selection: any) {
        if (this.UserLog.length > 0) {
        for (const item of this.UserLog) {
            item.actionstatus = selection.actionstatus;
        }
        this.dataService.ProcessLogActions(this.UserLog, selection);
        }
    }
}
