import { Injectable } from '@angular/core';

import * as Controls from '../../../partials/components/dynamicform/model/elements';
import { FormBase } from '../../../partials/components/dynamicform/model/base';
import * as OPTIONS from '../model';
import { ContentTypes, NormalRegex } from '../../../configs/settings';

@Injectable()
export class FormService {

  getControls(entity: OPTIONS.UserEntity, viewType: number) {

    switch (viewType) {
        case 1:
          // create account
          return this.CreateAccountControls(entity);
        case 2:
          // edit profile
          return this.EditProfileControls(entity);
        case 3:
          // change email
          return this.ChangeEmailControls(entity);
        case 4:
          // change password
          return this.ChangePasswordControls(entity);
        case 5:
          // change user type
          return this.ChangeUserTypeControls(entity);
    }
    return this.CreateAccountControls(entity);
  }

  CreateAccountControls(entity: OPTIONS.UserEntity) {
        const controls: FormBase<any>[] = [];

        const _user_types = [];
        for (const type of ContentTypes.USER_TYPES) {
            _user_types.push({
               key: type.value,
               value: type.title
            });
        }
        controls.push( new Controls.Dropdown({
            key: 'type',
            label: 'Select Type',
            value: entity.type.toString(),
            options: _user_types,
            order: 0,
        }));
        controls.push(new Controls.Textbox({
            key: 'firstname',
            label: '',
            value: entity.firstname,
            placeholder: 'First Name',
            minLength: 3,
            maxLength: 50,
            order: 1,
        }));
        controls.push(new Controls.Textbox({
            key: 'lastname',
            label: '',
            value: entity.lastname,
            placeholder: 'Last Name',
            minLength: 3,
            maxLength: 50,
            order: 2,
        }));
        controls.push(new Controls.Textbox({
            key: 'username',
            label: '',
            value: entity.username,
            placeholder: 'User Name',
            minLength: 5,
            maxLength: 15,
            required: true,
            pattern: NormalRegex.USERNAME_REGEX,
            order: 3,
        }));
        controls.push(new Controls.Textbox({
            key: 'password',
            type: 'password',
            label: '',
            value: entity.password,
            placeholder: 'Password',
            minLength: 5,
            maxLength: 20,
            required: true,
            pattern: NormalRegex.PASSWORD_REGEX,
            order: 4,
        }));
        controls.push(new Controls.Textbox({
            key: 'cpassword',
            type: 'password',
            label: '',
            placeholder: 'Confirm Password',
            value: entity.cpassword,
            minLength: 5,
            maxLength: 20,
            required: true,
            order: 5,
        }));
        controls.push(new Controls.Textbox({
            key: 'email',
            label: '',
            placeholder: 'Email',
            value: entity.email,
            required: true,
            email: true,
            order: 6,
        }));
        return controls.sort((a, b) => a.order - b.order);
  }

  getLoginControls(entity: OPTIONS.LOGINENTITY) {
        const controls: FormBase<any>[] = [];

        controls.push(new Controls.Textbox({
            key: 'username',
            label: '',
            value: entity.username,
            placeholder: 'User Name',
            required: true,
            minLength: 5,
            maxLength: 20,
            order: 1,
        }));
        controls.push(new Controls.Textbox({
            key: 'password',
            type: 'password',
            label: '',
            value: entity.password,
            placeholder: 'Password',
            required: true,
            minLength: 5,
            maxLength: 20,
            order: 2,
        }));
        controls.push(new Controls.CheckBox({
            key: 'rememberme',
            label: 'Remember Me',
            value: entity.rememberme,
            checked: entity.rememberme,
            required: true,
            order: 3,
        }));
        return controls.sort((a, b) => a.order - b.order);
  }

  EditProfileControls(entity: OPTIONS.UserEntity) {
        const controls: FormBase<any>[] = [];
        const _user_types = [];
        for (const type of ContentTypes.USER_TYPES) {
            _user_types.push({
               key: type.value,
               value: type.title
            });
        }
        controls.push( new Controls.Dropdown({
            key: 'type',
            label: 'Select Type',
            value: entity.type.toString(),
            options: _user_types,
            order: 0,
        }));
        controls.push(new Controls.TinyMyceEditor({
            key: 'aboutme',
            label: 'About Me',
            value: entity.aboutme,
            placeholder: 'About Me',
            minLength: 3,
            maxLength: 300,
            order: 1,
        }));
        controls.push(new Controls.Textbox({
            key: 'website',
            label: '',
            value: entity.website,
            placeholder: 'Website',
            minLength: 3,
            maxLength: 150,
            order: 2,
        }));
        controls.push(new Controls.Textbox({
            key: 'firstname',
            label: '',
            value: entity.firstname,
            placeholder: 'First Name',
            minLength: 3,
            maxLength: 50,
            order: 3,
        }));
        controls.push(new Controls.Textbox({
            key: 'lastname',
            label: '',
            value: entity.lastname,
            placeholder: 'Last Name',
            minLength: 3,
            maxLength: 50,
            order: 4,
        }));
        controls.push(new Controls.RadioButtonList({
            key: 'gender',
            label: 'Gender',
            value: entity.gender,
            required: true,
            order: 5,
            options: [{
              key: 'Male', value: 'Male'
            }, {
              key: 'Female', value: 'Female'
            }]
        }));
        controls.push(new Controls.RadioButtonList({
            key: 'relationshipstatus',
            label: 'Relationship Status',
            value: entity.relationshipstatus,
            required: true,
            order: 6,
            options: [{
              key: 'Single', value: 'Single'
            }, {
              key: 'Taken', value: 'Taken'
            }]
        }));
        controls.push(new Controls.RadioButtonList({
            key: 'isallowbirthday',
            label: 'Display my age',
            value: entity.isallowbirthday,
            required: true,
            order: 7,
            options: [{
              key: '1', value: 'Display my age'
            }, {
              key: '0', value: 'Don\'t display my age'
            }]
        }));
        controls.push(new Controls.Textbox({
            key: 'hometown',
            label: '',
            value: entity.hometown,
            placeholder: 'Home Town',
            order: 8,
        }));
        controls.push(new Controls.Textbox({
            key: 'currentcity',
            label: '',
            value: entity.currentcity,
            placeholder: 'Current City',
            order: 9,
        }));
        controls.push(new Controls.Textbox({
            key: 'zipcode',
            label: '',
            value: entity.zipcode,
            placeholder: 'Zipcode',
            order: 10,
        }));
        return controls.sort((a, b) => a.order - b.order);
    }

    ChangeEmailControls(entity: OPTIONS.UserEntity) {
        const controls: FormBase<any>[] = [];
        controls.push(new Controls.Textbox({
            key: 'email',
            label: '',
            placeholder: 'Email',
            value: entity.email,
            required: true,
            email: true,
            order: 6,
        }));
        return controls.sort((a, b) => a.order - b.order);
    }

    ChangePasswordControls(entity: OPTIONS.UserEntity) {
        const controls: FormBase<any>[] = [];
        controls.push(new Controls.Textbox({
            key: 'password',
            type: 'password',
            label: '',
            value: entity.password,
            placeholder: 'Password',
            minLength: 5,
            maxLength: 20,
            required: true,
            pattern: NormalRegex.PASSWORD_REGEX,
            order: 4,
        }));
        controls.push(new Controls.Textbox({
            key: 'cpassword',
            type: 'password',
            label: '',
            placeholder: 'Confirm Password',
            value: entity.cpassword,
            minLength: 5,
            maxLength: 20,
            required: true,
            order: 5,
        }));
        return controls.sort((a, b) => a.order - b.order);
    }

    ChangeUserTypeControls(entity: OPTIONS.UserEntity) {
        const controls: FormBase<any>[] = [];
        const _user_types = [];
        for (const type of ContentTypes.USER_TYPES) {
            _user_types.push({
               key: type.value,
               value: type.title
            });
        }
        controls.push( new Controls.Dropdown({
            key: 'type',
            label: 'Select Type',
            value: entity.type.toString(),
            options: _user_types,
            order: 0,
        }));
        return controls.sort((a, b) => a.order - b.order);
    }
}
