import { Component, OnInit, EventEmitter, Input, Output } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import { select } from '@angular-redux/store';
import { VideoAPIActions } from '../reducer/actions';
import { DataService } from '../services/data.service';
import { Router } from '@angular/router';
import { CoreAPIActions } from '../../core/reducer/actions';

@Component({
  selector: 'app-list',
  templateUrl: './list.html'
})
export class UserListComponent implements OnInit {

  constructor(
    private actions: VideoAPIActions,
    private coreActions: CoreAPIActions,
    private dataService: DataService,
    private router: Router
  ) {}

  @Input() isActionGranded = false;
  @select(['videos', 'posts'])
  readonly Data$: Observable<any>;

  @select(['videos', 'loading'])
  readonly loading$: Observable<boolean>;

  @select(['videos', 'pagination'])
  readonly pagination$: Observable<any>;

  @select(['videos', 'selectall'])
  readonly selectAll$: Observable<any>;

  @Output() View = new EventEmitter<any>();
  @Output() SelectedItems = new EventEmitter<any>();

  selectall = false;

  ngOnInit() {
    this.selectAll$.subscribe((selectall: boolean) => {
      this.selectall = selectall;
      this.checkChange();
    });
  }

  /* action trigger */
  viewRecord(obj, event) {
      this.router.navigate(['/videos/profile/' + obj.videoid]);
      event.stopPropagation();
  }

  // Since we're observing an array of items, we need to set up a 'trackBy'
  // parameter so Angular doesn't tear down and rebuild the list's DOM every
  // time there's an update.
  getKey(_, item: any) {
    return item.videoid;
  }

  /* pagination click event */
  PaginationChange(value: number) {
      // update filter option to query database
      this.actions.applyFilter({ attr: 'pagenumber', value: value });
      // update pagination current page (to hightlight selected page)
      this.actions.updatePaginationCurrentPage({ currentpage: value });
  }

  delete(item: any, index: number, event) {
    if (!this.isActionGranded) {
      this.coreActions.Notify({
          title: 'Permission Denied',
          text: '',
          css: 'bg-danger' });
      return;
  }
    const r = confirm('Are you sure you want to delete selected record?');
    if (r === true) {
      this.dataService.DeleteRecord(item, index);
    }
  }
  processChange() {
    this.actions.selectAll(this.selectall);
}

checkChange() {
    this.Data$.subscribe(items => {
      const _items = [];
      for (const item of items) {
        if (item.Selected) {
          _items.push(item);
        }
      }
      this.SelectedItems.emit(_items);
  });
}
}
