import { Component, OnInit, Input } from '@angular/core';
import { FormService } from '../services/form.service';
import { DataService } from '../services/data.service';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { CoreAPIActions } from '../../core/reducer/actions';
import { select, select$ } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';

@Component({
    selector: 'viewmodal',
    templateUrl: './model.html',
    providers: [FormService, DataService]
})
export class ViewComponent implements OnInit {

  @Input() Info: any;
  title: string;
  data: any;

  showLoader = false;
  heading: string;
  controls: any[];
  Categories: any = [];

  list: any[] = [];
  constructor(
    public activeModal: NgbActiveModal,
    private service: FormService,
    private dataService: DataService,
    private coreActions: CoreAPIActions) { }

    @select(['videos', 'categories'])
    readonly categories$: Observable<any>;
  ngOnInit() {
      this.title = this.Info.title;
      if (this.Info.viewType === 0) {
        // upload video thumbnails (direct uploader)
        this.controls = this.service.getCoverControls(this.Info.data, this.Info.auth);
      } else if (this.Info.viewType === 1) {
        this.categories$.subscribe((categories: any) => {
              this.Categories = categories;
              // edit video
              this.controls = this.service.getVideoEditControls(this.Info.data);
              console.log(this.controls);
              this.updateCategories();
        });
      } else {
        this.controls = this.service.getControls(this.Info.data, this.Info.auth);
      }
  }
  updateCategories() {
    if (this.controls !== undefined) {
        if (this.controls.length > 0) {
            for (const control of this.controls) {
                if (control.key === 'categories') {
                    for (const category of this.Categories) {
                        let term = category.term;
                        if (term === '') {
                            term = category.title;
                        }
                        control.options.push({
                            key: term,
                            value: category.title
                        });
                    }
                }
            }
        }
    }
}

  SubmitForm(payload) {
       // permission check
       if (this.Info.isActionGranded !== undefined) {
        if (!this.Info.isActionGranded) {
               this.coreActions.Notify({
                   title: 'Permission Denied',
                   text: '',
                   css: 'bg-danger' });
               return;
        }
    }
    if (this.Info.viewType === 0) {
        this.Info.data.thumbs = payload.thumbs;
        // this.showLoader = true;
        this.activeModal.close({
          data: this.Info.data
        });
    } else if (this.Info.viewType === 1) {
        // edit video information
        payload.videoid = this.Info.data.videoid;
        payload.username = this.Info.data.username;
        const _status = 'Updated';
        this.showLoader = true;
        console.log(payload);
        this.dataService.EditRecord([payload]).subscribe((data: any) => {
          if (data.status === 'error') {
            this.coreActions.Notify({
                title: data.message,
                text: '',
                css: 'bg-error' });
          } else {
                this.coreActions.Notify({
                    title: 'Record ' + _status + ' Successfully',
                    text: '',
                    css: 'bg-success' });

                this.activeModal.close({
                    data: data.record,
                    status: _status
                });
          }
          this.showLoader = false;
      }, err => {
          this.showLoader = false;
          this.coreActions.Notify({ title: 'Error Occured', text: '', css: 'bg-danger' });
      });
    }

  }
  close() {
     this.activeModal.dismiss('Cancel Clicked');
  }

}
