import { Injectable } from '@angular/core';
import { VideoAPIActions } from '../reducer/actions';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { select } from '@angular-redux/store';

import { SettingsService } from './settings.service';
import { CoreAPIActions } from '../../core/reducer/actions';


@Injectable()
export class DataService {

  // headers  = new Headers({ 'Content-Type': 'application/json' }); // ... Set content type to JSON

  constructor(

    private settings: SettingsService,
    private http: HttpClient,
    private actions: VideoAPIActions,
    private coreActions: CoreAPIActions) { }

  LoadRecords(FilterOptions) {
      const URL = this.settings.getApiOptions().load;
      console.log(URL);
      console.log(FilterOptions);
      this.actions.loadStarted();
      this.http.post(URL, JSON.stringify(FilterOptions))
        .subscribe((data: any) => {
          // update core data
          this.actions.loadSucceeded(data);
          if (data.categories.length > 0) {
              // if enabled, api send list of categories too
              // update categories in state
              this.actions.updateCategories(data.categories);
          }
          // update list stats
          this.coreActions.refreshListStats(
              { totalrecords: data.records,
                pagesize:  FilterOptions.pagesize,
                pagenumber: FilterOptions.pagenumber
              });
      }, err => {
          this.actions.loadFailed(err);
      });

  }

  LoadCategories() {
    const URL = this.settings.getApiOptions().load_categories;
    this.http.post(URL, JSON.stringify({ type: 0, status: 1, issummary: false, isdropdown: true}))
    .subscribe((data: any) => {
        this.actions.updateCategories(data.categorylist);
    }, err => {
        this.actions.loadFailed(err);
    });
}

LoadYoutubeCategories() {
    const URL = this.settings.getApiOptions().youtube;
    this.http.post(URL, {})
    .subscribe((data: any) => {
        this.actions.loadYoutubeCategories(data.categorylist);
    }, err => {
        this.actions.loadFailed(err);
    });
}

SearchYoutube(FilterOptions) {

    const URL = this.settings.getApiOptions().fetch_youtube;
    this.actions.loadStarted();
    this.http.post(URL, JSON.stringify(FilterOptions))
    .subscribe((data: any) => {
        // update core data
        if (data.status === 'error') {
            this.coreActions.Notify({
                title: data.message,
                text: '',
                css: 'bg-success' });
        } else {
            this.actions.loadYoutubeSearchResult(data.posts);
        }
    }, err => {
        this.actions.loadFailed(err);
    });
}

    AddYoutubeVideos(arr) {
    return this.http.post(this.settings.getApiOptions().yt_proc, JSON.stringify(arr));
    }

    AddFFMPEGVideos(arr) {
    return this.http.post(this.settings.getApiOptions().proc_ffmpeg_videos, JSON.stringify(arr));
    }

    AddAWSVideos(arr) {
    return this.http.post(this.settings.getApiOptions().aws_proc, JSON.stringify(arr));
    }

    EncodeVideo(obj) {
    return this.http.post(this.settings.getApiOptions().encodevideo, JSON.stringify([obj]));
    }

    AddRecord(obj) {
        return this.http.post(this.settings.getApiOptions().proc, JSON.stringify(obj));
    }

    EditRecord(obj) {
        return this.http.post(this.settings.getApiOptions().editvideo, JSON.stringify(obj));
    }

    DeletePhotos(obj) {
        return this.http.post(this.settings.getApiOptions().deletevideo, JSON.stringify(obj));
    }
    UpdateThumbnail(obj) {
        return this.http.post(this.settings.getApiOptions().update_video_thumb, JSON.stringify(obj));
    }
    /* get single record */
    GetInfo(videoid: number) {
        const URL = this.settings.getApiOptions().getinfo;
        return this.http.post(URL, JSON.stringify({videoid}));
    }

  DeleteRecord(item, index) {
    item.actionstatus = 'delete';
        const arr = [];
        arr.push(item);
        this.ProcessActions(arr, 'delete');
    }
    /* execute actions */
    ProcessActions(SelectedItems, status) {
    // apply changes directory instate
    this.actions.applyChanges({
        SelectedItems, status
    });

    // update database
    this.http.post(this.settings.getApiOptions().action, JSON.stringify(SelectedItems))
        .subscribe((data: any) => {
        // this.coreActions.Notify(data.message);
        let message = 'Operation Performed';
        if (status === 'delete') {
            message = 'Record Removed';
        }
        this.coreActions.Notify({
            title: message,
            text: '',
            css: 'bg-success' });
    }, err => {
        this.coreActions.Notify({ title: 'Error Occured', text: '', css: 'bg-danger' });
    });

    }
}
