import { Injectable } from '@angular/core';

import * as Controls from '../../../partials/components/dynamicform/model/elements';
import { FormBase } from '../../../partials/components/dynamicform/model/base';
import * as OPTIONS from '../videos.model';
import { AppConfig } from '../../../configs/app.config';
import { ContentTypes } from '../../../configs/settings';

@Injectable()
export class FormService {
    constructor(private config: AppConfig) {
    }
  getControls(entity: OPTIONS.VideoEntity, auth: any) {

    const controls: FormBase<any>[] = [];

       controls.push(new Controls.Textbox({
          key: 'tags',
          label: 'Tags',
          value: entity.tags,
          required: false,
          order: 2,
          maxLength: 1000,
          helpblock: `Enter one or more tags separated by comma`
       }));

       controls.push(new Controls.Dropdown({
            key: 'categories',
            label: 'Select Category',
            value: entity.categories,
            required: true,
            options: [],
            //  maxLength: 150,
            order: 3,
            colsize: 'col-md-3',
       }));

       controls.push( new Controls.Uploader({
          key: 'files',
          label: '',
          value: entity.files,
          required: false,
          helpblock: '',
          uploadoptions: {
              // photouploader: true, // target photo uploader (to format uploaded photos in proper layout)
              handlerpath: this.config.getConfig('host') + 'api/videos/upload',
              pickfilecaption: 'Select MP4 Videos',
              uploadfilecaption: 'Start Uploading',
              colcss: 'col-md-4',
              pickbuttoncss: 'btn btn-danger waves-effect waves-light',
              maxfilesize: '1000mb',
              chunksize: '8mb',
              headers: {},
              extensiontitle: 'MP4 Video Files',
              extensions: 'mp4',
              filepath: '',
              username: auth.User.username, // 'administrator',
              removehandler: '',
              maxallowedfiles: 5,
              showFileName: false, // show filename with media file
              showoriginalSize: false, // show media in original size
              value: entity.files
          },
          order: 0
      }));

      return controls.sort((a, b) => a.order - b.order);
  }

  getCoverControls(entity: OPTIONS.VideoThumbnailEntity, auth: any) {

    const controls: FormBase<any>[] = [];


    controls.push( new Controls.Uploader({
        key: 'thumbs',
        label: '',
        value: entity.thumbs,
        required: false,
        helpblock: '',
        uploadoptions: {
            // photouploader: true, // target photo uploader (to format uploaded photos in proper layout)
            handlerpath: this.config.getConfig('host') + 'api/videos/thumbuploads',
            pickfilecaption: 'Select Thumbnails',
            uploadfilecaption: 'Start Uploading',
            colcss: 'col-md-4',
            pickbuttoncss: 'btn btn-danger waves-effect waves-light',
            maxfilesize: '8mb',
            chunksize: '8mb',
            headers: {},
            extensiontitle: 'Images Files',
            extensions: 'jpg,jpeg,png',
            filepath: '',
            username: auth.User.username, // 'administrator',
            removehandler: '',
            maxallowedfiles: 1000,
            showFileName: false, // show filename with media file
            showoriginalSize: false, // show media in original size
            value: entity.thumbs
        },
        order: 4
    }));

      return controls.sort((a, b) => a.order - b.order);
  }

  getVideoEditControls(entity: OPTIONS.VideoEntity) {

    const controls: FormBase<any>[] = [];

      controls.push(new Controls.Textbox({
          key: 'title',
          label: 'Title',
          value: entity.title,
          required: true,
          order: 0,
          maxLength: 300,
          placeholder: 'Enter title'
          // helpblock: `Enter post title`
      }));

      controls.push(new Controls.TextArea({
            key: 'description',
            label: 'Description',
            value: entity.description,
            required: false,
            helpblock: `Enter description`,
            order: 1
       }));

       controls.push(new Controls.Textbox({
          key: 'tags',
          label: 'Tags',
          value: entity.tags,
          required: false,
          order: 2,
          maxLength: 1000,
          helpblock: `Enter one or more tags separated by comma`
       }));

       controls.push(new Controls.Dropdown({
            key: 'categories',
            label: 'Select Category',
            value: entity.categories,
            required: true,
            options: [],
            //  maxLength: 150,
            order: 3,
            colsize: 'col-md-12',
       }));

      return controls.sort((a, b) => a.order - b.order);
  }
}
