import { Injectable } from '@angular/core';

import * as OPTIONS from '../videos.model';
import { iStyleConfig, iUploadOptions } from '../../core/core.model';
import { AppConfig } from '../../../configs/app.config';
import { CoreService } from '../../core/coreService';
import { ButtonCSS, ICONCSS, ThemeCSS } from '../../../configs/themeSettings';
import { ContentTypes } from '../../../configs/settings';

@Injectable()
export class SettingsService {

  // configurations
  private apiOptions: OPTIONS.IAPIOptions;
  private Config: iStyleConfig;
  private uploadOptions: iUploadOptions;
  private toolbarOptions: any;
  private searchOptions: any;

  constructor(private coreService: CoreService, private config: AppConfig) {
    const APIURL = config.getConfig('host');
      this.apiOptions = {
            load: APIURL + 'api/videos/load',
            getinfo: APIURL + 'api/videos/getinfo',
            action: APIURL + 'api/videos/action',
            youtube: APIURL + 'api/videos/get_yt_categories',
            fetch_youtube: APIURL + 'api/videos/fetch_youtube',
            remove_audio: APIURL + 'api/videos/remove_audio',
            proc: APIURL + 'api/videos/proc_new',
            editvideo:  APIURL + 'api/videos/editvideos',
            deletevideo: APIURL + 'api/videos/deletevideo',
            encodevideo: APIURL + 'api/ffmpeg/encode',
            load_categories: APIURL + 'api/category/load',
            proc_ffmpeg_videos: APIURL + 'api/videos/ffmpeg_proc',
            aws_proc: APIURL + 'api/videos/aws_proc',
            yt_proc: APIURL + 'api/videos/yt_proc',
            update_video_thumb: APIURL + 'api/videos/update_thumbnail'
      };

      this.init_toolbar_options();
      this.init_search_options();
  }

  init_search_options() {
    this.searchOptions = {
        showpanel: true, // show, hide whole panel
        showSearchPanel: true,
        showAdvanceSearchLink: true,
        term: '',
        topselectioncheck: true,
        navList: [],
        filters: [
        {
            id: 1, title: 'Featured', value: 1, default_value: 3, selected: false, attr: 'isfeatured'
        },
        {
         id: 3, title: 'Adult', value: 1, default_value: 2, selected: false, attr: 'isadult'
        }],
        dropdownFilters: [{
            id: 1,
            value: 0,
            group: 'datefilter',
            caption: 'Added',
            attr: 'datefilter',
            options: [{
                id: 1, title: 'Today', value: 1,
            },
            {
                id: 2, title: 'This Week', value: 2
            },
            {
                id: 3, title: 'This Month', value: 3
            },
            {
                id: 4, title: 'All Time', value: 0
            }]
        }, {
            id: 2,
            value: 'added_date desc',
            group: 'order',
            caption: 'Order',
            attr: 'order',
            options: [{
                id: 1, title: 'Uploaded Date', value: 'date_added desc'
            }, {
                id: 2, title: 'Caption ASC', value: 'title asc',
            }, {
                id: 4, title: 'Caption DESC', value: 'title desc'
            }, {
                id: 5, title: 'Most Viewed', value: 'views desc'
            }, {
                id: 7, title: 'Most Rated', value: 'liked desc'
            }, {
                id: 6, title: 'Approved', value: 'isreviewed desc'
            }, {
                id: 7, title: 'Unapproved', value: 'isreviewed asc'
            }, {
                id: 7, title: 'Enabled', value: 'isenabled desc'
            }, {
                id: 7, title: 'Disabled', value: 'isenabled asc'
            }]
        }],
        checkFilters: [{
            id: 4,
            value: 0,
            group: 'type',
            caption: 'Media Type',
            attr: 'type',
            options: [{
                id: 4, title: 'Videos', value: 0,
            },
            {
                id: 5, title: 'Audio', value: 1
            }]
         }, {
           id: 1,
           value: 2,
           group: 'isenabled',
           caption: 'Enable Status',
           attr: 'isenabled',
           options: [{
               id: 4, title: 'Active', value: 1,
           },
           {
               id: 5, title: 'Inactive', value: 0
           },
           {
             id: 6, title: 'Any', value: 2
          }]
        },
        {
           id: 1,
           value: 2,
           group: 'isreviewed',
           caption: 'Approval Status',
           attr: 'isreviewed',
           options: [{
               id: 4, title: 'Approved', value: 1,
           },
           {
               id: 5, title: 'Under Review', value: 0
           },
           {
               id: 6, title: 'Any', value: 2
           }]
        }, {
                id: 1,
                value: 3,
                group: 'privacy',
                caption: 'Privacy',
                attr: 'isprivate',
                options: [{
                    id: 1, title: 'Private', value: 1,
                },
                {
                    id: 2, title: 'Public', value: 0
                },
                {
                    id: 3, title: 'Unlisted', value: 2
                },
            {
                id: 3, title: 'Any', value: 3
            }]
         }
        ],
        categories: [],
        selectedcategory: '',
        singleaction: false,
        actions: this.navActions()
      };
  }

  navActions () {
      return [
        {
             id: 1,
             title: 'Direct Uploader',
             tooltip: 'Upload Videos',
             row: 2,
             icon: 'icon-movie',
             css: 'btn waves-effect m-b-5 waves-light btn-block btn-primary',
             event: 'add'
         },
         {
            id: 2,
            title: 'FFMPEG Uploader',
            tooltip: 'Upload videos using FFMPEG Uploader',
            row: 2,
            icon: 'icon-movie',
            css: 'btn waves-effect m-b-5 waves-light btn-block btn-primary',
            event: 'ffmpeg'
        },
        {
            id: 3,
            title: 'Youtube Uploader',
            tooltip: 'Upload videos using Youtube Uploader',
            row: 2,
            icon: 'icon-movie',
            css: 'btn waves-effect m-b-5 waves-light btn-block btn-primary',
            event: 'youtube'
        },
        {
            id: 4,
            title: 'AWS Uploader',
            tooltip: 'Upload videos using AWS Uploader',
            row: 2,
            icon: 'icon-movie',
            css: 'btn waves-effect m-b-5 waves-light btn-block btn-primary',
            event: 'aws'
        }
   ];
  }

  init_toolbar_options() {
    this.toolbarOptions = {
        showtoolbar: true,
        showsecondarytoolbar: true,
        showcheckAll: true,
        navbarcss: ThemeCSS.NAVBAR_CSS,
        left_options: [],
        left_caption: '',
        right_caption: '',
        right_options: [],
        actions: []
    };
    for (const type of ContentTypes.MEDIA_TYPES) {
        this.toolbarOptions.left_options.push({
            title: type.title,
            ismultiple: false,
            icon: '', // icon-sort-amount-desc position-left
            Options: [],
            clickevent: 'f_type',
            value: type.value,
            tooltip: 'Load ' + type.title
       });
    }
    this.toolbarOptions.actions.push({
        title: 'Mark As',
        showOnMainBar: false,
        ismultiple: true,
        icon: '',
        Options: [
            { id: '0',
              title: 'Approve',
              value: 1,
              actionstatus: 'approve',
              attr: 'isreviewed',
              isclick: true,
              clickevent: 'm_markas',
              icon: '',
              css: ButtonCSS.SUCCESS_BUTTON,
              tooltip: 'Approve selected records'
            },
        { id: '1', title: 'Enable', value: 1, actionstatus: 'enable',
         attr: 'isenabled', isclick: true, clickevent: 'm_markas',  icon: '',
         css: ButtonCSS.SUCCESS_BUTTON, tooltip: 'Enable selected records' },
        { id: '2', title: 'Disable', value: 0, actionstatus: 'disable',
        attr: 'isenabled', isclick: true, clickevent: 'm_markas',  icon: '',
        css: ButtonCSS.SUCCESS_BUTTON, tooltip: 'Disable selected records' },
        { id: '0', title: 'Normal', value: 1, actionstatus: 'featured',
         attr: 'isfeatured', isclick: true, clickevent: 'm_markas',  icon: '',
         css: ButtonCSS.SUCCESS_BUTTON, tooltip: 'Mark as featured' },
        { id: '1', title: 'Featured', value: 1, actionstatus: 'featured',
         attr: 'isfeatured', isclick: true, clickevent: 'm_markas',  icon: '',
         css: ButtonCSS.SUCCESS_BUTTON, tooltip: 'Mark as featured' },
        { id: '3', title: 'Adult', value: 1, actionstatus: 'adult', isclick: true,
         attr: 'isadult', clickevent: 'm_markas',  icon: '',
         css: ButtonCSS.SUCCESS_BUTTON, tooltip: 'Mark as adult' },
        { id: '4', title: 'Non Adult', value: 0, actionstatus: 'nonadult',
        attr: 'isadult', isclick: true, clickevent: 'm_markas',  icon: '',
        css: ButtonCSS.SUCCESS_BUTTON, tooltip: 'Mark as non adult' },
        { id: '1', title: 'Private', value: 1, actionstatus: 'private',
         attr: 'privacy', isclick: true, clickevent: 'm_markas',  icon: '',
         css: ButtonCSS.SUCCESS_BUTTON, tooltip: 'Mark as private' },
        { id: '2', title: 'Public', value: 0, actionstatus: 'public',
         attr: 'privacy', isclick: true, clickevent: 'm_markas',  icon: '',
         css: ButtonCSS.SUCCESS_BUTTON, tooltip: 'Mark as public' },
         { id: '2', title: 'Delete', value: 0, actionstatus: 'delete',
         css: ButtonCSS.DANGER_BUTTON,
        attr: '', isclick: true, clickevent: 'm_markas', icon:  ICONCSS.DELETE_ICON, tooltip: 'Delete selected records' }]
     });
  }

  /* navigation for profile */
  init_search_profile_options() {
    return {
        showpanel: true, // show, hide whole panel
        showSearchPanel: false,
        term: '',
        topselectioncheck: false,
        navList: this.navList(),
        filters: [],
        dropdownFilters: [],
        categories: [],
        selectedcategory: '',
        singleaction: false,
        actions: this.navActions()
      };
  }

  navList() {
        return [
            { id: 1, title: 'Edit', clickevent: true, event: 'edit', css: '', url: '', icon: '' },
            { id: 2, title: 'Delete', clickevent: true, event: 'delete', css: '', url: '', icon: '' }
        ];
    }

  getApiOptions() {
     return this.apiOptions;
  }

  getConfigs() {
      return this.Config;
  }

  getUploadOptions() {
      return this.uploadOptions;
  }

  getToolbarOptions() {
     return this.toolbarOptions;
  }

  getSearchOptions() {
     return this.searchOptions;
  }

  getSearchOptionsProfile() {
        return this.init_search_profile_options();
  }

  getInitObject(): OPTIONS.VideoEntity {
    return {
        videoid: 0,
        username: '',
        title: '',
        description: '',
        tags: '',
        categories: '',
        files: []

    };
}
}
