import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { select } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';

// services
import { SettingsService } from '../../services/settings.service';
import { DataService } from '../../services/data.service';
import { FormService } from '../../services/form.service';

// shared services
import { CoreService } from '../../../core/coreService';
import { CoreAPIActions } from '../../../core/reducer/actions';

// reducer actions
import { VideoAPIActions } from '../../reducer/actions';
import { fadeInAnimation } from '../../../../animations/core';
import { AppConfig } from '../../../../configs/app.config';


@Component({
  templateUrl: './process.html',
  encapsulation: ViewEncapsulation.None,
  animations: [ fadeInAnimation ]
})
export class AWSUPloaderComponent implements OnInit {

  constructor(
    private dataService: DataService,
    private coreService: CoreService,
    private coreActions: CoreAPIActions,
    private actions: VideoAPIActions,
    private router: Router
    ) {}

    // Website own video categories
    @select(['videos', 'categories'])
    readonly categories$: Observable<any>;

    @select(['videos', 'loading'])
    readonly loading$: Observable<any>;

    @select(['videos', 'isloaded'])
    readonly isloaded$: Observable<any>;

    @select(['videos', 'uploadedfiles'])
    readonly uploadedfiles$: Observable<any>;

    @select(['users', 'auth'])
    readonly auth$: Observable<any>;

    formHeading = 'Upload Videos';
    Auth: any = {};
    showLoader = false;
    Panel = 0; // 0: uploader, 1: input form
    VideoInfo: any = {
       tags: '',
       category: ''
    };

   ngOnInit() {
        // load required categories
        this.dataService.LoadCategories();

        this.auth$.subscribe((auth: any) => {
            this.Auth = auth;
        });
   }

  OnUploadedVideos(videos: any) {
      // prepare output
      for (const video of videos) {
          video.id = this.makeid();
          video.title = '',
          video.sf = video.filename; // backend api need sf as source video file
          video.description = '';
          video.categories = '';
          video.privacy = 0;
          video.username = this.Auth.User.username;
          video.galleryid = 0;
          video.type = 0;
      }

      // update video files in video reducer
      this.actions.uploadedVideoFiles(videos);
  }

  nextPanel() {
    this.Panel = 1;
  }

  backPanel() {
     this.Panel = 0;
  }

  saveChanges() {
    this.uploadedfiles$.subscribe((files: any) => {
        if (files.length > 0) {
           for (const video of files) {
               video.tags = this.VideoInfo.tags;
               video.categories = this.VideoInfo.category;
           }
           this.processVideos(files);
        }
    });
  }

  processVideos(videos: void) {
    this.actions.loadStarted();
    this.dataService.AddAWSVideos(videos).subscribe((data: any) => {

         if (data.status === 'error') {
             this.coreActions.Notify({
                 title: data.message,
                 text: '',
                 css: 'bg-error' });
         } else {
            // clean up search result
            this.actions.resetPublishing();
            // display message
            this.coreActions.Notify({
             title: 'Uploaded videos submitted for processing',
             text: '',
             css: 'bg-success' });

            // enable reload action to refresh data
            this.actions.reloadList();
            // redirect to videos page
            this.router.navigate(['/videos']);
         }
         this.actions.loadEnd(); // hide progressbar
     }, err => {
         this.actions.loadEnd(); // hide progressbar
         this.coreActions.Notify({ title: 'Error Occured', text: '', css: 'bg-danger' });
     });
  }
  makeid() {

    let text = '';
    const possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

    for (let i = 0; i < 5; i++ ) {
      text += possible.charAt(Math.floor(Math.random() * possible.length));
    }
    return text;
  }
}
