import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { select, select$ } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';

import { Router, ActivatedRoute } from '@angular/router';

// services
import { SettingsService } from './services/settings.service';
import { DataService } from './services/data.service';

// shared services
import { CoreService } from '../core/coreService';
import { CoreAPIActions } from '../core/reducer/actions';

// reducer actions
import { VideoAPIActions } from './reducer/actions';
import { fadeInAnimation } from '../../animations/core';

import { PermissionService } from '../../pages/users/services/permission.service';

@Component({
  templateUrl: './videos.html',
  encapsulation: ViewEncapsulation.None,
  animations: [ fadeInAnimation ],
  host: { '[@fadeInAnimation]': '' }
})
export class VideosComponent implements OnInit {

  constructor(
    private settingService: SettingsService,
    private dataService: DataService,
    private coreService: CoreService,
    private coreActions: CoreAPIActions,
    private actions: VideoAPIActions,
    public permission: PermissionService,
    private router: Router,
    private route: ActivatedRoute
    ) {}

   @select(['videos', 'filteroptions'])
   readonly filteroptions$: Observable<any>;

   @select(['videos', 'categories'])
   readonly categories$: Observable<any>;

   @select(['videos', 'itemsselected'])
   readonly isItemSelected$: Observable<any>;

   @select(['videos', 'isloaded'])
   readonly isloaded$: Observable<any>;

   @select(['users', 'auth'])
   readonly auth$: Observable<any>;

     // permission logic
     isAccessGranted = false; // Granc access on resource that can be full access or read only access with no action rights
     isActionGranded = false; // Grand action on resources like add / edit /delete

   heading = 'Videos';
   subheading = 'Management';
   SearchOptions: any;
   ToolbarOptions: any;

   SelectedItems: any; // selected items in list by check / uncheck options
   isItemsSelected = false; // check the status of items there or not
   FilterOptions: any; // local copy of observable query filters

   IsLoaded = false;

   ngOnInit() {
        // user authentication & access right management
        // full resource access key and readonly key can be generated via roles management
        this.auth$.subscribe((auth: any) => {
            const FullAccessID = '1521153486644';
            const ReadOnlyAccessID = '1521395130448';
            if (this.permission.GrandResourceAccess(false, FullAccessID, ReadOnlyAccessID, auth.Role)) {
                this.isAccessGranted = true;
                if (this.permission.GrandResourceAction(FullAccessID, auth.Role)) {
                    this.isActionGranded = true;
                }
            }
        });


        this.SearchOptions = this.settingService.getSearchOptions();
        this.ToolbarOptions = this.settingService.getToolbarOptions();

        this.filteroptions$.subscribe(options => {
            this.FilterOptions = options;
            if (options.track_filter) {
                this.dataService.LoadRecords(options);
                // reset track filter to false again
                options.track_filter = false;
                this.actions.updateFilterOptions(options);
            }
        });
        this.isloaded$.subscribe((loaded: boolean) => {
            if (! loaded) {
                this.dataService.LoadRecords(this.FilterOptions);
            }
        });

      this.isItemSelected$.subscribe((selectedItems: boolean) => {
        this.isItemsSelected = selectedItems;
      });

      this.categories$.subscribe(categories => {
          // prepare data for navigation format
          const _categories = [];
          _categories.push({
             id: '',
             text: ' -- Select --'
          });
          for (const category of categories) {
              _categories.push({
                  id: category.term,
                  text: category.title
              });
          }
          this.SearchOptions.categories = _categories;
      });
      this.route.params.subscribe(params => {
                // this.Params = params;
                if (params['tagname'] !== undefined) {
                    console.log('tag filter initiated');
                    this.FilterOptions.tags = params['tagname'];
                    this.FilterOptions.track_filter = true; // to force triggering load event via obvervable subscription
                    this.actions.updateFilterOptions(this.FilterOptions);
                }
                if (params['catname'] !== undefined) {
                    this.FilterOptions.categories_str = params['catname'];
                    this.FilterOptions.track_filter = true; // to force triggering load event via obvervable subscription
                    this.actions.updateFilterOptions(this.FilterOptions);
                }
                if (params['uname'] !== undefined) {
                    console.log('user filter initiated');
                    console.log(params['uname']);
                    this.FilterOptions.username = params['uname'];
                    this.FilterOptions.track_filter = true; // to force triggering load event via obvervable subscription
                    this.actions.updateFilterOptions(this.FilterOptions);
                }
       });
  }

  loadRecords(options: any) {
    this.dataService.LoadRecords(options);
  }
  selectAll(selectall: boolean) {
     this.actions.selectAll(selectall);
  }
  /* toolbar actions */
  toolbaraction(selection: any) {
    if (!this.isActionGranded) {
        this.coreActions.Notify({
            title: 'Permission Denied',
            text: '',
            css: 'bg-danger' });
        return;
    }
     switch (selection.action) {
          case 'add':
              this.AddRecord();
              return;
          case 'ffmpeg':
              this.router.navigate(['/videos/ffmpeg']);
              return;
          case 'aws':
              this.router.navigate(['/videos/aws']);
              return;
          case 'youtube':
              this.router.navigate(['/videos/youtube']);
              return;
          case 'm_markas':
              this.ProcessActions(selection.value);
              return;
          case 'f_type':
              console.log('f_type clicked ' + selection.value);
              this.actions.applyFilter({ attr: 'type', value: selection.value });
              break;
          case 'f_isapproved':
              this.actions.applyFilter({ attr: 'isapproved', value: selection.value });
              break;
          case 'f_status':
              this.actions.applyFilter({ attr: 'isdisabled', value: selection.value });
              break;
           case 'f_featured':
              this.actions.applyFilter({ attr: 'isfeatured', value: selection.value });
              break;
          case 'f_adult':
              this.actions.applyFilter({ attr: 'isadult', value: selection.value });
              break;
          case 'pagesize':
              this.actions.applyFilter({ attr: 'pagesize', value: selection.value });
              break;

          case 'f_private':
              this.actions.applyFilter({ attr: 'privacy', value: selection.value });
              break;
          
          case 'm_filter':
              this.actions.applyFilter({ attr: 'datefilter', value: selection.value });
              break;
          case 'sort':
              this.actions.applyFilter({ attr: 'direction', value: selection.value });
              break;
      }
  }

  /* find records event trigger */
  FindRecords(filters: any) {
    const _filterOptions = filters.filters;
     // reset some attributes if search / find record is used to avoid any confusion in search listing
     _filterOptions.tags = '';
     _filterOptions.categories_str = '';
     _filterOptions.username = '';
    _filterOptions.pagenumber = 1;
    _filterOptions.track_filter = true; // to force triggering load event via obvervable subscription
    this.actions.updateFilterOptions(_filterOptions);
}

   AddRecord() {
        this.router.navigate(['/videos/directuploader/0']);
   }


  getSelectedItems(arr: any) {
      this.SelectedItems = arr;
      if (this.SelectedItems.length > 0) {
         this.isItemsSelected = true;
      } else {
        this.isItemsSelected = false;
      }
   }

   ProcessActions(selection: any) {
    if (this.SelectedItems.length > 0) {
       for (const item of this.SelectedItems) {
           item.actionstatus = selection.actionstatus;
       }
       this.dataService.ProcessActions(this.SelectedItems, selection);
    }
 }
}
