import { NgModule } from '@angular/core';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { Routes, RouterModule } from '@angular/router';

import { VideosComponent } from './videos.component';
import { VideoProfileModule } from './profile/process.module';

import { UserListComponent } from './partials/list.component';
import { ViewComponent } from './partials/modal.component';
import { VideoProfileComponent } from './profile/process.component';

// uploaders
import { DirectVideoUploaderComponent } from './uploaders/direct/process.component';
import { DirectVideoUploadModule } from './uploaders/direct/process.module';

import { EmbedYoutubeUploaderComponent } from './uploaders/youtube/process.component';
import { YoutubeVideoModule } from './uploaders/youtube/process.module';

import { FFMPEGUPloaderComponent } from './uploaders/ffmpeg/process.component';
import { FFMPEGVideoModule } from './uploaders/ffmpeg/process.module';

import { AWSUPloaderComponent } from './uploaders/aws/process.component';
import { AWSVideoModule } from './uploaders/aws/process.module';
/* services */
import { SettingsService } from './services/settings.service';
import { DataService } from './services/data.service';

/* actions */
import { VideoAPIActions } from './reducer/actions';

import { PartialModule } from '../../partials/shared.module';

const routes: Routes = [{
  path: '',
  data: {
      title: 'Videos Management',
      urls: [{title: 'Dashboard', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Management'}]
    },
  component: VideosComponent
}, {
  path: 'tag/:tagname',
  data: {
    title: 'Videos Management',
    urls: [{title: 'Dashboard', url: '/'},
    {title: 'Videos', url: '/videos'},
    {title: 'Management'}]
  },
component: VideosComponent
}, {
  path: 'category/:catname',
  data: {
    title: 'Videos Management',
    urls: [{title: 'Dashboard', url: '/'},
    {title: 'Videos', url: '/videos'},
    {title: 'Management'}]
  },
component: VideosComponent
}, {
  path: 'user/:uname',
  data: {
    title: 'Videos Management',
    urls: [{title: 'Dashboard', url: '/'},
    {title: 'Videos', url: '/videos'},
    {title: 'Management'}]
  },
component: VideosComponent
}, {
  path: 'profile/:id',
  data: {
      title: 'Video Information',
      urls: [{title: 'Dashboard', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Video Information'}]
    },
    component: VideoProfileComponent
}, {
  path: 'directuploader/:id',
  data: {
      title: 'Direct Video Uploaders',
      urls: [{title: 'Dashboard', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Direct Video Uploader'}]
    },
    component: DirectVideoUploaderComponent
}, {
  path: 'youtube',
  data: {
      title: 'Youtube Uploader',
      urls: [{title: 'Dashboard', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Upload Youtube Videos'}]
    },
    component: EmbedYoutubeUploaderComponent
}, {
  path: 'ffmpeg',
  data: {
      title: 'FFMPEG Uploader',
      urls: [{title: 'Dashboard', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'FFMPEG Video Uploader'}]
    },
    component: FFMPEGUPloaderComponent
}, {
  path: 'aws',
  data: {
      title: 'AWS Video Uploader',
      urls: [{title: 'Dashboard', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'AWS Upload Videos'}]
    },
    component: AWSUPloaderComponent
}];

@NgModule({
  imports: [
      CommonModule,
      FormsModule,
      PartialModule,
      VideoProfileModule,
      DirectVideoUploadModule,
      YoutubeVideoModule,
      FFMPEGVideoModule,
      AWSVideoModule,
      NgbModule.forRoot(),
      RouterModule.forChild(routes)
  ],
  declarations: [
      VideosComponent,
      UserListComponent,
      ViewComponent
  ],
  entryComponents: [
    ViewComponent
  ],
  exports: [
    VideosComponent
  ],
  providers: [
    SettingsService, DataService, VideoAPIActions
  ]
})
export class VideosModule { }
