import { NgModule } from '@angular/core';
import { RouterModule } from '@angular/router';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';

import { FooterComponent } from './footer.component';
import { NavBarComponent } from './navbar.component';
import { SideBarComponent } from './sidebar.component';

import { PaginationComponent } from './components/pagination/pagination.component';
import { LoaderComponent } from './components/loader/loader.component';
import { PageHeaderComponent } from './components/pageheader/pageheader.component';
import { AlertComponent } from './components/alert/alert.component';

import { SpinnerModule} from './components/loader/types/spinner.module';


import { NoRecordFoundComponent } from './bootstrap4/components/norecord/norecord.component';

/* services */
import { NotifyService } from './components/pnotify/pnotify.service';

/* Select 2 */
import { Select2Directive } from './directives/select2.directive';

/* pluloader */
import { PlUploadDirective } from './directives/uploader/plupload';
// import { PlImageReplacerComponent } from './directives/image-replacer/imagereplacer';
/* Core Service */
import { CoreService } from '../pages/core/coreService';
import { CoreAPIActions } from '../pages/core/reducer/actions';
/* validators */
// import { BirthYearValidatorDirective } from './components/dynamicform/validators/birthyear';

/* bootstrap 4 components */
import { BootstrapNavigationComponent } from './bootstrap4/components/navigation/navigation.component';
import { BootstrapAlertComponent } from './bootstrap4/components/alert/alert.component';
import { BootstrapToolbarComponent } from './bootstrap4/components/toolbar/toolbar.component';

/* Dynamic Form Components */
import { DynamicFormComponent } from './components/dynamicform/form.component';
import { DynamicFormControlComponent } from './components/dynamicform/form.control';
import { DynamicModalFormComponent } from './components/dynamicform/dynamic-modal-form';
import { FileDisplayComponent } from './components/dynamicform/components/display/display.component';
import { DisplayAudioComponent } from './components/dynamicform/components/display/audio/audio.component';
import { DisplayImageComponent } from './components/dynamicform/components/display/image/image.component';
import { DisplayVideoComponent } from './components/dynamicform/components/display/video/video.component';
import { DisplayFileComponent } from './components/dynamicform/components/display/file/file.component';

import { TinymceModule } from 'angular2-tinymce';
import {SelectModule} from 'ng2-select';


// video uploader
import { VideoUploadComponent } from '../pages/videos/uploaders/ffmpeg/partials/uploader';
@NgModule({
  imports: [
    CommonModule,
    RouterModule,
    SpinnerModule,
    FormsModule,
    ReactiveFormsModule,
    TinymceModule.withConfig({}),
    SelectModule
  ],
  declarations: [
    BootstrapNavigationComponent,
    BootstrapAlertComponent,
    BootstrapToolbarComponent,
    FooterComponent,
    NavBarComponent,
    SideBarComponent,
    PaginationComponent,
    LoaderComponent,
    PageHeaderComponent,
    Select2Directive,
    AlertComponent,
    DynamicFormComponent,
    DynamicFormControlComponent,
    DynamicModalFormComponent,
    FileDisplayComponent,
    DisplayAudioComponent,
    DisplayImageComponent,
    DisplayVideoComponent,
    DisplayFileComponent,
    PlUploadDirective,
    NoRecordFoundComponent,
    VideoUploadComponent
  ],
  exports: [
    BootstrapNavigationComponent,
    BootstrapAlertComponent,
    BootstrapToolbarComponent,
    FooterComponent,
    NavBarComponent,
    SideBarComponent,
    PaginationComponent,
    LoaderComponent,
    PageHeaderComponent,
    Select2Directive,
    AlertComponent,
    DynamicFormComponent,
    DynamicFormControlComponent,
    DynamicModalFormComponent,
    FileDisplayComponent,
    DisplayAudioComponent,
    DisplayImageComponent,
    DisplayVideoComponent,
    DisplayFileComponent,
    PlUploadDirective,
    NoRecordFoundComponent,
    VideoUploadComponent
  ],
  providers: [NotifyService, CoreService, CoreAPIActions]
})
export class PartialModule {}
