import { Inject, Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';

@Injectable()
export class AppConfig {

    configUrl = '../assets/account/configs'; // ../assets/account/configs (for application integration), 'assets/configs' for development

    private config: any = {
        host: '[HOST]'
    };

    // global js values passed from external application
    private global_vars: any = {
        userid: 0,
        username: '',
        culture: '',
        message: '',
        videouploadoption: 0,
        isdelete: true // optional disable delete in case of app in demo purpose for public.
    };


    private env:    any = null;

    constructor(private http: HttpClient) {
         if (window['Ang_UID'] !== undefined) {
            this.global_vars.userid = parseInt(window['Ang_UID'], 10);
         }
         if (window['Ang_UserName'] !== undefined) {
            this.global_vars.username = window['Ang_UserName'];
         }
         if (window['Ang_AppMessage'] !== undefined) {
            this.global_vars.message = window['Ang_AppMessage'];
         }
         if (window['Ang_Culture'] !== undefined) {
            this.global_vars.culture = window['Ang_Culture'];
         }
         if (window['Ang_VideoUploaderOption'] !== undefined) {
            this.global_vars.videouploadoption = window['Ang_VideoUploaderOption'];
         }

         if (window['Ang_en_delete'] !== undefined) {
            this.global_vars.isdelete = window['Ang_en_delete'];
         }
    }

    /**
     * Use to get the data found in the second file (config file)
     */
    public getConfig(key: any) {
        return this.config[key];
    }

    /**
     * Use to get the data found in the first file (env file)
     */
    public getEnv(key: any) {
        return this.env[key];
    }

    /**
     * Use to get the value of global variable
     */
    public getGlobalVar(key: any) {
        return this.global_vars[key];
    }

    /**
     * This method:
     *   a) Loads "env.json" to get the current working environment (e.g.: 'production', 'development')
     *   b) Loads "config.[env].json" to get all env's variables (e.g.: 'config.development.json')
     */
    public load() {

        this.http.get(this.configUrl + '/env.json')
            .subscribe((env: any) => {
                this.env = env;
                console.log('env is');
                console.log(this.env);
                let configUrl = 'config.development.json';
                switch (this.env.env) {
                    case 'production':
                        configUrl = 'config.production.json';
                    break;
                    case 'development':
                        configUrl = 'config.development.json';
                    break;
                }
                this.http.get(this.configUrl + '/' + configUrl)
                    .subscribe((config: any) => {
                        this.config = config;
                        console.log('config is');
                        console.log(this.config);
                }, err => {
                    console.error('Error loading config file....');
                });
        }, err => {
            console.log(err);
            console.error('Error loading env file....');
        });
    }
}
