import { Injectable } from '@angular/core';
import { UserAPIActions } from '../reducer/actions';
import { CookieService } from 'ngx-cookie-service';
import { HttpClient } from '@angular/common/http';
import { SettingsService } from './settings.service';

// Actions
import { CoreAPIActions } from '../../core/reducer/actions';
import { VideoAPIActions } from '../../videos/reducer/actions';

@Injectable()
export class UserService {

  constructor(

    private settings: SettingsService,
    private http: HttpClient,
    private actions: UserAPIActions,
    private cookieService: CookieService,
    private coreActions: CoreAPIActions,
    private videoActions: VideoAPIActions) { }

    SignOut() {
        this.cookieService.delete('_AUTH');
        this.actions.SignOut();
    }
   CheckAuthentication() {
       const _auth = this.cookieService.get('_AUTH');
       if ( _auth !== undefined && _auth !== '') {
            // authenticated (authenticate until user full data receiving)
            this.actions.Authenticate({
                isAuthenticated: true,
                User: JSON.parse(_auth)
            });
            this.coreActions.Notify({
                title: 'Logged in successfully',
                text: '',
                css: 'bg-success' });
             /*this.GetInfo(parseInt(_auth, 10)).subscribe( (data: any) => {
                if (data.status === 'error') {
                    this.coreActions.Notify({
                        title: data.message,
                        text: '',
                        css: 'bg-success' });
                } else {
                    const obj = {
                        isAuthenticated: true,
                        User: data.post
                    };
                    console.log('AUTHENTICATION');
                    this.actions.Authenticate(obj);
                }
           });*/
       }
  }

  AuthorizeUser(userid: number) {
    this.GetInfo(userid).subscribe( (data: any) => {
        if (data.status === 'error') {
            this.coreActions.Notify({
                title: data.message,
                text: '',
                css: 'bg-success' });
        } else {
            const obj = {
                isAuthenticated: true,
                User: data.post
            };
            this.actions.Authenticate(obj);
            // update filter of all contents with authenticated user
            this.videoActions.updateFilter(obj.User);
        }
   });

  }
  GetInfo(userid: number) {
      const URL = this.settings.getApiOptions().getinfo;
      return this.http.post(URL, JSON.stringify({userid}));
  }

}
