import { Injectable } from '@angular/core';
import { UserAPIAction, UserAPIActions } from '../reducer/actions';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { select } from '@angular-redux/store';

import { SettingsService } from './settings.service';
import { CoreAPIActions } from '../../core/reducer/actions';


@Injectable()
export class DataService {

  // headers  = new Headers({ 'Content-Type': 'application/json' }); // ... Set content type to JSON

  constructor(

    private settings: SettingsService,
    private http: HttpClient,
    private actions: UserAPIActions,
    private coreActions: CoreAPIActions) { }

  LoadRecords(FilterOptions) {
      const URL = this.settings.getApiOptions().load;
      this.actions.loadStarted();
      this.http.post(URL, JSON.stringify(FilterOptions))
        .subscribe((data: any) => {
          // update core data
          this.actions.loadSucceeded(data);
          if (data.categories.length > 0) {
              // if enabled, api send list of categories too
              // update categories in state
              this.actions.updateCategories(data.categories);
          }
          // update list stats
          this.coreActions.refreshListStats(
              { totalrecords: data.records,
                pagesize:  FilterOptions.pagesize,
                pagenumber: FilterOptions.pagenumber
              });
      }, err => {
          this.actions.loadFailed(err);
      });
  }
  AddRecord(obj: any) {
      let API_URL = '';
      // 2: edit profile, 3: change email, 4: change password
      switch (obj.viewType) {
         case 2:
             API_URL = this.settings.getApiOptions().proc;
         break;
         case 3:
             API_URL = this.settings.getApiOptions().cemail;
         break;
         case 4:
            API_URL = this.settings.getApiOptions().chpass;
         break;
      }
      return this.http.post(API_URL, JSON.stringify(obj));
  }

    UpdateThumb(info: any, images: any) {
        const URL = this.settings.getApiOptions().updatethumb;
        const param: any = {};
        param.userid = info.userid;
        for (const image of images) {
            param.picturename = image.fname;
        }
        console.log(param);
        this.http.post(URL, JSON.stringify(param))
        .subscribe((data: any) => {
            if (data.status === 'error') {
                this.coreActions.Notify({
                    title: data.message,
                    text: '',
                    css: 'bg-danger' });
            } else {
                this.actions.UpdateThumb(data.record);
                this.coreActions.Notify({
                    title: 'Profile Photo Updated',
                    text: '',
                    css: 'bg-success' });
            }
        }, err => {
            this.coreActions.Notify({
                title: err,
                text: '',
                css: 'bg-danger' });
        });
    }
  /* get single record */
  GetInfo(userid: number) {
      const URL = this.settings.getApiOptions().getinfo;
      return this.http.post(URL, JSON.stringify({userid}));
  }

  Authenticate(user: any) {
        const URL = this.settings.getApiOptions().authenticate;
        return this.http.post(URL, JSON.stringify(user));
  }

  GetUserLog(username: string) {
        const URL = this.settings.getApiOptions().userlog;
        return this.http.post(URL, JSON.stringify({username}));
  }
  DeleteRecord(item, index) {
    item.actionstatus = 'delete';
        const arr = [];
        arr.push(item);
        this.ProcessActions(arr, 'delete');
    }
    /* execute actions */
    ProcessActions(SelectedItems, status) {
    // apply changes directory instate
    this.actions.applyChanges({
        SelectedItems, status
    });

    const arr = [];
    for (const item of SelectedItems) {
        arr.push({
            userid: item.userid,
            actionstatus: item.actionstatus
        });
    }
    // update database
    this.http.post(this.settings.getApiOptions().action, JSON.stringify(arr))
        .subscribe((data: any) => {
        // this.coreActions.Notify(data.message);
        let message = 'Operation Performed';
        if (status === 'delete') {
            message = 'Record Removed';
        }
        this.coreActions.Notify({
            title: message,
            text: '',
            css: 'bg-success' });
    }, err => {
        this.coreActions.Notify({ title: 'Error Occured', text: '', css: 'bg-danger' });
    });

    }
    /* execute actions */
    ProcessLogActions(SelectedItems, status) {
        this.coreActions.Notify({
            title: 'Feature not yet implemented',
            text: '',
            css: 'bg-success' });
    }

}
