import { Injectable } from '@angular/core';

import * as OPTIONS from '../model';
import { iUploadOptions } from '../../core/core.model';
import { AppConfig } from '../../../configs/app.config';
import { CoreService } from '../../core/coreService';
import { ButtonCSS, ICONCSS, ThemeCSS } from '../../../configs/themeSettings';

@Injectable()
export class SettingsService {

  // configurations
  private apiOptions: OPTIONS.IAPIOptions;
  private uploadOptions: iUploadOptions;
  private toolbarOptions: any;
  private searchOptions: any;

  constructor(private coreService: CoreService, private config: AppConfig) {
      const APIURL = config.getConfig('host');
      this.apiOptions = {
            load: APIURL + 'api/user/load',
            getinfo: APIURL + 'api/user/getinfo',
            action: APIURL + 'api/user/action',
            proc: APIURL + 'api/user/proc',
            updatethumb: APIURL + 'api/user/updatethumb',
            cemail: APIURL + 'api/user/cemail',
            chpass: APIURL + 'api/user/chpass_acc',
            ctype: APIURL + 'api/user/ctype',
            userlog: APIURL + 'api/user/userlog',
            authenticate: APIURL + 'api/user/authenticate'
      };

      this.init_toolbar_options();
      this.init_search_options();

  }
  init_search_options() {
      this.searchOptions = this.init_search_main_options();
  }
  /* navigation for listing */
  init_search_main_options() {
    return {
        showpanel: true, // show, hide whole panel
        showSearchPanel: true,
        term: '',
        topselectioncheck: true,
        showAdvanceSearchLink: true,
        navList: [],
        filters: [],
        dropdownFilters: [{
         id: 1,
         value: 0,
         group: 'datefilter',
         caption: 'Registered',
         attr: 'datefilter',
         options: [{
             id: 1, title: 'Today', value: 1,
         },
         {
             id: 2, title: 'This Week', value: 2
         },
         {
             id: 3, title: 'This Month', value: 3
         },
         {
             id: 4, title: 'All Time', value: 0
         }]
        }, {
            id: 1,
            value: 'register_date desc',
            group: 'order',
            caption: 'Registered',
            attr: 'order',
            options: [{
                id: 1, title: 'Account Created', value: 'register_date desc'
            }, {
                id: 2, title: 'Name', value: 'firstname asc, username asc',
            }, {
                id: 4, title: 'Most Viewed', value: 'views desc'
            }, {
                id: 5, title: 'Last Signed In', value: 'last_login desc'
            }]
        }],
        checkFilters: [{
            id: 1,
            value: 3,
            group: 'usertype',
            caption: 'User Type',
            attr: 'type',
            options: [{
                id: 0, title: 'Admin', value: 1,
            },
            {
                id: 1, title: 'Member', value: 0
            },
            {
              id: 2, title: 'Any', value: 3
           }]
         }, {
           id: 1,
           value: 2,
           group: 'status',
           caption: 'User Status',
           attr: 'isenabled',
           options: [{
               id: 1, title: 'Active', value: 1,
           },
           {
               id: 2, title: 'Inactive', value: 0
           },
           {
             id: 3, title: 'Any', value: 2
          }]
        },
        {
           id: 2,
           value: '',
           group: 'gender',
           caption: 'Gender',
           attr: 'gender',
           options: [{
               id: 4, title: 'Male', value: 'Male',
           },
           {
               id: 5, title: 'Female', value: 'Female'
           },
           {
               id: 6, title: 'Any', value: ''
           }]
        }
        ],
        categories: [],
        selectedcategory: '',
        singleaction: false,
        actions: [
             {
                 id: 1,
                 title: 'Create Account',
                 tooltip: 'Create new account',
                 row: 1,
                 icon: 'icon-file-plus',
                 options: {},
                 css: 'btn waves-effect m-b-5 waves-light btn-block btn-success',
                 event: 'add'
             }
        ]
      };
  }
  /* navigation for profile */
  init_search_profile_options() {
    return {
        showpanel: true, // show, hide whole panel
        showSearchPanel: false,
        term: '',
        topselectioncheck: false,
        navList: this.navList(),
        filters: [],
        dropdownFilters: [],
        categories: [],
        selectedcategory: '',
        singleaction: false,
        actions: [
             {
                 id: 1,
                 title: 'Create Account',
                 tooltip: 'Create new account',
                 row: 1,
                 icon: 'icon-file-plus',
                 options: {},
                 css: 'btn waves-effect m-b-5 waves-light btn-block btn-success',
                 event: 'add'
             }
        ]
      };
  }

  navList() {
      return [
            { id: 8, title: 'View Profile', clickevent: true, event: 'toggle_viewprofile', css: '', url: '', icon: '' },
            { id: 1, title: 'Edit Profile', clickevent: true, event: 'edit_profile', css: '', url: '', icon: '' },
            { id: 2, title: 'Change Password', clickevent: true, event: 'change_password', css: '', url: '', icon: '' },
            { id: 3, title: 'Change Email', clickevent: true, event: 'change_email', css: '', url: '', icon: '' },
            { id: 7, title: 'Change Member Type', clickevent: true, event: 'change_usertype', css: '', url: '', icon: '' },
            { id: 8, title: 'View History', clickevent: true, event: 'toggle_userhistory', css: '', url: '', icon: '' }
            // { id: 7, title: 'View History', clickevent: false, css: '', url: '/users/history/[id]', icon: '' },
      ];
  }


  init_toolbar_options() {
    this.toolbarOptions = {
        showtoolbar: false,
        showsecondarytoolbar: true,
        showcheckAll: true,
        navbarcss: ThemeCSS.NAVBAR_CSS,
        left_options: [],
        left_caption: 'Filter:',
        right_caption: '',
        right_options: [],
        actions: []
    };

     this.toolbarOptions.actions.push({
        title: 'Mark As',
        showOnMainBar: false,
        ismultiple: true,
        icon: '',
        Options: [
        { id: '1', title: 'Enable', value: 1, actionstatus: 'enable',
         attr: 'isenabled', isclick: true, clickevent: 'm_markas',  icon: '',
         css: ButtonCSS.SUCCESS_BUTTON, tooltip: 'Enable selected records' },
        { id: '2', title: 'Disable', value: 0, actionstatus: 'disable',
        attr: 'isenabled', isclick: true, clickevent: 'm_markas',  icon: '',
        css: ButtonCSS.SUCCESS_BUTTON, tooltip: 'Disable selected records' },
        { id: '2', title: 'Archive', value: 0, actionstatus: 'delete',
         css: ButtonCSS.DANGER_BUTTON,
        attr: '', isclick: true, clickevent: 'm_markas', icon:  ICONCSS.DELETE_ICON, tooltip: 'Archive selected records' }]
     });
  }
  init_toolbar_user_log_options() {
    const options = {
        showtoolbar: false,
        showsecondarytoolbar: true,
        showcheckAll: true,
        navbarcss: ThemeCSS.NAVBAR_CSS,
        left_options: [],
        left_caption: 'Filter:',
        right_caption: '',
        right_options: [],
        actions: []
    };
    options.actions.push({
        title: 'Mark As',
        showOnMainBar: false,
        ismultiple: true,
        icon: '',
        Options: [
        { id: '2', title: 'Delete All', value: 0, actionstatus: 'delete',
         css: ButtonCSS.DANGER_BUTTON,
        attr: '', isclick: true, clickevent: 'm_log_markas', icon:  ICONCSS.DELETE_ICON, tooltip: 'Clear all log' }]
     });
     return options;
  }
  getApiOptions() {
     return this.apiOptions;
  }

  getUploadOptions() {
      return this.uploadOptions;
  }

  getToolbarOptions() {
     return this.toolbarOptions;
  }

  getUserLogToolbarOptions() {
      return this.init_toolbar_user_log_options();
  }

  getSearchOptions() {
     return this.searchOptions;
  }

  getSearchOptionsProfile() {
      return this.init_search_profile_options();
  }
  getInitObject(): OPTIONS.UserEntity {
    return {
       userid: 0,
       type: 0,
       username: '',
       email: '',
       firstname: '',
       lastname: '',
       opassword: '',
       password: '',
       cpassword: '',
       aboutme: '',
       gender: '',
       website: '',
       isallowbirthday: 1,
       relationshipstatus: 'Single',
       hometown: '',
       currentcity: '',
       zipcode: '',
       countryname: '',
       interests: '',
       description: '',
       files: [],
       picture: ''
    };
 }
}
