import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { select, select$ } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';

import { Router, ActivatedRoute } from '@angular/router';

// services
import { SettingsService } from '../services/settings.service';
import { DataService } from '../services/data.service';

// shared services
import { CoreService } from '../../core/coreService';
import { CoreAPIActions } from '../../core/reducer/actions';

// reducer actions
import { VideoAPIActions } from '../reducer/actions';
import { fadeInAnimation } from '../../../animations/core';

import { AppConfig } from '../../../configs/app.config';

@Component({
  templateUrl: './favorites.html',
  encapsulation: ViewEncapsulation.None,
  animations: [ fadeInAnimation ],
  host: { '[@fadeInAnimation]': '' }
})
export class FavoritedVideosComponent implements OnInit {

  constructor(
    private settingService: SettingsService,
    private dataService: DataService,
    private coreService: CoreService,
    private coreActions: CoreAPIActions,
    private actions: VideoAPIActions,
    private router: Router,
    private route: ActivatedRoute,
    private config: AppConfig
    ) {}

   @select(['videos', 'filteroptions'])
   readonly filteroptions$: Observable<any>;

   // @select(['videos', 'categories'])
   // readonly categories$: Observable<any>;

   @select(['videos', 'itemsselected'])
   readonly isItemSelected$: Observable<any>;

   @select(['videos', 'favorite_isloaded'])
   readonly isloaded$: Observable<any>;

   @select(['videos', 'favorite_records'])
   readonly records$: Observable<any>;

   @select(['videos', 'favorite_pagination'])
   readonly pagination$: Observable<any>;

   @select(['users', 'auth'])
    readonly auth$: Observable<any>;

   SearchOptions: any;
   ToolbarOptions: any;

   SelectedItems: any; // selected items in list by check / uncheck options
   isItemsSelected = false; // check the status of items there or not
   FilterOptions: any; // local copy of observable query filters

   IsLoaded = false;
   Records = 0;
   Pagination: any = {};
   User: any = {};
   ngOnInit() {

        // reset list stats
        this.coreActions.resetStats();

        this.SearchOptions = this.settingService.getSearchOptions();
        this.ToolbarOptions = this.settingService.getToolbarOptions(1);

        this.filteroptions$.subscribe(options => {
            this.FilterOptions = options;
            if (options.track_filter) {
                this.dataService.LoadRecords(options);
                // reset track filter to false again
                options.track_filter = false;
                this.actions.updateFilterOptions(options);
            }
        });

        this.records$.subscribe(records => {
            this.Records = records;
        });

        this.pagination$.subscribe(pagination => {
            this.Pagination = pagination;
        });

        this.auth$.subscribe(Info => {
            this.User = Info.User;
        });

        this.isloaded$.subscribe((loaded: boolean) => {
            if (! loaded) {
                this.dataService.LoadFavoriteRecords(this.FilterOptions);
            } else {
                 // loaded data from reducer store (cache)
                 // update pagination (records & pagesize on load)
                 this.refreshStats();
            }
        });

      this.isItemSelected$.subscribe((selectedItems: boolean) => {
        this.isItemsSelected = selectedItems;
      });

  }

  loadRecords(options: any) {
      options.loadfavorites = true;
      this.dataService.LoadFavoriteRecords(options);
  }

  selectAll(selectall: boolean) {

     this.actions.selectAll({
         type : 1,
         checked: selectall
     });
  }

  /* toolbar actions */
  toolbaraction(selection: any) {
     switch (selection.action) {
          case 'add':
              this.AddRecord();
              return;
          case 'm_markas':
              this.ProcessActions(selection.value);
              return;
          case 'f_type':
              this.actions.applyFilter({ attr: 'type', value: selection.value });
              break;
          case 'pagesize':
              this.actions.applyFilter({ attr: 'pagesize', value: selection.value });
              break;
          case 'f_private':
              this.actions.applyFilter({ attr: 'privacy', value: selection.value });
              break;
          case 'm_filter':
              this.actions.applyFilter({ attr: 'datefilter', value: selection.value });
              break;
          case 'sort':
              this.actions.applyFilter({ attr: 'direction', value: selection.value });
              break;
      }
  }

  /* find records event trigger */
  FindRecords(filters: any) {
        const _filterOptions = filters.filters;
        // reset some attributes if search / find record is used to avoid any confusion in search listing
        _filterOptions.tags = '';
        _filterOptions.categories_str = '';
        _filterOptions.username = '';
        _filterOptions.pagenumber = 1;
        _filterOptions.track_filter = true; // to force triggering load event via obvervable subscription
        this.actions.updateFilterOptions(_filterOptions);
    }

   AddRecord() {
       switch (this.config.getGlobalVar('videouploadoption')) {
           case 0:
                // ffmpeg uploader
                this.router.navigate(['/videos/ffmpeg']);
           break;
           case 1:
               // youtube uploader
               this.router.navigate(['/videos/youtube']);
           break;
           case 2:
               // direct uploader
               this.router.navigate(['/videos/directuploader/0']);
           break;
           case 3:
                // aws uploader
                this.router.navigate(['/videos/aws']);
           break;
       }

   }

  getSelectedItems(arr: any) {
      this.SelectedItems = arr;
      if (this.SelectedItems.length > 0) {
         this.isItemsSelected = true;
      } else {
         this.isItemsSelected = false;
      }
   }

   ProcessActions(selection: any) {
        if (this.SelectedItems.length > 0) {
            for (const item of this.SelectedItems) {
                item.actionstatus = selection.actionstatus;
            }
            this.dataService.ProcessActions(this.SelectedItems, selection, 1);
        }
    }

    refreshStats() {
        this.actions.refresh_pagination({
            type: 0, // 0: my , 1: favorites, 2: liked, 3: playlist
            totalrecords: this.Records,
            pagesize:  this.FilterOptions.pagesize
       });
       // refresh list states
       this.coreActions.refreshListStats({
            totalrecords: this.Records,
            pagesize:  this.FilterOptions.pagesize,
            pagenumber: this.Pagination.currentPage
       });
    }
}
