import { Component, OnInit, EventEmitter, Output } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import { select } from '@angular-redux/store';
import { VideoAPIActions } from '../reducer/actions';
import { DataService } from '../services/data.service';
import { Router } from '@angular/router';

// import { CoreService } from '../../core/coreService';
@Component({
  selector: 'app-favlist',
  templateUrl: './favlist.html'
})
export class FavListComponent implements OnInit {

  constructor(
    private actions: VideoAPIActions,
    private dataService: DataService,
    private router: Router
  ) {}

  @select(['videos', 'favorite_posts'])
  readonly Data$: Observable<any>;

  @select(['videos', 'loading'])
  readonly loading$: Observable<boolean>;

  @select(['videos', 'favorite_pagination'])
  readonly pagination$: Observable<any>;

  @select(['videos', 'selectall'])
  readonly selectAll$: Observable<any>;

  @Output() View = new EventEmitter<any>();
  @Output() SelectedItems = new EventEmitter<any>();

  selectall = false;

  ngOnInit() {
    this.selectAll$.subscribe((selectall: boolean) => {
      this.selectall = selectall;
      this.checkChange();
    });
  }

  // Since we're observing an array of items, we need to set up a 'trackBy'
  // parameter so Angular doesn't tear down and rebuild the list's DOM every
  // time there's an update.
  getKey(_, item: any) {
    return item.videoid;
  }

  /* pagination click event */
  PaginationChange(value: number) {
      // update filter option to query database
      this.actions.applyFilter({ attr: 'pagenumber', value: value });
      // update pagination current page (to hightlight selected page)
      this.actions.updatePaginationCurrentPage({ currentpage: value });
  }

  delete(item: any, index: number, event) {
    const r = confirm('Are you sure you want to delete selected record?');
    if (r === true) {
      this.dataService.DeleteRecord(item, index, 1);
    }
  }

  processChange() {
     this.actions.selectAll({
        type : 1,
        checked: this.selectall
     });
  }

checkChange() {
    this.Data$.subscribe(items => {
      const _items = [];
      for (const item of items) {
        if (item.Selected) {
          _items.push(item);
        }
      }
      this.SelectedItems.emit(_items);
  });
}
}
