import { VideoAPIAction, VideoAPIActions, VideosBLL } from './actions';
import { IVideoState, IPagination, IFilterOption } from './model';
import { tassign } from 'tassign';
import { Action } from 'redux';

const INITIAL_STATE: IVideoState = {
    posts: [],
    records: 0,
    loading: false,
    error: null,
    pagination: IPagination,
    filteroptions: IFilterOption,
    categories: [],
    yt_categories: [],
    yt_result: [],
    selectall: false,
    itemsselected: false,
    isloaded: false,
    uploadedfiles: [],
    favorite_posts: [],
    favorite_records: 0,
    favorite_isloaded: false,
    favorite_pagination: IPagination,
    liked_posts: [],
    liked_records: 0,
    liked_loaded: false,
    liked_pagination: IPagination,
    playlist_posts: [],
    playlist_records: 0,
    playlist_loaded: false,
    playlist_pagination: IPagination,
};

export function createVideosReducer() {
  return function videosReducer(state: IVideoState = INITIAL_STATE,
    a: Action): IVideoState {

    const action = a as VideoAPIAction;

    const bll = new VideosBLL();
    if (!action.meta) {
      return state;
    }
    switch (action.type) {
          case VideoAPIActions.IS_ITEM_SELECTED:
             return tassign(state, { itemsselected: action.payload });

          case VideoAPIActions.SELECT_ALL:
             return bll.selectAll(state, action);

          case VideoAPIActions.LOAD_STARTED:
            return tassign(state, { loading: true, error: null});

          case VideoAPIActions.LOAD_SUCCEEDED:
            return bll.loadSucceeded(state, action);

          case VideoAPIActions.LOAD_FAVORITE_SUCCEEDED:
            return bll.loadFavoriteSucceeded(state, action);

          case VideoAPIActions.LOAD_LIKED_SUCCEEDED:
            return bll.loadLikeSucceeded(state, action);

          case VideoAPIActions.LOAD_PLAYLIST_SUCCEEDED:
            return bll.loadPlaylistSucceeded(state, action);

          case VideoAPIActions.LOAD_FAILED:
              return tassign(state, { loading: false, error: action.error});

          /* update wholefilter options */
          case VideoAPIActions.UPDATE_FILTER_OPTIONS:
              return tassign(state, { filteroptions: Object.assign({}, state.filteroptions, action.payload) });

          /* update specific filter option */
          case VideoAPIActions.APPLY_FILTER:
             return bll.applyFilterChanges(state, action);

          /* update pagination current page */
          case VideoAPIActions.UPDATE_PAGINATION_CURRENTPAGE:
             return bll.updatePagination(state, action);

          /* add record */
          case VideoAPIActions.ADD_RECORD:
             return bll.addRecord(state, action);

          /* update record state */
          case VideoAPIActions.UPDATE_RECORD:
             return bll.updateRecord(state, action);

          case VideoAPIActions.UPDATE_CATEGORIES:
             return tassign(state, { categories: action.payload });

          /* apply changes (multiple selection items e.g delete selected records or enable selected records) */
          case VideoAPIActions.APPLY_CHANGES:
             return bll.applyChanges(state, action);

          case VideoAPIActions.UPDATE_USER:
             return bll.updateUserFilter(state, action);

          // Youtube Uploader Reducer Calls
          case VideoAPIActions.LOAD_YT_CATEGORIES:
             return tassign(state, { yt_categories: action.payload });

          case VideoAPIActions.LOAD_YT_SEARCH:
             return tassign(state, { yt_result: action.payload, loading: false });

          // remove loader
          case VideoAPIActions.LOAD_END:
             return tassign(state, { loading: false });

          // cleanup youtube search result
          case VideoAPIActions.REMOVE_YT_SEARCH:
             return tassign(state, { yt_result: [] });

          case VideoAPIActions.UPLOADED_FILES:
             return tassign(state, { uploadedfiles: action.payload });


          case VideoAPIActions.RESET_PUBLISHING:
             return tassign(state, { uploadedfiles: [] });

          case VideoAPIActions.UPDATE_VIDEO_FILE:
             return bll.updateVideoFile(state, action);

          case VideoAPIActions.REFRESH_PAGINATION:
             return bll.refreshpagination(state, action);

          case VideoAPIActions.REFRESH_DATA:
             const filterOptions = state.filteroptions;
             filterOptions.track_filter = true;
             return tassign(state, { filteroptions: Object.assign({}, state.filteroptions, filterOptions) });
      }

    return state;
  };
}
