import { Injectable } from '@angular/core';
import { VideoAPIActions } from '../reducer/actions';
import { HttpClient } from '@angular/common/http';
import { SettingsService } from './settings.service';
import { CoreAPIActions } from '../../core/reducer/actions';
import { AppConfig } from '../../../configs/app.config';

@Injectable()
export class DataService {

  constructor(

    private settings: SettingsService,
    private http: HttpClient,
    private config: AppConfig,
    private actions: VideoAPIActions,
    private coreActions: CoreAPIActions) { }

  LoadRecords(FilterOptions) {
      FilterOptions.loadliked = false;
      FilterOptions.loadfavorites = false;
      FilterOptions.loadPlayList = false;
      const URL = this.settings.getApiOptions().load;
      this.actions.loadStarted();
      this.http.post(URL, JSON.stringify(FilterOptions))
        .subscribe((data: any) => {
          // update core data
          this.actions.loadSucceeded(data);
          if (data.categories.length > 0) {
              // if enabled, api send list of categories too
              // update categories in state
              this.actions.updateCategories(data.categories);
          }
          // update list stats
          this.coreActions.refreshListStats(
              { totalrecords: data.records,
                pagesize:  FilterOptions.pagesize,
                pagenumber: FilterOptions.pagenumber
              });
      }, err => {
          this.actions.loadFailed(err);
      });
  }

  LoadFavoriteRecords(FilterOptions) {
        FilterOptions.loadfavorites = true;
        const URL = this.settings.getApiOptions().load;
        this.actions.loadStarted();
        this.http.post(URL, JSON.stringify(FilterOptions))
        .subscribe((data: any) => {
            // update core data
            this.actions.loadFavoriteSucceeded(data);
            // update list stats
            this.coreActions.refreshListStats(
                { totalrecords: data.records,
                pagesize:  FilterOptions.pagesize,
                pagenumber: FilterOptions.pagenumber
                });
        }, err => {
            this.actions.loadFailed(err);
        });
  }

  LoadLikedRecords(FilterOptions) {
         FilterOptions.loadliked = true;
        const URL = this.settings.getApiOptions().load;
        this.actions.loadStarted();
        this.http.post(URL, JSON.stringify(FilterOptions))
        .subscribe((data: any) => {
            // update core data
            this.actions.loadLikeSucceeded(data);
            // update list stats
            this.coreActions.refreshListStats(
                {
                    totalrecords: data.records,
                    pagesize:  FilterOptions.pagesize,
                    pagenumber: FilterOptions.pagenumber
                });
        }, err => {
            this.actions.loadFailed(err);
        });
  }

  LoadPlaylistRecords(FilterOptions) {
        const URL = this.settings.getApiOptions().load;
        this.actions.loadStarted();
        FilterOptions.loadPlayList = true;
        this.http.post(URL, JSON.stringify(FilterOptions))
        .subscribe((data: any) => {
            // update core data
            this.actions.loadPlaylistSucceeded(data);
            // update list stats
            this.coreActions.refreshListStats(
                { totalrecords: data.records,
                pagesize:  FilterOptions.pagesize,
                pagenumber: FilterOptions.pagenumber
                });
        }, err => {
            this.actions.loadFailed(err);
        });
  }
  LoadCategories() {
        const URL = this.settings.getApiOptions().load_categories;
        this.http.post(URL, JSON.stringify({ type: 0, status: 1, issummary: false, isdropdown: true}))
        .subscribe((data: any) => {
            this.actions.updateCategories(data.categorylist);
        }, err => {
            this.actions.loadFailed(err);
        });
    }

  LoadYoutubeCategories() {
        const URL = this.settings.getApiOptions().youtube;
        this.http.post(URL, {})
        .subscribe((data: any) => {
            this.actions.loadYoutubeCategories(data.categorylist);
        }, err => {
            this.actions.loadFailed(err);
        });
   }

   SearchYoutube(FilterOptions) {

        const URL = this.settings.getApiOptions().fetch_youtube;
        this.actions.loadStarted();
        this.http.post(URL, JSON.stringify(FilterOptions))
        .subscribe((data: any) => {
            // update core data
            if (data.status === 'error') {
                this.coreActions.Notify({
                    title: data.message,
                    text: '',
                    css: 'bg-success' });
            } else {
                this.actions.loadYoutubeSearchResult(data.posts);
            }
        }, err => {
            this.actions.loadFailed(err);
        });
    }

  AddRecord(obj) {
    return this.http.post(this.settings.getApiOptions().proc, JSON.stringify(obj));
  }
  AddYoutubeVideos(arr) {
    return this.http.post(this.settings.getApiOptions().yt_proc, JSON.stringify(arr));
  }

  AddFFMPEGVideos(arr) {
    return this.http.post(this.settings.getApiOptions().proc_ffmpeg_videos, JSON.stringify(arr));
  }

  AddAWSVideos(arr) {
    return this.http.post(this.settings.getApiOptions().aws_proc, JSON.stringify(arr));
  }

  EncodeVideo(obj) {
    return this.http.post(this.settings.getApiOptions().encodevideo, JSON.stringify([obj]));
  }

  EditRecord(obj) {
    return this.http.post(this.settings.getApiOptions().editvideo, JSON.stringify(obj));
  }

  DeletePhotos(obj) {
     return this.http.post(this.settings.getApiOptions().deletevideo, JSON.stringify(obj));
  }

  Authorize_Author(obj) {
    return this.http.post(this.settings.getApiOptions().authorize_author, JSON.stringify(obj));
  }

  UpdateThumbnail(obj) {
    return this.http.post(this.settings.getApiOptions().update_video_thumb, JSON.stringify(obj));
  }

  /* get single record */
  GetInfo(videoid: number) {
      const URL = this.settings.getApiOptions().getinfo;
      return this.http.post(URL, JSON.stringify({videoid}));
  }

  DeleteRecord(item, index, type) {
        if (this.config.getGlobalVar('isdelete')) {
            item.actionstatus = 'delete';
            const arr = [];
            arr.push(item);
            this.ProcessActions(arr, 'delete', type);
        }
  }
    /* execute actions */
    ProcessActions(SelectedItems, status, type) {
         // apply changes directory instate
         this.actions.applyChanges({
            SelectedItems, status,
            type: type
        });

        // update database
        this.http.post(this.settings.getApiOptions().action, JSON.stringify(SelectedItems))
            .subscribe((data: any) => {
            // this.coreActions.Notify(data.message);
            let message = 'Operation Performed';
            if (status === 'delete') {
                message = 'Record removed from your list';
            }
            this.coreActions.Notify({
                title: message,
                text: '',
                css: 'bg-success' });
        }, err => {
            this.coreActions.Notify({ title: 'Error Occured', text: '', css: 'bg-danger' });
        });

    }
}
