import { Injectable } from '@angular/core';

import * as OPTIONS from '../videos.model';
import { iStyleConfig, iUploadOptions } from '../../core/core.model';
import { AppConfig } from '../../../configs/app.config';
import { CoreService } from '../../core/coreService';
import { ButtonCSS, ICONCSS, ThemeCSS } from '../../../configs/themeSettings';

@Injectable()
export class SettingsService {

  // configurations
  private apiOptions: OPTIONS.IAPIOptions;
  private Config: iStyleConfig;
  private uploadOptions: iUploadOptions;
  private toolbarOptions: any;
  private searchOptions: any;

  constructor(private coreService: CoreService, private config: AppConfig) {
    const APIURL = config.getConfig('host');
      this.apiOptions = {
            load: APIURL + 'api/videos/load',
            getinfo: APIURL + 'api/videos/getinfo_acc',
            action: APIURL + 'api/videos/action',
            youtube: APIURL + 'api/videos/get_yt_categories',
            fetch_youtube: APIURL + 'api/videos/fetch_youtube',
            remove_audio: APIURL + 'api/videos/remove_audio',
            proc: APIURL + 'api/videos/proc_new',
            yt_proc: APIURL + 'api/videos/yt_proc',
            editvideo:  APIURL + 'api/videos/editvideos_macc',
            deletevideo: APIURL + 'api/videos/deletevideo',
            encodevideo: APIURL + 'api/ffmpeg/encode',
            load_categories: APIURL + 'api/category/load',
            proc_ffmpeg_videos: APIURL + 'api/videos/ffmpeg_proc',
            aws_proc: APIURL + 'api/videos/aws_proc',
            authorize_author: APIURL + 'api/videos/authorize_author',
            update_video_thumb: APIURL + 'api/videos/update_thumbnail'
      };

      this.init_search_options();
  }

  init_search_options() {
    this.searchOptions = {
        showpanel: true, // show, hide whole panel
        showSearchPanel: true,
        term: '',
        topselectioncheck: true,
        singleaction: false,
        actions: this.navActions()
      };
  }

  navActions () {
      return [
        {
             id: 3,
             title: 'Upload',
             tooltip: 'Upload Videos',
             icon: 'icon-movie',
             options: {},
             css: 'btn btn-success',
             event: 'add'
         }
   ];
  }

  init_toolbar_options(type: number) {
    this.toolbarOptions = {
        showtoolbar: false,
        showsecondarytoolbar: true,
        showcheckAll: true,
        navbarcss: ThemeCSS.NAVBAR_CSS,
        left_options: [],
        left_caption: '',
        right_caption: '',
        right_options: [],
        actions: []
    };

    if (type === 0) {
        // my videos options
        this.toolbarOptions.actions.push({
            title: 'Mark As',
            showOnMainBar: false,
            ismultiple: true,
            icon: '',
            Options: [
            { id: '1', title: 'Private', value: 1, actionstatus: 'private',
             attr: 'privacy', isclick: true, clickevent: 'm_markas',  icon: '',
             css: ButtonCSS.PRIMARY_BUTTON, tooltip: 'Mark as private' },
            { id: '2', title: 'Public', value: 0, actionstatus: 'public',
             attr: 'privacy', isclick: true, clickevent: 'm_markas',  icon: '',
             css: ButtonCSS.PRIMARY_BUTTON, tooltip: 'Mark as public' },
             { id: '2', title: 'Delete', value: 0, actionstatus: 'delete',
             css: ButtonCSS.DANGER_BUTTON,
            attr: '', isclick: true, clickevent: 'm_markas', icon:  ICONCSS.DELETE_ICON, tooltip: 'Delete selected records' }]
         });
    } else if (type === 1) {
        // my favorite options
        this.toolbarOptions.actions.push({
            title: 'Mark As',
            showOnMainBar: false,
            ismultiple: true,
            icon: '',
            Options: [
             { id: '2', title: 'Delete', value: 0, actionstatus: 'delete_fav',
             css: ButtonCSS.DANGER_BUTTON,
             attr: '', isclick: true, clickevent: 'm_markas',
             icon:  ICONCSS.DELETE_ICON,
             tooltip: 'Remove selected item from favorited list' }]
         });
    } else if (type === 2) {
        // my liked options
        this.toolbarOptions.actions.push({
            title: 'Mark As',
            showOnMainBar: false,
            ismultiple: true,
            icon: '',
            Options: [
             { id: '2', title: 'Delete', value: 0, actionstatus: 'delete_like',
             css: ButtonCSS.DANGER_BUTTON,
             attr: '', isclick: true,
             clickevent: 'm_markas',
             icon:  ICONCSS.DELETE_ICON,
              tooltip: 'Remove selected record from liked list' }]
         });
    } else if (type === 3) {
        // my playlist options
        this.toolbarOptions.actions.push({
            title: 'Mark As',
            showOnMainBar: false,
            ismultiple: true,
            icon: '',
            Options: [
             { id: '2', title: 'Delete', value: 0, actionstatus: 'delete_playlist',
             css: ButtonCSS.DANGER_BUTTON,
             attr: '', isclick: true,
             clickevent: 'm_markas',
             icon:  ICONCSS.DELETE_ICON,
              tooltip: 'Delete selected playlsits' }]
         });
    }

  }

  getApiOptions() {
     return this.apiOptions;
  }

  getConfigs() {
      return this.Config;
  }

  getUploadOptions() {
      return this.uploadOptions;
  }

  getToolbarOptions(type: number) {
    this.init_toolbar_options(type);
     return this.toolbarOptions;
  }

  getSearchOptions() {
     return this.searchOptions;
  }

  getInitObject(): OPTIONS.VideoEntity {
    return {
        videoid: 0,
        username: '',
        title: '',
        description: '',
        tags: '',
        categories: '',
        files: []
    };
}
}
