import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { select } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';

// services
import { SettingsService } from '../../services/settings.service';
import { DataService } from '../../services/data.service';
import { FormService } from '../../services/form.service';

// shared services
import { CoreService } from '../../../core/coreService';
import { CoreAPIActions } from '../../../core/reducer/actions';

// reducer actions
import { VideoAPIActions } from '../../reducer/actions';
import { fadeInAnimation } from '../../../../animations/core';
import { AppConfig } from '../../../../configs/app.config';


@Component({
  templateUrl: './process.html',
  encapsulation: ViewEncapsulation.None,
  animations: [ fadeInAnimation ]
})
export class FFMPEGUPloaderComponent implements OnInit {

  constructor(
    private dataService: DataService,
    private coreService: CoreService,
    private coreActions: CoreAPIActions,
    private actions: VideoAPIActions,
    private router: Router
    ) {}

    // Website own video categories
    @select(['videos', 'categories'])
    readonly categories$: Observable<any>;

    @select(['videos', 'loading'])
    readonly loading$: Observable<any>;

    @select(['videos', 'isloaded'])
    readonly isloaded$: Observable<any>;

    @select(['videos', 'uploadedfiles'])
    readonly uploadedfiles$: Observable<any>;

    @select(['users', 'auth'])
    readonly auth$: Observable<any>;

    formHeading = 'Upload Videos';
    Auth: any = {};

    UploadedFiles: any = [];
    showSubmitBtn = false;
    selectedFile: any = {};
    EncodingIndex = 0; // track encoding file
    showLoader = false;
    Panel = 0; // 0: uploader, 1: input form
    VideoInfo: any = {
       tags: '',
       category: ''
    };

   ngOnInit() {
        // load required categories
        this.dataService.LoadCategories();

        this.auth$.subscribe((auth: any) => {
            this.Auth = auth;
        });
   }

  OnUploadedVideos(videos: any) {
      // prepare output
      for (const video of videos) {
          video.id = this.makeid();
          video.sf = video.filename;
          video.pf = video.fileIndex + '-' + this.makeid() + '_pub.mp4',
          video.status = 'start';
          video.errorcode = 0;
          video.progress = 0;
          video.progressStyle = {
            'width': '0%'
          }, // for progress bar
          video.title = '',
          video.description = '';
          video.categories = '';
          // categories_arr: $scope.Info.categories_arr,
          video.categories = '';
          video.privacy = 0;
          video.username = this.Auth.User.username;
          video.galleryid = 0;
          video.thumbs = [];
          video.processID = 0;
          video.panelstatus = 0; // 0: progress bar, 1: show processing, 2: preview video
          video.defaultImg = '';
      }

      // update video files in video reducer
      this.actions.uploadedVideoFiles(videos);

      this.publish();
  }

  publish() {

      this.uploadedfiles$.subscribe((files: any) => {
          if (files.length > 0) {
             if (this.EncodingIndex < files.length) {
                 // update selected video in video reducer store
                 // this.actions.selectedVideoFile(files[this.EncodingIndex]);
                 this.selectedFile = files[this.EncodingIndex];
                 this.encodeVideo();
             } else {
                 this.finalOutput();
             }
          }
      });
  }

  encodeVideo() {
    this.selectedFile.status = 'progress';

    this.dataService.EncodeVideo({
        sf: this.selectedFile.sf,
        pf: this.selectedFile.pf,
        tp: 0,
        username: this.selectedFile.username
    }).subscribe((data: any) => {
          if (parseInt(data.ecode, 10) > 0) {
             this.selectedFile.errorcode = parseInt(data.ecode, 10);
             this.selectedFile.edesc = data.edesc;
             this.EncodingIndex++;
             this.publish();
          } else {
            this.selectedFile.processID = data.procid;
            this.selectedFile.IntervalID =  setInterval(() => {
                this.getProgress();
              }, 1000);
            }
            // update video output in reducer
            this.actions.updateVideoFile(this.selectedFile);
     }, err => {
          this.coreActions.Notify({ title: 'Error occured while encoding video', text: '', css: 'bg-danger' });
     });
  }

  getProgress() {
      this.dataService.EncodeVideo({
          pid: this.selectedFile.processID,
          tp: 1,
          username: this.selectedFile.username
      }).subscribe((data: any) => {
          this.selectedFile.progress = parseInt(data.status, 10);
          this.selectedFile.progressStyle = {
            'width': this.selectedFile.progress + '%'
          };
          if (parseInt(data.status, 10) >= 100) {
              this.selectedFile.status = 'completed';
              if (this.selectedFile.IntervalID !== 0) {
                  clearInterval(this.selectedFile.IntervalID);
              }
              // update video output in reducer
              this.actions.updateVideoFile(this.selectedFile);
              this.getInfo();
          }
        }, err => {
              this.coreActions.Notify({ title: 'Error occured while encoding video', text: '', css: 'bg-danger' });
        });
  }

  getInfo() {
    this.selectedFile.panelstatus = 1; // 0: progress bar, 1: show processing, 2: preview video
    this.dataService.EncodeVideo({
        pid: this.selectedFile.processID,
        sf: this.selectedFile.sf,
        pf: this.selectedFile.pf,
        tp: 2,
        username: this.selectedFile.username
    }).subscribe((data: any) => {
            if (data.status === 'OK') {
              this.selectedFile.panelstatus = 2; // 0: progress bar, 1: show processing, 2: preview video
              this.selectedFile.errorcode = data.ecode;
              this.selectedFile.pf = data.fname;
              this.selectedFile.duration = data.dur;
              this.selectedFile.dursec = data.dursec;
              this.selectedFile.tfile = data.tfile;
              this.selectedFile.edesc = data.edesc;
              this.selectedFile.isenable = data.isenable;
              this.selectedFile.fIndex = data.fIndex;
              this.selectedFile.img_url = data.img_url;
              console.log('thumb index');
              console.log(data.fIndex);
              if (data.fIndex !== undefined) {
                  for (let i = 1; i <= 14; i++) {
                      let _name = '';
                      if (i <= 9) {
                        _name = data.fIndex + '00' + i + '.jpg';
                      } else {
                        _name = data.fIndex + '0' + i + '.jpg';
                      }
                      let _selected = false;
                      if (i === 8) {
                        this.selectedFile.defaultImg = _name;
                        _selected = true;
                      } else {
                        _selected = false;
                      }
                      this.selectedFile.thumbs.push({
                          id: i,
                          filename: _name,
                          selected: _selected
                      });
                  }
              }
              // update video output in reducer
              this.actions.updateVideoFile(this.selectedFile);
          } else {
            // $scope.ffmpeg.EncodingObj[$scope.ffmpeg.publishedFiles].status = "checkfailed";
          }

          this.EncodingIndex++;
          this.publish();

      }, err => {
            this.coreActions.Notify({ title: 'Error occured while encoding video', text: '', css: 'bg-danger' });
      });
  }

  finalOutput() {
     console.log('final output triggered');
     this.showSubmitBtn = true;
  }

  updateSelectedThumb(thumb, file, event) {
    file.defaultImg = thumb.filename;
    this.selectedFile.defaultImg = thumb.filename;
    // reset all thumbnail selection.
    if (this.selectedFile.thumbs.length > 0) {
       for (const thumbnail of this.selectedFile.thumbs) {
          if (thumbnail.id === thumb.id) {
             thumbnail.selected = true;
          } else {
             thumbnail.selected = false;
          }
       }
    }
    this.actions.updateVideoFile(this.selectedFile);
    event.stopPropagation();
  }

  nextPanel() {
     this.Panel = 1;
  }

  saveChanges() {
    this.uploadedfiles$.subscribe((files: any) => {
        if (files.length > 0) {
           for (const video of files) {
               video.tags = this.VideoInfo.tags;
               video.categories = this.VideoInfo.category;
           }
           this.processVideos(files);
        }
    });
  }

  processVideos(videos: void) {
    this.actions.loadStarted();
    this.dataService.AddFFMPEGVideos(videos).subscribe((data: any) => {

         if (data.status === 'error') {
             this.coreActions.Notify({
                 title: data.message,
                 text: '',
                 css: 'bg-error' });
         } else {
            // clean up search result
            this.actions.resetPublishing();
            // display message
            this.coreActions.Notify({
             title: 'All uploaded videos processed successfully',
             text: '',
             css: 'bg-success' });

            // enable reload action to refresh data
            this.actions.reloadList();
            // redirect to videos page
            this.router.navigate(['/videos']);
         }
         this.actions.loadEnd(); // hide progressbar
     }, err => {
         this.actions.loadEnd(); // hide progressbar
         this.coreActions.Notify({ title: 'Error Occured', text: '', css: 'bg-danger' });
     });
  }
  makeid() {

    let text = '';
    const possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

    for (let i = 0; i < 5; i++ ) {
      text += possible.charAt(Math.floor(Math.random() * possible.length));
    }
    return text;
  }
}
