import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { select } from '@angular-redux/store';
import { Observable } from 'rxjs/Observable';

// services
import { SettingsService } from '../../services/settings.service';
import { DataService } from '../../services/data.service';
import { FormService } from '../../services/form.service';

// shared services
import { CoreService } from '../../../core/coreService';
import { CoreAPIActions } from '../../../core/reducer/actions';

// reducer actions
import { VideoAPIActions } from '../../reducer/actions';
import { fadeInAnimation } from '../../../../animations/core';
import { AppConfig } from '../../../../configs/app.config';


@Component({
  templateUrl: './process.html',
  encapsulation: ViewEncapsulation.None,
  animations: [ fadeInAnimation ]
})
export class EmbedYoutubeUploaderComponent implements OnInit {

  constructor(
    private dataService: DataService,
    private coreService: CoreService,
    private coreActions: CoreAPIActions,
    private actions: VideoAPIActions,
    private router: Router
    ) {}

    // Website own video categories
    @select(['videos', 'categories'])
    readonly categories$: Observable<any>;

    // Youtube categories
    @select(['videos', 'yt_categories'])
    readonly YoutubeCategories$: Observable<any>;

    // Youtube Search Result
    @select(['videos', 'yt_result'])
    readonly YoutubeSearchResult$: Observable<any>;

    @select(['videos', 'loading'])
    readonly loading$: Observable<any>;

    @select(['videos', 'isloaded'])
    readonly isloaded$: Observable<any>;

    @select(['users', 'auth'])
    readonly auth$: Observable<any>;

    YoutubeQuery: any = {
        term: '',
        advanceoption: false,
        username: '',
        youtubecategory: '',
        order: 0,
        date: 0,
    };

    YoutubeOrderList: any = [
        { id: 0, title: 'Date' },
        { id: 1, title: 'Rating' },
        { id: 2, title: 'Relevance' },
        { id: 3, title: 'Title' },
        { id: 4, title: 'VideoCount' },
        { id: 5, title: 'ViewCount' }];

    YoutubeDateList: any = [
        { id: 3, title: 'All Time' },
        { id: 0, title: 'Today' },
        { id: 1, title: 'This Week' },
        { id: 2, title: 'This Month' }];

    YoutubeResult: any = {
        tags: '',
        category: ''
    };

    YoutubeSearchResult: any = [];

    formHeading = 'Upload Videos';

    IsLoaded = false;

    step1 = true;
    step2 = false;

    Auth: any = {};

    // Categories = [];
   ngOnInit() {
        // load youtube categories
        this.dataService.LoadYoutubeCategories();

        this.isloaded$.subscribe((loaded: boolean) => {
                this.IsLoaded = loaded;
                if (! this.IsLoaded) {
                    this.router.navigate(['/videos/']);
                }
        });
        this.auth$.subscribe((auth: any) => {
            this.Auth = auth;
        });
        this.YoutubeSearchResult$.subscribe((result: any) => {
            this.YoutubeSearchResult = result;
        });

       // fetch param from url
       /*this.route.params.subscribe(params => {
           // initialize controls with default values
           this.Info = this.settingService.getInitObject();
           // this.initializeControls(this.Info);
       });*/

       /*this.categories$.subscribe((categories: any) => {
            this.Categories = categories;
            // this.updateCategories();
      });*/
   }


   submitYoutube() {
      this.dataService.SearchYoutube(this.YoutubeQuery);
   }

   showAdvanceOptions(event: any) {
       this.YoutubeQuery.advanceoption = true;
       event.stopPropagation();
   }

   nextstep() {
       this.step1 = false;
       this.step2 = true;
   }

   saveChanges() {
       console.log('submitted');
       console.log(this.YoutubeSearchResult);

       if (this.YoutubeSearchResult.length === 0) {
            this.coreActions.Notify({
                title: 'Please select atleast one video to proceed',
                text: '',
                css: 'bg-danger' });
            return;
       }

       for (const item of this.YoutubeSearchResult) {
           item.tags = this.YoutubeResult.tags;
           item.categories = this.YoutubeResult.category;
       }

       this.actions.loadStarted();
       this.dataService.AddYoutubeVideos(this.YoutubeSearchResult).subscribe((data: any) => {

            if (data.status === 'error') {
                this.coreActions.Notify({
                    title: data.message,
                    text: '',
                    css: 'bg-error' });
            } else {
               // clean up search result
               this.actions.cleanupYoutubeSearchResult();
               // display message
               this.coreActions.Notify({
                title: 'Videos processed successfully',
                text: '',
                css: 'bg-success' });

                // enable reload action to refresh data
                this.actions.reloadList();
               // redirect to videos page
               this.router.navigate(['/videos']);
            }
            this.actions.loadEnd(); // hide progressbar
        }, err => {
            this.actions.loadEnd(); // hide progressbar
            this.coreActions.Notify({ title: 'Error Occured', text: '', css: 'bg-danger' });
        });
   }

   removeYt_Video(index, event) {
        if (index > -1) {
            this.YoutubeSearchResult.splice(index, 1);
        }
        event.stopPropagation = true;
   }
}
