import { NgModule } from '@angular/core';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { Routes, RouterModule } from '@angular/router';

import { VideosComponent } from './videos.component';

import { EditVideoComponent } from './edit/process.component';
import { VideoEditModule } from './edit/process.module';

import { ViewComponent } from './partials/modal.component';

// Other pages
import { FavoritedVideosComponent } from './favorites/favorites.component';
import { LikedVideosComponent } from './liked/liked.component';
import { PlaylistVideosComponent } from './playlists/playlist.component';

// Partial Components
import { ListComponent } from './partials/list.component';
import { FavListComponent } from './partials/favlist.component';
import { LikedListComponent } from './partials/likedlist.component';
import { PlayListPartialComponent } from './partials/playlist.component';
// video uploaders
import { DirectVideoUploaderComponent } from './uploaders/direct/process.component';
import { DirectVideoUploadModule } from './uploaders/direct/process.module';

import { EmbedYoutubeUploaderComponent } from './uploaders/youtube/process.component';
import { YoutubeVideoModule } from './uploaders/youtube/process.module';

import { FFMPEGUPloaderComponent } from './uploaders/ffmpeg/process.component';
import { FFMPEGVideoModule } from './uploaders/ffmpeg/process.module';

import { AWSUPloaderComponent } from './uploaders/aws/process.component';
import { AWSVideoModule } from './uploaders/aws/process.module';

/* services */
import { SettingsService } from './services/settings.service';
import { DataService } from './services/data.service';

/* actions */
import { VideoAPIActions } from './reducer/actions';

import { PartialModule } from '../../partials/shared.module';
import { NavigationMenuIndex } from '../../configs/settings';

const routes: Routes = [{
  path: '',
  data: {
      topmenuIndex: NavigationMenuIndex.TOPMENU_VIDEOS_INDEX,
      leftmenuIndex: NavigationMenuIndex.VIDEOS_MY_INDEX,
      title: 'My Account',
      urls: [{title: 'My Account', url: '/'},
      {title: 'Manage Videos'}]
    },
  component: VideosComponent
}, {
  path: 'edit/:id',
  data: {
      topmenuIndex: NavigationMenuIndex.TOPMENU_VIDEOS_INDEX,
      leftmenuIndex: NavigationMenuIndex.VIDEOS_MY_INDEX,
      title: 'My Account',
      urls: [{title: 'My Account', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Edit Video Information'}]
    },
    component: EditVideoComponent
}, {
  path: 'directuploader/:id',
  data: {
      topmenuIndex: NavigationMenuIndex.TOPMENU_VIDEOS_INDEX,
      leftmenuIndex: NavigationMenuIndex.VIDEOS_MY_INDEX,
      title: 'My Account',
      urls: [{title: 'My Account', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Upload Videos'}]
    },
    component: DirectVideoUploaderComponent
}, {
  path: 'youtube',
  data: {
      topmenuIndex: NavigationMenuIndex.TOPMENU_VIDEOS_INDEX,
      leftmenuIndex: NavigationMenuIndex.VIDEOS_MY_INDEX,
      title: 'My Account',
      urls: [{title: 'My Account', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Upload Videos'}]
    },
    component: EmbedYoutubeUploaderComponent
}, {
  path: 'ffmpeg',
  data: {
      topmenuIndex: NavigationMenuIndex.TOPMENU_VIDEOS_INDEX,
      leftmenuIndex: NavigationMenuIndex.VIDEOS_MY_INDEX,
      title: 'My Account',
      urls: [{title: 'My Account', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Upload Videos'}]
    },
    component: FFMPEGUPloaderComponent
}, {
  path: 'aws',
  data: {
      topmenuIndex: NavigationMenuIndex.TOPMENU_VIDEOS_INDEX,
      leftmenuIndex: NavigationMenuIndex.VIDEOS_MY_INDEX,
      title: 'My Account',
      urls: [{title: 'My Account', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Upload Videos'}]
    },
    component: AWSUPloaderComponent
}, {
  path: 'favorites',
  data: {
      topmenuIndex: NavigationMenuIndex.TOPMENU_VIDEOS_INDEX,
      leftmenuIndex: NavigationMenuIndex.VIDEOS_FAVORITES_INDEX,
      title: 'My Account',
      urls: [{title: 'My Account', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Favorited Videos'}]
    },
    component: FavoritedVideosComponent
}, {
  path: 'liked',
  data: {
      topmenuIndex: NavigationMenuIndex.TOPMENU_VIDEOS_INDEX,
      leftmenuIndex: NavigationMenuIndex.VIDEOS_LIKED_INDEX,
      title: 'My Account',
      urls: [{title: 'My Account', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'Liked Videos'}]
    },
    component: LikedVideosComponent
}, {
  path: 'playlists',
  data: {
      topmenuIndex: NavigationMenuIndex.TOPMENU_VIDEOS_INDEX,
      leftmenuIndex: NavigationMenuIndex.VIDEOS_PLAYLIST_INDEX,
      title: 'My Account',
      urls: [{title: 'My Account', url: '/'},
      {title: 'Videos', url: '/videos'},
      {title: 'My Playlists'}]
    },
    component: PlaylistVideosComponent
}];

@NgModule({
  imports: [
      CommonModule,
      FormsModule,
      PartialModule,
      DirectVideoUploadModule,
      YoutubeVideoModule,
      FFMPEGVideoModule,
      AWSVideoModule,
      VideoEditModule,
      NgbModule.forRoot(),
      RouterModule.forChild(routes)
  ],
  declarations: [
      VideosComponent,
      ListComponent,
      FavListComponent,
      LikedListComponent,
      PlayListPartialComponent,
      ViewComponent,
      LikedVideosComponent,
      PlaylistVideosComponent,
      FavoritedVideosComponent
  ],
  entryComponents: [ViewComponent],
  exports: [
    VideosComponent
  ],
  providers: [
    SettingsService, DataService, VideoAPIActions
  ]
})
export class VideosModule { }
