import { Component, Input, Output, OnInit, OnChanges, EventEmitter, ChangeDetectionStrategy  } from '@angular/core';
import { FormGroup } from '@angular/forms';

import { FormBase } from './model/base';
import { ControlService } from './services/control.service';
import { ThemeCSS } from '../../../configs/themeSettings';
@Component({
  selector: 'dynamic-modal-form',
  changeDetection: ChangeDetectionStrategy.OnPush,
  templateUrl: './dynamic-modal-form.html',
  providers: [ ControlService ]
})
export class DynamicModalFormComponent implements OnInit, OnChanges {

  @Input() controls: FormBase<any>[] = [];
  @Input() showCancel = true;
  @Input() showModal = true;
  @Input() submitText = 'Submit';
  @Input() submitCss = '';
  @Output() OnClose = new EventEmitter<any>();
  @Output() OnSubmit = new EventEmitter<any>();
  @Output() OnDropdownSelection = new EventEmitter<any>();
  @Output() FileRemoved = new EventEmitter<any>();

  tempSubmitText = '';
  @Input() showLoader = false;
  disableSubmit = false;
  form: FormGroup;
  payLoad = '';
  isSubmit = false;
  submitBtnCss = ThemeCSS.SUBMIT_BUTTON_CSS;

  constructor(private qcs: ControlService) {  }

  ngOnInit() {
    if (this.submitCss !== '') {
       this.submitBtnCss = this.submitCss;
    }
    this.generateForm();
  }

  ngOnChanges() {
     this.generateForm();
  }

  generateForm() {
    if (this.controls.length > 0) {
      this.form = this.qcs.toFormGroup(this.controls);
    }
  }

  uploadCompleted(files: any) {
    this.submitText = this.tempSubmitText;
    this.disableSubmit = false;
  }

  uploadProgress(status: any) {
      this.disableSubmit = true;
      this.tempSubmitText = this.submitText;
      this.submitText = 'Uploading...';
  }

  OnDropdownSelectionChange(payload: any) {
      this.OnDropdownSelection.emit(payload);
  }

  OnFileRemoved(payload: any) {
      this.FileRemoved.emit(payload);
  }

  onSubmit() {
      this.isSubmit = true;
      if (this.form.valid) {
        this.payLoad = this.form.value;
        this.OnSubmit.emit(this.payLoad);
      }
  }

  close() {
     this.OnClose.emit(true);
  }
}
