import { Injectable } from '@angular/core';
import { PaginationEntity, PaginationLinkEntity } from './pagination.model';

@Injectable()
export class PaginationService {

     currentPage = 0;
     totalRecords = 0;
     pageSize = 20;
     showFirst = 1;
     showLast = 1;
     paginationstyle = 0;
     totalLinks = 7;
     prevCss = 'previous';
     nextCss = 'next';
     urlpath = '';

     totalPages = 0;

     PaginationLinks: PaginationLinkEntity[] = [];

     constructor(entity: PaginationEntity) {
        this.currentPage = entity.currentPage;
        this.totalRecords = entity.totalRecords;
        this.pageSize = entity.pageSize;
        this.showFirst = entity.showFirst;
        this.showLast = entity.showLast;
        this.paginationstyle = entity.paginationstyle;
        this.totalLinks = entity.totalLinks;
        if (entity.prevCss) {
          this.prevCss = entity.prevCss;
        }
        if (entity.nextCss) {
          this.prevCss = entity.nextCss;
        }
        this.urlpath = entity.urlpath;
     }

    ProcessPagination(): PaginationLinkEntity[] {

      let firstbound = 0;
      let lastbound = 0;
      let tooltip = '';

      if (this.totalRecords > this.pageSize) {

          this.totalPages  = Math.ceil(this.totalRecords / this.pageSize);

          if (this.currentPage > 1) {

                if (this.showFirst === 1 && this.paginationstyle !== 2) {
                    firstbound = 1;
                    lastbound = firstbound + this.pageSize - 1;
                    tooltip = 'showing ' + firstbound + ' - ' + lastbound + ' records of ' + this.totalRecords + ' records';
                    // First Link
                    this.addLink(1, '#/' + this.urlpath, 'First', tooltip, 'first');
                }

                firstbound = ((this.totalPages - 1) * this.pageSize);
                lastbound = firstbound + this.pageSize - 1;

                if (lastbound > this.totalRecords) {
                  lastbound = this.totalRecords;
                }

                tooltip = 'showing ' + firstbound + ' - ' + lastbound + ' records of ' + this.totalRecords + ' records';
                // Previous Link Enabled
                let pid = (this.currentPage - 1);
                if (pid < 1)  { pid = 1; }

                let prevPageCss = '';
                let prevIcon = 'Prev';

                if (this.paginationstyle === 2) {
                    if (this.prevCss !== '') {
                      prevPageCss = this.prevCss;
                    }
                    prevIcon = '&larr; Previous';
                }

                let _urlpath = '';
                if (this.urlpath !== '') {
                  _urlpath =  this.urlpath + '/' + pid;
                }
                this.addLink(pid, '#/' + _urlpath, prevIcon, tooltip, 'previous');

                // Normal Links
                if (this.paginationstyle !== 2) {
                  this.gen_links(this.urlpath);
                }

                if (this.currentPage < this.totalPages) {
                  this.set_prev_last_links(this.urlpath);
                }
           } else {
                if (this.paginationstyle !== 2) {
                  this.gen_links(this.urlpath);
                }
                if (this.currentPage < this.totalPages) {
                  this.set_prev_last_links(this.urlpath);
                }
            }
          }
        return this.PaginationLinks;

    }

    gen_links(urlpath) {

      let firstbound = 0;
      let lastbound = 0;
      let tooltip = '';

      const Links: number[]  = this.SimplePagination();
      if (Links.length > 0) {
            const _this = this;
            for (const link of Links) {

                 firstbound = ((link - 1) * _this.pageSize) + 1;
                 lastbound = firstbound + _this.pageSize - 1;
                 if (lastbound > _this.totalRecords) {
                    lastbound = _this.totalRecords;
                 }
                 tooltip = 'showing ' + firstbound + ' - ' + lastbound + ' records  of ' + _this.totalRecords + ' records';

                 let _urlpath = '';
                 if (urlpath !== '') {
                    _urlpath =  urlpath + '/' + link;
                 }
                 _this.addLink(link, '#/' + _urlpath, link, tooltip, '');
            }
       }
    }

    set_prev_last_links(urlpath) {

         let firstbound: number = ((this.currentPage) * this.pageSize) + 1;
         let lastbound: number = firstbound + this.pageSize - 1;

         if (lastbound > this.totalRecords) {
            lastbound = this.totalRecords;
         }

         let tooltip = 'showing ' + firstbound + ' - ' + lastbound + ' records of ' + this.totalRecords + ' records';

         // Next Link
         let pid = (this.currentPage + 1);
         if (pid > this.totalPages) {
            pid = this.totalPages;
         }

         let nextPageCss = '';
         let nextPageIcon = 'Next';

         if (this.paginationstyle === 2) {
              if (this.nextCss !== '') {
                 nextPageCss = this.nextCss;
              }
              nextPageIcon = 'Next &rarr;';
         }

         this.addLink(pid, '#/' + urlpath, nextPageIcon, tooltip, 'next');

         if (this.showLast === 1 && this.paginationstyle !== 2)  {
             // Last Link
             firstbound = ((this.totalPages - 1) * this.pageSize) + 1;
             lastbound = firstbound + this.pageSize - 1;
             if (lastbound > this.totalRecords) {
                lastbound = this.totalRecords;
             }
             tooltip = 'showing ' + firstbound + ' - ' + lastbound + ' records of ' + this.totalRecords + ' records';

             let _urlpath = '';
             if (urlpath !== '') {
                _urlpath =  urlpath + '/' + this.totalPages;
             }
             this.addLink(lastbound, '#/' + _urlpath, 'Last', tooltip, 'last');
        }

    }

    addLink(id, url, name, tooltip, css) {
        this.PaginationLinks.push({'id' : id, 'url' : url, 'name' : name, 'tooltip' : tooltip, 'css' : css});
    }

    SimplePagination(): number[] {
         const arr = [];
         if (this.totalPages < this.totalLinks) {
           for (let cnt = 1; cnt <= this.totalPages; cnt++) {
              arr[cnt - 1] = cnt;
           }
         } else {
           const startindex  = this.currentPage;
           let lowerbound: number = startindex - Math.floor(this.totalLinks / 2);
           let upperbound = startindex + Math.floor(this.totalLinks / 2);
           if (lowerbound < 1) {
               // calculate the difference and increment the upper bound
              upperbound = upperbound + (1 - lowerbound);
              lowerbound = 1;
            }
            // if upperbound is greater than total page is
            if (upperbound > this.totalPages) {
              // calculate the difference and decrement the lower bound
              lowerbound = lowerbound - (upperbound - this.totalPages);
              upperbound = this.totalPages;
            }
            let counter = 0;
            for (let i = lowerbound; i <= upperbound; i++) {
               arr[counter] = i;
               counter ++;
            }
         }
         return arr;
    }

}
