﻿using Jugnoon.BLL;
using Jugnoon.Entity;
using System;
using System.Collections.Generic;
using System.IO;
using System.Web.Mvc;
using Jugnoon.Utility;
using Jugnoon.Settings;
using Jugnoon.Scripts;
using Newtonsoft.Json;
using System.Drawing;

namespace JgnEnt.Areas.api.Controllers
{
    public class albumController : Controller
    {
        // GET: api/categories/load
        public ActionResult load()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<AlbumEntity>(json);

            data.order = data.order + " " + data.direction;
            var _posts = GalleryBLLC.LoadItems(data);

            var _categories = new List<CategoryDataEntity>();
            var _records = 0;
            /* setup thumb path */
            if (!data.loadall)
            {
                foreach (var ph in _posts)
                {
                    ph.customize_date = UtilityBLL.CustomizeDate((DateTime)ph.created_at, DateTime.Now);
                    ph.url = AlbumUrlConfig.PrepareUrl(ph, 0);
                    if (ph.type == 0)
                    {
                        // photo album
                        // photo functionality not exist in this build
                    }
                    else
                    {
                        // video or audio thumb
                        if (ph.picturename.Contains("http"))
                        {
                            // third party thumb
                            ph.picturename = ph.picturename;
                        }
                        else
                        {
                            // normal video or audio type
                            bool isaudio = false;
                            if (ph.type == 1)
                                isaudio = true;

                            var vd = new VideoDataEntity();
                            vd.thumbfilename = ph.picturename;
                            vd.username = ph.username;
                            vd.thumb_url = "none";
                            ph.picturename = UGeneral.ProcessVideoThumb("", vd, isaudio, new ListItems());
                        }
                        ph.img_url = ph.picturename; // ph.img_url used in app
                    }
                }
                               
                if (data.galleryid == 0)
                    _records = GalleryBLLC.Count(data);

                var _type = 0;
                switch (data.type)
                {
                    case 0:
                        _type = 5; // photos
                        break;
                    case 1:
                        _type = 0; // videos
                        break;
                    case 2:
                        _type = 7; // audio
                        break;
                }
               
                if (data.loadstats)
                {
                    _categories = CategoryBLL.LoadItems(new CategoryEntity()
                    {
                        id = 0,
                        type = _type,
                        mode = 0,
                        status = 2,
                        parentid = -1,
                        order = "level asc", // don't change this
                        issummary = false,
                        isdropdown = true,
                        loadall = true // load all data
                    });
                }
            }
            
            return Json(new { posts = _posts, records = _records, categories = _categories }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
        }

        
        // GET: admin/categories/getinfo
        public ActionResult getinfo()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<AlbumEntity>(json);
            data.issummary = false;
            data.isdropdown = false;
            data.nofilter = true;
            var _posts = GalleryBLLC.LoadItems(data);
            if(_posts.Count == 0)
            {
                return Json(new { status = "error", message = "No record found" }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
            }

            var ph = _posts[0];
            if (ph.type == 0)
            {
                // photo functionality not exist in this build
            }
            else
            {
                // video or audio thumb
                if (ph.picturename.Contains("http"))
                {
                    // third party thumb
                    ph.picturename = ph.picturename;
                    ph.img_url = ph.picturename;
                }
                else
                {
                    // normal video or audio type
                    bool isaudio = false;
                    if (ph.type == 1)
                        isaudio = true;

                    var vd = new VideoDataEntity();
                    vd.thumbfilename = ph.picturename;
                    vd.username = ph.username;
                    vd.thumb_url = "none";
                    ph.picturename = UGeneral.ProcessVideoThumb("", vd, isaudio, new ListItems());
                    ph.img_url = ph.picturename;
                }

                ph.videos = VideoBLL.LoadItems(new VideoEntity()
                {
                    galleryid = ph.galleryid,
                    ispublic = false
                });
                foreach (var video in ph.videos)
                {
                    video.picturename = Jugnoon.Scripts.UGeneral.ProcessVideoThumb("", video, false, new Jugnoon.Scripts.ListItems()); // default set
                    video.customize_date = UtilityBLL.CustomizeDate((DateTime)video.created_at, DateTime.Now);
                }
            }

            if (_posts[0].username != "")
            {
                var user = UserBLL.LoadItems(new MemberEntity()
                {
                    nofilter = true,
                    username = _posts[0].username,
                });
                if (user.Count > 0)
                {
                    _posts[0].author = user[0];
                    _posts[0].author.img_url = UserUrlConfig.ProfilePhoto(_posts[0].author.username, _posts[0].author.picturename, 0); // default set
                }
            }

            return Json(new { post = _posts[0] }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
        }

        // GET: Quick Album Info
        // mostly required album core information e.g username, title, categories, tags
        public ActionResult get_sm_info()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<AlbumEntity>(json);
            data.issummary = false;
            data.isdropdown = false;
            data.nofilter = true;
            var _posts = GalleryBLLC.LoadItems(data);
            if (_posts.Count == 0)
            {
                return Json(new { status = "error", message = "No record found" }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
            }
                        
            return Json(new { status = "success", post = _posts[0] }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
        }


        // check ownership of video (for myaccount purpose)
        public ActionResult authorize_author()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<AlbumDataEntity>(json);
            var isaccess = GalleryBLLC.Check(data.galleryid, data.username);
            return Json(new { isaccess = isaccess });
        }

        /*public ActionResult updatecover()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<AlbumDataEntity>(json);

            if(data.picturename != "" && data.picturename != "none")
            {
                GalleryBLLC.Update_Field_V3(data.galleryid, data.picturename, "picturename");
                data.img_url = PhotoUrlConfig.PreparePhotoUrl(data.username, data.picturename, 0);
            }
            return Json(new { status = "updated", record = data }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
        }*/
        // proc admin version
        public ActionResult proc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<AlbumDataEntity>>(json);

            var model = data[0];

            if (model.tags != "")
            {
                // validate tags
                if (!TagsBLL.Validate_Tags(model.tags))
                {
                    return Json(new { status = "error", message = "Invalid tags" });
                }
            }
            
            if (UtilityBLL.isLongWordExist(model.title) || UtilityBLL.isLongWordExist(model.title))
            {
                return Json(new { status = "error", message = "Invalid title" });
            }

            if(model.galleryid > 0)
            {
                var vd = new AlbumDataEntity();
                vd.title = UtilityBLL.processNull(model.title, 0);
                vd.description = Jugnoon.Scripts.UGeneral.SanitizeText(model.description);
                vd.tags = UtilityBLL.processNull(model.tags, 0);
                vd.galleryid = model.galleryid;
                if (model.categories != null)
                    vd.categories = string.Join(",", model.categories);
                vd.iscomments = (byte)model.iscomments;
                vd.isadult = (byte)model.isadult;
                vd.privacy = (byte)model.privacy;
                vd.liked = model.liked;
                vd.disliked = model.disliked;
                vd.views = model.views;
                vd.favorites = model.favorites;
                vd.isdisabled = model.isdisabled;
                vd.isapproved = model.isapproved;
                vd.isfeatured = model.isfeatured;

                GalleryBLLC.Update(vd, true);
            }
            else
            {
                int _isapproved = 1; // enabled approval status
                if (Site_Settings.Content_Approval == 0)
                    _isapproved = 0;

                string value = "";
                if (model.description != null)
                {
                    value = UtilityBLL.Process_Content_Text(UGeneral.SanitizeText(model.description));
                    if (value.Length > 2000)
                        value = value.Substring(0, 1999);
                }

                var gal = new AlbumDataEntity();
                if (model.categories != null)
                    gal.categories = string.Join(",", model.categories);
                gal.username = model.username;
                if (model.title != null)
                    gal.title = model.title;

                gal.description = value;
                if (model.tags != null)
                    gal.tags = model.tags;

                gal.isdisabled = 1;
                gal.isapproved = (byte)_isapproved;
                gal.type = (byte)model.type;
                gal.mode = 0;
                gal.privacy = (byte)model.privacy;

                if (model.privacy == 2)
                {
                    string authkey = Guid.NewGuid().ToString();
                    if (authkey.Length > 10)
                        authkey = authkey.Substring(0, 9);
                    gal.authkey = authkey;
                }
                gal.picturename = model.picturename;
                model = GalleryBLLC.Add(gal, true);
                // model.img_url = PhotoUrlConfig.PreparePhotoUrl(model.username, model.picturename, 0);
            }                               

            // Process tags
            if (model.tags != "")
            {
                int tag_type = 0; // represent videos
                switch (model.type)
                {
                    case 0:
                        tag_type = 2; // photos
                        break;
                    case 1:
                        tag_type = 0; // videos
                        break;
                    case 2:
                        tag_type = 4; // audio
                        break;
                }
                TagsBLL.Process_Tags(model.tags, tag_type, 0);
            }

            return Json(new { status = "success", record = model, message = "processed" });
        }

        // proc my account version
        public ActionResult proc_macc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<AlbumDataEntity>(json);

            var model = data;

            if (model.tags != "")
            {
                // validate tags
                if (!TagsBLL.Validate_Tags(model.tags))
                {
                    return Json(new { status = "error", message = "Invalid tags" });
                }
            }

            if (UtilityBLL.isLongWordExist(model.title) || UtilityBLL.isLongWordExist(model.title))
            {
                return Json(new { status = "error", message = "Invalid title" });
            }

            if (model.galleryid > 0)
            {
                var vd = new AlbumDataEntity();
                vd.title = UtilityBLL.processNull(model.title, 0);
                vd.description = Jugnoon.Scripts.UGeneral.SanitizeText(model.description);
                vd.tags = UtilityBLL.processNull(model.tags, 0);
                vd.galleryid = model.galleryid;
                if (model.categories != null)
                    vd.categories = string.Join(",", model.categories);
                //vd.iscomments = (byte)model.iscomments;
                //vd.isadult = (byte)model.isadult;
                vd.privacy = (byte)model.privacy;
                //vd.liked = model.liked;
                //vd.disliked = model.disliked;
                //vd.views = model.views;
                //vd.favorites = model.favorites;
                //vd.isdisabled = model.isdisabled;
                //vd.isapproved = model.isapproved;
                //vd.isfeatured = model.isfeatured;

                GalleryBLLC.Update(vd, true);
            }
            else
            {
                int _isapproved = 1; // enabled approval status
                if (Site_Settings.Content_Approval == 0)
                    _isapproved = 0;

                string value = "";
                if (model.description != null)
                {
                    value = UtilityBLL.Process_Content_Text(UGeneral.SanitizeText(model.description));
                    if (value.Length > 2000)
                        value = value.Substring(0, 1999);
                }

                var gal = new AlbumDataEntity();
                if (model.categories != null)
                    gal.categories = string.Join(",", model.categories);
                gal.username = model.username;
                if (model.title != null)
                    gal.title = model.title;

                gal.description = value;
                if (model.tags != null)
                    gal.tags = model.tags;

                gal.isdisabled = 1;
                gal.isapproved = (byte)_isapproved;
                gal.type = (byte)model.type;
                gal.mode = 0;
                gal.privacy = (byte)model.privacy;

                if (model.privacy == 2)
                {
                    string authkey = Guid.NewGuid().ToString();
                    if (authkey.Length > 10)
                        authkey = authkey.Substring(0, 9);
                    gal.authkey = authkey;
                }
                gal.picturename = model.picturename;
                model = GalleryBLLC.Add(gal, true);
                // model.img_url = PhotoUrlConfig.PreparePhotoUrl(model.username, model.picturename, 0);
            }

            // Process tags
            if (model.tags != "")
            {
                int tag_type = 0; // represent videos
                switch (model.type)
                {
                    case 0:
                        tag_type = 2; // photos
                        break;
                    case 1:
                        tag_type = 0; // videos
                        break;
                    case 2:
                        tag_type = 4; // audio
                        break;
                }
                TagsBLL.Process_Tags(model.tags, tag_type, 0);
            }

            return Json(new { status = "success", record = model, message = "processed" });
        }

        public ActionResult uploads()
        {
            if (Request.Files.Count > 0)
            {
                int chunk = Request["chunk"] != null ? int.Parse(Request["chunk"]) : 0;
                string fileName = Request["name"] != null ? Request["name"] : string.Empty;

                string _fileName = fileName.Remove(fileName.LastIndexOf(".")) + "-" + Guid.NewGuid().ToString().Substring(0, 6) + "" + fileName.Remove(0, fileName.LastIndexOf("."));

                string UserName = "";

                if (Request.Headers["UName"] != null)
                {
                    UserName = Request.Headers["UName"].ToString();
                }

                string dirPath = Server.MapPath(Request.ApplicationPath) + "/contents/member/" + UserName + "/photos/";

                FileStream fs;
                using (fs = new FileStream(Path.Combine(dirPath, _fileName), chunk == 0 ? FileMode.Create : FileMode.Append))
                {
                    byte[] buffer = new byte[Request.Files[0].InputStream.Length];
                    Request.Files[0].InputStream.Read(buffer, 0, buffer.Length);

                    fs.Write(buffer, 0, buffer.Length);
                }

                string fileType = System.IO.Path.GetExtension(_fileName);
                string fileIndex = fileName.Replace(fileType, "");
                string elementid = Guid.NewGuid().ToString().Substring(0, 6);

                /* generate thumb */
                // create small size thumb
                // string strPath = Server.MapPath(Request.ApplicationPath) + "contents/blog";

                string orignalfilename = dirPath + "" + _fileName;
                string thumbfilename = dirPath + "thumbs/" + _fileName;
                string midthumbfilename = dirPath + "midthumbs/" + _fileName;

                // create mid size thumb
                Jugnoon.Utility.Image.CreateCropThumbnail(orignalfilename, Site_Settings.MediumThumbWidth, Site_Settings.MediumThumbHeight, midthumbfilename);

                // create small size thumb
                Bitmap mp = Jugnoon.Utility.Image.CreateThumbnail(orignalfilename, Site_Settings.Thumb_Width, Site_Settings.Thumb_Height);
                if (mp == null)
                {
                    return Json(new { status = "success", message = Resources.vsk.message_photos_05 });
                }
                mp.Save(thumbfilename);

                string url = ""; // PhotoUrlConfig.PreparePhotoUrl(UserName, _fileName, 2);

                return Json(new { jsonrpc = "2.0", result = "OK", fname = _fileName, url = url, filetype = fileType, filename = _fileName, fileIndex = fileIndex, eleid = elementid }, JsonRequestBehavior.AllowGet);
            }
            else
            {
                return Json(new { result = "Error" });
            }
        }

        public ActionResult removefile()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<AlbumDataEntity>>(json);

            string RootPath = Server.MapPath(Request.ApplicationPath);
            foreach (var Record in data)
            {
                string dirpath = Server.MapPath(Request.ApplicationPath) + "//contents//member//" + Record.username + "//photos//";
                string tempPath = dirpath + "thumbs/";
                string mediumPath = dirpath + "medium/";
                if (!Record.picturename.StartsWith("http"))
                {
                    if (System.IO.File.Exists(dirpath + Record.picturename))
                        System.IO.File.Delete(dirpath + Record.picturename);
                    if (System.IO.File.Exists(tempPath + Record.picturename))
                        System.IO.File.Delete(tempPath + Record.picturename);
                    if (System.IO.File.Exists(mediumPath + Record.picturename))
                        System.IO.File.Delete(mediumPath + Record.picturename);
                }
            }

            return Json(new { status = "success", message = "Files removed successfully" });
        }

        // GET: admin/categories/action
        public ActionResult action()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<AlbumEntity>>(json);

            GalleryBLLC.ProcessAction(data);

            return Json(new { status = "success", message = "Operation performed successfully" });
        }
    }
}
