﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using Jugnoon.Entity;
using Jugnoon.Settings;
using Jugnoon.Utility;
using Jugnoon.BLL;
using System.Drawing;
using System.IO;
using Newtonsoft.Json;

namespace JgnEnt.Areas.api.Controllers
{
    public class categoriesController : Controller
    {
        // GET: api/categories/load
        public ActionResult load()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<CategoryEntity>(json);
            data.order = data.order + " " + data.direction;
            var _posts = CategoryBLL.LoadItems(data);
            foreach(var item in _posts)
            {
                if(item.picturename.StartsWith("http"))
                {
                    item.img_url = item.picturename;
                }
                else
                {
                    item.img_url = Config.GetUrl("contents/category/" + item.picturename);
                }
            }
            var _records = 0;
            if (data.id == 0)
                _records = CategoryBLL.Count(data);

            return Json(new { posts = _posts, records = _records });
        }
        public ActionResult load_dropdown()
        {
            var _entity = new CategoryEntity();
            _entity.isdropdown = true;
            _entity.issummary = false;
            _entity.parentid = -1;
            _entity.order = "level asc";
            _entity.pagesize = 10000; // all categories
            _entity.type = 100; // load all (no type filter)
            _entity.status = 2;

            var _posts = CategoryBLL.LoadItems(_entity);
                       
            return Json(new { posts = _posts });
        }
        // GET: admin/categories/getinfo
        public ActionResult getinfo()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<CategoryEntity>>(json);
            var _posts = CategoryBLL.LoadItems(data[0]);
            if (_posts.Count > 0)
            {
                _posts[0].img_url = CategoryUrlConfig.PrepareImageUrl(_posts[0]);
                return Json(new { status = "success", post = _posts[0] });
            }
            else
            {
                return Json(new { status = "error", message = "no record found" });
            }
        }

        // GET: admin/categories/proc
        public ActionResult proc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<CategoryDataEntity>(json);
            /* if (!data.permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            } */

            data.id = CategoryBLL.Process(data);

            return Json(new { status = "success", record = data, message = "Record added successfully" });
        }

        // GET: admin/categories/updatethumb
        public ActionResult updatethumb()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<CategoryEntity>>(json);
            /*if (!data[0].permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            }*/

            CategoryBLL.Update_Field(data[0].id, "picturename", data[0].picturename);
            string dirPath = Server.MapPath(Request.ApplicationPath) + "/contents/category/";
            if (System.IO.File.Exists(dirPath + data[0].picturename))
                System.IO.File.Delete(dirPath + data[0].picturename);

            return Json(new { status = "success", message = "Record updated successfully" });
        }

        // GET: admin/categories/removethumb
        public ActionResult removethumb()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<CategoryEntity>>(json);
            /* if (!data[0].permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            } */
            string dirPath = Server.MapPath(Request.ApplicationPath) + "/contents/category/";
            if (System.IO.File.Exists(dirPath + data[0].picturename))
                System.IO.File.Delete(dirPath + data[0].picturename);

            return Json(new { status = "success", message = "File removed successfully" });
        }

        // GET: admin/categories/removefile
        public ActionResult removefile()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<CategoryEntity>>(json);
            /* if (!data[0].permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            }*/
            string RootPath = Server.MapPath(Request.ApplicationPath);
            foreach (CategoryEntity Record in data)
            {
                string dirPath = RootPath + "/contents/category/";
                string tempPath = dirPath + "temp/";
                if (System.IO.File.Exists(dirPath + Record.picturename))
                    System.IO.File.Delete(dirPath + Record.picturename);
                if (System.IO.File.Exists(tempPath + Record.picturename))
                    System.IO.File.Delete(tempPath + Record.picturename);
            }

            return Json(new { status = "success", message = "Files removed successfully" });
        }

        // GET: admin/categories/remove
        public ActionResult remove()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<CategoryEntity>>(json);
            /* if (!data[0].permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            } */
            CategoryBLL.Delete((short)data[0].id);

            return Json(new { status = "success", message = "Record removed successfully" });
        }

        // GET: admin/categories/action
        public ActionResult action()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<CategoryEntity>>(json);
            /* if (!data[0].permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            } */
            CategoryBLL.ProcessAction(data);

            return Json(new { status = "success", message = "Operation performed successfully" });
        }

        // GET: admin/categories/uploads
        public ActionResult uploads()
        {
            if (Request.Files.Count > 0)
            {
                int chunk = Request["chunk"] != null ? int.Parse(Request["chunk"]) : 0;
                string fileName = Request["name"] != null ? Request["name"] : string.Empty;

                string _fileName = fileName.Remove(fileName.LastIndexOf(".")) + "-" + Guid.NewGuid().ToString().Substring(0, 6) + "" + fileName.Remove(0, fileName.LastIndexOf("."));

                string dirPath = Server.MapPath(Request.ApplicationPath) + "/contents/category/temp/";

                FileStream fs;
                using (fs = new FileStream(Path.Combine(dirPath, _fileName), chunk == 0 ? FileMode.Create : FileMode.Append))
                {
                    byte[] buffer = new byte[Request.Files[0].InputStream.Length];
                    Request.Files[0].InputStream.Read(buffer, 0, buffer.Length);

                    fs.Write(buffer, 0, buffer.Length);
                }
                
                string fileType = System.IO.Path.GetExtension(_fileName);
                string fileIndex = fileName.Replace(fileType, "");
                string elementid = Guid.NewGuid().ToString().Substring(0, 6);

                /* generate thumb */
                // create small size thumb
                string strPath = Server.MapPath(Request.ApplicationPath) + "/contents/category";
                string orignalfilename = strPath + "/temp/" + _fileName;
                string thumbfilename = strPath + "/" + _fileName;
                Bitmap mp = Jugnoon.Utility.Image.CreateThumbnail(orignalfilename, Site_Settings.Category_Thumbnail_Width, Site_Settings.Category_Thumbnail_Height);
                if (mp == null)
                {
                    return Json(new { result = "Error" });
                }
                mp.Save(thumbfilename);

                Jugnoon.Utility.Image.SaveJpeg(thumbfilename, mp, 90);

                string url = Config.GetUrl("contents/category/" + _fileName);

                return Json(new { jsonrpc = "2.0", result = "OK", fname = _fileName, url = url, filetype = fileType, filename = _fileName, fileIndex = fileIndex, eleid = elementid }, JsonRequestBehavior.AllowGet);
            }
            else
            {
                return Json(new { result = "Error" });
            }
        }

    }
}