﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Jugnoon.Settings;
using System.IO;
using Newtonsoft.Json;
using Jugnoon.Entity;
using Jugnoon.Utility;
using Jugnoon.Scripts;
using System.Collections;
using Jugnoon.videos;
using Jugnoon.BLL;

namespace JgnEnt.Areas.api.Controllers
{
    public class ffmpegController : Controller
    {
        // GET: api/ffmpeg
          // GET: api/ffmpeg
        // Store all encoding processes in static (shared) list of mediahandler objects
        // Each mediahandler object, control single encoding process accross application
        // static object required to manage each encoding separately in shared environment 
        //where there is chances of concurrent encoding request at a time.
        public static List<MediaHandler> _lst = new List<MediaHandler>();

        //
        // GET: /api/video/
        public ActionResult Index()
        {
            Dictionary<string, string> _response = new Dictionary<string, string>();
            _response["status"] = "success";
            _response["message"] = "API - Home";

            return Json(_response, JsonRequestBehavior.AllowGet);
        }


        //
        // GET: /api/video/upload
        public ActionResult upload()
        {
            if (Request.Files.Count > 0)
            {
                var prepare_List = new Dictionary<string, string>();
                prepare_List["jsonrpc"] = "2.0";
                prepare_List["id"] = "id";
                prepare_List["result"] = "OK";
                var fileName = "";
                try
                {
                    int chunk = Request["chunk"] != null ? int.Parse(Request["chunk"]) : 0;
                    fileName = Request["name"] != null ? Request["name"] : string.Empty;
                    var _extension = Path.GetExtension(fileName);
                    fileName = fileName.Replace(_extension, "");
                    
                    fileName = Jugnoon.Utility.UtilityBLL.ReplaceSpaceWithHyphin(fileName) + "" + _extension;
                    string upath = "";
                    if (Request.Headers["UName"] != null)
                        upath = Request.Headers["UName"].ToString();

                    string uploadPath = VideoUrlConfig.Source_Video_Path(upath);  // Server.MapPath(Request.ApplicationPath) + "/contents/member/" + upath + "/default";

                    FileStream fs;
                    using (fs = new FileStream(Path.Combine(uploadPath, fileName), chunk == 0 ? FileMode.Create : FileMode.Append))
                    {
                        byte[] buffer = new byte[Request.Files[0].InputStream.Length];
                        Request.Files[0].InputStream.Read(buffer, 0, buffer.Length);

                        fs.Write(buffer, 0, buffer.Length);
                    }

                    string url = VideoUrlConfig.Source_Video_Url(upath) + "/" + fileName; // "/contents/member/" + upath + "/default/" + fileName;
                    string fileType = System.IO.Path.GetExtension(fileName);
                    string fileIndex = fileName.Replace(fileType, "");
                    string elementid = ""; // not used in this case
                    prepare_List["fname"] = fileName;
                    prepare_List["url"] = url;
                    prepare_List["host"] = Config.GetUrl();
                    prepare_List["filetype"] = fileType;
                    prepare_List["filename"] = fileName;
                    prepare_List["fileIndex"] = fileIndex;
                    prepare_List["eleid"] = elementid;

                    //return Json(new { fname = fileName, url, host = Config.GetUrl(), fileType, filename = fileName, fileIndex });
                    return Json(prepare_List);
                }
                catch(Exception ex)
                {
                    var _message = new System.Text.StringBuilder();
                    _message.Append("<h4>Video Upload Error</h4>");
                    _message.Append("<p>Error:" + ex.Message + "</p>");
                    _message.Append("<p>FileName: " + fileName);

                    MailProcess.Send_Mail(Config.Return_Email(), "Upload Error, description", _message.ToString());

                    prepare_List["status"] = "Error";
                    prepare_List["description"] = ex.Message;

                    return Json(prepare_List);
                }
            }
            else
            {
                var prepare_List = new Dictionary<string, string>();
                prepare_List["result"] = "failed";
                return Json(prepare_List);
            }
        }
              
        //
        // GET: /api/video/published
        public ActionResult published()
        {
            Response.ContentType = "text/plain";

            var json = new StreamReader(Request.InputStream).ReadToEnd();
            List<VideoFiles> data = JsonConvert.DeserializeObject<List<VideoFiles>>(json);
            string publishedPath = VideoUrlConfig.Published_Video_Path(data[0].username);

            var _log = new List<VideoLog>();
            foreach (VideoFiles file in data)
            {
                if (System.IO.File.Exists(publishedPath + "/" + file.filename))
                {
                    // file uploaded successfully.

                    // write code to upload published file to cloud storage e.g amazon s3 here.

                    string ErrorCode = file.errorcode;
                    string fileName = file.filename; // published filename
                    // rename published file to avoid duplicate issue
                    string newfileName = Guid.NewGuid().ToString().Substring(0, 15) + System.IO.Path.GetExtension(fileName);
                    System.IO.File.Copy(publishedPath + "/" + file.filename, publishedPath + "/" + newfileName);
                    // renmove older file
                    System.IO.File.Delete(publishedPath + "/" + file.filename);


                    string sourcefilename = file.sfle; // source file name
                    string duration = file.duration;
                    string durationsec = file.durationsec; // video duration in seconds


                    // write code to save file information in database


                    // write code to delete original video if not required after publishing
                    if (System.IO.File.Exists(publishedPath + "/" + file.sfle))
                    {
                        System.IO.File.Delete(publishedPath + "/" + file.sfle);
                    }

                    // file operation completed, create log
                    _log.Add(new VideoLog()
                    {
                        errorcode = file.errorcode,
                        message = "File processed successfully",
                        publishedfile = file.filename,
                        sourcefile = file.sfle,
                        status = "success"
                    });

                }
                else
                {
                    // file not found
                    _log.Add(new VideoLog()
                    {
                        errorcode = file.errorcode,
                        message = "published file not found",
                        publishedfile = file.filename,
                        sourcefile = file.sfle,
                        status = "error"
                    });
                }
            }
            // complete published operation
            var _report = new PublishedReport()
            {
                status = "success",
                message = "All videos processed successfully",
                report = _log
            };

            return Json(_report, JsonRequestBehavior.AllowGet);

        }

        //
        // GET: /api/video/encode
        public ActionResult encode()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<EncodeFFMPEGVideo>>(json);
            int ActionType = data[0].tp;
            string username = data[0].username;

            var _response = new Dictionary<string, string>();
            _response["encodeoutput"] = "2.0";

            string Source = "";
            string Published = "";
            string ProcessID = "";
            switch (ActionType)
            {
                case 0:
                    // encode video
                    Source = data[0].sf;
                    Published = data[0].pf;

                    if (Source != "" && Published != null)
                    {
                        var _mhandler = new MediaHandler();
                        string RootPath = Server.MapPath(Request.ApplicationPath);
                        _mhandler.FFMPEGPath = Encoding_Settings.FFMPEGPATH;
                        _mhandler.InputPath = UrlConfig.Upload_Path(username, "default");
                        _mhandler.OutputPath = UrlConfig.Upload_Path(username, "flv");
                        _mhandler.BackgroundProcessing = true;
                        _mhandler.FileName = Source;
                        _mhandler.OutputFileName = Published.Replace(Path.GetExtension(Published), "");
                        _mhandler.Parameters = "-s 640x380 -c:v libx264 -preset medium -crf 22 -b:v 500k -b:a 128k -profile:v baseline -level 3.1"; // Site_Settings.MP4_480p_Settings;
                        _mhandler.OutputExtension = ".mp4";
                        _mhandler.vinfo = _mhandler.ProcessMedia();
                        if (_mhandler.vinfo.ErrorCode > 0)
                        {
                            // remove file if failed to publish properly
                            if (System.IO.File.Exists(RootPath + "/" + _mhandler.InputPath))
                                System.IO.File.Delete(RootPath + "/" + _mhandler.InputPath);

                            _response["encodeoutput"] = "2.0";
                            _response["ecode"] = _mhandler.vinfo.ErrorCode.ToString();
                            _response["edesc"] = _mhandler.vinfo.FFMPEGOutput.ToString();


                            var _message = new System.Text.StringBuilder();
                            _message.Append("<h4>Video Upload Error</h4>");
                            _message.Append("<p>Error:" + _mhandler.vinfo.ErrorCode + " _ _ " + _mhandler.vinfo.ErrorMessage + "</p>");
                            _message.Append("<p>Source FileName: " + Source);
                            _message.Append("<p>Published FileName: " + Published);
                            MailProcess.Send_Mail(Config.Return_Email(), "Publishing Error, description", _message.ToString());


                            return Json(_response, JsonRequestBehavior.AllowGet);
                        }
                        else
                        {
                            // _mhandler.vinfo.ProcessID = Guid.NewGuid().ToString(); // unique guid to attach with each process to identify proper object on progress bar and get info request
                            _lst.Add(_mhandler);
                            _response["encodeoutput"] = "2.0";
                            _response["ecode"] = _mhandler.vinfo.ErrorCode.ToString();
                            _response["procid"] = _mhandler.vinfo.ProcessID; // _mhandler.vinfo.ProcessID;
                            return Json(_response, JsonRequestBehavior.AllowGet);
                        }
                    }
                    break;
                case 1:
                    // get progress status
                    ProcessID = data[0].pid;
                    if (ProcessID != "")
                    {
                        string completed_process = "0";
                        if (_lst.Count > 0)
                        {
                            int i = 0;
                            for (i = 0; i <= _lst.Count - 1; i++)
                            {
                                if (_lst[i].vinfo.ProcessID == ProcessID)
                                {
                                    completed_process = Math.Round(_lst[i].vinfo.ProcessingCompleted, 2).ToString();
                                }
                            }
                        }

                        _response["encodeoutput"] = "2.0";
                        _response["status"] = completed_process;
                        return Json(_response, JsonRequestBehavior.AllowGet);
                    }

                    break;
                case 2:
                    // get information
                    ProcessID = data[0].pid;
                    if (ProcessID != "")
                    {
                        if (_lst.Count > 0)
                        {
                            int i = 0;
                            for (i = 0; i <= _lst.Count - 1; i++)
                            {
                                if (_lst[i].vinfo.ProcessID == ProcessID)
                                {
                                    _response["status"] = "OK";
                                    _response["ecode"] = _lst[i].vinfo.ErrorCode.ToString();
                                    _response["fname"] = _lst[i].vinfo.FileName.ToString();

                                    _response["dur"] = _lst[i].vinfo.Duration.ToString();
                                    _response["dursec"] = _lst[i].vinfo.Duration_Sec.ToString();


                                    // remove from list of corrent processes if processes reach this point
                                    // store all information of completed process and remove it from list of concurrent processes
                                    // e.g
                                    VideoInfo current_uploaded_video_info = _lst[i].vinfo;
                                    _lst.Remove(_lst[i]);
                                    
                                    // Validation 
                                    int plength = 0;
                                    string path = UrlConfig.Upload_Path(username, "flv") + "\\" + data[0].pf;
                                    if (System.IO.File.Exists(path))
                                    {
                                        FileInfo flv_info = new FileInfo(path);
                                        plength = (int)flv_info.Length;
                                    }
                                    if (plength == 0)
                                    {
                                        var _message = new System.Text.StringBuilder();
                                        _message.Append("<h4>Video Publishing Error</h4>");
                                        _message.Append("<p>Error: 0kb file generated</p>");
                                        _message.Append("<p>Source FileName: " + Source);
                                        _message.Append("<p>Published FileName: " + Published);
                                        MailProcess.Send_Mail(Config.Return_Email(), "Publishing Error, description", _message.ToString());
                                    }

                                    // ii: add meta information to mp4 video
                                    try
                                    {
                                        var mp4med = new MediaHandler();
                                        mp4med.MP4BoxPath = Encoding_Settings.MP4BoxPath;
                                        string _mp4_temp_path = "\"" + UrlConfig.Upload_Path(username, "flv") + "/" + data[0].pf + "\"";
                                        string meta_filename = data[0].sf.Replace(".mp4", "_meta.mp4");
                                        mp4med.Parameters = "-isma -hint -add " + _mp4_temp_path + "";
                                        mp4med.FileName = meta_filename;
                                        mp4med.InputPath = UrlConfig.Upload_Path(username, "flv");
                                        mp4med.Set_MP4_Buffering();

                                        // check whether file created
                                        string pubPath = UrlConfig.Upload_Path(username, "flv");
                                        if (System.IO.File.Exists(pubPath + "\\" + meta_filename))
                                        {
                                            // remove temp mp4 file
                                            if (System.IO.File.Exists(pubPath + "" + data[0].pf))
                                                System.IO.File.Delete(pubPath + "\\" + data[0].pf);

                                            _response["fname"] = meta_filename;
                                        }
                                        else
                                        {
                                            // file not created by mp4box
                                            // rename published mp4 as _meta.mp4
                                            System.IO.File.Move(pubPath + "\\" + data[0].pf, pubPath + "\\" + meta_filename);
                                            _response["fname"] = meta_filename;
                                        }
                                    }
                                    catch(Exception ex)
                                    {
                                        var _message = new System.Text.StringBuilder();
                                        _message.Append("<h4>Video Meta Information Error</h4>");
                                        _message.Append("<p>Error: " + ex.Message + "</p>");
                                        _message.Append("<p>Source FileName: " + Source);
                                        _message.Append("<p>Published FileName: " + Published);
                                        MailProcess.Send_Mail(Config.Return_Email(), "Publishing Error, description", _message.ToString());
                                    }

                                    _response["isenable"] = "1";

                                    // Thumb Grabbing Script
                                    string thumb_start_index = "";
                                    try
                                    {
                                        var med = new MediaHandler();
                                        med.FFMPEGPath = Encoding_Settings.FFMPEGPATH;
                                        med.InputPath = UrlConfig.Upload_Path(username, "default"); // RootPath + "\\" + SourcePath;
                                        med.OutputPath = UrlConfig.Upload_Path(username, "thumbs"); // RootPath + "\\" + PublishedPath;
                                        med.FileName = data[0].sf; // source file
                                        thumb_start_index = med.FileName.Replace(Path.GetExtension(med.FileName), "_");
                                        med.Image_Format = "jpg";
                                        med.VCodec = "image2"; //optional
                                        med.ACodec = "";
                                        med.ImageName = thumb_start_index;
                                        med.Multiple_Thumbs = true;
                                        med.ThumbMode = 0;
                                        med.No_Of_Thumbs = 15;
                                        med.Thumb_Start_Position = 5; // start grabbing thumbs from 5th second
                                                                      //if (this.BackgroundProcessing)
                                                                      //    med.BackgroundProcessing = true;
                                        int width = Site_Settings.Thumb_Width;
                                        if (width > 0)
                                            med.Width = width;
                                        int height = Site_Settings.Thumb_Height;
                                        if (height > 0)
                                            med.Height = height;
                                        var tinfo = med.Grab_Thumb();
                                        if (tinfo.ErrorCode > 0)
                                        {
                                            // Error occured in grabbing thumbs - Rollback process
                                            _response["ecode"] = "1006";
                                            _response["edesc"] = "Grabbing thumbs from video failed";

                                            var _message = new System.Text.StringBuilder();
                                            _message.Append("<h4>Thumb Generation Error</h4>");
                                            _message.Append("<p>Error: " + _response["edesc"] + "</p>");
                                            _message.Append("<p>Source FileName: " + Source);
                                            _message.Append("<p>Published FileName: " + Published);
                                            MailProcess.Send_Mail(Config.Return_Email(), "Thumb Generation Error, description", _message.ToString());

                                            // call rollback script here
                                            return Json(_response, JsonRequestBehavior.AllowGet);
                                        }

                                        // Validate Thumbs
                                        path = UrlConfig.Upload_Path(username, "thumbs") + "/" + thumb_start_index;
                                        if (!System.IO.File.Exists(path + "004.jpg") || !System.IO.File.Exists(path + "008.jpg") || !System.IO.File.Exists(path + "011.jpg"))
                                        {
                                            // thumb failed try again grabbing thumbs from published video
                                            med.InputPath = UrlConfig.Upload_Path(username, "default");
                                            med.FileName = data[0].pf; // grab thumb from encoded video
                                            tinfo = med.Grab_Thumb();
                                            if (tinfo.ErrorCode > 0)
                                            {
                                                // Error occured in grabbing thumbs - Rollback process
                                                _response["ecode"] = "1006";
                                                _response["edesc"] = "Grabbing thumbs from video failed";
                                                // rollback script here
                                                var _message = new System.Text.StringBuilder();
                                                _message.Append("<h4>Thumb Generation Error</h4>");
                                                _message.Append("<p>Error: " + _response["edesc"] + "</p>");
                                                _message.Append("<p>Source FileName: " + Source);
                                                _message.Append("<p>Published FileName: " + Published);
                                                MailProcess.Send_Mail(Config.Return_Email(), "Thumb Generation Error, description", _message.ToString());
                                                return Json(_response, JsonRequestBehavior.AllowGet);
                                            }
                                            // Disable Video
                                            if (!System.IO.File.Exists(path + "004.jpg") || !System.IO.File.Exists(path + "008.jpg") || !System.IO.File.Exists(path + "011.jpg"))
                                            {
                                                _response["isenable"] = "0"; // disable video - thumbs not grabbed properly.
                                            }
                                        }
                                    }
                                    catch (Exception ex)
                                    {
                                        _response["ecode"] ="1010";
                                        _response["edesc"] = ex.Message;

                                        var _message = new System.Text.StringBuilder();
                                        _message.Append("<h4>Thumb Generation Error</h4>");
                                        _message.Append("<p>Error: " + ex.Message + "</p>");
                                        _message.Append("<p>Source FileName: " + Source);
                                        _message.Append("<p>Published FileName: " + Published);
                                        MailProcess.Send_Mail(Config.Return_Email(), "Thumb Generation Error, description", _message.ToString());
                                    }
                                    
                                    _response["tfile"] = thumb_start_index + "" + "008.jpg";
                                    _response["fIndex"] = thumb_start_index;
                                    _response["img_url"] = Config.GetUrl("contents/member/" + username + "/thumbs/"); // + _response["tfile"]);
                                }
                            }
                        }
                        return Json(_response, JsonRequestBehavior.AllowGet);
                    }

                    break;
                case 3:
                    // final check
                    ProcessID = data[0].pid;
                    Source = data[0].sf;
                    Published = data[0].pf;

                    if (ProcessID != "" && Source != "" && Published != "")
                    {
                        if (_lst.Count > 0)
                        {
                            int i = 0;
                            for (i = 0; i <= _lst.Count - 1; i++)
                            {
                                if (_lst[i].vinfo.ProcessID == ProcessID)
                                {
                                    if (_lst[i].vinfo.ProcessingCompleted >= 100)
                                    {
                                        // check whether published file uploaded properly
                                        string publishedPath = UrlConfig.Upload_Path(username, "flv");

                                        if (!System.IO.File.Exists(publishedPath + "/" + Published))
                                        {
                                            _response["status"] = "INVALID";// published file not found
                                        }
                                        else
                                        {
                                            _response["encodeoutput"] = "2.0";
                                            _response["status"] = "OK";
                                        }
                                    }
                                }
                            }
                        }

                        return Json(_response, JsonRequestBehavior.AllowGet);
                    }
                    break;
            }
            _response["status"] = "INVALID";
            return Json(_response, JsonRequestBehavior.AllowGet);
        }


        public static string GetErrorCode(string ProcessID)
        {
            string ErrorCode = "0";
            if (_lst.Count > 0)
            {
                int i = 0;
                for (i = 0; i <= _lst.Count - 1; i++)
                {
                    if (_lst[i].vinfo.ProcessID == ProcessID)
                    {
                        ErrorCode = _lst[i].vinfo.ErrorCode.ToString();
                    }
                }
            }

            return ErrorCode;
        }

        public static string GetFFMPEGOutPut(string ProcessID)
        {
            string Output = "";
            if (_lst.Count > 0)
            {
                int i = 0;
                for (i = 0; i <= _lst.Count - 1; i++)
                {
                    if (_lst[i].vinfo.ProcessID == ProcessID)
                    {
                        Output = _lst[i].vinfo.FFMPEGOutput.ToString();
                    }
                }
            }

            return Output;
        }
    }
}
