﻿using Jugnoon.BLL;
using Jugnoon.Entity;
using System;
using System.Collections.Generic;
using System.IO;
using System.Web.Mvc;
using Jugnoon.Utility;
using Jugnoon.Settings;
using Newtonsoft.Json;
using System.Drawing;
using DevOne.Security.Cryptography.BCrypt;
using System.Web.Http.Cors;

namespace JgnEnt.Areas.api.Controllers
{
    [EnableCors(origins: "*", headers: "*", methods: "*")]
    public class userController : Controller
    {
        public ActionResult authenticate()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var model = JsonConvert.DeserializeObject<UserDataEntity>(json);

            // validate member
            // Update Password Validation Script
            if (model.username == "" || model.password == "")
            {
                return Json(new { status = "error", message = "Enter UserName & Password" }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
            }

            // IP Address tracking and processing
            string ipaddress = Request.ServerVariables["REMOTE_ADDR"].ToString();
            if (BlockIPBLL.Validate_IP(ipaddress))
            {
                return Json(new { status = "error", message = Resources.vsk.ipblockedmsg }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
            }

            int Membertype = 0;
            int Readonly = 0;
            var _lst = UserBLL.Get_Hash_Password(model.username, true);
            if (_lst.Count == 0)
            {
                return Json(new { status = "error", message = "Login failed, please enter correct user name & password" }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
            }

            var _memberprocess = new UserBLL();
            // check encrypted password
            if (_lst[0].password.Length < 20)
            {
                // backward compatibility
                // check existing user passwords with old system
                if (!_memberprocess.Validate_Member(model.username, model.password, true))
                {
                    return Json(new { status = "error", message = "Login failed, please enter correct user name & password" }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
                }

                Membertype = Convert.ToInt32(UserBLL.Return_Value(model.username, "type"));
                Readonly = Convert.ToInt32(UserBLL.Return_Value(model.username, "readonly"));
            }
            else
            {
                // check encrypted password with user typed password
                bool matched = BCryptHelper.CheckPassword(model.password, _lst[0].password);
                if (!matched)
                {
                    return Json(new { status = "error", message = "Login failed, please enter correct user name & password" }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
                }

                if (_lst[0].isenabled == 0)
                {
                    return Json(new { status = "error", message = "Your account is temporary suspended, please contact site administrator" }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
                }
                Membertype = _lst[0].type; // type
                Readonly = _lst[0].@readonly;
            }


            if (Membertype == 0)
            {
                return Json(new { status = "error", message = "Admininstrator Access Denied" }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
            }


            if (Site_Settings.Store_IPAddress)
            {
                // Store IP Address Log 
                User_IPLogBLL.Process_Ipaddress_Log(model.username, ipaddress);
            }

            // Update Last Login Activity of User
            UserBLL.Update_Field_V3(model.username, "last_login", DateTime.Now);
            // member is validated

            var checkPath = Server.MapPath(Request.ApplicationPath) + "/contents/member/" + model.username.ToLower();
            if (!System.IO.Directory.Exists(checkPath))
            {
                Directory_Process.CreateRequiredDirectories(checkPath);
            }

            var User = UserBLL.LoadItems(new MemberEntity()
            {
                username = model.username
            });

            if (User.Count > 0)
            {
                foreach (var item in User)
                {
                    item.img_url = UserUrlConfig.ProfilePhoto(item.username, item.picturename, 0); // default set
                    item.url = UserUrlConfig.ProfileUrl(item.username);
                }
                return Json(new { status = "success", message = "Authenticated", user = User[0] }, JsonRequestBehavior.AllowGet);
            }
            else
            {
                return Json(new { status = "error", message = "Error Occured" }, JsonRequestBehavior.AllowGet);
            }

        }
        // GET: admin/categories/load
        public ActionResult load()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<MemberEntity>(json);
            if (!data.order.Contains("desc") && !data.order.Contains("asc"))
                data.order = data.order + " " + data.direction;
            var _posts = UserBLL.LoadItems(data);

            /* setup thumb path */
            foreach (var item in _posts)
            {
                item.img_url = UserUrlConfig.ProfilePhoto(item.username, item.picturename, 0); // default set
                item.url = UserUrlConfig.ProfileUrl(item.username);
            }

            var _records = 0;
            if (data.userid == 0)
                _records = UserBLL.Count(data);

            var _categories = new List<CategoryDataEntity>();
            if (data.loadstats)
            {
                _categories = CategoryBLL.LoadItems(new CategoryEntity()
                {
                    id = 0,
                    type = 2,
                    mode = 0,
                    status = 2,
                    parentid = -1,
                    order = "level asc", // don't change this
                    issummary = false,
                    isdropdown = true,
                    loadall = true // load all data
                });
            }
            return Json(new { posts = _posts, categories = _categories, records = _records }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
        }

        // sends
        // user info
        // language list
        // settings

        public ActionResult getData()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<MemberEntity>(json);

            data.nofilter = false;
            data.issummary = false;
            data.isdropdown = false;
            var _posts = UserBLL.LoadItems(data);
            if (_posts.Count == 0)
            {
                return Json(new { status = "error", message = "Error fetching record" });
            }
            _posts[0].img_url = UserUrlConfig.ProfilePhoto(_posts[0].username, _posts[0].picturename, 0); // default set
            _posts[0].url = UserUrlConfig.ProfileUrl(_posts[0].username);
            if (_posts[0].aboutme != null && _posts[0].aboutme.Length > 0)
                _posts[0].aboutme = UtilityBLL.CompressCodeBreak(_posts[0].aboutme);

            var languages = LanguageBLLC.Load(new LanguageEntity()
            {
                iscache = true,
                isselected = 1,
                isdefault = 2,
                status = 1,
                order = "isdefault desc"
            });

            var settings = ConfigurationBLL.LoadItmes(new ConfigEntity()
            {
                iscache = true,
                type = 0 // should be 1 in production
            });

            return Json(new { post = _posts[0], languages = languages, settings = settings });
        }

        // GET: api/user/getinfo
        public ActionResult getinfo()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<MemberEntity>(json);

            data.nofilter = false;
            data.issummary = false;
            data.isdropdown = false;
            var _posts = UserBLL.LoadItems(data);
            if (_posts.Count == 0)
            {
                return Json(new { status = "error", message = "Error fetching record" });
            }
            _posts[0].img_url = UserUrlConfig.ProfilePhoto(_posts[0].username, _posts[0].picturename, 0); // default set
            _posts[0].url = UserUrlConfig.ProfileUrl(_posts[0].username);
            if (_posts[0].aboutme != null && _posts[0].aboutme.Length > 0)
                _posts[0].aboutme = UtilityBLL.CompressCodeBreak(_posts[0].aboutme);
            return Json(new { status="ok", post = _posts[0] });
        }

        // get user info with role data
        // GET: api/user/getinfo
        public ActionResult getinfo_role()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<MemberEntity>(json);

            data.nofilter = false;
            data.issummary = false;
            data.isdropdown = false;
            var _posts = UserBLL.LoadItems(data);
            if (_posts.Count == 0)
            {
                return Json(new { status = "error", message = "Error fetching record" });
            }
            _posts[0].img_url = UserUrlConfig.ProfilePhoto(_posts[0].username, _posts[0].picturename, 0); // default set
            _posts[0].url = UserUrlConfig.ProfileUrl(_posts[0].username);
            if (_posts[0].aboutme != null && _posts[0].aboutme.Length > 0)
                _posts[0].aboutme = UtilityBLL.CompressCodeBreak(_posts[0].aboutme);

            var _roles = new List<RoleDataEntity>();
            if (_posts[0].roleid > 0)
            {
                _roles = RoleBLL.LoadItems(new RoleEntity()
                {
                    id = _posts[0].roleid
                });
                if ( _roles.Count > 0)
                {
                    foreach (var post in _posts)
                    {
                        _roles[0].permissions = RolePermission.LoadItems(new RoleDPermissionEntity()
                        {
                            roleid = _roles[0].id,
                            order = "id desc",
                            pagesize = 5000,
                            pagenumber = 1
                        });
                    }
                }
            }

            return Json(new { status = "ok", post = _posts[0], role = _roles });
        }

        public ActionResult userlog()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var entity = JsonConvert.DeserializeObject<UserIPEntity>(json);

            var _data = User_IPLogBLL.LoadItems(entity);

            return Json(new { posts = _data });
        }

        // admin version (without validating existing password
        public ActionResult chpass()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<UserDataEntity>(json);
            
            var model = data;
            string encrypted_password = BCryptHelper.HashPassword(model.password, BCryptHelper.GenerateSalt(6));

            UserBLL.Update_Field_V3(model.username, "password", encrypted_password);

            return Json(new { status = "success", message = "Password updated" });

        }
        // myaccount version (need to validate old password before changing email)
        public ActionResult chpass_acc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<UserDataEntity>(json);
            
            var model = data;

            if (model.opassword == null)
            {
                return Json(new { status = "error", message = "Error occured while processing your request" });
            }


            if (model.opassword != null)
            {
                // Update Password Validation Script
                var _lst = UserBLL.Get_Hash_Password(model.username);
                if (_lst.Count == 0)
                {
                    // No user account found based on username search
                    return Json(new { status = "error", message = Resources.vsk.message_emailoptions_03 });
                }
                // check encrypted password
                if (_lst[0].password.Length < 20)
                {
                    var _memberprocess = new UserBLL();
                    // backward compatibility
                    // check existing user passwords with old system
                    if (!_memberprocess.Validate_Member(model.username, model.opassword, false))
                    {
                        return Json(new { status = "error", message = Resources.vsk.message_emailoptions_03 });
                    }
                }
                else
                {
                    // check encrypted password with user typed password
                    bool matched = BCryptHelper.CheckPassword(model.opassword, _lst[0].password);
                    if (!matched)
                    {
                        return Json(new { status = "error", message = Resources.vsk.message_emailoptions_03 });
                    }
                }
                // change password

                string encrypted_password = BCryptHelper.HashPassword(model.password, BCryptHelper.GenerateSalt(6));
                UserBLL.Update_Field_V3(model.username, "password", encrypted_password);

                ManageAccount_MailTemplateProcess(model.username);

            }
            else
            {
                string encrypted_password = BCryptHelper.HashPassword(model.password, BCryptHelper.GenerateSalt(6));

                UserBLL.Update_Field_V3(model.username, "password", encrypted_password);

            }

            return Json(new { status = "success", message = "Password updated" });

        }
        public ActionResult cemail()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<UserDataEntity>(json);
           /* if (!data.permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            } */

            var model = data;

            if (model.password != null)
            {
                // validate email address and password.
                List<UserDataEntity> _lst = UserBLL.Get_Hash_Password(model.username);
                if (_lst.Count == 0)
                {
                    return Json(new { status = "error", message = Resources.vsk.message_emailoptions_03 });
                }
                // check encrypted password
                if (_lst[0].password.Length < 20)
                {
                    // backward compatibility
                    if (!UserBLL.Validate_Member_Email(model.email, model.password))
                    {
                        return Json(new { status = "error", message = Resources.vsk.message_emailoptions_03 });
                    }
                }
                else
                {
                    // check encrypted password with user typed password
                    bool matched = BCryptHelper.CheckPassword(model.password, _lst[0].password);
                    if (!matched)
                    {
                        return Json(new { status = "error", message = Resources.vsk.message_emailoptions_03 });
                    }
                }

                string val_key = Guid.NewGuid().ToString().Substring(0, 10);
                // update user validation key
                UserBLL.Update_Field_V3(model.username, "val_key", val_key);

                // send mail validation request on new email address
                Emailo_MailTemplateProcess(model.email, model.username, val_key);

            }
            else
            {
                UserBLL.Update_Field_V3(model.username, "email", model.email);
            }

            return Json(new { status = "success", message = "Password updated" });
        }

        public ActionResult ctype()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<UserDataEntity>(json);
            /* if (!data.permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            } */

            var model = data;

            UserBLL.Update_Field_V3(model.username, "type", model.type.ToString());

            return Json(new { status = "success", message = "Password updated" });
        }

        public ActionResult updatethumb()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var model = JsonConvert.DeserializeObject<UserDataEntity>(json);
            if (model.userid == 0)
            {
                return Json(new { status = "error", message = "No info found!" });
            }
            var entity = new MemberEntity();
            entity.nofilter = false;
            entity.issummary = false;
            entity.isdropdown = false;
            entity.userid = model.userid;
            var _posts = UserBLL.LoadItems(entity);
            if (_posts.Count == 0)
            {
                return Json(new { status = "error", message = "Error fetching record" });
            }

            _posts[0].picturename = model.picturename;
            var record = UserBLL.Update_User_Profile(_posts[0]);
            record.img_url = UserUrlConfig.ProfilePhoto(record.username, record.picturename, 0);
            return Json(new { status = "success", record = record, message = "Profile updated successfully" });

        }

        public ActionResult updaterole()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var model = JsonConvert.DeserializeObject<UserDataEntity>(json);
            if (model.userid == 0)
            {
                return Json(new { status = "error", message = "No info found!" });
            }
            if (model.roleid > 0)
              UserBLL.Update_Field_V3(model.userid, "roleid", model.roleid);
            
            return Json(new { status = "success", message = "User role updated successfully" });

        }
        
        public ActionResult proc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<UserDataEntity>(json);
            /* if (!data[0].permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            }*/

            var model = data;

            if (data.userid > 0)
            {
                /* Update Profile Logic */
                // check for long words
                if (UtilityBLL.isLongWordExist(model.aboutme))
                {
                    return Json(new { status = "errror", message = Resources.vsk.message_04 });
                }

                // data validation
                model.aboutme = UtilityBLL.processNull(model.aboutme, 399);
                model.website = UtilityBLL.processNull(model.website, 128);
                model.hometown = UtilityBLL.processNull(model.hometown, 50);
                model.currentcity = UtilityBLL.processNull(model.currentcity, 50);
                model.interests = UtilityBLL.processNull(model.interests, 300);

                model.firstname = UtilityBLL.processNull(model.firstname, 0);
                model.lastname = UtilityBLL.processNull(model.lastname, 0);
                model.zipcode = UtilityBLL.processNull(model.zipcode, 0);

                var record = UserBLL.Update_User_Profile(model);

                record.img_url = UserUrlConfig.ProfilePhoto(record.username, record.picturename, 0);

                return Json(new { status = "success", record = record, message = "Profile updated successfully" });
            }
            else
            {

                var _memberprocess = new UserBLL();
                if (_memberprocess.Check_UserName(data.username))
                {
                    return Json(new { status = "error", message = "Username not available" });
                }

                if (_memberprocess.Check_Email(data.email))
                {
                    return Json(new { status = "error", message = "Email already exist" });
                }

                string gender = "Male";

                // validation key processing
                string val_key = "none";
                int isenabled = 1; // user account activated
                // Add Member
                int type = data.type;

                // Credits and Default Space Allocation
                int credits = 0;
                int userrole_id = 0;

                int atype = 0;

                var entity = new UserDataEntity()
                {
                    accounttype = (short)atype,
                    username = data.username,
                    firstname = model.firstname,
                    lastname = model.lastname,
                    password = BCryptHelper.HashPassword(data.password, BCryptHelper.GenerateSalt(6)),
                    email = data.email,
                    countryname = "",
                    isenabled = (byte)isenabled,
                    picturename = "none",
                    gender = gender,
                    birthdate = DateTime.Now,
                    type = (byte)type,
                    credits = credits,
                    roleid = userrole_id
                };

                data.userid = UserBLL.Add(entity, val_key);
                // Create Required Directories
                Directory_Process.CreateRequiredDirectories(Server.MapPath(Request.ApplicationPath) + "/contents/member/" + data.username.ToLower());


                // setup url / picture url for app use only
                model.picturename = "none";
                model.isenabled = (byte)isenabled;
                model.img_url = UserUrlConfig.ProfilePhoto(model.username, model.picturename, 0); // default set
                model.url = UserUrlConfig.ProfileUrl(model.username);

                return Json(new { status = "success", record = data, message = "Account created successfully" });

            }

        }

        // GET: admin/categories/uploads
        public ActionResult uploads()
        {
            if (Request.Files.Count > 0)
            {
                int chunk = Request["chunk"] != null ? int.Parse(Request["chunk"]) : 0;
                string fileName = Request["name"] != null ? Request["name"] : string.Empty;

                string _fileName = fileName.Remove(fileName.LastIndexOf(".")) + "-" + Guid.NewGuid().ToString().Substring(0, 6) + "" + fileName.Remove(0, fileName.LastIndexOf("."));

                string UserName = "";

                if (Request.Headers["UName"] != null)
                {
                    UserName = Request.Headers["UName"].ToString();
                }

                string dirPath = Server.MapPath(Request.ApplicationPath) + "//contents//member//" + UserName + "//photos//";
                if (!Directory.Exists(dirPath))
                {
                    Directory_Process.CreateRequiredDirectories(Server.MapPath(Request.ApplicationPath) + "//contents//member//" + UserName);
                }
                FileStream fs;
                using (fs = new FileStream(Path.Combine(dirPath, _fileName), chunk == 0 ? FileMode.Create : FileMode.Append))
                {
                    byte[] buffer = new byte[Request.Files[0].InputStream.Length];
                    Request.Files[0].InputStream.Read(buffer, 0, buffer.Length);

                    fs.Write(buffer, 0, buffer.Length);
                }

                string fileType = System.IO.Path.GetExtension(_fileName);
                string fileIndex = fileName.Replace(fileType, "");
                string elementid = Guid.NewGuid().ToString().Substring(0, 6);

                /* generate thumb */
                // create small size thumb
                string strPath = Server.MapPath(Request.ApplicationPath) + "//contents//member//" + UserName + "//photos//";
                string orignalfilename = strPath + "" + _fileName;
                string thumbfilename = strPath + "thumbs/" + _fileName;
                Bitmap mp = Jugnoon.Utility.Image.CreateThumbnail(orignalfilename, Site_Settings.Thumb_Width, Site_Settings.Thumb_Height);
                if (mp == null)
                {
                    return Json(new { result = "Error" });
                }
                mp.Save(thumbfilename);

                Jugnoon.Utility.Image.SaveJpeg(thumbfilename, mp, 90);

                string url = UserUrlConfig.ProfilePhoto(UserName, _fileName, 0);
                return Json(new { jsonrpc = "2.0", result = "OK", fname = _fileName, url = url, filetype = fileType, filename = _fileName, fileIndex = fileIndex, eleid = elementid }, JsonRequestBehavior.AllowGet);
            }
            else
            {
                return Json(new { result = "Error" });
            }
        }

        public ActionResult removefile()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<MemberEntity>>(json);

            string RootPath = Server.MapPath(Request.ApplicationPath);
            foreach (var Record in data)
            {
                string dirPath = RootPath + "/contents/category/";
                string tempPath = dirPath + "temp/";
                if (!data[0].picturename.StartsWith("http"))
                {
                    if (System.IO.File.Exists(dirPath + Record.picturename))
                        System.IO.File.Delete(dirPath + Record.picturename);
                    if (System.IO.File.Exists(tempPath + Record.picturename))
                        System.IO.File.Delete(tempPath + Record.picturename);
                }

            }

            return Json(new { status = "success", message = "Files removed successfully" });
        }
        // GET: admin/categories/action
        public ActionResult action()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<MemberEntity>>(json);
            /*if (!data[0].permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            } */

            UserBLL.ProcessAction(data);

            return Json(new { status = "success", message = "Operation performed successfully" });
        }

        // send email validation
        private void Emailo_MailTemplateProcess(string emailaddress, string username, string key)
        {
            //if sending mail option enabled
            if (Config.isEmailEnabled())
            {
                var lst = MailTemplateBLL.Get_Record("USREMLCREQ");
                if (lst.Count > 0)
                {
                    string subject = MailProcess.Process2(lst[0].subject, "\\[username\\]", username);
                    string contents = MailProcess.Process2(lst[0].contents, "\\[username\\]", username);
                    contents = MailProcess.Process2(contents, "\\[key_url\\]", key);
                    contents = MailProcess.Process2(contents, "\\[emailaddress\\]", emailaddress);

                    MailProcess.Send_Mail(emailaddress, subject, contents);
                }
            }
        }
        // notification for changing email options
        private void Emailo_MailTemplateProcess_EmlOptions(string username)
        {
            //if sending mail option enabled
            if (Config.isEmailEnabled())
            {
                string emailaddress = UserBLL.Return_Value(username, "email");
                var lst = MailTemplateBLL.Get_Record("USREMLOPT");
                if (lst.Count > 0)
                {
                    string subject = MailProcess.Process2(lst[0].subject, "\\[username\\]", username);
                    string contents = MailProcess.Process2(lst[0].contents, "\\[username\\]", username);

                    MailProcess.Send_Mail(emailaddress, subject, contents);
                }
            }
        }

        private void ManageAccount_MailTemplateProcess(string username)
        {
            //if sending mail option enabled
            if (Config.isEmailEnabled())
            {
                string emailaddress = UserBLL.Return_Value(username, "email");
                var lst = MailTemplateBLL.Get_Record("USRPASSCHN");
                if (lst.Count > 0)
                {
                    string subject = MailProcess.Process2(lst[0].subject, "\\[username\\]", username);
                    string contents = MailProcess.Process2(lst[0].contents, "\\[username\\]", username);

                    MailProcess.Send_Mail(emailaddress, subject, contents);
                }
            }
        }

    }
}

