﻿using Jugnoon.BLL;
using Jugnoon.Entity;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web.Mvc;
using Jugnoon.Utility;
using Newtonsoft.Json;
using Jugnoon.Settings;
using System.Collections;
using Jugnoon.Scripts;

namespace JgnEnt.Areas.api.Controllers
{
    public class videosController : Controller
    {   

        // GET: admin/videos/load
        public ActionResult load()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<VideoEntity>(json);
            data.issummary = false;
            data.isdropdown = false;
            if (!data.order.Contains("asc") && !data.order.Contains("desc") && data.direction != "")
                data.order = data.order + " " + data.direction;

            data.order = data.order + " " + data.direction;
            
            var _posts = VideoBLL.LoadItems(data);

            /* setup thumb path */
            foreach (var item in _posts)
            {
                item.picturename = UGeneral.ProcessVideoThumb("", item, false, new Jugnoon.Scripts.ListItems()); // default set
                item.url = VideoUrlConfig.PrepareUrl(item);
                item.author_url = UserUrlConfig.ProfileUrl(item.username);
                item.customize_date = UtilityBLL.CustomizeDate((DateTime)item.created_at, DateTime.Now);
            }
            var _records = 0;
            if (data.videoid == 0)
                _records = VideoBLL.Count(data);
            var _type = 0;
            switch (data.type)
            {
                case 0:
                    _type = 0; // videos
                    break;
                case 1:
                    _type = 7; // audio
                    break;
            }
            var _categories = new List<CategoryDataEntity>();
            if (data.loadstats)
            {
                _categories = CategoryBLL.LoadItems(new CategoryEntity()
                {
                    id = 0,
                    type = _type,
                    mode = 0,
                    status = 2,
                    parentid = -1,
                    order = "level asc", // don't change this
                    issummary = false,
                    isdropdown = true,
                    loadall = true // load all data
                });
            }

            return Json(new { posts = _posts, records = _records, categories = _categories }, System.Web.Mvc.JsonRequestBehavior.AllowGet);
        }
        

        public ActionResult ffmpeg_proc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<SaveVideoInfo>>(json);

            return Json(VideoBLL.ffmpeg_proc(data));
        }

        public ActionResult direct_proc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<SaveVideoInfo>>(json);

            return Json(VideoBLL.direct_proc(data));
        }
        
        /* yotuube fetcher */
        public ActionResult yt_proc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<VideoDataEntity>>(json);

            return Json(VideoBLL.yt_proc(data));
        }
        /* embed script */
        public ActionResult embed_proc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<VideoDataEntity>>(json);

            return Json(VideoBLL.embed_proc(data));
        }


        /* get youtube supported categories */
        public ActionResult get_yt_categories()
        {
            var _yt = new YoutubeCategories();
            var model = new YoutubeEntity
            {
                OrderList = JgnEnt.Areas.admin.models.videos.youtube.GeneralTypes.OrderList(),
                YoutubeCategories = _yt.GetVideoCategories(),
                DateList = JgnEnt.Areas.admin.models.videos.youtube.GeneralTypes.DateList(),
                uploaddate = 3
            };

            return Json(new { categorylist = model });
        }

        public ActionResult audio_proc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<SaveVideoInfo>>(json);

            return Json(VideoBLL.audio_proc(data));
        }

        public ActionResult fetch_youtube()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<YoutubeEntity>(json);

            if (data.term == "")
            {
                return Json(new { status = "error", message = "please enter term" });
            }
            var obj = new VideoBLL();
            if (data.term.StartsWith("http"))
                return Json(obj.yt_single_proc(data)); // single youtube video
            else
                return Json(obj.yt_multiple_proc(data)); // multiple youtube videos
        }


        // for admin video information purpose
        // GET: api/videos/getinfo
        public ActionResult getinfo()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<VideoEntity>(json);
            data.issummary = false;
            data.isdropdown = false;
            data.nofilter = true;
            var _posts = VideoBLL.LoadItems(data);
            if (_posts.Count == 0)
            {
                return Json(new { status = "error", message = "no record found" });
            }

            _posts[0].picturename = Jugnoon.Scripts.UGeneral.ProcessVideoThumb("", _posts[0], false, new Jugnoon.Scripts.ListItems()); // default set
            _posts[0].customize_date = UtilityBLL.CustomizeDate((DateTime)_posts[0].created_at, DateTime.Now);
            var model = _posts[0];
            // Player processing
            // Video Path settings
            string VideoPath = "";
            string PictureName = "";
            if (model.type == 0)
            {
                // video settings
                VideoPath = Config.Return_FLV_Video_Url(model.flv_url, model.username) + "/" + model.videofilename;
                if (model.thumb_url != "none")
                    PictureName = model.thumb_url;
                else
                    PictureName = Config.Return_Video_Thumb_Url(model.thumb_url, model.username) + "/" + model.thumbfilename;
                string MP4Path = "";
                if (VideoPath.EndsWith(".webm"))
                    MP4Path = VideoPath.Replace(".webm", ".mp4");
            }
            else
            {
                // audio settings
                VideoPath = Config.Return_MP3_Audio_Url(model.flv_url, model.username) + "/" + model.videofilename;
                if (model.thumb_url != "none")
                    PictureName = model.thumb_url;
                else if (model.coverurl.StartsWith("http"))
                    PictureName = model.coverurl;
                else
                    PictureName = Config.Return_Video_Thumb_Url(model.thumb_url, model.username) + "/" + model.thumbfilename;
            }

            // script specifically for updating default video thumbnail
            _posts[0].thumb_url = UGeneral.ProcessVideoThumb("", _posts[0], false, new ListItems()); // default set
            // setup thumb index 
            if (_posts[0].thumb_url.Contains("_"))
                _posts[0].picturename = _posts[0].thumb_url.Remove(_posts[0].thumb_url.LastIndexOf("_"));
            else
                _posts[0].picturename = _posts[0].thumb_url;
                       
            if (_posts[0].username != "")
            {
                var user = UserBLL.LoadItems(new MemberEntity()
                {
                    nofilter = true,
                    username = _posts[0].username,
                });
                if (user.Count > 0)
                {
                    _posts[0].author = user[0];
                    _posts[0].author.img_url = UserUrlConfig.ProfilePhoto(_posts[0].author.username, _posts[0].author.picturename, 0); // default set
                }
            }

            string _script = model.embed_script;
            string _videoUrl = "";
            if (model.preview_url != null && model.preview_url != "")
                _videoUrl = model.preview_url;
            else
                _videoUrl = model.flv_url;

            model.player = new PlayerEntity();
            model.player.embedscript = "";
            model.player.youtubeid = "";

            model.player.type = model.type;

            if (model.coverurl != "")
                model.player.picturename = model.coverurl;
            else
                model.player.picturename = PictureName;

            if (model.type == 0)
            {
                // video player settings
                // If cloud enabled
                if (_script != "" || model.youtubeid != "")
                {
                    // third party embed video script enabled
                    if (_script == "")
                    {
                        model.player.youtubeid = "https://www.youtube.com/embed/" + model.youtubeid.Replace("/watch?v=", "");
                    }
                    else
                    {
                        model.player.embedscript = _script;
                    }
                }
                else if (model.videofilename.EndsWith("mp4"))
                {
                    if (TokenBLL.isTokenEnabled)
                    {
                        // Token Based Stream Authentication Enabled
                        // Recommended to stream on HTML5 Players
                        string maxParam = VideoBLL.GenerateBandwidthParam();
                        string Token = TokenBLL.Add();
                        model.player.url = "/stream/token.ashx?f=" + model.videofilename + "&u=" + model.username + "&tk=" + Token + "" + maxParam;
                        if (model.originalvideofilename.EndsWith(".png"))
                            model.player.picturename = "/contents/member/" + model.username + "/thumbs/" + model.originalvideofilename;
                    }
                    else if (VideoBLL.EnableHttpStreaming)
                    {
                        // Http Streaming Enabled
                        // Recommended to stream on HTML5 Players
                        string maxParam = VideoBLL.GenerateBandwidthParam();
                        model.player.url = "/stream/stream.ashx?f=" + model.videofilename + "&u=" + model.username + "" + maxParam;
                    }
                    else
                    {
                        // Direct Play
                        model.player.url = VideoPath;
                    }
                }
                else
                {
                    // load and play videos normally
                    model.player.url = VideoPath;
                }
            }
            else
            {
                // audio player settings
                model.player.url = VideoPath;
                model.player.picturename = PictureName;
            }

            return Json(new { post = model });
        }

        // for myaccount purpose (allow you to edit video information)
        public ActionResult getinfo_acc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<VideoEntity>(json);
            data.issummary = false;
            data.isdropdown = false;
            data.nofilter = true;
            var _posts = VideoBLL.LoadItems(data);
            if (_posts.Count == 0)
            {
                return Json(new { status = "error", message = "no record found" });
            }

            // script specifically for updating default video thumbnail
            _posts[0].thumb_url = UGeneral.ProcessVideoThumb("", _posts[0], false, new ListItems()); // default set
            // setup thumb index 
            if (_posts[0].thumb_url.Contains("_"))
                _posts[0].picturename = _posts[0].thumb_url.Remove(_posts[0].thumb_url.LastIndexOf("_"));
            else
                _posts[0].picturename = _posts[0].thumb_url;

            return Json(new { status = "success", post = _posts[0] });
        }

        /* edit video api */
        public ActionResult proc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<VideoDataEntity>>(json);

            var model = data[0];

            if (model.tags != "")
            {
                // validate tags
                if (!TagsBLL.Validate_Tags(model.tags))
                {
                    return Json(new { status = "error", message = "Invalid tags" });
                }
            }


            if (UtilityBLL.isLongWordExist(model.title) || UtilityBLL.isLongWordExist(model.title))
            {
                return Json(new { status = "error", message = "Invalid title" });
            }


            var vd = new VideoDataEntity();
            vd.title = model.title;
            vd.description = Jugnoon.Scripts.UGeneral.SanitizeText(model.description);
            vd.tags = model.tags;
            vd.videoid = model.videoid;
            if (model.categories != null)
                vd.categories = string.Join(",", model.categories);
            vd.iscomments = (byte)model.iscomments;
            vd.isratings = (byte)model.isratings;
            vd.isprivate = (byte)model.isprivate;
            vd.views = model.views;
            vd.liked = model.liked;
            vd.disliked = model.disliked;
            vd.avg_rating = (byte)model.avg_rating;
            vd.total_rating = model.total_rating;
            vd.ratings = (byte)model.avg_rating * model.ratings;
            vd.isenabled = (byte)model.isenabled;
            vd.isreviewed = (byte)model.isreviewed;
            vd.isfeatured = (byte)model.isfeatured;

            VideoBLL.Update_VideoInfo_Adm(vd);

            // Process tags
            if (model.tags != "")
            {
                int tag_type = 0; // represent videos
                TagsBLL.Process_Tags(model.tags, tag_type, 0);
            }

            return Json(new { status = "success", record = vd, message = "updated" });
        }

        /* edit video for angular 5 app */
        public ActionResult editvideos()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<VideoDataEntity>>(json);

            var model = data[0];

            if (model.tags != "")
            {
                // validate tags
                if (!TagsBLL.Validate_Tags(model.tags))
                {
                    return Json(new { status = "error", message = "Invalid tags" });
                }
            }


            if (UtilityBLL.isLongWordExist(model.title) || UtilityBLL.isLongWordExist(model.title))
            {
                return Json(new { status = "error", message = "Invalid title" });
            }


            var vd = new VideoDataEntity();
            vd.title = model.title;
            vd.description = UGeneral.SanitizeText(model.description);
            vd.tags = model.tags;
            vd.videoid = model.videoid;
            if (model.categories != null)
                vd.categories = string.Join(",", model.categories);
            //vd.iscomments = (byte)model.iscomments;
            //vd.isratings = (byte)model.isratings;
            //vd.isprivate = (byte)model.isprivate;
            //vd.views = model.views;
            //vd.liked = model.liked;
            //vd.disliked = model.disliked;
            //vd.avg_rating = (byte)model.avg_rating;
            //vd.total_rating = model.total_rating;
            //vd.ratings = (byte)model.avg_rating * model.ratings;
            ///vd.isenabled = (byte)model.isenabled;
            //vd.isreviewed = (byte)model.isreviewed;
            //vd.isfeatured = (byte)model.isfeatured;

            VideoBLL.Update_VideoInfo_Adm_V2(vd);

            // Process tags
            if (model.tags != "")
            {
                int tag_type = 0; // represent videos
                TagsBLL.Process_Tags(model.tags, tag_type, 0);
            }

            return Json(new { status = "success", record = vd, message = "updated" });
        }

        /* edit video for myaccount app */
        public ActionResult editvideos_macc()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<VideoDataEntity>(json);

            var model = data;

            if (model.videoid == 0)
            {
                return Json(new { status = "error", message = "Record ID Missing" });
            }
            if (model.tags != "")
            {
                // validate tags
                if (!TagsBLL.Validate_Tags(model.tags))
                {
                    return Json(new { status = "error", message = "Invalid tags" });
                }
            }


            if (UtilityBLL.isLongWordExist(model.title) || UtilityBLL.isLongWordExist(model.title))
            {
                return Json(new { status = "error", message = "Invalid title" });
            }


            var vd = new VideoDataEntity();
            vd.title = model.title;
            vd.description = UGeneral.SanitizeText(model.description);
            vd.tags = model.tags;
            vd.videoid = model.videoid;
            if (model.categories != null)
                vd.categories = string.Join(",", model.categories);
            vd.iscomments = (byte)model.iscomments;
            vd.isratings = (byte)model.isratings;
            vd.isprivate = (byte)model.isprivate;
            //vd.views = model.views;
            //vd.liked = model.liked;
            //vd.disliked = model.disliked;
            //vd.avg_rating = (byte)model.avg_rating;
            //vd.total_rating = model.total_rating;
            //vd.ratings = (byte)model.avg_rating * model.ratings;
            ///vd.isenabled = (byte)model.isenabled;
            //vd.isreviewed = (byte)model.isreviewed;
            //vd.isfeatured = (byte)model.isfeatured;

            VideoBLL.Update_Video_Info(vd);

            // Process tags
            if (model.tags != "")
            {
                int tag_type = 0; // represent videos
                TagsBLL.Process_Tags(model.tags, tag_type, 0);
            }

            return Json(new { status = "success", record = vd, message = "updated" });
        }
        public ActionResult upload()
        {
            return Json(VideoBLL.upload(""), JsonRequestBehavior.AllowGet);
        }

        // GET: admin/categories/action
        public ActionResult action()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<VideoEntity>>(json);
            /* if (!data[0].permission.canupdate)
            {
                return Json(new { status = "error", message = "You have no permission to perform this action" });
            } */

            VideoBLL.ProcessAction(data);
            //var jsonResult = Json(new { posts = _posts, records = _records }, JsonRequestBehavior.AllowGet);
            //jsonResult.MaxJsonLength = int.MaxValue;
            //return jsonResult;
            return Json(new { status = "success", message = "Operation performed successfully" });
        }

        public ActionResult proc_new()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<List<VideoDataEntity>>(json);

            foreach (var _rec in data)
            {
                _rec.isenabled = 1; // by default enabled
                if (Site_Settings.Content_Approval == 1)
                    _rec.isreviewed = 1;
                else
                    _rec.isreviewed = 0;

                var _counter = 1;
                foreach (var thumb in _rec.thumbs)
                {
                    var _prefix = "00";
                    if (_counter <= 9)
                        _prefix = "00";
                    else if (_counter <= 99)
                        _prefix = "0";
                    else
                        _prefix = "";


                    /* save base64 image in physical path */
                    //byte[] image = Convert.FromBase64String(thumb.filename.Replace("data:image/png;base64,", ""));
                    string thumbFileName = Guid.NewGuid().ToString().Substring(0,6) + _rec.videofilename.Remove(_rec.videofilename.LastIndexOf(".")) + "_" + _prefix + "" + _counter + ".png";
                    _counter++;
                    string old_path = VideoUrlConfig.Thumbs_Path(_rec.username) + "/" + thumb.filename;
                    string new_path = VideoUrlConfig.Thumbs_Path(_rec.username) + "/" + thumbFileName;

                    System.IO.File.Move(old_path, new_path);
                    //System.IO.File.WriteAllBytes(path, image);

                    //if (thumb.selected)
                    _rec.thumbfilename = thumbFileName;

                    //_rec.tfile = _rec.thumbfilename;
                }
                try
                {
                    // in case of direct uploader, there is no published video, source video is published.
                    // shift source video to published directory
                    var SourcePath = VideoUrlConfig.Source_Video_Path(_rec.username); // no source video there
                    var publishedPath = VideoUrlConfig.Published_Video_Path(_rec.username);
                    var thumbsPath = VideoUrlConfig.Thumbs_Path(_rec.username);
                    if (System.IO.File.Exists(publishedPath + "/" + _rec.videofilename))
                        System.IO.File.Delete(publishedPath + "/" + _rec.videofilename);

                    System.IO.File.Move(SourcePath + "/" + _rec.videofilename, publishedPath + "/" + _rec.videofilename);
                
                }
                catch (Exception ex)
                {
                    var err = ex.Message;

                }

                string _publish_path = "none";
                string _thumb_path = "none";

                var Item = new VideoDataEntity()
                {
                    title = _rec.title,
                    description = _rec.description,
                    tags = _rec.tags,
                    galleryid = _rec.galleryid,
                    categoryid = 0,
                    categories = string.Join(",", _rec.categories),
                    duration = _rec.duration,
                    originalvideofilename = _rec.videofilename,
                    videofilename = _rec.videofilename,
                    thumbfilename = _rec.thumbfilename,
                    isenabled = _rec.isenabled,
                    isreviewed = _rec.isreviewed,
                    username = _rec.username,
                    type = _rec.type,
                    duration_sec = _rec.duration_sec,
                    isprivate = 0, // _rec.privacy,
                    authkey = "",
                    flv_url = _publish_path,
                    thumb_url = _thumb_path,
                    org_url = "",
                    isexternal = 0,
                    youtubeid = "",
                    istagsreviewed = 1,
                    mode = 0,
                    ispublished = 1,
                    errorcode = _rec.errorcode,
                    ipaddress = Request.ServerVariables["REMOTE_ADDR"].ToString(),
                    embed_script = "",
                    // submittype = 2 // ffmpeg target type
                };


                Item.isreviewed = 1;
                // Approved by Default
                if (Site_Settings.Content_Approval == 0)
                {
                    // Moderator Review Required
                    Item.isreviewed = 0;
                }
                //XSS CLEANUP
                // not working on medium trust
                //string content = Sanitizer.GetSafeHtmlFragment(txt_content.Value);
                // Compress Code => remove \r\t etc which create unnecessary line breaking issues
                string content = UtilityBLL.CompressCode(Item.description);

                // Process Contents -> links, bbcodes etc
                content = UtilityBLL.Process_Content_Text(content);


                // Generate Gallery Preview
                //Content = GalleryBLLC.Generate_Blog_Gallery_Previews(Content)


                Item.videoid = VideoBLL.Process_Info(Item);

                if (Item.tags != "")
                {
                    int tag_type = 0; // represent videos
                    if (Item.type == 1)
                        tag_type = 4; // represent audio file
                    TagsBLL.Process_Tags(Item.tags, tag_type, 0);
                }
            }
            return Json(new { status = "success", message = "video processed successfully" });

        }


        public ActionResult thumbuploads()
        {
            if (Request.Files.Count > 0)
            {
                int chunk = Request["chunk"] != null ? int.Parse(Request["chunk"]) : 0;
                string fileName = Request["name"] != null ? Request["name"] : string.Empty;

                string _fileName = fileName.Remove(fileName.LastIndexOf(".")) + "-" + Guid.NewGuid().ToString().Substring(0, 6) + "" + fileName.Remove(0, fileName.LastIndexOf("."));

                string UserName = "";

                if (Request.Headers["UName"] != null)
                {
                    UserName = Request.Headers["UName"].ToString();
                }

                string dirPath = Server.MapPath(Request.ApplicationPath) + "//contents//member//" + UserName + "//thumbs//";

                FileStream fs;
                using (fs = new FileStream(Path.Combine(dirPath, _fileName), chunk == 0 ? FileMode.Create : FileMode.Append))
                {
                    byte[] buffer = new byte[Request.Files[0].InputStream.Length];
                    Request.Files[0].InputStream.Read(buffer, 0, buffer.Length);

                    fs.Write(buffer, 0, buffer.Length);
                }

                string fileType = System.IO.Path.GetExtension(_fileName);
                string fileIndex = fileName.Replace(fileType, "");
                string elementid = Guid.NewGuid().ToString().Substring(0, 6);

                /* generate thumb */
                // create small size thumb
                // string strPath = Server.MapPath(Request.ApplicationPath) + "contents/blog";

                //string uploadPath = UrlConfig.Photo_Upload_Path(UserName, 0); // original photo
                //string miduploadpath = UrlConfig.Photo_Upload_Path(UserName, 1); // mid thumb photo
                //string thumbuploadpath = UrlConfig.Photo_Upload_Path(UserName, 2); // thumb photo

                string uploadcompletepath = dirPath + "\\" + _fileName;
                string miduploadcompletepath = dirPath + "\\" + "thumb_" + _fileName;
                //string thumbcompletepath = thumbuploadpath + "\\" + _fileName;

                Jugnoon.Utility.Image.Generate_Thumbs(uploadcompletepath, "", miduploadcompletepath, Site_Settings.Thumb_Width, Site_Settings.Thumb_Height, Site_Settings.Thumb_Width, Site_Settings.Thumb_Height);
                
                string url = VideoUrlConfig.Thumb_Url(UserName) + "\\" + "thumb_" + _fileName;

                return Json(new { jsonrpc = "2.0", result = "OK", fname = _fileName, url = url, filetype = fileType, filename = "thumb_" + _fileName, fileIndex = fileIndex, eleid = elementid }, JsonRequestBehavior.AllowGet);
            }
            else
            {
                return Json(new { result = "Error" });
            }
        }

        // check ownership of video (for myaccount purpose)
        public ActionResult authorize_author()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<VideoDataEntity>(json);
            var isaccess = VideoBLL.Check(data.videoid, data.username);
            return Json(new { isaccess = isaccess });
        }

        public ActionResult update_thumbnail()
        {
            var json = new StreamReader(Request.InputStream).ReadToEnd();
            var data = JsonConvert.DeserializeObject<VideoDataEntity>(json);

            if (data.videoid == 0)
            {
                return Json(new { status = "error", message = "no record found" });
            }
            if (data.picturename == "")
            {
                return Json(new { status = "error", message = "no thumbnail selected" });
            }

            VideoBLL.Update_Field_V3(data.videoid, data.picturename, "thumb_url");

            return Json(new { status = "success", message = "Video thumbnail updated" });
        }
        
    }
}
