﻿using Jugnoon.BLL;
using Jugnoon.Entity;
using Jugnoon.models;
using Jugnoon.Utility;
using System.Threading;
using System.Web.Mvc;
using Jugnoon.Settings;

namespace JgnEnt.Controllers
{
    public class HomeController : Controller
    {
        // GET: Home
        public ActionResult Index() 
        {
            ViewBag.title = Site_Settings.Page_Caption;

            if (Request.Params["lng"] != null)
            {
                string culturename = Request.Params["lng"].ToString();
                //string language = LanguageBLLC.Return_CultureName(culturename);
                UtilityBLL.WriteCookie("CultureInfo", culturename);
                
                return Redirect(Config.GetUrl());
            }
            if (Request.Params["theme"] != null)
            {
                string theme = Request.Params["theme"].ToString();
                UtilityBLL.WriteCookie("VSKTheme", theme);

                return Redirect(Config.GetUrl());
            }
            var currentCulture = Thread.CurrentThread.CurrentCulture;

            return View();
        }

        public ActionResult contact()
        {
            ViewBag.title= Resources.vsk.contactus + " | " + Site_Settings.Page_Caption;

            var model = new ContactUsViewModel();
            model.Message = "";
            model.AlertType = AlertTypes.Error;
            return View(model);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult contact(ContactUsViewModel model, string action)
        {
            ViewBag.title = Resources.vsk.contactus + " | " + Site_Settings.Page_Caption;

            if (ModelState.IsValid)
            {
                SendMessage(model, Site_Settings.Admin_Mail);

                model.Message = "Message Sent";
                model.AlertType = AlertTypes.Success;
                return View(model);
            }

            // if not validated
            model.Message = ValidationError.HasErrors(ModelState);
            return View(model);
        }

        public static void SendMessage(ContactUsViewModel entity, string email)
        {
            //if sending mail option enabled
            if (Config.isEmailEnabled())
            {
                System.Collections.Generic.List<MailTemplateEntity> lst = MailTemplateBLL.Get_Record("CONTACTUS");
                if (lst.Count > 0)
                {
                    string subject = MailProcess.Process2(lst[0].subject, "\\[fullname\\]", entity.SenderName);

                    string contents = MailProcess.Process2(lst[0].contents, "\\[fullname\\]", entity.SenderName);
                    contents = MailProcess.Process2(contents, "\\[phone\\]", entity.PhoneNumber);
                    contents = MailProcess.Process2(contents, "\\[email\\]", entity.EmailAddress);
                    contents = MailProcess.Process2(contents, "\\[message\\]", entity.Body);

                    MailProcess.Send_Mail(email, subject, contents);
                }
            }
        }
        public ActionResult ipblocked()
        {
            ViewBag.title= Resources.vsk.ipaddressblocked + " | " + Site_Settings.Page_Caption;

            return View();
        }

        public ActionResult privacy()
        {
            ViewBag.title= Resources.vsk.privacypolicy + " | " + Site_Settings.Page_Caption;

            return View();
        }

        public ActionResult terms()
        {
            ViewBag.title= Resources.vsk.termsofuse + " | " + Site_Settings.Page_Caption;

            return View();
        }

        public ActionResult sitemap()
        {
            ViewBag.title= Resources.vsk.sitemap + " | " + Site_Settings.Page_Caption;

            return View();
        }
        public ActionResult about()
        {
            ViewBag.title= Resources.vsk.about + " | " + Site_Settings.Page_Caption;

            return View();
        }
        public ActionResult validateadult()
        {
            ViewBag.title= Resources.vsk.adultwarning_title + " | " + Site_Settings.Page_Caption;

            return View();
        }

        private ActionResult validateemail()
        {
            if (Request.Params["key"] != null && Request.Params["user"] != null && Request.Params["eml"] != null)
            {
                // validate key / username
                string key = Request.Params["key"].ToString();
                string user = Request.Params["user"].ToString();

                var _members = new UserBLL();

                if (_members.Check_Key(user, key))
                {
                    // update email address
                    UserBLL.Update_Field_V3(user, "Email", Request.Params["eml"].ToString());
                    // send mail
                    EmailValidate_MailTemplateProcess(user);

                    return Redirect(Config.GetUrl("account/emailoptions?status=activated"));
                }
                else
                {
                    // key validation failed
                    return Redirect(Config.GetUrl("account/emailoptions?status=notvalidate"));
                }
            }
            else
            {
                // key validation failed
                return Redirect(Config.GetUrl("account/emailoptions?status=notvalidate"));
            }
        }
        // notification for changing email options
        private void EmailValidate_MailTemplateProcess(string username)
        {
            //if sending mail option enabled
            if (Config.isEmailEnabled())
            {
                string emailaddress = UserBLL.Return_Value(username, "email");

                var lst = MailTemplateBLL.Get_Record("USREMLCHNG");
                if (lst.Count > 0)
                {

                    string subject = MailProcess.Process2(lst[0].subject, "\\[username\\]", username);
                    string contents = MailProcess.Process2(lst[0].contents, "\\[username\\]", username);

                    MailProcess.Send_Mail(emailaddress, subject, contents);
                }
            }
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult validateadult(string submit)
        {
            switch (submit)
            {
                case "enter":
                    return (Enter());
                case "canel":
                    return (Cancel());
                default:
                    return (View());
            }
        }

        private ActionResult Enter()
        {
            // start adult session for this member
            Session["adultmember"] = Resources.vsk.adult;

            if (Request.Params["surl"] != null)
                return Redirect(Request.Params["surl"].ToString());
            else
                return Redirect(Config.GetUrl());
        }

        private ActionResult Cancel()
        {
            return Redirect(Config.GetUrl());
        }

    }
}