﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using Jugnoon.Entity;
using Jugnoon.Utility;
using Jugnoon.BLL;
using Jugnoon.models;
using System.Text;
using Jugnoon.Settings;
using Jugnoon.Scripts;
using JgnEnt.models.ViewModels.Albums;

namespace JgnEnt.Controllers
{
    public class albumsController : Controller
    {
        // GET: albums
        #region Feed
        // feed atom url format /albums/atom/photos or /albums/atom/videos or /albums/atom/audio
        public ActionResult atom(string type)
        {
            string sXml = GalleryBLLC.generateATOM(new AlbumEntity()
            {
                type = returnAlbumType(type),
                pagenumber = 1,
                ispublic = true,
                order = "galleryid desc",
                pagesize = 10
            }, Config.GetUrl("albums/"));

            return this.Content(sXml, "text/xml");
        }
        // feed rss url format /albums/rss/photos or /albums/rss/videos or /albums/rss/audio
        public ActionResult rss(string type)
        {
            string sXml = GalleryBLLC.generateRSS(new AlbumEntity()
            {
                type = returnAlbumType(type),
                pagenumber = 1,
                ispublic = true,
                order = "galleryid desc",
                pagesize = 10
            });

            return this.Content(sXml, "text/xml");
        }
        #endregion

        #region Public Region

        // GET: videos
        public ActionResult Index(string order, string filter, string type, int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;

            if (type == null)
                type = "photos";

            string _order = "g.added_date desc";
            int _dateFilter = 0;
            int _isFeatured = 3; // 0: normal, 1: featured, 2: premium/paid, 3: all
            string _title = Resources.vsk.recentlyadded + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums;
            var _HeadingTitle = _title;

            // pagination
            var DefaultUrl = Config.GetUrl("albums/" + type);
            var PaginationUrl = Config.GetUrl("albums/" + type + "/page/[p]/");

            // order
            var selectedOrder = "Recent";
            var selectedFilter = "All Time";

            if (order != null)
            {
                DefaultUrl = Config.GetUrl("albums/" + type + "/"  + order.ToLower().Trim());
                PaginationUrl = Config.GetUrl("albums/" + type + "/" + order.ToLower().Trim() + "/[p]/");
                switch (order)
                {
                    case "mostviewed":
                        selectedOrder = "Most Viewed";
                        _order = "v.views desc, g.added_date desc";
                        _title = Resources.vsk.mostviewed + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums; //Resources.vsk.albums_view_title;
                        _HeadingTitle = _title;
                        break;
                    case "toprated":
                        selectedOrder = "Top Rated";
                        _order = "v.avg_rating desc, v.views desc";
                        _title = Resources.vsk.toprated + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums;
                        _HeadingTitle = _title;
                        break;
                    case "featured":
                        selectedOrder = "Featured";
                        _order = "g.added_date desc";
                        _isFeatured = 1;
                        _title = Resources.vsk.featured + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums;
                        _HeadingTitle = _title;
                        break;
                }
                _HeadingTitle = _title;
            }

            if (filter != null)
            {
                // pagination setting
                if (filter == "today" || filter == "thisweek" || filter == "thismonth")
                {
                    DefaultUrl = Config.GetUrl("albums/" + type + "/added/" + filter.ToLower().Trim());
                    PaginationUrl = Config.GetUrl("albums/" + type + "/added/" + filter.ToLower().Trim() + "/[p]/");
                }

                switch (filter)
                {
                    case "today":
                        selectedFilter = "Today";
                        _dateFilter = 1; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added Today";
                        break;
                    case "thisweek":
                        selectedFilter = "This Week";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Week";
                        break;
                    case "thismonth":
                        selectedFilter = "This Month";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Month";
                        break;
                }
                _HeadingTitle = _title;
            }

            if (pagenumber > 1)
            {
                _HeadingTitle = _HeadingTitle + " - Page: " + pagenumber;
            }


            /* List Initialization */
            var ListEntity = new AlbumListViewModel()
            {
                AlbumType = returnAlbumType(type),
                AlbumTypePath = type,
                isListStatus = true,
                isListNav = true,
                // setup list navigation urls
                Navigation = prepareFilterLinks("/albums/", selectedOrder, selectedFilter, type),
                QueryOptions = new AlbumEntity()
                {
                    type = returnAlbumType(type),
                    pagenumber = (int)pagenumber,
                    term = "",
                    iscache = true,
                    ispublic = true,
                    isfeatured = _isFeatured, // 0: normal, 1: featured, 2: premium/paid, 3: all
                    pagesize = Site_Settings.Galleries_Page_Size,
                    datefilter = _dateFilter,
                    order = _order
                },
                ListObject = new ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Link, // 0: div, 1: a
                    showRating = true,
                    showViews = true,
                    showDate = true
                },
                HeaderTextCss = "mb30",
                DefaultUrl = DefaultUrl,
                PaginationUrl = PaginationUrl,
                NoRecordFoundText = Resources.vsk.norecordfound,
                HeadingTitle = _HeadingTitle
            };

            ListEntity.TotalRecords = GalleryBLLC.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = GalleryBLLC.LoadItems(ListEntity.QueryOptions);

            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;
            //ViewBag.description = _description;

            return View(ListEntity);
        }

        // GET: albums/category
        public ActionResult category(string title, string order, string filter, string type, int? pagenumber)
        {
            if (type == null)
                type = "photos";

            if (title == null)
                return Redirect(Config.GetUrl("albums/" + type));

            if (pagenumber == null)
                pagenumber = 1;
                      
            string _term = UtilityBLL.ReplaceHyphinWithSpace(title);
            string categoryName = UtilityBLL.UppercaseFirst(_term);

            string _order = "g.added_date desc";
            int _dateFilter = 0;
            int _isFeatured = 3; // 0: normal, 1: featured, 2: premium/paid, 3: all
            string _title = categoryName + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums;
            var _HeadingTitle = _title;

            // pagination
            var DefaultUrl = Config.GetUrl("albums/" + type + "/category/" + title);
            var PaginationUrl = Config.GetUrl("albums/" + type + "/category/" + title + "/[p]/");

            // order
            var selectedOrder = "Recent";
            var selectedFilter = "All Time";

            if (order != null)
            {
                DefaultUrl = Config.GetUrl("albums/" + type + "/category/" + title + "/" + order.ToLower().Trim());
                PaginationUrl = Config.GetUrl("albums/" + type + "/category/" + title + "/" + order.ToLower().Trim() + "/[p]/");

                switch (order)
                {
                    case "mostviewed":
                        selectedOrder = "Most Viewed";
                        _order = "v.views desc, g.added_date desc";
                        _title = categoryName + " " + Resources.vsk.mostviewed + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums; //Resources.vsk.albums_view_title;
                        _HeadingTitle = _title;
                        break;
                    case "toprated":
                        selectedOrder = "Top Rated";
                        _order = "v.avg_rating desc, v.views desc";
                        _title = categoryName + " " + Resources.vsk.toprated + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums;
                        _HeadingTitle = _title;
                        break;
                    case "featured":
                        selectedOrder = "Featured";
                        _order = "g.added_date desc";
                        _isFeatured = 1;
                        _title = categoryName + " " + Resources.vsk.featured + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums;
                        _HeadingTitle = _title;
                        break;
                }
                _HeadingTitle = _title;
            }
            if (filter != null)
            {
                // pagination setting
                if (filter == "today" || filter == "thisweek" || filter == "thismonth")
                {
                    if (order != null)
                    {
                        DefaultUrl = Config.GetUrl("albums/" + type + "/category/filter/" + title + "/" + filter.ToLower().Trim() + "/" + order);
                        PaginationUrl = Config.GetUrl("albums/" + type + "/category/filter/" + title + "/" + filter.ToLower().Trim() + "/" + order + "/[p]/");
                    }
                    else
                    {
                        DefaultUrl = Config.GetUrl("albums/" + type + "/category/filter/" + title + "/" + filter.ToLower().Trim());
                        PaginationUrl = Config.GetUrl("albums/" + type + "/category/filter/" + title + "/" + filter.ToLower().Trim() + "/[p]/");
                    }
                }

                switch (filter)
                {
                    case "today":
                        selectedFilter = "Today";
                        _dateFilter = 1; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added Today";
                        break;
                    case "thisweek":
                        selectedFilter = "This Week";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Week";
                        break;
                    case "thismonth":
                        selectedFilter = "This Month";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Month";
                        break;
                }
                _HeadingTitle = _title;
            }

            if (pagenumber > 1)
            {
                _HeadingTitle = _HeadingTitle + " - Page: " + pagenumber;
            }

            /* List Initialization */
            var ListEntity = new AlbumListViewModel()
            {
                AlbumType = returnAlbumType(type),
                AlbumTypePath = type,
                isListStatus = true,
                isListNav = true,
                Navigation = prepareCategoryFilterLinks("/albums/" + type + "/category/", title, selectedOrder, selectedFilter, type),
                QueryOptions = new AlbumEntity()
                {
                    type = returnAlbumType(type),
                    pagenumber = (int)pagenumber,
                    categories_str = _term,
                    term = "",
                    iscache = true,
                    ispublic = true,
                    isfeatured = _isFeatured, // 0: normal, 1: featured, 2: premium/paid, 3: all
                    pagesize = Site_Settings.Galleries_Page_Size,
                    datefilter = _dateFilter,
                    order = _order
                },
                ListObject = new ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Link, // 0: div, 1: a
                    showRating = true,
                    showViews = true,
                    showDate = true
                },
                HeadingTitle = _HeadingTitle,
                DefaultUrl = DefaultUrl,
                PaginationUrl = PaginationUrl,
                NoRecordFoundText = Resources.vsk.norecordfound,
            };

            ListEntity.TotalRecords = GalleryBLLC.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = GalleryBLLC.LoadItems(ListEntity.QueryOptions);

            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;
            // ViewBag.description = _description;

            return View(ListEntity);
        }

        // GET: albums/tag
        public ActionResult tag(string title, string order, string filter, string type, int? pagenumber)
        {
            if (type == null)
                type = "photos";

            if (title == null)
                return Redirect(Config.GetUrl("albums/" + type));

            if (pagenumber == null)
                pagenumber = 1;

            string _term = UtilityBLL.ReplaceHyphinWithSpace(title);
            string categoryName = UtilityBLL.UppercaseFirst(_term);

            string _order = "g.added_date desc";
            int _dateFilter = 0;
            int _isFeatured = 3; // 0: normal, 1: featured, 2: premium/paid, 3: all
            string _title = "Filter " + UtilityBLL.UppercaseFirst(type) + " Albums by label " + categoryName;
            var _HeadingTitle = _title;

            // pagination
            var DefaultUrl = Config.GetUrl("albums/" + type + "/tag/" + title);
            var PaginationUrl = Config.GetUrl("albums/" + type + "/tag/" + title + "/[p]/");

            // order
            var selectedOrder = "Recent";
            var selectedFilter = "All Time";


            if (order != null)
            {
                DefaultUrl = Config.GetUrl("albums/" + type + "/tag/" + title + "/" + order.ToLower().Trim());
                PaginationUrl = Config.GetUrl("albums/" + type + "/tag/" + title + "/" + order.ToLower().Trim() + "/[p]/");

                switch (order)
                {
                    case "mostviewed":
                        selectedOrder = "Most Viewed";
                        _order = "v.views desc, g.added_date desc";
                        _title = categoryName + " " + Resources.vsk.mostviewed + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums; //Resources.vsk.albums_view_title;
                        _HeadingTitle = _title;
                        break;
                    case "toprated":
                        selectedOrder = "Top Rated";
                        _order = "v.avg_rating desc, v.views desc";
                        _title = categoryName + " " + Resources.vsk.toprated + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums;
                        _HeadingTitle = _title;
                        break;
                    case "featured":
                        selectedOrder = "Featured";
                        _order = "g.added_date desc";
                        _isFeatured = 1;
                        _title = categoryName + " " + Resources.vsk.featured + " " + UtilityBLL.UppercaseFirst(type) + " " + Resources.vsk.albums;
                        _HeadingTitle = _title;
                        break;
                }
                _HeadingTitle = _title;
            }
            if (filter != null)
            {
                // pagination setting
                if (filter == "today" || filter == "thisweek" || filter == "thismonth")
                {
                    if (order != null)
                    {
                        DefaultUrl = Config.GetUrl("albums/" + type + "/tag/filter/" + title + "/" + filter.ToLower().Trim() + "/" + order);
                        PaginationUrl = Config.GetUrl("albums/" + type + "/tag/filter/" + title + "/" + filter.ToLower().Trim() + "/" + order + "/[p]/");
                    }
                    else
                    {
                        DefaultUrl = Config.GetUrl("albums/" + type + "/tag/filter/" + title + "/" + filter.ToLower().Trim());
                        PaginationUrl = Config.GetUrl("albums/" + type + "/tag/filter/" + title + "/" + filter.ToLower().Trim() + "/[p]/");
                    }
                }

                switch (filter)
                {
                    case "today":
                        selectedFilter = "Today";
                        _dateFilter = 1; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added Today";
                        break;
                    case "thisweek":
                        selectedFilter = "This Week";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Week";
                        break;
                    case "thismonth":
                        selectedFilter = "This Month";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Month";
                        break;
                }
                _HeadingTitle = _title;
            }

            /* List Initialization */
            var ListEntity = new AlbumListViewModel()
            {
                AlbumType = returnAlbumType(type),
                AlbumTypePath = type,
                isListStatus = true,
                isListNav = true,
                Navigation = prepareCategoryFilterLinks("/albums/" + type + "/tag/", title, selectedOrder, selectedFilter, type),
                QueryOptions = new AlbumEntity()
                {
                    type = returnAlbumType(type),
                    pagenumber = (int)pagenumber,
                    tags = _term,
                    term = "",
                    iscache = true,
                    ispublic = true,
                    isfeatured = _isFeatured, // 0: normal, 1: featured, 2: premium/paid, 3: all
                    pagesize = Site_Settings.Galleries_Page_Size,
                    datefilter = _dateFilter,
                    order = _order
                },
                ListObject = new ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Link, // 0: div, 1: a
                    showRating = true,
                    showViews = true,
                    showDate = true
                },
                HeadingTitle = _HeadingTitle,
                DefaultUrl = DefaultUrl,
                PaginationUrl = PaginationUrl,
                NoRecordFoundText = Resources.vsk.norecordfound,
            };

            ListEntity.TotalRecords = GalleryBLLC.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = GalleryBLLC.LoadItems(ListEntity.QueryOptions);

            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;
            // ViewBag.description = _description;

            return View(ListEntity);
        }

        // GET: albums/archive
        public ActionResult archive(string type, string month, int year, string order, int? pagenumber)
        {
            if (type == null)
                type = "photos";

            if (pagenumber == null)
                pagenumber = 1;


            string _title = UtilityBLL.UppercaseFirst(type) + " Albums Archived | " + month + " " + year; // Resources.vsk.albums_archive_default_title.Replace("{MN}", month).Replace("{YR}", year.ToString());
            var _HeadingTitle = _title;

            /* List Initialization */
            var ListEntity = new AlbumListViewModel()
            {
                AlbumType = returnAlbumType(type),
                AlbumTypePath = type,
                isListStatus = true,
                isListNav = false,
                QueryOptions = new AlbumEntity()
                {
                    type = returnAlbumType(type),
                    pagenumber = (int)pagenumber,
                    month = UtilityBLL.ReturnMonth(month),
                    year = year,
                    term = "",
                    iscache = true,
                    ispublic = true,
                    pagesize = Site_Settings.Galleries_Page_Size,
                },
                ListObject = new ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Link, // 0: div, 1: a
                    showRating = true,
                    showViews = true,
                    showDate = true
                },
                HeadingTitle = _HeadingTitle,
                HeaderTextCss = "mb30",
                DefaultUrl = Config.GetUrl("albums/" + type + "/archive/" + month + "/" + year + "/"),
                PaginationUrl = Config.GetUrl("albums/" + type + "/archive/" + month + "/" + year + "/[p]/"),
                NoRecordFoundText = Resources.vsk.norecordfound,
            };

            ListEntity.TotalRecords = GalleryBLLC.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = GalleryBLLC.LoadItems(ListEntity.QueryOptions);
            }

            if (pagenumber > 1)
            {
                _title = _title + " - Page: " + pagenumber;
            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;
            //ViewBag.description = _description;

            return View(ListEntity);
        }

        // GET: albums/categories
        public ActionResult categories(string type, int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;

            if (type == null)
                type = "photos";

            string _title = UtilityBLL.UppercaseFirst(type) + " Album Categories";
            if (pagenumber > 1)
                _title = _title + " - Page: " + pagenumber;

            /* List Initialization */
            var ListEntity = new AlbumCategoryListViewModel()
            {
                AlbumType = returnAlbumType(type),
                AlbumTypePath = type,
                isListStatus = true,
                isListNav = false,
                QueryOptions = new CategoryEntity()
                {
                    type = returnAlbumCategoryType(type),
                    pagenumber = (int)pagenumber,
                    status = 1,
                    term = "",
                    iscache = true,
                    pagesize = 30,
                    order = "categoryname asc",
                },
                ListObject = new ListItems()
                {
                    ListType = ListType.Grid, // 0: grid 1: list
                    ColWidth = "col-md-6 col-sm-12",
                },
                Path = "albums/", // category url path
                DefaultUrl = Config.GetUrl("albums/" + type + "/categories"),
                PaginationUrl = Config.GetUrl("albums/" + type + "/categories/[p]"),
                NoRecordFoundText = Resources.vsk.norecordfound,
                HeadingTitle = _title
            };

            ListEntity.TotalRecords = CategoryBLL.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = CategoryBLL.LoadItems(ListEntity.QueryOptions);

            }
            /* Page Title Adjustment */
            if (pagenumber > 1)
            {
                _title = _title + " - Page: " + pagenumber;
                //_description = _description + ", Page: " + pagenumber;
            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;

            return View(ListEntity);
        }

        // GET: albums/archivelist
        public ActionResult archivelist(string type)
        {
            if (type == null)
                type = "photos";

            string path = "photos/";
            string mediaType = "Photos";
            string subTitle = "Photo";
            switch (type)
            {
                case "videos":
                    path = "videos/";
                    mediaType = "Videos";
                    subTitle = "Video";
                    break;
                case "audio":
                    path = "audio/";
                    mediaType = "Audio";
                    subTitle = "Audio";
                    break;
            }
            /* List Initialization */
            var ListEntity = new AlbumArchiveListModelView()
            {
                AlbumType = returnAlbumType(type),
                AlbumTypePath = type,
                Path = path,
                HeadingTitle = subTitle + " Albums Archive List",
                isAll = true
            };
            // Page Title
            ViewBag.title = ListEntity.HeadingTitle + " | " + Site_Settings.Page_Caption;

            return View(ListEntity);
        }

        // GET: albums/taglist
        public ActionResult taglist(string term, string type, int? pagenumber)
        {
            if (type == null)
                type = "photos";

            if (pagenumber == null)
                pagenumber = 1;

            int tagType = (int)TagsBLL.Types.Photos;
            string mediaPath = "photos/";
            string media = "Photos";
            switch (type)
            {
                case "videos":
                    tagType = (int)TagsBLL.Types.Videos;
                    media = "Videos";
                    mediaPath = "videos/";
                    break;
                case "audio":
                    tagType = (int)TagsBLL.Types.Audio;
                    mediaPath = "audio/";
                    media = "Audio";
                    break;
            }
            string _title = media + " Albums Tag Clouds";
            /* List Initialization */
            var ListEntity = new AlbumTagListModelView()
            {
                AlbumType = returnAlbumType(type),
                AlbumTypePath = type,
                pagenumber = (int)pagenumber,
                TotalRecords = 100, // display 100 tags per page
                Type = tagType, // represent videos
                Path = mediaPath,
                DefaultUrl = Config.GetUrl("albums/" + type + "/taglist"),
                PaginationUrl = Config.GetUrl("albums/" + type + "/taglist/[p]/"),
                NoRecordFoundText = Resources.vsk.norecordfound,
                Action = "/albums/" + type + "/taglist", // for search tags
                HeadingTitle = _title
            };

           
            //string _description = Resources.vsk.albums_tag_meta;

            if (term != null && term.Length > 0)
            {
                ListEntity.Term = UtilityBLL.CleanSearchTerm(Server.UrlDecode(term).Trim());
                ListEntity.HeadingTitle = media + " Tags | " + ListEntity.Term;
                ListEntity.DefaultUrl = Config.GetUrl("albums/" + type + "/taglist/search/" + term);
                ListEntity.PaginationUrl = Config.GetUrl("albums/" + type + "/taglist/search/" + term + "/[p]");

                _title = ListEntity.HeadingTitle;
            }
            if (pagenumber > 1)
            {
                _title = _title + " - Page: " + pagenumber;
            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;

            return View(ListEntity);
        }

        private AlbumListFilterViewModel prepareFilterLinks(string url, string selectedOrder, string selectedFilter, string type)
        {
            // type represent /photos/ , /videos/ or /albums/ 
            return new AlbumListFilterViewModel()
            {
                selectedFilter = selectedFilter,
                selectedOrder = selectedOrder,
                order_rating_url = url + "" + type + "/toprated",
                order_recent_url = url + "" + type + "/recent",
                order_view_url = url + "" + type + "/mostviewed",
                order_featured_url = url + "" + type + "/featured",
                filter_date_alltime_url = url + "" + type,
                filter_date_thismonth_url = url + "" + type + "/added/thismonth",
                filter_date_thisweek_url = url + "" + type + "/added/thisweek",
                filter_date_today_url = url + "" + type + "/added/today"
            };
        }

        private AlbumListFilterViewModel prepareCategoryFilterLinks(string url, string title, string selectedOrder, string selectedFilter, string type)
        {
            // type represent /photos/ , /videos/ or /albums/ 
            return new AlbumListFilterViewModel()
            {
                selectedFilter = selectedFilter,
                selectedOrder = selectedOrder,
                order_rating_url = url + "/" + title + "/toprated",
                order_recent_url = url + "/" + title + "/recent",
                order_view_url = url + "/" + title + "/mostviewed",
                order_featured_url = url + "/" + title + "/featured",
                filter_date_alltime_url = url + "/" + title,
                filter_date_thismonth_url = url + "/filter/" + title + "/thismonth",
                filter_date_thisweek_url = url + "/filter/" + title + "/thisweek",
                filter_date_today_url = url + "/filter/" + title + "/today"
            };
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult taglist(TagListModelView model)
        {
            return Redirect(Config.GetUrl("albums/" + model.Type + "/taglist/search/" + Server.UrlEncode(UtilityBLL.CleanSearchTerm(model.Query))));
        }
        #endregion

        #region Search Feature
        // GET: albums/taglist
        public ActionResult search(string term, string filter, string type, int? pagenumber)
        {
            if (type == null)
                type = "photos";

            if (pagenumber == null)
                pagenumber = 1;

            // if term null, redirect to videos home page
            if (term == null)
                return Redirect("/albums/" + type);

            var _term_url = term; // UtilityBLL.ReplaceSpaceWithHyphin_v2(term);

            // if search option not used in advance search
            if (term == "skip-search")
                term = "";
            else
                term = UtilityBLL.CleanSearchTerm(Server.UrlDecode(term).Trim()); //UtilityBLL.ReplaceHyphinWithSpace(term.ToLower()).Trim();

            var DefaultUrl = Config.GetUrl("albums/" + type + "/search/" + _term_url);
            var PaginationUrl = Config.GetUrl("albums/" + type + "/search/" + _term_url + "/[p]/");

            var _dateFilter = 0; // 1: today, 2: this week, 3: this month, 0: all time
            var selectedFilter = "All Time";

            int _isFeatured = 3; // 0: normal, 1: featured, 2: premium/paid, 3: all
            string _title = UtilityBLL.UppercaseFirst(term, true) + " " + Resources.vsk.albums;

            if (filter != null)
            {
                // pagination setting
                DefaultUrl = Config.GetUrl("albums/" + type + "/search/filter/" + filter.ToLower().Trim() + "/" + _term_url);
                PaginationUrl = Config.GetUrl("albums/" + type + "/search/filter/" + filter.ToLower().Trim() + "/" + _term_url + "/[p]/");

                switch (filter)
                {
                    case "today":
                        selectedFilter = "Today";
                        _dateFilter = 1; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added Today";
                        break;
                    case "thisweek":
                        selectedFilter = "This Week";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Week";
                        break;
                    case "thismonth":
                        selectedFilter = "This Month";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Month";
                        break;
                }
            }

            string order = "g.added_date desc";
            string author = "";
            int searchtype = 3; // broad search
            string categories = null;
            if (Request.Params["cat"] != null)
            {
                categories = Request.Params["cat"].ToString();
            }

            bool isAdvanceSearch = false;
            var selectedOrder = "Recent";
            if (Request.Params["o"] != null)
            {
                isAdvanceSearch = true;
                switch (Request.Params["o"])
                {
                    case "0":
                        order = "g.added_date desc";
                        break;
                    case "1":
                        selectedOrder = "Most Viewed";
                        order = "v.views desc, g.added_date desc";
                        break;
                    case "3":
                        selectedOrder = "Top Rated";
                        order = "v.liked desc, v.avg_rating desc";
                        break;
                    case "2":
                        selectedOrder = "Featured";
                        order = "v.isfeatured desc, g.added_date desc";
                        break;
                }
            }
            if (Request.Params["usr"] != null)
                author = Request.Params["usr"].ToString();
            if (Request.Params["isfr"] != null)
                _isFeatured = Convert.ToInt32(Request.Params["isfr"]);
            if (Request.Params["dfilter"] != null)
                _dateFilter = Convert.ToInt32(Request.Params["dfilter"]);

            /* List Initialization */
            var ListEntity = new AlbumListViewModel()
            {
                AlbumType = returnAlbumType(type),
                AlbumTypePath = type,
                isListStatus = true,
                isListNav = true,
                HeadingTitle = _title,
                // setup list navigation urls
                Navigation = prepareFilterLinks(DefaultUrl, selectedOrder, selectedFilter, type),
                QueryOptions = new AlbumEntity()
                {
                    type = returnAlbumType(type),
                    pagenumber = (int)pagenumber,
                    searchtype = searchtype, // 0: category match, 1: tag match, 2: both category / tag, 3: broad search
                    categories_str = categories,
                    term = term,
                    username = author,
                    iscache = false,
                    ispublic = true,
                    isfeatured = _isFeatured, // all
                    pagesize = Site_Settings.Galleries_Page_Size,
                    order = order,
                },
                ListObject = new Jugnoon.Scripts.ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Div, // 0: div, 1: a
                    showRating = true
                },
                DefaultUrl = DefaultUrl,
                PaginationUrl = PaginationUrl,
                NoRecordFoundText = Resources.vsk.norecordfound,
            };

            ListEntity.TotalRecords = GalleryBLLC.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = GalleryBLLC.LoadItems(ListEntity.QueryOptions);

            }
            if (isAdvanceSearch)
            {
                var urlbuilder = new StringBuilder();
                urlbuilder.Append("?o=" + Request.Params["o"]);
                if (Request.Params["usr"] != null)
                    urlbuilder.Append("&usr=" + Request.Params["usr"]);
                if (Request.Params["cat"] != null)
                    urlbuilder.Append("&cat=" + Request.Params["cat"]);
                if (Request.Params["isfr"] != null)
                    urlbuilder.Append("&isfr=" + Request.Params["isfr"]);
                if (Request.Params["dfilter"] != null)
                    urlbuilder.Append("&dfilter=" + Request.Params["dfilter"]);
                //if (Request.Params["query"] != null)
                //    urlbuilder.Append("&query=" + Request.Params["query"]);

                ListEntity.DefaultUrl = DefaultUrl + "?" + urlbuilder.ToString();
                ListEntity.PaginationUrl = PaginationUrl + "?" + urlbuilder.ToString();
            }


            /* Page Title Adjustment */
            if (pagenumber > 1)
            {
                _title = _title + " - Page: " + pagenumber;
            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;

            return View(ListEntity);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult queryresult(JgnEnt.models.ViewModels.List.SearchListModel model)
        {
            if (model.Type == null)
                model.Type = "photos";
            return Redirect(Config.GetUrl("albums/" + model.Type + "/search/" + Server.UrlEncode(UtilityBLL.CleanSearchTerm(model.Query))));
        }
        // GET: albums/searchoptions
        public ActionResult searchoptions(string type)
        {
            if (type == null)
                type = "photos";

            ViewBag.title = "Advance " + UtilityBLL.UppercaseFirst(type) + " Album Search | " + Site_Settings.Page_Caption;

            var model = new AdvanceSearchModelView();

            model.CategoryList = CategoryBLL.LoadItems(new CategoryEntity()
            {
                iscache = true,
                type = returnAlbumCategoryType(type),
                status = 1,
                pagesize = 1000
            });

            // order list
            var _orderList = new Dictionary<string, string>();
            _orderList.Add(Resources.vsk.recentlyadded, "0");
            _orderList.Add(Resources.vsk.mostviewed, "1");
            _orderList.Add(Resources.vsk.toprated, "3");
            _orderList.Add(Resources.vsk.featured, "2");

            model.orderlist = _orderList;

            // date added
            var _dateList = new Dictionary<string, string>();
            _dateList.Add(Resources.vsk.alltime, "0");
            _dateList.Add(Resources.vsk.today, "1");
            _dateList.Add(Resources.vsk.thisweek, "2");
            _dateList.Add(Resources.vsk.thismonth, "3");
            model.datelist = _dateList;

            model.date_added = 0; // all time default selected

            model.isFeatured = 3; // both selected

            return View(model);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult searchoptions(AdvanceSearchModelView model, string action)
        {
            if (ModelState.IsValid)
            {
                var str = new StringBuilder();

                if (model.order == null)
                    model.order = "0";

                var _term = "skip-search";
                if (model.term != null && model.term != "")
                    _term = Server.UrlEncode(UtilityBLL.CleanSearchTerm(UtilityBLL.processNull(model.term, 0)));

                str.Append("?o=" + model.order);
                if (model.username != "" && model.username != null)
                    str.Append("&usr=" + Server.UrlEncode(UtilityBLL.CleanSearchTerm(UtilityBLL.processNull(model.username, 0))));
                if (model.categories != null)
                {
                    if (model.categories.Length > 0)
                        str.Append("&cat=" + string.Join(",", model.categories));

                }
                if (model.isFeatured != 3)
                    str.Append("&isfr=" + model.isFeatured);

                if (model.date_added > 0)
                    str.Append("&dfilter=" + model.date_added);

                string _path = "photos";
                switch(model.AlbumType)
                {
                    case 1:
                        _path = "videos";
                        break;
                    case 2:
                        _path = "audio";
                        break;
                }
                return Redirect(Config.GetUrl("albums/" + _path + "/search/" + _term + "/" + str.ToString()));
            }

            model.Message = ValidationError.HasErrors(ModelState);
            return View(model);
        }

        private int returnAlbumType(string type)
        {
            var AlbumType = (int)GalleryBLLC.Types.Photos;
            switch (type)
            {
                case "videos":
                    AlbumType = (int)GalleryBLLC.Types.Videos;
                    break;
                case "audio":
                    AlbumType = (int)GalleryBLLC.Types.Audio;
                    break;
            }
            return AlbumType;
        }

        private int returnAlbumCategoryType(string type)
        {
            var CategoryType = (int)CategoryBLL.Types.Photos;
            switch (type)
            {
                case "videos":
                    CategoryType = (int)GalleryBLLC.Types.Videos;
                    break;
                case "audio":
                    CategoryType = (int)GalleryBLLC.Types.Audio;
                    break;
            }
            return CategoryType;
        }
        #endregion
    }
}