﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using Jugnoon.Entity;
using Jugnoon.Utility;
using Jugnoon.BLL;
using Jugnoon.models;
using System.Text;
using Jugnoon.Settings;
using Jugnoon.Scripts;
using JgnEnt.models.ViewModels.videos;

namespace JgnEnt.Controllers
{
    public class audioController : Controller
    {

        private int MediaType = 1; // 0: Videos, 1: audio
        #region Feed
        public ActionResult atom()
        {
            string sXml = VideoBLL.generateATOM(new VideoEntity()
            {
                type = MediaType,
                pagenumber = 1,
                isenabled = 1,
                isreviewed = 1,
                order = "videoid desc",
                pagesize = 10
            }, Config.GetUrl("audio/"));

            return this.Content(sXml, "text/xml");
        }

        public ActionResult rss()
        {
            string sXml = VideoBLL.generateRSS(new VideoEntity()
            {
                type = MediaType,
                pagenumber = 1,
                isenabled = 1,
                isreviewed = 1,
                order = "videoid desc",
                pagesize = 10
            });

            return this.Content(sXml, "text/xml");
        }
        #endregion

        #region Public Region

        // GET: videos
        public ActionResult Index(string order, string filter, int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;


            string _order = "v.date_added desc";
            int _dateFilter = 0;
            int _isFeatured = 3; // 0: normal, 1: featured, 2: premium/paid, 3: all
            string _title = Resources.vsk.recentlyadded + " " + Resources.vsk.videos;
            var _HeadingTitle = _title;

            // pagination
            var DefaultUrl = Config.GetUrl("audio/");
            var PaginationUrl = Config.GetUrl("audio/page/[p]/");

            // order
            var selectedOrder = "Recent";
            var selectedFilter = "All Time";


            if (order != null)
            {
                DefaultUrl = Config.GetUrl("audio/" + order.ToLower().Trim());
                PaginationUrl = Config.GetUrl("audio/" + order.ToLower().Trim() + "/[p]/");
                switch (order)
                {
                    case "mostviewed":
                        selectedOrder = "Most Viewed";
                        _order = "v.views desc, v.date_added desc";
                        _title = Resources.vsk.mostviewed + " " + Resources.vsk.videos; //Resources.vsk.videos_view_title;
                        _HeadingTitle = _title;
                        break;
                    case "toprated":
                        selectedOrder = "Top Rated";
                        _order = "v.avg_rating desc, v.views desc";
                        _title = Resources.vsk.toprated + " " + Resources.vsk.videos;
                        _HeadingTitle = _title;
                        break;
                    case "featured":
                        selectedOrder = "Featured";
                        _order = "v.date_added desc";
                        _isFeatured = 1;
                        _title = Resources.vsk.featured + " " + Resources.vsk.videos;
                        _HeadingTitle = _title;
                        break;
                }
                _HeadingTitle = _title;
            }

            if (filter != null)
            {
                // pagination setting
                if (filter == "today" || filter == "thisweek" || filter == "thismonth")
                {
                    DefaultUrl = Config.GetUrl("audio/added/" + filter.ToLower().Trim());
                    PaginationUrl = Config.GetUrl("audio/added/" + filter.ToLower().Trim() + "/[p]/");
                }

                switch (filter)
                {
                    case "today":
                        selectedFilter = "Today";
                        _dateFilter = 1; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added Today";
                        break;
                    case "thisweek":
                        selectedFilter = "This Week";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Week";
                        break;
                    case "thismonth":
                        selectedFilter = "This Month";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Month";
                        break;
                }
                _HeadingTitle = _title;
            }

            if (pagenumber > 1)
            {
                _HeadingTitle = _HeadingTitle + " - Page: " + pagenumber;
            }


            /* List Initialization */
            var ListEntity = new VideoListViewModel()
            {
                isListStatus = true,
                isListNav = true,
                // setup list navigation urls
                Navigation = prepareFilterLinks("/audio/", selectedOrder, selectedFilter),
                QueryOptions = new VideoEntity()
                {
                    type = MediaType,
                    pagenumber = (int)pagenumber,
                    term = "",
                    iscache = true,
                    ispublic = true,
                    isfeatured = _isFeatured, // 0: normal, 1: featured, 2: premium/paid, 3: all
                    pagesize = Site_Settings.Video_Page_Size,
                    datefilter = _dateFilter,
                    order = _order,
                },
                ListObject = new ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Link, // 0: div, 1: a
                    showRating = true,
                    showViews = true,
                    showDate = true
                },
                HeaderTextCss = "mb30",
                DefaultUrl = DefaultUrl,
                PaginationUrl = PaginationUrl,
                NoRecordFoundText = Resources.vsk.norecordfound,
                HeadingTitle = _HeadingTitle
            };

            ListEntity.TotalRecords = VideoBLL.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = VideoBLL.LoadItems(ListEntity.QueryOptions);

            }
            
            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;
    
            return View(ListEntity);
        }


        // GET: audio/category
        public ActionResult category(string title, string order, string filter, int? pagenumber)
        {
            if (title == null)
            {
                return Redirect(Config.GetUrl("audio/"));
            }
            if (pagenumber == null)
                pagenumber = 1;

            string _term = UtilityBLL.ReplaceHyphinWithSpace(title);
            string categoryName = UtilityBLL.UppercaseFirst(_term);

            string _order = "date_added desc";
            int _dateFilter = 0;
            int _isFeatured = 3; // 0: normal, 1: featured, 2: premium/paid, 3: all
            string _title = Resources.vsk.videos_category_default_title.Replace("{CN}", categoryName);
            var _HeadingTitle = _title;

            // pagination
            var DefaultUrl = Config.GetUrl("audio/category/" + title);
            var PaginationUrl = Config.GetUrl("audio/category/" + title + "/[p]/");

            // order
            var selectedOrder = "Recent";
            var selectedFilter = "All Time";

            if (order != null)
            {
                DefaultUrl = Config.GetUrl("audio/category/" + title + "/" + order.ToLower().Trim());
                PaginationUrl = Config.GetUrl("audio/category/" + title + "/" + order.ToLower().Trim() + "/[p]/");

                switch (order)
                {
                    case "mostviewed":
                        selectedOrder = "Most Viewed";
                        _order = "v.views desc, v.date_added desc";
                        _title = categoryName + " " + Resources.vsk.mostviewed + " " + Resources.vsk.videos; //Resources.vsk.videos_view_title;
                        _HeadingTitle = _title;
                        break;
                    case "toprated":
                        selectedOrder = "Top Rated";
                        _order = "v.avg_rating desc, v.views desc";
                        _title = categoryName + " " + Resources.vsk.toprated + " " + Resources.vsk.videos;
                        _HeadingTitle = _title;
                        break;
                    case "featured":
                        selectedOrder = "Featured";
                        _order = "v.date_added desc";
                        _isFeatured = 1;
                        _title = categoryName + " " + Resources.vsk.featured + " " + Resources.vsk.videos;
                        _HeadingTitle = _title;
                        break;
                }
                _HeadingTitle = _title;
            }
            if (filter != null)
            {
                // pagination setting
                if (filter == "today" || filter == "thisweek" || filter == "thismonth")
                {
                    if (order != null)
                    {
                        DefaultUrl = Config.GetUrl("audio/category/filter/" + title + "/" + filter.ToLower().Trim() + "/" + order);
                        PaginationUrl = Config.GetUrl("audio/category/filter/" + title + "/" + filter.ToLower().Trim() + "/" + order + "/[p]/");
                    }
                    else
                    {
                        DefaultUrl = Config.GetUrl("audio/category/filter/" + title + "/" + filter.ToLower().Trim());
                        PaginationUrl = Config.GetUrl("audio/category/filter/" + title + "/" + filter.ToLower().Trim() + "/[p]/");
                    }
                }

                switch (filter)
                {
                    case "today":
                        selectedFilter = "Today";
                        _dateFilter = 1; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added Today";
                        break;
                    case "thisweek":
                        selectedFilter = "This Week";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Week";
                        break;
                    case "thismonth":
                        selectedFilter = "This Month";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Month";
                        break;
                }
                _HeadingTitle = _title;
            }

            if (pagenumber > 1)
            {
                _HeadingTitle = _HeadingTitle + " - Page: " + pagenumber;
            }

            /* List Initialization */
            var ListEntity = new VideoListViewModel()
            {
                isListStatus = true,
                isListNav = true,
                Navigation = prepareCategoryFilterLinks("/audio/category/", title, selectedOrder, selectedFilter),

                QueryOptions = new VideoEntity()
                {
                    type = MediaType,
                    pagenumber = (int)pagenumber,
                    categories_str = _term,
                    iscache = false,
                    term = "",
                    ispublic = true,
                    isfeatured = _isFeatured, // 0: normal, 1: featured, 2: premium/paid, 3: all
                    pagesize = Site_Settings.Video_Page_Size,
                    datefilter = _dateFilter,
                    order = _order,
                },
                ListObject = new ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Link, // 0: div, 1: a
                    showRating = true,
                    showViews = true,
                    showDate = true
                },
                HeadingTitle = _HeadingTitle,
                DefaultUrl = DefaultUrl,
                PaginationUrl = PaginationUrl,
                NoRecordFoundText = Resources.vsk.norecordfound,
            };

            ListEntity.TotalRecords = VideoBLL.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = VideoBLL.LoadItems(ListEntity.QueryOptions);

            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;

            return View(ListEntity);
        }

        // GET: audio/tag
        public ActionResult tag(string title, string order, string filter, int? pagenumber)
        {
            if (title == null)
            {
                return Redirect(Config.GetUrl("audio/"));
            }
            if (pagenumber == null)
                pagenumber = 1;

            string _term = UtilityBLL.ReplaceHyphinWithSpace(title);
            string categoryName = UtilityBLL.UppercaseFirst(_term);

            string _order = "v.date_added desc";
            int _dateFilter = 0;
            int _isFeatured = 3; // 0: normal, 1: featured, 2: premium/paid, 3: all
            string _title = Resources.vsk.videos_tag_title.Replace("{TG}", categoryName);
            var _HeadingTitle = _title;

            // pagination
            var DefaultUrl = Config.GetUrl("audio/category/" + title);
            var PaginationUrl = Config.GetUrl("audio/category/" + title + "/[p]/");

            // order
            var selectedOrder = "Recent";
            var selectedFilter = "All Time";


            if (order != null)
            {
                DefaultUrl = Config.GetUrl("audio/tag/" + title + "/" + order.ToLower().Trim());
                PaginationUrl = Config.GetUrl("audio/tag/" + title + "/" + order.ToLower().Trim() + "/[p]/");

                switch (order)
                {
                    case "mostviewed":
                        selectedOrder = "Most Viewed";
                        _order = "v.views desc, v.date_added desc";
                        _title = categoryName + " " + Resources.vsk.mostviewed + " " + Resources.vsk.videos; //Resources.vsk.videos_view_title;
                        _HeadingTitle = _title;
                        break;
                    case "toprated":
                        selectedOrder = "Top Rated";
                        _order = "v.avg_rating desc, v.views desc";
                        _title = categoryName + " " + Resources.vsk.toprated + " " + Resources.vsk.videos;
                        _HeadingTitle = _title;
                        break;
                    case "featured":
                        selectedOrder = "Featured";
                        _order = "v.date_added desc";
                        _isFeatured = 1;
                        _title = categoryName + " " + Resources.vsk.featured + " " + Resources.vsk.videos;
                        _HeadingTitle = _title;
                        break;
                }
                _HeadingTitle = _title;
            }
            if (filter != null)
            {
                // pagination setting
                if (filter == "today" || filter == "thisweek" || filter == "thismonth")
                {
                    if (order != null)
                    {
                        DefaultUrl = Config.GetUrl("audio/tag/filter/" + title + "/" + filter.ToLower().Trim() + "/" + order);
                        PaginationUrl = Config.GetUrl("audio/tag/filter/" + title + "/" + filter.ToLower().Trim() + "/" + order + "/[p]/");
                    }
                    else
                    {
                        DefaultUrl = Config.GetUrl("audio/tag/filter/" + title + "/" + filter.ToLower().Trim());
                        PaginationUrl = Config.GetUrl("audio/tag/filter/" + title + "/" + filter.ToLower().Trim() + "/[p]/");
                    }
                }

                switch (filter)
                {
                    case "today":
                        selectedFilter = "Today";
                        _dateFilter = 1; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added Today";
                        break;
                    case "thisweek":
                        selectedFilter = "This Week";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Week";
                        break;
                    case "thismonth":
                        selectedFilter = "This Month";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Month";
                        break;
                }
                _HeadingTitle = _title;
            }

            if (pagenumber > 1)
            {
                _HeadingTitle = _HeadingTitle + " - Page: " + pagenumber;
            }

            /* List Initialization */
            var ListEntity = new VideoListViewModel()
            {
                isListStatus = true,
                isListNav = true,
                Navigation = prepareCategoryFilterLinks("/audio/tag/", title, selectedOrder, selectedFilter),
                QueryOptions = new VideoEntity()
                {
                    type = MediaType,
                    pagenumber = (int)pagenumber,
                    tags = _term,
                    iscache = false,
                    term = "",
                    ispublic = true,
                    isfeatured = _isFeatured, // 0: normal, 1: featured, 2: premium/paid, 3: all
                    pagesize = Site_Settings.Video_Page_Size,
                    datefilter = _dateFilter,
                    order = _order,
                },
                ListObject = new ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Link, // 0: div, 1: a
                    showRating = true,
                    showViews = true,
                    showDate = true
                },
                HeadingTitle = _HeadingTitle,
                DefaultUrl = DefaultUrl,
                PaginationUrl = PaginationUrl,
                NoRecordFoundText = Resources.vsk.norecordfound,
            };

            ListEntity.TotalRecords = VideoBLL.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = VideoBLL.LoadItems(ListEntity.QueryOptions);

            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;
            // ViewBag.description = _description;

            return View(ListEntity);
        }

        // GET: audio/archive
        public ActionResult archive(string month, int year, string order, int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;


            string _title = Resources.vsk.videos_archive_default_title.Replace("{MN}", month).Replace("{YR}", year.ToString());
            var _HeadingTitle = _title;

            /* List Initialization */
            var ListEntity = new VideoListViewModel()
            {
                isListStatus = true,
                isListNav = false,
                QueryOptions = new VideoEntity()
                {
                    type = MediaType,
                    pagenumber = (int)pagenumber,
                    term = "",
                    month = UtilityBLL.ReturnMonth(month),
                    year = year,
                    searchtype = 0, // 0: category match, 1: tag match, 2: both category / tag, 3: broad search
                    iscache = false,
                    ispublic = true,
                    pagesize = Site_Settings.Video_Page_Size,
                },
                ListObject = new ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Link, // 0: div, 1: a
                    showRating = true,
                    showViews = true,
                    showDate = true
                },
                HeadingTitle = _HeadingTitle,
                HeaderTextCss = "mb30",
                DefaultUrl = Config.GetUrl("audio/archive/" + month + "/" + year + "/"),
                PaginationUrl = Config.GetUrl("audio/archive/" + month + "/" + year + "/[p]/"),
                NoRecordFoundText = Resources.vsk.norecordfound,
            };

            ListEntity.TotalRecords = VideoBLL.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = VideoBLL.LoadItems(ListEntity.QueryOptions);

            }


            if (pagenumber > 1)
            {
                _title = _title + " - Page: " + pagenumber;
            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;
            //ViewBag.description = _description;

            return View(ListEntity);
        }

        // GET: audio/categories
        public ActionResult categories(int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;

            string _title = "Video Categories";
            if (pagenumber > 1)
                _title = _title + " - Page: " + pagenumber;

            /* List Initialization */
            var ListEntity = new CategoryListViewModel_v2()
            {
                isListStatus = true,
                isListNav = false,
                QueryOptions = new CategoryEntity()
                {
                    type = (int)CategoryBLL.Types.Videos,
                    pagenumber = (int)pagenumber,
                    status = 1,
                    term = "",
                    iscache = true,
                    pagesize = 30,
                    order = "categoryname asc",
                },
                ListObject = new ListItems()
                {
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Link, // 0: div, 1: a
                    showRating = true,
                    showViews = true,
                    showDate = true
                },
                Path = "audio/", // category url path
                DefaultUrl = Config.GetUrl("audio/categories"),
                PaginationUrl = Config.GetUrl("audio/categories/[p]"),
                NoRecordFoundText = Resources.vsk.norecordfound,
                HeadingTitle = _title
            };

            ListEntity.TotalRecords = CategoryBLL.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = CategoryBLL.LoadItems(ListEntity.QueryOptions);

            }
            /* Page Title Adjustment */
            if (pagenumber > 1)
            {
                _title = _title + " - Page: " + pagenumber;
                //_description = _description + ", Page: " + pagenumber;
            }
            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;
            return View(ListEntity);
        }

        // GET: audio/archivelist
        public ActionResult archivelist()
        {
            /* List Initialization */
            var ListEntity = new ArchiveListModelView()
            {
                Type = 0, // represent videos
                Path = "audio/",
                HeadingTitle = Resources.vsk.videos_archivelist_title,
                isAll = true
            };
            // Page Title
            ViewBag.title = Resources.vsk.videos_archivelist_title + " | " + Site_Settings.Page_Caption;

            return View(ListEntity);
        }

        // GET: audio/taglist
        public ActionResult taglist(string term, int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;

            /* List Initialization */
            var ListEntity = new TagListModelView()
            {
                pagenumber = (int)pagenumber,
                TotalRecords = 100, // display 100 tags per page
                Type = (int)TagsBLL.Types.Audio,
                Path = "audio/",
                DefaultUrl = Config.GetUrl("audio/taglist"),
                PaginationUrl = Config.GetUrl("audio/taglist/[p]/"),
                NoRecordFoundText = Resources.vsk.norecordfound,
                Action = "/audio/taglist", // for search tags
                HeadingTitle = Resources.vsk.audio_tags_home
            };

            string _title = Resources.vsk.audio_tags_home;

            if (term != null && term.Length > 0)
            {
                ListEntity.Term = UtilityBLL.CleanSearchTerm(Server.UrlDecode(term).Trim());
                ListEntity.HeadingTitle = "Audio Tags | " + ListEntity.Term;
                ListEntity.DefaultUrl = Config.GetUrl("audio/taglist/search/" + term);
                ListEntity.PaginationUrl = Config.GetUrl("audio/taglist/search/" + term + "/[p]");

                _title = ListEntity.HeadingTitle;
            }
            // Page Title


            if (pagenumber > 1)
            {
                _title = _title + " - Page: " + pagenumber;
            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;

            return View(ListEntity);
        }

        private VideoListFilterViewModel prepareFilterLinks(string url, string selectedOrder, string selectedFilter)
        {
            return new VideoListFilterViewModel()
            {
                selectedFilter = selectedFilter,
                selectedOrder = selectedOrder,
                order_rating_url = url + "toprated",
                order_recent_url = url + "recent",
                order_view_url = url + "mostviewed",
                order_featured_url = url + "featured",
                filter_date_alltime_url = url,
                filter_date_thismonth_url = url + "added/thismonth",
                filter_date_thisweek_url = url + "added/thisweek",
                filter_date_today_url = url + "added/today"
            };
        }

        private VideoListFilterViewModel prepareCategoryFilterLinks(string url, string title, string selectedOrder, string selectedFilter)
        {
            return new VideoListFilterViewModel()
            {
                selectedFilter = selectedFilter,
                selectedOrder = selectedOrder,
                order_rating_url = url + "" + title + "/toprated",
                order_recent_url = url + "" + title + "/recent",
                order_view_url = url + "" + title + "/mostviewed",
                order_featured_url = url + "" + title + "/featured",
                filter_date_alltime_url = url + "" + title,
                filter_date_thismonth_url = url + "filter/" + title + "/thismonth",
                filter_date_thisweek_url = url + "filter/" + title + "/thisweek",
                filter_date_today_url = url + "filter/" + title + "/today"
            };
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult taglist(TagListModelView model)
        {
            return Redirect(Config.GetUrl("audio/taglist/search/" + Server.UrlEncode(UtilityBLL.CleanSearchTerm(model.Query))));
        }
        #endregion

        #region Search Feature
        // GET: audio/taglist
        public ActionResult search(string term, string filter, int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;

            // if term null, redirect to videos home page
            if (term == null)
                return Redirect("/audio/");


            var _term_url = term; // UtilityBLL.ReplaceSpaceWithHyphin_v2(term);

            // if search option not used in advance search
            if (term == "skip-search")
                term = "";
            else
                term = UtilityBLL.CleanSearchTerm(Server.UrlDecode(term).Trim()); //UtilityBLL.ReplaceHyphinWithSpace(term.ToLower()).Trim();

            var DefaultUrl = Config.GetUrl("audio/search/" + _term_url);
            var PaginationUrl = Config.GetUrl("audio/search/" + _term_url + "/[p]/");

            var _dateFilter = 0; // 1: today, 2: this week, 3: this month, 0: all time
            var selectedFilter = "All Time";

            int _isFeatured = 3; // 0: normal, 1: featured, 2: premium/paid, 3: all
            string _title = UtilityBLL.UppercaseFirst(term, true) + " " + Resources.vsk.videos;

            if (filter != null)
            {
                // pagination setting
                DefaultUrl = Config.GetUrl("audio/search/filter/" + filter.ToLower().Trim() + "/" + _term_url);
                PaginationUrl = Config.GetUrl("audio/search/filter/" + filter.ToLower().Trim() + "/" + _term_url + "/[p]/");

                switch (filter)
                {
                    case "today":
                        selectedFilter = "Today";
                        _dateFilter = 1; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added Today";
                        break;
                    case "thisweek":
                        selectedFilter = "This Week";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Week";
                        break;
                    case "thismonth":
                        selectedFilter = "This Month";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Month";
                        break;
                }
            }

            string order = "v.date_added desc";
            string author = "";
            int searchtype = 3; // broad search
            string categories = null;
            if (Request.Params["cat"] != null)
            {
                categories = Request.Params["cat"].ToString();
            }

            bool isAdvanceSearch = false;
            var selectedOrder = "Recent";
            if (Request.Params["o"] != null)
            {
                isAdvanceSearch = true;
                switch (Request.Params["o"])
                {
                    case "0":
                        order = "v.date_added desc";
                        break;
                    case "1":
                        selectedOrder = "Most Viewed";
                        order = "v.views desc, v.date_added desc";
                        break;
                    case "3":
                        selectedOrder = "Top Rated";
                        order = "v.liked desc, v.avg_rating desc";
                        break;
                    case "2":
                        selectedOrder = "Featured";
                        order = "v.isfeatured desc, v.date_added desc";
                        break;
                }
            }
            if (Request.Params["usr"] != null)
                author = Request.Params["usr"].ToString();
            if (Request.Params["isfr"] != null)
                _isFeatured = Convert.ToInt32(Request.Params["isfr"]);
            if (Request.Params["dfilter"] != null)
                _dateFilter = Convert.ToInt32(Request.Params["dfilter"]);

            /* List Initialization */
            var ListEntity = new VideoListViewModel()
            {
                isListStatus = true,
                isListNav = true,
                // setup list navigation urls
                Navigation = prepareFilterLinks(DefaultUrl, selectedOrder, selectedFilter),
                QueryOptions = new VideoEntity()
                {
                    type = MediaType,
                    pagenumber = (int)pagenumber,
                    searchtype = searchtype, // 0: category match, 1: tag match, 2: both category / tag, 3: broad search
                    term = term,
                    datefilter = _dateFilter,
                    categories_str = categories,
                    username = author,
                    ispublic = true,
                    iscache = false,
                    isfeatured = _isFeatured, // all
                    pagesize = 20,
                    order = order,
                },
                ListObject = new Jugnoon.Scripts.ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    ItemType = ItemType.Div, // 0: div, 1: a
                    showRating = true
                },
                DefaultUrl = DefaultUrl,
                PaginationUrl = PaginationUrl,
                NoRecordFoundText = Resources.vsk.norecordfound,
            };

            ListEntity.TotalRecords = VideoBLL.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = VideoBLL.LoadItems(ListEntity.QueryOptions);

            }
            if (isAdvanceSearch)
            {
                var urlbuilder = new StringBuilder();
                urlbuilder.Append("?o=" + Request.Params["o"]);
                if (Request.Params["usr"] != null)
                    urlbuilder.Append("&usr=" + Request.Params["usr"]);
                if (Request.Params["cat"] != null)
                    urlbuilder.Append("&cat=" + Request.Params["cat"]);
                if (Request.Params["isfr"] != null)
                    urlbuilder.Append("&isfr=" + Request.Params["isfr"]);
                if (Request.Params["dfilter"] != null)
                    urlbuilder.Append("&dfilter=" + Request.Params["dfilter"]);
                //if (Request.Params["query"] != null)
                //    urlbuilder.Append("&query=" + Request.Params["query"]);

                ListEntity.DefaultUrl = DefaultUrl + "?" + urlbuilder.ToString();
                ListEntity.PaginationUrl = PaginationUrl + "?" + urlbuilder.ToString();
            }


            /* Page Title Adjustment */
            if (pagenumber > 1)
            {
                _title = _title + " - Page: " + pagenumber;
            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;

            return View(ListEntity);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult queryresult(JgnEnt.models.ViewModels.List.SearchListModel model)
        {
            return Redirect(Config.GetUrl("audio/search/" + Server.UrlEncode(UtilityBLL.CleanSearchTerm(model.Query))));
        }
        // GET: audio/searchoptions
        public ActionResult searchoptions()
        {
            ViewBag.title = Resources.vsk.advancevideosearch + " | " + Site_Settings.Page_Caption;

            var model = new AdvanceSearchModelView();

            var _type = CategoryBLL.Types.Videos;
            if (MediaType == 1)
                _type = CategoryBLL.Types.Audio;

            model.CategoryList = CategoryBLL.LoadItems(new CategoryEntity()
            {
                iscache = true,
                type = (int)_type,
                status = 1,
                pagesize = 1000
            });

            // order list
            var _orderList = new Dictionary<string, string>();
            _orderList.Add(Resources.vsk.recentlyadded, "0");
            _orderList.Add(Resources.vsk.mostviewed, "1");
            _orderList.Add(Resources.vsk.toprated, "3");
            _orderList.Add(Resources.vsk.featured, "2");

            model.orderlist = _orderList;

            // date added
            var _dateList = new Dictionary<string, string>();
            _dateList.Add(Resources.vsk.alltime, "0");
            _dateList.Add(Resources.vsk.today, "1");
            _dateList.Add(Resources.vsk.thisweek, "2");
            _dateList.Add(Resources.vsk.thismonth, "3");
            model.datelist = _dateList;

            model.date_added = 0; // all time default selected

            model.isFeatured = 3; // both selected

            return View(model);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult searchoptions(AdvanceSearchModelView model, string action)
        {
            if (ModelState.IsValid)
            {
                var str = new StringBuilder();

                if (model.order == null)
                    model.order = "0";

                var _term = "skip-search";
                if (model.term != null && model.term != "")
                    _term = Server.UrlEncode(UtilityBLL.CleanSearchTerm(UtilityBLL.processNull(model.username, 0)));

                str.Append("?o=" + model.order);
                if (model.username != "" && model.username != null)
                    str.Append("&usr=" + Server.UrlEncode(UtilityBLL.CleanSearchTerm(UtilityBLL.processNull(model.username, 0))));
                if (model.categories != null)
                {
                    if (model.categories.Length > 0)
                        str.Append("&cat=" + string.Join(",", model.categories));

                }
                if (model.isFeatured != 2)
                    str.Append("&isfr=" + model.isFeatured);

                if (model.date_added > 0)
                    str.Append("&dfilter=" + model.date_added);

                return Redirect(Config.GetUrl("audio/search/" + _term + "/" + str.ToString()));
            }

            model.Message = ValidationError.HasErrors(ModelState);
            return View(model);
        }
        #endregion
    }

}