﻿using Jugnoon.models;
using System;
using System.Collections.Generic;
using System.Web.Mvc;
using Jugnoon.BLL;
using Jugnoon.Utility;
using Jugnoon.Entity;
using System.Collections;
using Jugnoon.Settings;
using Jugnoon.Scripts;
using JgnEnt.models.ViewModels.Channels;
using System.Text;

namespace JgnEnt.Controllers
{
    public class channelsController : Controller
    {
        // GET: channels
        public ActionResult Index(string order, string filter, int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;


            string _order = "register_date desc";
            int _dateFilter = 0;
           
            string _title = Resources.vsk.recentlyadded + " " + Resources.vsk.channels;
            var _HeadingTitle = _title;

            // pagination
            var DefaultUrl = Config.GetUrl("channels/");
            var PaginationUrl = Config.GetUrl("channels/page/[p]/");

            // order
            var selectedOrder = "Recent";
            var selectedFilter = "All Time";


            if (order != null)
            {
                DefaultUrl = Config.GetUrl("channels/" + order.ToLower().Trim());
                PaginationUrl = Config.GetUrl("channels/" + order.ToLower().Trim() + "/[p]/");
                switch (order)
                {
                    case "mostviewed":
                        selectedOrder = "Most Viewed";
                        _order = "views desc, register_date desc";
                        _title = Resources.vsk.mostviewed + " " + Resources.vsk.photos;
                        _HeadingTitle = _title;
                        break;
                }
                _HeadingTitle = _title;
            }

            if (filter != null)
            {
                // pagination setting
                if (filter == "today" || filter == "thisweek" || filter == "thismonth")
                {
                    DefaultUrl = Config.GetUrl("channels/added/" + filter.ToLower().Trim());
                    PaginationUrl = Config.GetUrl("channels/added/" + filter.ToLower().Trim() + "/[p]/");
                }

                switch (filter)
                {
                    case "today":
                        selectedFilter = "Today";
                        _dateFilter = 1; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added Today";
                        break;
                    case "thisweek":
                        selectedFilter = "This Week";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Week";
                        break;
                    case "thismonth":
                        selectedFilter = "This Month";
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Month";
                        break;
                }
                _HeadingTitle = _title;
            }

            if (pagenumber > 1)
            {
                _HeadingTitle = _HeadingTitle + " - Page: " + pagenumber;
            }


            /* List Initialization */
            var ListEntity = new ChannelListViewModel()
            {
                isListStatus = true,
                isListNav = true,
                // setup list navigation urls
                Navigation = prepareFilterLinks("/channels/", selectedOrder, selectedFilter),
                QueryOptions = new MemberEntity()
                {
                    pagenumber = (int)pagenumber,
                    term = "",
                    iscache = true,
                    ispublic = true,
                    pagesize = 18,
                    datefilter = _dateFilter,
                    order = _order,
                },
                ListObject = new ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    showViews = true,
                    showDate = true
                },
                DefaultUrl = DefaultUrl,
                PaginationUrl = PaginationUrl,
                NoRecordFoundText = Resources.vsk.norecordfound,
                HeadingTitle = _HeadingTitle
            };

            ListEntity.TotalRecords = UserBLL.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = UserBLL.LoadItems(ListEntity.QueryOptions);

            }
            
            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;

            return View(ListEntity);
        }

        #region Search
        // GET: channels/search
        public ActionResult search(string term, string filter, int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;

            // if term null, redirect to videos home page
            if (term == null)
                return Redirect("/channels/");

            var _term_url = term;

            // if search option not used in advance search
            if (term == "skip-search")
                term = "";
            else
                term = UtilityBLL.CleanSearchTerm(Server.UrlDecode(term).Trim()); //UtilityBLL.ReplaceHyphinWithSpace(term.ToLower()).Trim();

            var DefaultUrl = Config.GetUrl("channels/search/" + _term_url);
            var PaginationUrl = Config.GetUrl("channels/search/" + _term_url + "/[p]/");

            var _dateFilter = 0; // 1: today, 2: this week, 3: this month, 0: all time
            
            string _title = UtilityBLL.UppercaseFirst(term, true) + " " + Resources.vsk.qa;

            if (filter != null)
            {
                // pagination setting
                DefaultUrl = Config.GetUrl("channels/search/filter/" + filter.ToLower().Trim() + "/" + _term_url);
                PaginationUrl = Config.GetUrl("channels/search/filter/" + filter.ToLower().Trim() + "/" + _term_url + "/[p]/");

                switch (filter)
                {
                    case "today":
                        _dateFilter = 1; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added Today";
                        break;
                    case "thisweek":
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Week";
                        break;
                    case "thismonth":
                        _dateFilter = 2; // 1: today, 2: this week, 3: this month, 0: all time
                        _title = _title + " | Added This Month";
                        break;
                }
            }

            string order = "register_date desc";
            string author = "";
            int searchtype = 3; // broad search
            string categories = null;
            if (Request.Params["cat"] != null)
            {
                categories = Request.Params["cat"].ToString();
            }

            bool isAdvanceSearch = false;
            
            if (Request.Params["o"] != null)
            {
                isAdvanceSearch = true;
                switch (Request.Params["o"])
                {
                    case "0":
                        order = "register_date desc";
                        break;
                    case "1":
                        order = "views desc, register_date desc";
                        break;
                }
            }
            if (Request.Params["usr"] != null)
                author = Request.Params["usr"].ToString();
            if (Request.Params["dfilter"] != null)
                _dateFilter = Convert.ToInt32(Request.Params["dfilter"]);

            /* List Initialization */
            var ListEntity = new ChannelListViewModel()
            {
                isListStatus = true,
                isListNav = false,
                QueryOptions = new MemberEntity()
                {
                    pagenumber = (int)pagenumber,
                    searchtype = searchtype, // 0: category match, 1: tag match, 2: both category / tag, 3: broad search
                    term = term,
                    username = author,
                    iscache = false,
                    ispublic = true,
                    datefilter = _dateFilter,
                    pagesize = Site_Settings.QA_Page_Size,
                    order = order,
                },
                ListObject = new ListItems()
                {
                    ColWidth = "col-md-4",
                    ListType = ListType.Grid, // 0: grid 1: list
                    showViews = true,
                    showDate = true
                },
                NoRecordFoundText = Resources.vsk.norecordfound,
            };

            ListEntity.TotalRecords = UserBLL.Count(ListEntity.QueryOptions);
            if (ListEntity.TotalRecords > 0)
            {
                ListEntity.DataList = UserBLL.LoadItems(ListEntity.QueryOptions);
            }
            if (isAdvanceSearch)
            {
                var urlbuilder = new StringBuilder();
                urlbuilder.Append("?o=" + Request.Params["o"]);
                if (Request.Params["usr"] != null)
                    urlbuilder.Append("&usr=" + Request.Params["usr"]);
                if (Request.Params["cat"] != null)
                    urlbuilder.Append("&cat=" + Request.Params["cat"]);
                if (Request.Params["dfilter"] != null)
                    urlbuilder.Append("&dfilter=" + Request.Params["dfilter"]);
               
                ListEntity.DefaultUrl = DefaultUrl + "?" + urlbuilder.ToString();
                ListEntity.PaginationUrl = PaginationUrl + "?" + urlbuilder.ToString();
            }

            /* Page Title Adjustment */
            if (pagenumber > 1)
            {
                _title = _title + " - Page: " + pagenumber;
            }

            ViewBag.title = _title + " | " + Site_Settings.Page_Caption;

            return View(ListEntity);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult queryresult(JgnEnt.models.ViewModels.List.SearchListModel model)
        {
            return Redirect(Config.GetUrl("channels/search/" + Server.UrlEncode(UtilityBLL.CleanSearchTerm(model.Query))));
        }

        // GET: qa/searchoptions
        public ActionResult searchoptions()
        {
            ViewBag.title = "Channel Search | " + Site_Settings.Page_Caption;

            var model = new AdvanceSearchModelView();
            
            // order list
            var _orderList = new Dictionary<string, string>();
            _orderList.Add(Resources.vsk.recentlyadded, "0");
            _orderList.Add("Most Viewed", "1");
            model.orderlist = _orderList;

            // date added
            var _dateList = new Dictionary<string, string>();
            _dateList.Add(Resources.vsk.alltime, "0");
            _dateList.Add(Resources.vsk.today, "1");
            _dateList.Add(Resources.vsk.thisweek, "2");
            _dateList.Add(Resources.vsk.thismonth, "3");
            model.datelist = _dateList;

            model.date_added = 0; // all time default selected

            //model.isFeatured = 3; // both selected

            return View(model);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult searchoptions(AdvanceSearchModelView model, string action)
        {
            if (ModelState.IsValid)
            {
                var str = new StringBuilder();

                if (model.order == null)
                    model.order = "0";

                var _term = "skip-search";
                if (model.term != null && model.term != "")
                    _term = Server.UrlEncode(UtilityBLL.CleanSearchTerm(UtilityBLL.processNull(model.term, 0)));

                str.Append("?o=" + model.order);
                if (model.username != "" && model.username != null)
                    str.Append("&usr=" + Server.UrlEncode(UtilityBLL.CleanSearchTerm(UtilityBLL.processNull(model.username, 0))));
                if (model.categories != null)
                {
                    if (model.categories.Length > 0)
                        str.Append("&cat=" + string.Join(",", model.categories));

                }
                //if (model.isFeatured != 3)
                //    str.Append("&isfr=" + model.isFeatured);

                if (model.date_added > 0)
                    str.Append("&dfilter=" + model.date_added);

                return Redirect(Config.GetUrl("channels/search/" + _term + "/" + str.ToString()));
            }

            model.Message = ValidationError.HasErrors(ModelState);
            return View(model);
        }

        #endregion

        /*public ActionResult resize()
        {
            string FileName = "";
            string username = "";
            int isCloud = 0;
            int width = 800;
            int height = 600;
           
            // load no photo if original photo not found or error arises
            string strPath = Server.MapPath(Request.ApplicationPath) + "\\assets\\img\\default_image.jpg";

            if (Request.Params["file"] != null)
            {
                FileName = Request.Params["file"].ToString();
            }

            if (Request.Params["cloud"] != null)
                isCloud = Convert.ToInt32(Request.Params["cloud"]);

            if (Request.Params["size"] != null)
            {
                string[] arr = Request.Params["size"].ToString().Split(char.Parse("x"));
                if (arr.Length > 0)
                {
                    width = Convert.ToInt32(arr[0].Trim());
                    height = Convert.ToInt32(arr[1].Trim());
                }
            }
            if (Request.Params["u"] != null)
                username = Request.Params["u"].ToString();

            var ht = new Hashtable();
            if (isCloud == 0)
            {
                strPath = Server.MapPath(Request.ApplicationPath) + "/contents/member/" + username + "/photos/" + FileName;
            }

            if (Request.Params["size"] != null)
            {
                string[] arr = Request.Params["size"].ToString().Split(char.Parse("x"));
                if (arr.Length > 0)
                {
                    width = Convert.ToInt32(arr[0].Trim());
                    height = Convert.ToInt32(arr[1].Trim());
                }
            }

            var ImageID = Guid.NewGuid().ToString().Substring(0, 6);
            if (System.IO.File.Exists(strPath))
            {
                ht = new Hashtable();
                //FileStream fInfo = new FileStream(strPath, FileMode.Open);
                byte[] bFile;
                bFile = Jugnoon.Utility.Image.Generate_Online_Thumbnail(strPath, width, height);
                ht.Add(ImageID, bFile);
                //fInfo.Close();
            }

            var arrImg = (byte[])ht[ImageID];

            return base.File(arrImg, "image/jpeg");
        }*/

        private ChannelListFilterViewModel prepareFilterLinks(string url, string selectedOrder, string selectedFilter)
        {
            return new ChannelListFilterViewModel()
            {
                selectedFilter = selectedFilter,
                selectedOrder = selectedOrder,
                order_recent_url = url + "recent",
                order_view_url = url + "mostviewed",
                filter_date_alltime_url = url,
                filter_date_thismonth_url = url + "added/thismonth",
                filter_date_thisweek_url = url + "added/thisweek",
                filter_date_today_url = url + "added/today"
            };
        }
    }
}