﻿using Jugnoon.models;
using System;
using System.Web;
using System.Web.Mvc;
using Jugnoon.Utility;
using Jugnoon.BLL;
using DevOne.Security.Cryptography.BCrypt;
using Jugnoon.Settings;
using System.Security.Claims;
using Microsoft.AspNet.Identity;
using Microsoft.Owin.Security;

namespace JgnEnt.Controllers
{
    public class loginController : Controller
    {
        private IAuthenticationManager AuthenticationManager
        {
            get
            {
                return HttpContext.GetOwinContext().Authentication;
            }
        }

        // GET: login
        public ActionResult Index()
        {
            ViewBag.title= Resources.vsk.signin;

            var model = new UserLoginViewModel();

            model.AlertType = AlertTypes.Error;

            if (Request.Params["status"] != null)
            {
                switch (Request.Params["status"].ToString())
                {
                    case "pchanged":
                        model.Message = Resources.vsk.passwordupdated;
                        break;
                    case "perror":
                        model.Message = Resources.vsk.passwordpdatefailed;
                        break;
                }
            }
           
            if (Request.Params["error"] == "failed")
            {
                model.Message = Resources.vsk.message_06;
            }

            return View(model);
        }
        
        //
        // POST: /login
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Index(UserLoginViewModel model, string action)
        {
            ViewBag.title= Resources.vsk.signin + " | " + Site_Settings.Page_Caption;

            if (ModelState.IsValid)
            {
                // validate member
                // Update Password Validation Script
                if (model.UserName== "" || model.Password == "")
                {
                    model.Message = Resources.vsk.enterusernamepassword;
                    return View(model);
                }

                // IP Address tracking and processing
                string ipaddress = Request.ServerVariables["REMOTE_ADDR"].ToString();
                if (BlockIPBLL.Validate_IP(ipaddress))
                {
                    model.Message = Resources.vsk.ipblockedmsg;
                    return View(model);
                }

                int Membertype = 0;
                int Readonly = 0;
                var _lst = UserBLL.Get_Hash_Password(model.UserName);
                if (_lst.Count == 0)
                {
                    // No user account found based on username search
                    model.Message = Resources.vsk.message_06; // "Login failed, please enter correct user name &amp; password."
                    return View(model);
                }

                var _memberprocess = new UserBLL();
                // check encrypted password
                if (_lst[0].password.Length < 20)
                {
                    // backward compatibility
                    // check existing user passwords with old system
                    if (!_memberprocess.Validate_Member(model.UserName, model.Password, false))
                    {
                        model.Message = Resources.vsk.message_06;  // "Login failed, please enter correct user name &amp; password."
                        return View(model);
                    }

                    Membertype = Convert.ToInt32(UserBLL.Return_Value(model.UserName, "type"));
                    Readonly = Convert.ToInt32(UserBLL.Return_Value(model.UserName, "readonly"));
                }
                else
                {
                    // check encrypted password with user typed password
                    bool matched = BCryptHelper.CheckPassword(model.Password, _lst[0].password);
                    if (!matched)
                    {
                        model.Message = Resources.vsk.message_06;  // "Login failed, please enter correct user name &amp; password."
                        return View(model);
                    }

                    if(_lst[0].isenabled == 0)
                    {
                        model.Message = "Account has been blocked by site moderator";
                        return View(model);
                    }
                    Membertype = _lst[0].type; // type
                    Readonly = _lst[0].@readonly;
                }

                string Role = "User";
                switch (Membertype)
                {
                    case 0:
                        Role = "User";
                        break;
                    case 1:
                        Role = "Admin";
                        break;
                    case 2:
                        Role = "PaidUser";
                        break;
                }

                if (Membertype == 1)
                {
                    if (Readonly == 1)
                        Role = "ReadOnlyAdmin";
                }
                

                if (Site_Settings.Store_IPAddress)
                {
                    // Store IP Address Log 
                    User_IPLogBLL.Process_Ipaddress_Log(model.UserName, ipaddress);
                }

                // Update Last Login Activity of User
                UserBLL.Update_Field_V3(model.UserName, "last_login", DateTime.Now);
                // member is validated

                var checkPath = Server.MapPath(Request.ApplicationPath) + "/contents/member/" + model.UserName.ToLower();
                if (!System.IO.Directory.Exists(checkPath))
                {
                    Directory_Process.CreateRequiredDirectories(checkPath);
                }

                // authorize user
                var identity = new ClaimsIdentity(DefaultAuthenticationTypes.ApplicationCookie);
                identity.AddClaim(new Claim(ClaimTypes.Name, model.UserName));
                identity.AddClaim(new Claim(ClaimTypes.Role, Role));
                identity.AddClaim(new Claim(ClaimTypes.NameIdentifier, _lst[0].userid.ToString()));
                identity.AddClaim(new Claim(ClaimTypes.Email, _lst[0].email));

                AuthenticationManager.SignIn(new AuthenticationProperties() { IsPersistent = model.RememberMe }, identity);
               
                if (Request.Params["ReturnUrl"] != null)
                    return Redirect(Request.Params["ReturnUrl"].ToString());
                else
                    return Redirect(Config.GetUrl("account/"));
            }

            // if not validated
            model.Message = ValidationError.HasErrors(ModelState);
            return View(model);
        }

        public ActionResult logout()
        {
            Request.GetOwinContext().Authentication.SignOut();

            Request.GetOwinContext().Authentication.SignOut(DefaultAuthenticationTypes.ApplicationCookie);

            HttpContext.GetOwinContext().Authentication.SignOut(DefaultAuthenticationTypes.ApplicationCookie);

            return Redirect(Config.GetUrl());
        }

        public ActionResult forgot()
        {
            ViewBag.title= Resources.vsk.forgotpassword_title + " | " + Site_Settings.Page_Caption;

            var model = new ForgotViewModel();
            model.AlertType = AlertTypes.Error;

            if (Request.Params["status"] != null)
            {
                switch (Request.Params["status"].ToString())
                {
                    case "invalid":
                        model.Message = Resources.vsk.invalidkey;
                        break;
                    case "sent":
                        model.Message = Resources.vsk.forgotpassword_msg; // Password recovery / update link has been sent on your email address. please click on it to reset your password
                        model.AlertType = AlertTypes.Success;
                        break;
                }
            }

            return View(model);
        }

        // POST: /login/forgot
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult forgot(ForgotViewModel model, string action)
        {
            ViewBag.title= Resources.vsk.forgotpassword_title + " | " + Site_Settings.Page_Caption;

            if (ModelState.IsValid)
            {
                var _memberprocess = new UserBLL();
                // validate username
                if (!_memberprocess.Check_UserName(model.UserName))
                {
                    model.Message = Resources.vsk.message_07; // User Name not validated, please make sure you enter valid User Name
                    return View(model);
                }

                // get email address
                string emailaddress = "";
                if (model.UserName.Contains("@"))
                    emailaddress = model.UserName;
                else
                    emailaddress = UserBLL.Return_Value(model.UserName, "email");
                string val_key = Guid.NewGuid().ToString().Substring(0, 10);
                // update val_key
                UserBLL.Update_Field_V3(model.UserName, "val_key", val_key);
                // send validation url in email
                string url = Config.GetUrl("/login/updatepass?u=" + model.UserName + "&k=" + val_key);
                MailTemplateProcess(emailaddress, model.UserName, url);

                return Redirect("/login/forgot?status=sent");
            }

            // if not validated
            model.Message = ValidationError.HasErrors(ModelState);
            return View(model);
        }


        private void MailTemplateProcess(string emailaddress, string username, string url)
        {
            //if sending mail option enabled
            if (Config.isEmailEnabled())
            {
                var lst = MailTemplateBLL.Get_Record("FORPASS");
                if (lst.Count > 0)
                {
                    string subject = MailProcess.Process2(lst[0].subject, "\\[username\\]", username);
                    string contents = MailProcess.Process2(lst[0].contents, "\\[username\\]", username);
                    contents = MailProcess.Process2(contents, "\\[url\\]", url);

                    MailProcess.Send_Mail(emailaddress, subject, contents);
                }
            }
        }

        public ActionResult updatepass()
        {
            ViewBag.title= Resources.vsk.updatepassword + " | " + Site_Settings.Page_Caption;

            var model = new UpdatePasswordModelView();
            model.AlertType = AlertTypes.Error;

            if (Request.Params["u"] != null && Request.Params["k"] != null)
            {
                // validate user info
                var _members = new UserBLL();
                string username =Server.UrlDecode(Request.Params["u"].ToString());
                string key = Server.UrlDecode(Request.Params["k"].ToString());

                if (!_members.Check_Key(username, key))
                {
                    return Redirect(Config.GetUrl("/login/forgot?status=invalid"));
                }

                string ipaddress = Request.ServerVariables["REMOTE_ADDR"].ToString();
                if (BlockIPBLL.Validate_IP(ipaddress))
                {
                    model.Message = Resources.vsk.ipblockedmsg;
                    return View(model);
                }
                model.UserName = username;
            }
            else
            {
                return Redirect("/login/forgot");
            }
            return View(model);
        }

        //
        // POST: /login/updatepass
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult updatepass(UpdatePasswordModelView model, string action)
        {
            ViewBag.title= Resources.vsk.updatepassword + " | " + Site_Settings.Page_Caption;

            if (ModelState.IsValid)
            {
                if (model.Password.Length > 3)
                {
                    // change password

                    string encrypted_password = BCryptHelper.HashPassword(model.Password, BCryptHelper.GenerateSalt(6));
                    UserBLL.Update_Field_V3(model.UserName, "password", encrypted_password);
                    return Redirect(Config.GetUrl("/login?status=pchanged"));
                }
                else
                {
                    return Redirect(Config.GetUrl("/login?status=perror"));
                }

            }
            model.Message = ValidationError.HasErrors(ModelState);
            return View(model);
        }
    }
}