﻿using JgnEnt.models.ViewModels.videos;
using System;
using System.Web.Mvc;
using Jugnoon.Utility;
using Jugnoon.BLL;
using Jugnoon.Settings;
using Jugnoon.Entity;
using Microsoft.AspNet.Identity;
using JgnEnt.models.ViewModels.Shared;
using JgnEnt.models.ViewModels.videos.Player;
using System.Collections.Generic;

namespace JgnEnt.Controllers
{
    public class mediaController : Controller
    {
        // GET: media
        public ActionResult Index(long? pid)
        {
            if (pid == null)
            {
                return Redirect(Config.GetUrl("videos"));
            }
            var model = new VideoViewModel();
            model.isAllowed = true;

            if (Request.Params["status"] != null)
            {
                if (Request.Params["status"] == "pending")
                    model.Message = Resources.vsk.video_message_02; // Video has been posted and will be appear shortly.
            }


            var _list = VideoBLL.LoadItems(new VideoEntity()
            {
                videoid = (long)pid,
                issummary = false,
                isdropdown = false,
                nofilter = true
            });

            if (_list.Count == 0)
            {
                model.isAllowed = false;
                model.DetailMessage = Resources.vsk.norecordfound;
                return View(model);
            }

            model.Data = _list[0];

            model.PreviewUrl = VideoUrlConfig.PrepareUrl(model.Data);

            if (model.Data.isadult == 1)
            {
                // 1:-> adult content, 0:-> non adult content
                if (Session["adultmember"] == null)
                {
                    return Redirect(Config.GetUrl("home/validateadult?surl=" + Server.UrlEncode(model.PreviewUrl) + ""));
                }
            }

            if (_list[0].isreviewed == 0)
            {
                model.Message = Resources.vsk.videopreview_msg_01; // Video is pending for moderator approval
            }
            if (_list[0].isenabled == 0)
            {
                model.isAllowed = false;
                model.DetailMessage = Resources.vsk.videopreview_msg_02; // Video has been blocked. contact site administrator for more detail
                return View(model);
            }

            var _Path = "videos/";
            if (_list[0].type == 1)
                _Path = "audio/";

            model.Info = new InfoViewModel()
            {
                ContentID = (long)pid,
                Description = model.Data.description,
                Added_Date = (DateTime)model.Data.created_at,
                UserName =model.Data.username,
                CategoryID = 0,
                CategoryName = string.Join(",",model.Data.categories),
                Tags = model.Data.tags,
                Liked = model.Data.liked,
                Disliked = model.Data.disliked,
                MediaType = model.Data.type,
                ShowLikeBox = true,
                Path = _Path
            };


            if (Site_Settings.Feature_Filter != 1)
            {
                // simple view statistics
                _list[0].views = _list[0].views + 1;
                VideoBLL.Update_Field_V3((long)pid, model.Data.views, "views");
            }
            else
            {
                // scalability wise not recommended
                // wide range of view statistics 
                //int ItemType = 1; // represent videos
                //ViewStatsBLL.Process_View_Stats((long)pid, ItemType, _list[0].views);
                _list[0].views = _list[0].views + 1;
                VideoBLL.Update_Field_V3((long)pid, model.Data.views, "views");
            }
            model.Data.views++;

            // Video Path settings
            string VideoPath = "";
            string PictureName = "";
            if (model.Data.type == 0)
            {
                // video settings
                VideoPath = Config.Return_FLV_Video_Url(model.Data.flv_url, model.Data.username) + "/" + model.Data.videofilename;
                if (model.Data.thumb_url != "none")
                    PictureName = model.Data.thumb_url;
                else
                    PictureName = Config.Return_Video_Thumb_Url(model.Data.thumb_url, model.Data.username) + "/" + model.Data.thumbfilename;
                string MP4Path = "";
                if (VideoPath.EndsWith(".webm"))
                    MP4Path = VideoPath.Replace(".webm", ".mp4");
            }
            else
            {
                // audio settings
                VideoPath = Config.Return_MP3_Audio_Url(model.Data.flv_url, model.Data.username) + "/" + model.Data.videofilename;
                if (model.Data.thumb_url != "none")
                    PictureName = model.Data.thumb_url;
                else if (_list[0].coverurl.StartsWith("http"))
                    PictureName = _list[0].coverurl;
                else
                    PictureName = Config.Return_Video_Thumb_Url(model.Data.thumb_url, model.Data.username) + "/" + model.Data.thumbfilename;

             
            }
                      
          
            string _script = _list[0].embed_script;
            string _videoUrl = "";
            if (_list[0].preview_url != null && _list[0].preview_url != "")
                _videoUrl = _list[0].preview_url;
            else
                _videoUrl = _list[0].flv_url;

            model.Player = new VideoJsModelView()
            {
                EmbedScript = "",
                Type = model.Data.type
            };

            if (_list[0].coverurl != "")
                model.Player.PictureUrl = _list[0].coverurl;
            else
                model.Player.PictureUrl = PictureName;
                       
            if (model.Data.type == 0)
            {
                // video player settings
                // if AWS Uploader Enabled
                if (_list[0].streamoutputs != null && _list[0].streamoutputs != "")
                {
                    // enabled advance adaptive streaming / mpeg-dash / hls
                    model.Player.enabledAWS = true;
                    model.Player.VideoFeed = new List<SupportedVideos>();
                    var supportedStreamOptions = _list[0].streamoutputs.Split(char.Parse(","));
                    foreach(var option in supportedStreamOptions)
                    {
                        /*if (option == "mpegdash")
                        {
                            //CloudFront.CreateCannedPrivateURL(_list[0].flv_url + "mpegdash.mpd", 3000),
                            model.Player.VideoFeed.Add(new SupportedVideos()
                            {
                                src = CloudFront.CreateCannedPrivateURL(_list[0].flv_url + "mpegdash.mpd", 900), 
                                type = "application/dash+xml"
                            });
                        }
                        
                        if (option == "hls")
                        {
                            // CloudFront.CreateCannedPrivateURL(_list[0].flv_url + "hls.m3u8", 3000),
                            model.Player.VideoFeed.Add(new SupportedVideos()
                            {
                                src = CloudFront.CreateCannedPrivateURL(_list[0].flv_url + "hls.m3u8", 900),
                                type = "application/x-mpegURL"
                            });
                        }
                        if (option == "mp4_480")
                        {
                            model.Player.VideoFeed.Add(new SupportedVideos()
                            {
                                src = CloudFront.CreateCannedPrivateURL(_list[0].flv_url + "menifest_480.mp4", 3000),
                                type = "video/mp4"
                            });
                        }*/
                        // enable hd streaming (make sure it is enabled in AWS Lambda to publish
                        /*if (option == "mp4_720")
                        {
                            model.Player.VideoFeed.Add(new SupportedVideos()
                            {
                                src = CloudFront.CreateCannedPrivateURL(_list[0].flv_url + "menifest_720.mp4", 3000),
                                type = "video/mp4"
                            });
                        }*/
                    }
                }
                // If cloud enabled
                else if (_script != "" || _list[0].youtubeid != "")
                {
                    // third party embed video script enabled
                    if (_script == "")
                    {
                        model.Player.EmbedScript = _list[0].youtubeid;
                    }
                    else
                    {
                        model.Player.EmbedScript = _script;
                    }
                }
                
                else if (_list[0].videofilename.EndsWith("mp4"))
                {
                   
                    if (TokenBLL.isTokenEnabled)
                    {
                        // Token Based Stream Authentication Enabled
                        // Recommended to stream on HTML5 Players
                        string maxParam = VideoBLL.GenerateBandwidthParam();
                        string Token = TokenBLL.Add();
                        model.Player.VideoUrl = "/stream/token.ashx?f=" + _list[0].videofilename + "&u=" + _list[0].username + "&tk=" + Token + "" + maxParam;
                        if (_list[0].originalvideofilename.EndsWith(".png"))
                            model.Player.PictureUrl = "/contents/member/" + _list[0].username + "/thumbs/" + _list[0].originalvideofilename;
                    }
                    else if (VideoBLL.EnableHttpStreaming)
                    {
                        // Http Streaming Enabled
                        // Recommended to stream on HTML5 Players
                        string maxParam = VideoBLL.GenerateBandwidthParam();
                        model.Player.VideoUrl = "/stream/stream.ashx?f=" + _list[0].videofilename + "&u=" + _list[0].username + "" + maxParam;
                    }
                    else
                    {
                        // Direct Play
                        model.Player.VideoUrl = VideoPath;
                    }
                }
                else
                {
                    // load and play videos normally
                    model.Player.VideoUrl = VideoPath;
                }
            }
            else
            {
                // audio player settings
                model.Player.VideoUrl = VideoPath;
                model.Player.PictureUrl = PictureName;
            }

            model.Action = new ActionViewModel()
            {
                ContentID = (long)pid,
                Liked = _list[0].liked,
                Disliked = _list[0].disliked,
                UserName = _list[0].username,
                Favorites = _list[0].favorites,
                VideoFileName = model.Data.videofilename,
                VideoPath = VideoPath,
                isRating = _list[0].isratings,
                Views = _list[0].views,
                Comments = _list[0].comments,
                Content_Type = Resources.vsk.video,
                isEmbed = true,
                MediaType = model.Data.type,
                Ratingtype = 1, // 0: like / dislike, 1: rating
                Avg_Ratings = _list[0].avg_rating,
                Ratings = _list[0].ratings,
                TotalRatings = _list[0].total_rating,
                isDownload = false
            };

            /* 
              related1.tags = tags;
                related1.title= _title;
                related1.videoid = VideoID;
                related1.categoryid = _list[0].categoryid;
                related1.AlbumId = _list[0].galleryid; */

            model.Comments = new CommentViewModel()
            {
                isComment = _list[0].iscomments
            };

            string meta_title = _list[0].title;
            if (meta_title == "")
                meta_title = "Media No " + _list[0].videoid;

            string _desc = UtilityBLL.StripHTML(_list[0].description);
            if (_desc == "")
                _desc = _list[0].title;
            else if (_desc.Length > 160)
                _desc = _desc.Substring(0, 160);

            ViewBag.title= meta_title + " | " + Site_Settings.Page_Caption;
            ViewBag.description = _desc + ", date uploaded: " + _list[0].created_at + ", VID: " + _list[0].videoid;

            return View(model);
        }

      
    }
}
 