﻿using Jugnoon.BLL;
using Jugnoon.models;
using Jugnoon.Settings;
using Jugnoon.Utility;
using Microsoft.Owin.Security;
using System;
using System.Security.Claims;
using System.Web;
using System.Web.Mvc;
using Jugnoon.Entity;
using DevOne.Security.Cryptography.BCrypt;
using Microsoft.AspNet.Identity;
using System.Threading.Tasks;
using JgnEnt.models.ViewModels.User;


namespace JgnEnt.Controllers
{
    public class signupController : Controller
    {

        private IAuthenticationManager AuthenticationManager
        {
            get
            {
                return HttpContext.GetOwinContext().Authentication;
            }
        }

        // GET: signup
        public ActionResult Index()
        {
            ViewBag.title= Resources.vsk.register_page_title;

            var _model = new UserSignupViewModel();
            if (Site_Settings.Feature_MemberRegistration == 0)
                _model.isBlocked = true;
            
            if (Request.IsAuthenticated)
            {
                var user = Request.GetOwinContext().Authentication.User;
                if(Auth.isAdmin(user))
                {
                    _model.isAdmin = true;
                    _model.AccountTypes = GeneralTypes.AccountTypes(false);
                    _model.Type = "0";
                }
            }
            return View(_model);
        }

        public ActionResult accountstatus()
        {
            ViewBag.title= Resources.vsk.account_activation_title;
            
            return View();
        }

        public ActionResult activate()
        {
            ViewBag.title= "Create Account | " + Site_Settings.Page_Caption;

            var _model = new ActivateAccountViewModel();

            if (Request.Params["key"] != null && Request.Params["user"] != null)
            {
                var _members = new UserBLL();
                // validate key
                string username = Request.Params["user"].ToString();
                if (_members.Check_Key(username, Request.Params["key"].ToString()))
                {
                    // activate account
                    _members.Update_IsEnabled(username, 1);

                    return Redirect("/login?status=activated");
                }
                else
                {
                    // key validation failed
                    _model.Message = Resources.vsk.registrationkeyvalidationfailed;
                }
            }
            else
            {
                _model.Message = Resources.vsk.noinformation;
            }

            return View(_model);
        }
        
        //
        // POST: /signup
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Index(UserSignupViewModel model, string action)
        {
            ViewBag.title= Resources.vsk.register_page_title;

            // for init purpose
            if(model.isAdmin)
            {
                model.AccountTypes = GeneralTypes.AccountTypes(false);
                model.Type = "0";
            }
            var AdminView = "~/Areas/admin/Views/members/create.cshtml";
            if (ModelState.IsValid)
            {
                model.AlertType = AlertTypes.Error;
                // IP Address tracking and processing
                string ipaddress = Request.ServerVariables["REMOTE_ADDR"].ToString();

                if (!model.isAdmin)
                {
                    // check for restricted usernames
                    string res_values = DictionaryBLL.Return_RestrictedUserNames();
                    if (res_values != "")
                    {
                        if (DictionaryBLL.isMatch(model.UserName, res_values))
                        {
                            model.Message = Resources.vsk.message_reg_03; // User name not available, please choose another one.
                            if (model.isAdmin)
                                return View(AdminView, model);
                            else
                                return View(model);
                        }
                    }
                   
                    if (BlockIPBLL.Validate_IP(ipaddress))
                    {
                        model.Message = Resources.vsk.ipblockedmsg;
                        if (model.isAdmin)
                            return View(AdminView, model);
                        else
                            return View(model);
                    }
                }
                else
                {
                    model.AccountTypes = GeneralTypes.AccountTypes(false);
                }
                

               

                var _memberprocess = new UserBLL();
                if (_memberprocess.Check_UserName(model.UserName))
                {
                    model.Message = Resources.vsk.message_reg_03; // User name not available, please choose another one.
                    if (model.isAdmin)
                        return View(AdminView, model);
                    else
                        return View(model);
                }

                if (_memberprocess.Check_Email(model.Email))
                {
                    model.Message = Resources.vsk.message_reg_04; // Email address is already exist.
                    if (model.isAdmin)
                        return View(AdminView, model);
                    else
                        return View(model);
                }

                string gender = "Male";
              
                // validation key processing
                string val_key = "none";
                int isenabled = 1; // user account activated
                if (Config.isRegistrationValidation() && !model.isAdmin)
                {
                    val_key = Guid.NewGuid().ToString().Substring(0, 10);
                    isenabled = 0; // user account deactivated
                }
                // Add Member
                int type = 0; // normal member
                if (model.isAdmin)
                {
                    type = Convert.ToInt32(model.Type);
                }
                // Credits and Default Space Allocation
                int credits = 0;
                int userrole_id = 0;

                int atype = 0;

                var entity = new UserDataEntity()
                {
                    accounttype = (short)atype,
                    username = model.UserName,
                    password = BCryptHelper.HashPassword(model.Password, BCryptHelper.GenerateSalt(6)),
                    email = model.Email,
                    countryname = "",
                    isenabled = (byte)isenabled,
                    gender = gender,
                    birthdate = DateTime.Now,
                    type = (byte)type,
                    credits = credits,
                    roleid = userrole_id
                };

                long userid = UserBLL.Add(entity, val_key);
                // Create Required Directories
                Directory_Process.CreateRequiredDirectories(Server.MapPath(Request.ApplicationPath) + "/contents/member/" + model.UserName.ToLower());

                if (model.isAdmin)
                {
                    return Redirect(Config.GetUrl("admin/members/?status=created"));
                }
                else
                {
                    // Send Mail
                    MailTemplateProcess(model.Email,  model.UserName, model.Password, val_key);

                    if (Config.isRegistrationValidation())
                    {
                       return Redirect("/signup/accountstatus?user=" + model.UserName + "");
                    }
                    else
                    {
                        string UserRole = "Normal";
                        // authorize user
                        var identity = new ClaimsIdentity(DefaultAuthenticationTypes.ApplicationCookie);
                        identity.AddClaim(new Claim(ClaimTypes.Name, model.UserName));
                        identity.AddClaim(new Claim(ClaimTypes.Role, UserRole));
                        identity.AddClaim(new Claim(ClaimTypes.NameIdentifier, userid.ToString()));
                        identity.AddClaim(new Claim(ClaimTypes.Email, model.Email));
                        
                        AuthenticationManager.SignIn(new AuthenticationProperties() { IsPersistent = false }, identity);
                        // Store IP Address Log 
                        User_IPLogBLL.Process_Ipaddress_Log(model.UserName, ipaddress);
                        if (Config.GetMembershipAccountUpgradeRedirect() == 1)
                            return Redirect("account/payment/packages?status=success");
                        else
                            return Redirect("account/?status=success");
                    }
                }
            }

            // if not validated
            model.Message = ValidationError.HasErrors(ModelState);
            if (model.isAdmin)
                return View(AdminView, model);
            else
                return View(model);
        }

        // mail processing
        private void MailTemplateProcess(string emailaddress, string username, string password, string key)
        {
            //if sending mail option enabled
            if (Config.isEmailEnabled())
            {
                var lst = MailTemplateBLL.Get_Record("USRREG");
                if (lst.Count > 0)
                {
                    string subject = MailProcess.Process2(lst[0].subject, "\\[username\\]", username);
                    string contents = MailProcess.Process2(lst[0].contents, "\\[username\\]", username);
                    contents = MailProcess.Process2(contents, "\\[password\\]", password);
                    string validation_url = Config.GetUrl("signup/activate?key=" + key.Trim() + "&user=" + username.Trim());
                    string url = "<a href=\"" + validation_url + "\">" + validation_url + "</a>";
                    contents = MailProcess.Process2(contents, "\\[key_url\\]", url);

                    MailProcess.Send_Mail(emailaddress, subject, contents);
                }
            }
        }
        private const string XsrfKey = "mevsk_33k22";
        //
        // POST: /Account/ExternalLogin
        [HttpPost]
        [AllowAnonymous]
        [ValidateAntiForgeryToken]
        public ActionResult ExternalLogin(string provider, string returnUrl)
        {
            // Request a redirect to the external login provider
            return new ChallengeResult(provider, Url.Action("googleredirect", "signup", new { ReturnUrl = returnUrl }));
        }

        [AllowAnonymous]
        public async Task<ActionResult> googleredirect(string returnUrl)
        {          
            var loginInfo = await AuthenticationManager.GetExternalLoginInfoAsync();
            if (loginInfo == null)
            {
                return RedirectToAction("Register", new { message = "Unable to authenticate with external login." });
            }

            var Email = loginInfo.Email;
            var claimId = loginInfo.ExternalIdentity;
            var model = new ExternalAuthModelView();
            foreach (var claim in claimId.Claims)
            {
                model.Issuer = claim.Issuer;
                if (claim.Type.Contains("/nameidentifier"))
                    model.NameIdentifier = claim.Value;
                else if (claim.Type.Contains("/givenname"))
                    model.GivenName = claim.Value;
                else if (claim.Type.Contains("/surname"))
                    model.SurName = claim.Value;
                else if (claim.Type.Contains("/name"))
                    model.Name = claim.Value;
                else if (claim.Type.Contains("/emailaddress"))
                    model.EmailAddress = claim.Value;
                else if (claim.Type.Contains("accesstoken"))
                    model.AccessToken = claim.Value;
                else if (claim.Type.Contains("pictureUrl"))
                    model.PictureUrl = claim.Value;
                else if (claim.Type.Contains("gender"))
                    model.Gender = claim.Value;
            }
                    
            // check email address
            var _member = new UserBLL();

            if (_member.Check_Email(model.EmailAddress))
            {
                // account already exist
                // fetch account data
                var Info = UserBLL.LoadItems(new MemberEntity()
                {
                    email = model.EmailAddress
                })[0];
                if(Info.password != "" && Info.issuer == "")
                {
                    // account registered with no external source
                    ViewBag.title= Resources.vsk.emailalreadyexist;
                    return View("InternalAccount", model);
                }
                else if(Info.isenabled == 0)
                {
                    // account blocked
                    ViewBag.title= Resources.vsk.accountblocked;
                    return View("AccountBlocked", model);
                }

                string UserRole = "Normal";
                if (Info.type == 1)
                    UserRole = "Admin";
                // authorize user
                var identity = new ClaimsIdentity(DefaultAuthenticationTypes.ApplicationCookie);
                identity.AddClaim(new Claim(ClaimTypes.Name, Info.username));
                identity.AddClaim(new Claim(ClaimTypes.Role, UserRole));
                identity.AddClaim(new Claim(ClaimTypes.NameIdentifier, Info.userid.ToString()));
                identity.AddClaim(new Claim(ClaimTypes.Email, model.EmailAddress));

                AuthenticationManager.SignIn(new AuthenticationProperties() { IsPersistent = false }, identity);

                // login in system and redirect to 
                return RedirectToLocal(returnUrl);
            }
            else
            {
                // account not exist
                // create account
                ViewBag.title= Resources.vsk.completeyourregistration;
                return View("ExternalRegistration", model);
            }

            // create account with credentials returned
           
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult extregistration(ExternalAuthModelView model, HttpPostedFileBase _file)
        {
            if (ModelState.IsValid)
            {
                model.AlertType = AlertTypes.Error;

                // check for restricted usernames
                string res_values = DictionaryBLL.Return_RestrictedUserNames();
                if (res_values != "")
                {
                    if (DictionaryBLL.isMatch(model.UserName, res_values))
                    {
                        model.Message = Resources.vsk.message_reg_03; // User name not available, please choose another one.
                        return View("ExternalRegistration", model);
                    }
                }

                // IP Address tracking and processing
                string ipaddress = Request.ServerVariables["REMOTE_ADDR"].ToString();
                if (BlockIPBLL.Validate_IP(ipaddress))
                {
                    model.Message = Resources.vsk.ipblockedmsg;
                    return View("ExternalRegistration", model);
                }

                var _memberprocess = new UserBLL();
                if (_memberprocess.Check_UserName(model.UserName))
                {
                    model.Message = Resources.vsk.message_reg_03; // User name not available, please choose another one.
                    return View("ExternalRegistration", model);
                }

                if (_memberprocess.Check_Email(model.EmailAddress))
                {
                    model.Message = Resources.vsk.message_reg_03; // Email address is already exist.
                    return View("ExternalRegistration", model);
                }

                string gender = "Male";

                // validation key processing
                string val_key = "none";
                int isenabled = 1; // user account activated
                int type = 0; // normal member
               
                // Credits and Default Space Allocation
                int credits = 0;
                int userrole_id = 0;

                int atype = 0;

                var entity = new UserDataEntity()
                {
                    accounttype = (short)atype,
                    username = model.UserName,
                    password = "", // no password set in external registration account
                    email = model.EmailAddress,
                    firstname = model.GivenName,
                    lastname = model.SurName,
                    countryname = "",
                    isenabled = (byte)isenabled,
                    gender = gender,
                    birthdate = DateTime.Now,
                    type = (byte)type,
                    credits = credits,
                    roleid = userrole_id,
                    issuer = model.Issuer
                };

                long userid = UserBLL.Add(entity, val_key);
                // Create Required Directories
                Directory_Process.CreateRequiredDirectories(Server.MapPath(Request.ApplicationPath) + "/contents/member/" + model.UserName.ToLower());

                // update thumb
                // Jugnoon.Helper.Member.UpdateProfileAvator(model.UserName, model.Picture, _file);

                // Send Mail
                MailTemplateProcess(model.EmailAddress, model.UserName, "", val_key);

                string UserRole = "Normal";
                
                // authorize user
                var identity = new ClaimsIdentity(DefaultAuthenticationTypes.ApplicationCookie);
                identity.AddClaim(new Claim(ClaimTypes.Name, model.UserName));
                identity.AddClaim(new Claim(ClaimTypes.Role, UserRole));
                identity.AddClaim(new Claim(ClaimTypes.NameIdentifier, userid.ToString()));
                identity.AddClaim(new Claim(ClaimTypes.Email, model.EmailAddress));

                AuthenticationManager.SignIn(new AuthenticationProperties() { IsPersistent = false }, identity);
                // Store IP Address Log 
                User_IPLogBLL.Process_Ipaddress_Log(model.UserName, ipaddress);

                if (Config.GetMembershipAccountUpgradeRedirect() == 1)
                    return Redirect("/account/payment/packages?status=success");
                else
                    return Redirect("/account/?status=success");
            }

            // if not validated
            model.Message = ValidationError.HasErrors(ModelState);
            return View(model);
        }


        private void AddErrors(IdentityResult result)
        {
            foreach (var error in result.Errors)
            {
                ModelState.AddModelError("", error);
            }
        }

        private ActionResult RedirectToLocal(string returnUrl)
        {
            if (Url.IsLocalUrl(returnUrl))
            {
                return Redirect(returnUrl);
            }
            return RedirectToAction("Index", "Home");
        }

        internal class ChallengeResult : HttpUnauthorizedResult
        {
            public ChallengeResult(string provider, string redirectUri)
                : this(provider, redirectUri, null)
            {
            }

            public ChallengeResult(string provider, string redirectUri, string userId)
            {
                LoginProvider = provider;
                RedirectUri = redirectUri;
                UserId = userId;
            }

            public string LoginProvider { get; set; }
            public string RedirectUri { get; set; }
            public string UserId { get; set; }

            public override void ExecuteResult(ControllerContext context)
            {
                var properties = new AuthenticationProperties { RedirectUri = RedirectUri };
                if (UserId != null)
                {
                    properties.Dictionary[XsrfKey] = UserId;
                }
                context.HttpContext.GetOwinContext().Authentication.Challenge(properties, LoginProvider);
            }
        }
    }
}