﻿
using System.Web.Mvc;
using JgnEnt.models.ViewModels.User;
using Jugnoon.Utility;
using Microsoft.AspNet.Identity;
using Jugnoon.BLL;
using System.Text;
using Jugnoon.models;
using Jugnoon.Entity;
using Jugnoon.Settings;
using Jugnoon.Scripts;

namespace JgnEnt.Controllers
{
    public class userController : Controller
    {        
        // GET: user
        public ActionResult Index(string username)
        {
            var model = new UserModelView();
            model.AlertType = AlertTypes.Success;

            model.ActiveIndex = 0; 

            if (username == null || username.ToLower() == "index")
            {
                if (Request.IsAuthenticated)
                {
                    model.UserName = HttpContext.User.Identity.GetUserName();
                }
                else
                {
                    string redirect_url = Config.GetUrl("account");
                    return Redirect(Config.GetUrl() + "log?ReturnUrl=" + redirect_url);
                }
            }
            else
            {
                // user key provided
                model.UserName = username;
            }

            if (Request.Params["status"] != null)
            {
                switch (Request.Params["status"].ToString())
                {
                    case "chnl_setting_changed":
                        // channel settings changed
                        model.Message =Resources.vsk.message_channel_01; // Channel setting has been changed
                        break;
                    case "chnl_theme_changed":
                        // channel theme changed
                        model.Message =Resources.vsk.message_channel_02; // Channel theme has been changed
                        break;
                    case "chnl_module_changed":
                        // channel module changed
                        model.Message =Resources.vsk.message_channel_03; // Channel module settings have been changed
                        break;
                    case "fvalfailed":
                        // friend invitation :-> failed to find request key / uid with friend request
                        model.Message =Resources.vsk.message_channel_04; // "Friend request validation failed!"
                        break;
                    case "fcancelled":
                        // friend invitation :-> request cancelled
                        model.Message =Resources.vsk.message_channel_05; // Friend request cancelled!
                        break;
                    case "fusrvalfailed":
                        // friend invitation :-> user validation failed
                        model.Message =Resources.vsk.message_channel_06; // Friend request, user validation failed!
                        break;
                    case "accepted":
                        // friend inviation :-> user accepted friend inviation
                        if (Request.Params["uid"] != null)
                            model.Message =Request.Params["uid"].ToString() + " become your friend";
                        else
                            model.Message =Resources.vsk.message_channel_07; // User has been added in your friend list successfully
                        break;
                    case "unfriend":
                        //ProcessUnfriend(this.username);
                        break;
                    case "unfriendcomplete":
                        model.Message ="You and " + model.UserName + " unfriend successfully!";
                        break;
                    case "afriendsuccess":
                        model.Message =Resources.vsk.message_channel_16; // "Friend request has been sent."
                        break;
                    case "100":
                        model.Message =Resources.vsk.message_channel_11; // Select user to send him/her friend request.
                        model.AlertType = AlertTypes.Error;
                        break;
                    case "101":
                        model.Message =Resources.vsk.message_channel_12; // "Sign In to send him/her friend request."
                        model.AlertType = AlertTypes.Error;
                        break;
                    case "103":
                        model.Message =Resources.vsk.message_channel_13; // You can't send friend request to your own profile.
                        model.AlertType = AlertTypes.Error;
                        break;
                    case "104":
                        model.Message =Resources.vsk.message_channel_14; // User not found may be disabled by Administrator.
                        model.AlertType = AlertTypes.Error;
                        break;
                    case "105":
                        model.Message =Resources.vsk.message_channel_17; // Current cuser already in your friend list or request is in pending mode
                        model.AlertType = AlertTypes.Error;
                        break;
                }
            }

            model.UserExist = true;

            if(Request.IsAuthenticated)
            {
                if (model.UserName == HttpContext.User.Identity.GetUserName())
                    model.OwnProfile = true;
            }
            var _list = UserBLL.LoadItems(new MemberEntity()
            {
                username = model.UserName,
                ispublic = true,
                loadall = true
            });
            if (_list.Count > 0)
            {
                // increment user views
                int _views = _list[0].views + 1;
                UserBLL.Update_Field_V3(model.UserName, "views", _views);

                model.user = _list[0];

                model.Nav = new NavModelView()
                {
                    username = model.UserName,
                    ActiveIndex = 100,
                    ShowVideos = true, //getValue(model.user.video_upload),
                    CountVideos = model.user.stat_videos,
                    ShowAudio = getValue(model.user.channel_isaudios),
                    CountAudio = model.user.stat_audios,
                    ShowForumPosts = true,  //getValue(model.user.foru),
                    CountTopics = model.user.stat_forum_posts,
                    ShowPhotos = getValue(model.user.channel_isphotos),
                    CountPhotos = model.user.stat_photos,
                    ShowQA = getValue(model.user.stat_qa),
                    CountQA = model.user.stat_qa
                };

                model.UserInfo = prepare_user_info(model);

                model.FullName = prepare_user_fullname(model);

                if (Site_Settings.Feature_Galleries == 1)
                {
                    model.albums = new AlbumListViewModel()
                    {
                        HeadingTitle = model.FullName + " " + Resources.vsk.albums,
                        BrowseText = Resources.vsk.seeall,
                        HeadingElement = "h3",
                        HeaderTextCss = "mb20 text-white",
                        ListObject = new Jugnoon.Scripts.ListItems()
                        {
                            ListType = ListType.Grid, // 0: grid 1: list
                            ItemType = ItemType.Link, // 0: div, 1: a
                            ColWidth = "col-md-4"
                        },
                       
                        BrowseUrl = Config.GetUrl("user/albums/" + model.UserName),
                        QueryOptions = new AlbumEntity()
                        {
                            username = model.UserName,
                            order = "g.added_date desc",
                            ispublic = true,
                            isfeatured = 3,
                            pagesize = 6
                        }
                    };
                }


                if (Site_Settings.Feature_Videos == 1)
                {
                    model.videos = new VideoListViewModel()
                    {
                        BrowseText = Resources.vsk.seeall,
                        HeadingTitle = model.FullName + " " + Resources.vsk.videos,
                        HeadingElement = "h3",
                        HeaderTextCss = "mb20 text-white",
                        ListObject = new Jugnoon.Scripts.ListItems()
                        {
                            ListType = ListType.Grid, // 0: grid 1: list
                            ItemType = ItemType.Link, // 0: div, 1: a
                            ColWidth = "col-md-4"
                        },
                        BrowseUrl = Config.GetUrl("user/videos/" + model.UserName),
                        QueryOptions = new VideoEntity()
                        {
                            username = model.UserName,
                            order = "v.date_added desc",
                            ispublic = true,
                            isfeatured = 3,
                            pagesize = 6,
                            type = 0 // 0: videos, 1: audio
                        }
                    };
                }

                if (Site_Settings.Feature_Audio == 1)
                {
                    model.audio = new VideoListViewModel()
                    {
                        HeadingTitle = model.FullName + " " + Resources.vsk.audiofiles,
                        BrowseText = Resources.vsk.seeall,
                        HeadingElement = "h3",
                        HeaderTextCss = "mb20 text-white",
                        ListObject = new Jugnoon.Scripts.ListItems()
                        {
                            ListType = ListType.Grid, // 0: grid 1: list
                            ItemType = ItemType.Link, // 0: div, 1: a
                            ColWidth = "col-md-4"
                        },
                        BrowseUrl = Config.GetUrl("user/audio/" + model.UserName),
                        QueryOptions = new VideoEntity()
                        {
                            username = model.UserName,
                            order = "v.date_added desc",
                            ispublic = true,
                            isfeatured = 3,
                            pagesize = 6,
                            type = 1 // 0: videos, 1: audio
                        }
                    };
                }

                ViewBag.title= model.FullName + "'s " + Resources.vsk.profile + " | " + Site_Settings.Page_Caption;
                ViewBag.description = ViewBag.title+ ", info: " + model.UserInfo + ", date joined: " + _list[0].created_at;
            }
            else
            {
                model.UserExist = false;
            }
            return View(model);
        }

        private bool getValue(int value)
        {
            if (value > 0)
                return true;
            else
                return false;
        }
        public ActionResult profile(string username)
        {
            var model = new UserModelView();
            model.AlertType = AlertTypes.Success;

            model.ActiveIndex = 10; // profile section

            if (username == null || username.ToLower() == "index")
            {
                if (Request.IsAuthenticated)
                {
                    model.UserName = HttpContext.User.Identity.GetUserName();
                }
                else
                {
                    string redirect_url = Config.GetUrl("account");
                    return Redirect(Config.GetUrl() + "log?ReturnUrl=" + redirect_url);
                }
            }
            else
            {
                // user key provided
                model.UserName = username;
            }
                     
            model.UserExist = true;

            if (Request.IsAuthenticated)
            {
                if (model.UserName == HttpContext.User.Identity.GetUserName())
                    model.OwnProfile = true;
            }


            var _list = UserBLL.LoadItems(new MemberEntity()
            {
                ispublic = true,
                username = model.UserName,
                loadall = true
            });
            if (_list.Count > 0)
            {
                model.user = _list[0];
                // load user profile detail
                model.profile = UserBLL.LoadItems(new MemberEntity()
                {
                    username = model.UserName,
                    loadall = true
                })[0];
                model.UserInfo = prepare_user_info(model);

                model.FullName = prepare_user_fullname(model);

                model.Nav = new NavModelView()
                {
                    username = model.UserName,
                    ActiveIndex = 100,
                    ShowVideos = true, //getValue(model.user.video_upload),
                    CountVideos = model.user.stat_videos,
                    ShowAudio = getValue(model.user.channel_isaudios),
                    CountAudio = model.user.stat_audios,
                    ShowForumPosts = true,  //getValue(model.user.foru),
                    CountTopics = model.user.stat_forum_posts,
                    ShowPhotos = getValue(model.user.channel_isphotos),
                    CountPhotos = model.user.stat_photos,
                    ShowQA = getValue(model.user.stat_qa),
                    CountQA = model.user.stat_qa
                };
            }
            else
            {
                model.UserExist = false;
            }

            
            ViewBag.title= model.FullName + "'s " + Resources.vsk.detailprofile + " | " + Site_Settings.Page_Caption;
        
            return View(model);
        }

        public ActionResult videos(string username, int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;

            var model = new UserModelView();
            model.AlertType = AlertTypes.Success;

            model.ActiveIndex = 8; // blog section

            if (username == null || username.ToLower() == "index")
            {
                if (Request.IsAuthenticated)
                {
                    model.UserName = HttpContext.User.Identity.GetUserName();
                }
                else
                {
                    string redirect_url = Config.GetUrl("account");
                    return Redirect(Config.GetUrl() + "log?ReturnUrl=" + redirect_url);
                }
            }
            else
            {
                // user key provided
                model.UserName = username;
            }

            model.UserExist = true;

            if (Request.IsAuthenticated)
            {
                if (model.UserName == HttpContext.User.Identity.GetUserName())
                    model.OwnProfile = true;
            }


            var _list = UserBLL.LoadItems(new MemberEntity()
            {
                username = model.UserName,
                ispublic = true,
                loadall = true
            });
            if (_list.Count > 0)
            {
                model.user = _list[0];

                model.UserInfo = prepare_user_info(model);

                model.FullName = prepare_user_fullname(model);

                var _title= model.FullName + "'s " + Resources.vsk.uploadedvideos + " (" + _list[0].stat_videos+ ")";

                model.Nav = new NavModelView()
                {
                    username = model.UserName,
                    ActiveIndex = 1,
                    ShowVideos = true, //getValue(model.user.video_upload),
                    CountVideos = model.user.stat_videos,
                    ShowAudio = getValue(model.user.channel_isaudios),
                    CountAudio = model.user.stat_audios,
                    ShowForumPosts = true,  //getValue(model.user.foru),
                    CountTopics = model.user.stat_forum_posts,
                    ShowPhotos = getValue(model.user.channel_isphotos),
                    CountPhotos = model.user.stat_photos,
                    ShowQA = getValue(model.user.stat_qa),
                    CountQA = model.user.stat_qa
                };

                /* List Settings */
                model.videos = new VideoListViewModel()
                {
                    isListStatus = false,
                    QueryOptions = new VideoEntity()
                    {
                        type = 0, // 0: videos, 1: audio
                        pagenumber = (int)pagenumber,
                        username = model.UserName,
                        iscache = false,
                        ispublic =true,
                        isfeatured = 3, // all
                        pagesize = Site_Settings.Video_Page_Size,
                        order = "v.date_added desc",
                    },
                    ListObject = new Jugnoon.Scripts.ListItems()
                    {
                        ColWidth = "col-md-4 col-masonry mb20",
                        ListType = ListType.Grid, // 0: grid 1: list
                        ItemType = ItemType.Link, // 0: div, 1: a
                        TitleLength = 40
                    },
                    HeadingTitle = _title,
                    HeaderTextCss = "mb30",
                    DefaultUrl = Config.GetUrl("user/videos/" + model.UserName),
                    PaginationUrl = Config.GetUrl("user/videos/" + model.UserName + "/[p]/"),
                    NoRecordFoundText = Resources.vsk.norecordfound,
                };
                model.videos.TotalRecords = VideoBLL.Count(model.videos.QueryOptions);
                if (model.videos.TotalRecords > 0)
                {
                    model.videos.DataList = VideoBLL.LoadItems(model.videos.QueryOptions);
                }

                ViewBag.title = _title + " | " + Site_Settings.Page_Caption;
            }
            else
            {
                model.UserExist = false;
            }
            return View(model);

        }
        public ActionResult audio(string username, int? pagenumber)
        {
            if (pagenumber == null)
                pagenumber = 1;

            var model = new UserModelView();
            model.AlertType = AlertTypes.Success;

            model.ActiveIndex = 8; // blog section

            if (username == null || username.ToLower() == "index")
            {
                if (Request.IsAuthenticated)
                {
                    model.UserName = HttpContext.User.Identity.GetUserName();
                }
                else
                {
                    string redirect_url = Config.GetUrl("account");
                    return Redirect(Config.GetUrl() + "log?ReturnUrl=" + redirect_url);
                }
            }
            else
            {
                // user key provided
                model.UserName = username;
            }

            model.UserExist = true;

            if (Request.IsAuthenticated)
            {
                if (model.UserName == HttpContext.User.Identity.GetUserName())
                    model.OwnProfile = true;
            }

            var _list = UserBLL.LoadItems(new MemberEntity()
            {
                username = model.UserName,
                ispublic = true,
                loadall = true
            });
            if (_list.Count > 0)
            {
                model.user = _list[0];

                model.UserInfo = prepare_user_info(model);

                model.FullName = prepare_user_fullname(model);

                model.Nav = new NavModelView()
                {
                    username = model.UserName,
                    ActiveIndex = 2,
                    ShowVideos = true, //getValue(model.user.video_upload),
                    CountVideos = model.user.stat_videos,
                    ShowAudio = getValue(model.user.channel_isaudios),
                    CountAudio = model.user.stat_audios,
                    ShowForumPosts = true,  //getValue(model.user.foru),
                    CountTopics = model.user.stat_forum_posts,
                    ShowPhotos = getValue(model.user.channel_isphotos),
                    CountPhotos = model.user.stat_photos,
                    ShowQA = getValue(model.user.stat_qa),
                    CountQA = model.user.stat_qa
                };
                var _title = model.FullName + "'s " + Resources.vsk.uploadedaudio + " (" + _list[0].stat_audios + ")";
                /* List Settings */
                model.videos = new VideoListViewModel()
                {
                    isListStatus = false,
                    QueryOptions = new VideoEntity()
                    {
                        type = 1, // 0: videos, 1: audio
                        pagenumber = (int)pagenumber,
                        username = model.UserName,
                        term = "",
                        iscache = false,
                        ispublic = true,
                        isfeatured = 3, // all
                        pagesize = Site_Settings.Video_Page_Size,
                        order = "v.date_added desc",
                    },
                    ListObject = new Jugnoon.Scripts.ListItems()
                    {
                        ColWidth = "col-md-4 col-masonry mb20",
                        ListType = ListType.Grid, // 0: grid 1: list
                        ItemType = ItemType.Link, // 0: div, 1: a
                        TitleLength = 40
                    },
                    HeadingTitle = _title,
                    DefaultUrl = Config.GetUrl("user/audio/" + model.UserName),
                    PaginationUrl = Config.GetUrl("user/audio/" + model.UserName + "/[p]/"),
                    NoRecordFoundText = Resources.vsk.norecordfound,
                };
                model.videos.TotalRecords = VideoBLL.Count(model.videos.QueryOptions);
                if (model.videos.TotalRecords > 0)
                {
                    model.videos.DataList = VideoBLL.LoadItems(model.videos.QueryOptions);
                }

                ViewBag.title = _title + " | " + Site_Settings.Page_Caption;
            }
            else
            {
                model.UserExist = false;
            }

            model.UserInfo = prepare_user_info(model);

            model.FullName = prepare_user_fullname(model);

            return View(model);
        }
    
        private string prepare_user_fullname(UserModelView model)
        {
            string _name = model.UserName;
            if (model.user.firstname != "")
                model.FullName = model.user.firstname + " " + model.user.lastname;

            return _name;
        }
        private string prepare_user_info(UserModelView model)
        {
            var info = new StringBuilder();
            info.Append(model.user.gender);
            if (model.user.relationshipstatus != "")
            {
                info.Append(", " + model.user.relationshipstatus);
            }
            if (model.user.hometown != "" || model.user.currentcity != "")
            {
                if (model.user.currentcity != "")
                {
                    info.Append(", " + model.user.hometown);
                }
                else
                {
                    info.Append(", " + model.user.currentcity);
                }
            }
            if (model.user.zipcode != "")
            {
                info.Append(", " + model.user.zipcode);
            }

            if (model.user.countryname != "")
            {
                info.Append(", " + model.user.countryname);
            }

            return info.ToString();
        }
    }
}