﻿using JgnEnt.models.ViewModels.videos;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Jugnoon.Utility;
using Jugnoon.BLL;
using Jugnoon.Settings;
using Jugnoon.Entity;
using JgnEnt.models.ViewModels.videos.Player;

namespace JgnEnt.Controllers
{
    public class videoframeController : Controller
    {
        // GET: media
        public ActionResult Index(long? pid)
        {
            var model = new VideoFrameViewModel();
            model.isAllowed = true;
            if (pid == null)
            {
                model.isAllowed = false;
                model.DetailMessage = Resources.vsk.norecordfound;
                return View(model);
            }

            if (Request.Params["status"] != null)
            {
                if (Request.Params["status"] == "pending")
                    model.Message = Resources.vsk.video_message_02; // Video has been posted and will be appear shortly.
            }


            var _list = VideoBLL.LoadItems(new VideoEntity()
            {
                videoid = (long)pid
            });

            if (_list.Count == 0)
            {
                model.isAllowed = false;
                model.DetailMessage = Resources.vsk.norecordfound;
                return View(model);
            }

            model.Data = _list[0];

            if (_list[0].isreviewed == 0)
            {
                model.Message = Resources.vsk.videopreview_msg_01; // Video is pending for moderator approval
            }
            if (_list[0].isenabled == 0)
            {
                model.isAllowed = false;
                model.DetailMessage = Resources.vsk.videopreview_msg_02; // Video has been blocked. contact site administrator for more detail
                return View(model);
            }

            if (Site_Settings.Feature_Filter != 1)
            {
                // simple view statistics
                _list[0].views = _list[0].views + 1;
                VideoBLL.Update_Field_V3((long)pid, model.Data.views, "views");
            }
            else
            {
                // scalability wise not recommended
                // wide range of view statistics 
                //int ItemType = 1; // represent videos
                //ViewStatsBLL.Process_View_Stats((long)pid, ItemType, _list[0].views);
            }
            model.Data.views++;

            // Video Path settings
            string VideoPath = "";
            string PictureName = "";
            if (model.Data.type == 0)
            {
                // video settings
                VideoPath = Config.Return_FLV_Video_Url(model.Data.flv_url, model.Data.username) + "/" + model.Data.videofilename;
                if (model.Data.thumb_url != "none")
                    PictureName = model.Data.thumb_url;
                else
                    PictureName = Config.Return_Video_Thumb_Url(model.Data.thumb_url, model.Data.username) + "/" + model.Data.thumbfilename;
                string MP4Path = "";
                if (VideoPath.EndsWith(".webm"))
                    MP4Path = VideoPath.Replace(".webm", ".mp4");
            }
            else
            {
                // audio settings
                VideoPath = Config.Return_MP3_Audio_Url(model.Data.flv_url, model.Data.username) + "/" + model.Data.videofilename;

                PictureName = Config.GetUrl() + "assets/img/default_profile.gif";
            }


            string _script = _list[0].embed_script;

            model.Player = new VideoJsModelView()
            {
                EmbedScript = "",
                Type = model.Data.type
            };

            if (_list[0].coverurl != "")
                model.Player.PictureUrl = _list[0].coverurl;
            else
                model.Player.PictureUrl = PictureName;

            if (model.Data.type == 0)
            {
                // video player settings
                // If cloud enabled
                if (_script != "" || _list[0].youtubeid != "")
                {
                    // third party embed video script enabled
                    if (_script == "")
                    {
                        model.Player.EmbedScript = _list[0].youtubeid;
                    }
                    else
                    {
                        model.Player.EmbedScript = _script;
                    }
                }
               
                else if (_list[0].videofilename.EndsWith("mp4"))
                {

                    if (TokenBLL.isTokenEnabled)
                    {
                        // Token Based Stream Authentication Enabled
                        // Recommended to stream on HTML5 Players
                        string maxParam = VideoBLL.GenerateBandwidthParam();
                        string Token = TokenBLL.Add();
                        model.Player.VideoUrl = "/stream/token.ashx?f=" + _list[0].videofilename + "&u=" + _list[0].username + "&tk=" + Token + "" + maxParam;
                        if (_list[0].originalvideofilename.EndsWith(".png"))
                            model.Player.PictureUrl = "/contents/member/" + _list[0].username + "/thumbs/" + _list[0].originalvideofilename;
                    }
                    else if (VideoBLL.EnableHttpStreaming)
                    {
                        // Http Streaming Enabled
                        // Recommended to stream on HTML5 Players
                        string maxParam = VideoBLL.GenerateBandwidthParam();
                        model.Player.VideoUrl = "/stream/stream.ashx?f=" + _list[0].videofilename + "&u=" + _list[0].username + "" + maxParam;
                    }
                    else
                    {
                        // Direct Play
                        model.Player.VideoUrl = VideoPath;
                    }
                }
                else
                {
                    // load and play videos normally
                    model.Player.VideoUrl = VideoPath;
                }
            }
            else
            {
                // audio player settings
                model.Player.VideoUrl = VideoPath;
            }
            string meta_title = _list[0].title;
            if (meta_title == "")
                meta_title = "Media No " + _list[0].videoid;

            string _desc = UtilityBLL.StripHTML(_list[0].description);
            if (_desc == "")
                _desc = _list[0].title;
            else if (_desc.Length > 160)
                _desc = _desc.Substring(0, 160);

            ViewBag.title = meta_title;
            ViewBag.description = _desc + ", date uploaded: " + _list[0].created_at + ", VID: " + _list[0].videoid;

            return View(model);
        }

        public ActionResult preview()
        {
            var model = new VideoPreviewViewModel();
            if (Request.Params["url"] == null)
            {
                model.isExist = false;
            }
            else
            {
                model.isExist = true;
                model.previewUrl = Server.HtmlDecode(Request.Params["url"]).Replace(",/videoframe/preview","");
            }
            return View(model);
        }
    }
}