using System;
using System.Web;
using System.Collections.Generic;
/* Site Own Namespaces */
using Jugnoon.Entity;
using Jugnoon.Utility;
using Jugnoon.EntityFramework;
using JgnEnt.Framework;
using System.Linq;

namespace Jugnoon.BLL
{
    public class AbuseReport
    {
        // Note: Abuse Report Important Terms

        // It can be use for flag videos, blogs and other contents
        // It can also be use for (mark as spam) any content such as (comments, message)

        // contentid:
        // Unique ID of content that represent either video, blog, comment, message etc

        // Type:
        // ........... 0: Videos.
        // ........... 1: Channel Comments
        // ........... 2: User Messages
        // ........... 3: Group Topics
        // ........... 4: Video Comments
        // ........... 5: Photos
        // ........... 6: Photo Comments
        // ........... 7: Blogs
        // ........... 8: Blog Comments
        //............ 9: Photo Galleries
        // ........... 10: QA
        // ........... 11: QA Answer

        // New Type:
        //............ 4: Every type of comments (channel, videos, photos, blogs, galleries etc)

        public AbuseReport()
        {
        }

        public enum Types
        {
            Videos = 0,
            Channels = 1,
            UserMessages = 2,
            Groups = 4,
            VideoComments = 4,
            Photos = 5,
            PhotoComments = 6,
            Blogs = 7,
            BlogComments = 8,
            PhotoGalleries = 9,
            QA = 11,
            ANAnswer = 100
        };

        public static bool Add(long ContentID, string UserName, string IPAddress, string Reason, int Type)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                dbCtx.Entry(new JgnEnt.Framework.abusereport()
                {
                    reportid = ContentID,
                    contentid = ContentID,
                    username = UserName,
                    ipaddress = IPAddress,
                    reason = Reason,
                    type = (byte)Type,
                    addeddate = DateTime.Now
                }).State = System.Data.Entity.EntityState.Added;

                DataBLL.SaveChanges(dbCtx);
            }
            return true;
        }
        
        public static bool Delete(long ContentID, int Type)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new abusereport { contentid = ContentID, type = (byte)Type };
                dbCtx.abusereports.Attach(entity);
                dbCtx.abusereports.Remove(entity);
                dbCtx.SaveChanges();
            }
 
            return true;
        }

        // delete single reports related to content
        public static bool Delete(long ReportID)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new abusereport { reportid = ReportID };
                dbCtx.abusereports.Attach(entity);
                dbCtx.abusereports.Remove(entity);
                dbCtx.SaveChanges();
            }
            
            return true;
        }

        public static bool Check_UserName(string username, long ContentID, int Type)
        {
            bool flag = false;
            using (var ctx = new mvcportalEntities())
            {
                var records = ctx.abusereports
                    .Where(p => p.username == username && p.contentid == ContentID && p.type == Type)
                    .Count();
                if (records > 0)
                    flag = true;
            }
            return flag;
        }

        public static bool Check_IPAddress(string ipaddress, long ContentID, int Type)
        {
            bool flag = false;
            using (var ctx = new mvcportalEntities())
            {
                var records = ctx.abusereports
                    .Where(p => p.ipaddress == ipaddress && p.contentid == ContentID && p.type == Type)
                    .Count();
                if (records > 0)
                    flag = true;
            }
            return flag;
        }
              

        public static int Count(long ContentID, int Type)
        {
            int Records = 0;
            using (var ctx = new mvcportalEntities())
            {
                Records = ctx.abusereports
                    .Where(p => p.contentid == ContentID && p.type == Type)
                    .Count();
                
            }
            return Records;
        }
        
        public static List<AbuseEntity> Load_Reports(string type)
        {
            var _list = new List<AbuseEntity>();

            using (var ctx = new mvcportalEntities())
            {
                IQueryable<abusereport> query = ctx.abusereports;
                if (type != "all")
                    query = query.Where(p => p.type == Convert.ToByte(type));

                _list = query.Select(p => new AbuseEntity
                {
                    reportid = p.reportid,
                    contentid = p.contentid,
                    username = p.username,
                    ipaddress = p.ipaddress,
                    reason = p.reason,
                    addeddate = p.addeddate,
                    type = p.type,
                    
                }).ToList();
            }
            
            return _list;
        }

        /* utility functions */
        public static string Process_V2(string content_username, long contentid, string reason, int type, int mediatype)
        {
            if (!HttpContext.Current.User.Identity.IsAuthenticated)
            {
                string sign_in = "<a href=\"" + Config.GetUrl() + "Login.aspx\" class=\"bold\">Sign In</a>";
                string sign_up = "<a href=\"" + Config.GetUrl() + "Register.aspx\" class=\"bold\">Sign Up</a>";
                return sign_in + " or " + sign_up + " to post report!";
            }
            string username = HttpContext.Current.User.Identity.Name;

            if (content_username == username)
            {
                return "You can't post abuse / spam report on your own content.";
            }

            if (AbuseReport.Check_UserName(username, contentid, type))
            {
                return "You already post abuse / spam report on this content.";
            }

            //***********************************
            // ENABLE comment if you want to validate report based on ip address
            //***********************************

            string ipaddress = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"];
            if (AbuseReport.Check_IPAddress(ipaddress, contentid, type))
            {
                return "Report already posted from this IP address.";
            }

            AbuseReport.Add(contentid, username, ipaddress, reason, type);

            int count_reports = AbuseReport.Count(contentid, type);
            if (count_reports > Config.Return_AbuseReport_Count())
            {
                int OldValue = 0;
                switch (type)
                {
                    case 0:
                        // disable video
                        VideoBLL.Update_Field_V3(contentid, (byte)0, "isenabled");
                        OldValue = Convert.ToInt32(VideoBLL.Get_Field_Value(contentid, "isenabled"));
                        break;
                    case 9:
                        // Disable Album
                        GalleryBLLC.Update_Field_V3(contentid, (byte)0, "isdisabled");
                        break;
                }

            }
            return "Report has been submitted successfully!";
        }

    }
}