﻿using System;
using System.Web;
using Jugnoon.Utility;
using System.Text;
using Jugnoon.Entity;
using System.Collections.Generic;
using JgnEnt.Framework;
using System.Linq;
using Jugnoon.EntityFramework;
using Jugnoon.Settings;
using System.Web.Caching;

namespace Jugnoon.BLL
{
    public class AdsBLL
    {
        // Note: AdsBLL handle advertisement on website

        // This BLL handles admin related activities on control panel.

        // Type:
        // ........... 1: Adult Section Ads -/ will popup if user access adult content
        // ........... 0: Non Adult Section Ads -/ will popup if user access normal content.


        public AdsBLL()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        public static bool Add_Script(string name, string adscript, int type)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                dbCtx.Entry(new advertisement()
                {
                    name = name,
                    adscript = adscript,
                    type = (byte)type
                }).State = System.Data.Entity.EntityState.Added;

                DataBLL.SaveChanges(dbCtx);
            }
            return true;
        }

        public static bool Update_Field_V3(long ID, dynamic Value, string FieldName)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.advertisements
                     .Where(p => p.adid == ID)
                     .FirstOrDefault<advertisement>();

                if(item != null)
                {
                    foreach (var prop in item.GetType().GetProperties())
                    {
                        if (prop.Name.ToLower() == FieldName.ToLower())
                        {
                            prop.SetValue(item, Value);
                        }
                    }
                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }
            return true;
        }
        public static bool Update_Script(int adid, string name, string adscript)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.advertisements
                     .Where(p => p.adid == adid)
                     .FirstOrDefault<advertisement>();

                if(item != null)
                {
                    item.name = name;
                    item.adscript = adscript;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }

            }
            
            return true;
        }

        public static bool Count_Script()
        {
            bool flag = false;
            using (var ctx = new mvcportalEntities())
            {
                if (ctx.advertisements.Count() > 0)
                    flag = true;
            }
            
            return flag;
        }
              

        public static string Fetch_Ad_Script(int id)
        {
            string Value = "";
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.advertisements
                     .Where(p => p.adid == id)
                     .FirstOrDefault<advertisement>();

                if(item != null)
                   Value = item.adscript;

            }
            
            return Value;
        }

        public static string Return_Ad_Script(int id)
        {
            if (HttpContext.Current.Application["a" + id] == null)
            {
                HttpContext.Current.Application["a" + id] = Fetch_Ad_Script(id);
            }
            return HttpContext.Current.Application["a" + id].ToString();
        }

        public static List<AdEntity> Load(AdEntity entity)
        {
            var _list = new List<AdEntity>();

            using (var ctx = new mvcportalEntities())
            {
                IQueryable<advertisement> Query = ctx.advertisements;
                Query = BuildQuery(Query, entity);
            
                if (entity.adid > 0)
                {
                    entity.loadall = true;

                    _list = LoadCompleteList(Query);
                }

                if (entity.issummary)
                {
                    _list = LoadSummaryList(Query);
                }
                else if (entity.isdropdown)
                {
                    _list = LoadDropdownList(Query);
                }
                else
                {
                    _list = LoadCompleteList(Query);
                }
            }

            return _list;
        }

        public static int Count(AdEntity entity)
        {
            if (!entity.iscache || Site_Settings.Cache_Duration == 0 || entity.pagenumber > 5)
            {
                return CountRecords(entity);
            }
            else
            {
                string key = GenerateKey("cnt_ads", entity);
                if (HttpContext.Current.Cache[key] == null)
                {
                    int total_records = CountRecords(entity);

                    HttpContext.Current.Cache.Add(key, CountRecords(entity), null, DateTime.Now.AddMinutes(Site_Settings.Cache_Duration), TimeSpan.Zero, CacheItemPriority.High, null);
                }
                return Convert.ToInt32(HttpContext.Current.Cache[key]);
            }
        }

        private static int CountRecords(AdEntity entity)
        {
            int Records = 0;
            using (var entities = new mvcportalEntities())
            {
                IQueryable<advertisement> Query = entities.advertisements;
                entity.iscount = true;
                entity.adid = 0;
                Query = BuildQuery(Query, entity);
                Records = Query.Count();
            }
            return Records;
        }
        private static string GenerateKey(string key, AdEntity entity)
        {
            StringBuilder str = new StringBuilder();
            return key + entity.datefilter + "" + entity.filter + "" + UtilityBLL.ReplaceSpaceWithHyphin(entity.order.ToLower()) + "" + entity.pagenumber + "" + entity.term + "" + entity.type;
        }

        private static IQueryable<advertisement> BuildQuery(IQueryable<advertisement> query, AdEntity entity)
        {
            query = Where(query, entity);
            if (entity.adid == 0 && !entity.iscount)
            {
                query = OrderBy(query, entity);
                query = Skip(query, entity);
                query = Limit(query, entity);
            }
            return query;
        }

        public static List<AdEntity> LoadCompleteList(IQueryable<advertisement> query)
        {
            return query.Select(p => new AdEntity
            {
                adid = (int)p.adid,
                name = p.name,
                adscript = p.adscript,
                type = p.type
            }).ToList();
        }
        public static List<AdEntity> LoadSummaryList(IQueryable<advertisement> query)
        {
            return query.Select(p => new AdEntity
            {
                adid = (int)p.adid,
                name = p.name,
                adscript = p.adscript,
                type = p.type
            }).ToList();
        }
        public static List<AdEntity> LoadDropdownList(IQueryable<advertisement> query)
        {
            return query.Select(p => new AdEntity
            {
                adid = (int)p.adid,
                name = p.name
            }).ToList();
        }

        private static IQueryable<advertisement> Where(IQueryable<advertisement> query, AdEntity entity)
        {
            if (entity.adid > 0)
                query = query.Where(p => p.adid == entity.adid);
            if (entity.type > -1)
                query = query.Where(p => p.type == entity.type);
            if(entity.term != "")
                query = query.Where(p => p.adscript.Contains(entity.term) || p.name.Contains(entity.term));
            return query;
        }
        private static IQueryable<advertisement> OrderBy(IQueryable<advertisement> query, AdEntity entity)
        {
            if(entity.order != "")
            {
                var orderlist = entity.order.Split(char.Parse(","));
                foreach(var order in orderlist)
                {
                    if(order.Contains("adid"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.adid);
                        else
                            query = query.OrderBy(p => p.adid);
                    }
                    else if (order.Contains("name"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.name);
                        else
                            query = query.OrderBy(p => p.name);
                    }
                }
            }
           
            return query;
        }
        private static IQueryable<advertisement> Skip(IQueryable<advertisement> query, AdEntity entity)
        {
            if (entity.pagenumber > 1)
                query = query.Skip(entity.pagesize * (entity.pagenumber - 1));

            return query;
        }
        private static IQueryable<advertisement> Limit(IQueryable<advertisement> query, AdEntity entity)
        {
            if (!entity.loadall)
                query = query.Take(entity.pagesize);

            return query;
        }

        public static string ProcessAction(List<AdEntity> list)
        {           
            return "Operation performed successfully";
        }
    }
}