﻿using System;
using System.Data;
using Jugnoon.Utility;
using System.Text;
using Jugnoon.Entity;
using System.Collections.Generic;
using System.Linq;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;
using Jugnoon.Settings;
using System.Web.Caching;
using System.Web;

namespace Jugnoon.BLL
{
    public class BlockIPBLL
    {
        // Note: BlockIPBLL used to take actions on certain ip address.


        public BlockIPBLL()
        {
            //
            // TODO: Add constructor logic here
            //
        }


        public static BlockIPEntity Add_IP(BlockIPEntity entity)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var data = new blockip()
                {
                    ipaddress = entity.ipaddress,
                    date_added = DateTime.Now
                };
                dbCtx.Entry(data).State = System.Data.Entity.EntityState.Added;

                DataBLL.SaveChanges(dbCtx);

                entity.id = data.id;
            }
            return entity;
        }

        public static bool Update_Field_V3(long ID, dynamic Value, string FieldName)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.blockips
                     .Where(p => p.id == ID)
                     .FirstOrDefault<blockip>();

                if(item != null)
                {
                    foreach (var prop in item.GetType().GetProperties())
                    {
                        if (prop.Name.ToLower() == FieldName.ToLower())
                        {
                            prop.SetValue(item, Value);
                        }
                    }
                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
                
            }
            return true;
        }

        public static bool Delete_IP(int id)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new blockip { id = id };
                dbCtx.blockips.Attach(entity);
                dbCtx.blockips.Remove(entity);
                dbCtx.SaveChanges();
            }
            
            return true;
        }

        
        public static bool Validate_IP(string ipaddress)
        {
            bool flag = false;
            using (var ctx = new mvcportalEntities())
            {
                var records = ctx.blockips
                    .Where(p => p.ipaddress == ipaddress)
                    .Count();
                if (records > 0)
                    flag = true;
            }
            return flag;
        }

        public static List<BlockIPEntity> Load(BlockIPEntity entity)
        {
            var _list = new List<BlockIPEntity>();

            using (var ctx = new mvcportalEntities())
            {
                IQueryable<blockip> Query = ctx.blockips;
                Query = BuildQuery(Query, entity);
                if (entity.id > 0)
                {
                    entity.loadall = true;

                    _list = LoadCompleteList(Query);
                }
                if (entity.issummary)
                {
                    _list = LoadSummaryList(Query);
                }
                else if (entity.isdropdown)
                {
                    _list = LoadDropdownList(Query);
                }
                else
                {
                    _list = LoadCompleteList(Query);
                }
            }

            return _list;
        }

        public static int Count(BlockIPEntity entity)
        {
            if (!entity.iscache || Site_Settings.Cache_Duration == 0 || entity.pagenumber > 5)
            {
                return CountRecords(entity);
            }
            else
            {
                string key = GenerateKey("cnt_blockip", entity);
                if (HttpContext.Current.Cache[key] == null)
                {
                    int total_records = CountRecords(entity);

                    HttpContext.Current.Cache.Add(key, CountRecords(entity), null, DateTime.Now.AddMinutes(Site_Settings.Cache_Duration), TimeSpan.Zero, CacheItemPriority.High, null);
                }
                return Convert.ToInt32(HttpContext.Current.Cache[key]);
            }
        }

        private static int CountRecords(BlockIPEntity entity)
        {
            int Records = 0;
            using (var entities = new mvcportalEntities())
            {
                IQueryable<blockip> Query = entities.blockips;
                entity.iscount = true;
                entity.id = 0;
                Query = BuildQuery(Query, entity);
                Records = Query.Count();
            }
            return Records;
        }
        private static string GenerateKey(string key, BlockIPEntity entity)
        {
            StringBuilder str = new StringBuilder();
            return key + entity.datefilter + "" + entity.filter + "" + UtilityBLL.ReplaceSpaceWithHyphin(entity.order.ToLower()) + "" + entity.pagenumber + "" + entity.term;
        }

        private static IQueryable<blockip> BuildQuery(IQueryable<blockip> query, BlockIPEntity entity)
        {
            query = Where(query, entity);
            if (entity.id == 0 && !entity.iscount)
            {
                query = OrderBy(query, entity);
                query = Skip(query, entity);
                query = Limit(query, entity);
            }
            return query;
        }

        public static List<BlockIPEntity> LoadCompleteList(IQueryable<blockip> query)
        {
            return query.Select(p => new BlockIPEntity
            {
                id = (int)p.id,
                ipaddress = p.ipaddress,
                date_added = p.date_added,
            }).ToList();
        }
        public static List<BlockIPEntity> LoadSummaryList(IQueryable<blockip> query)
        {
            return query.Select(p => new BlockIPEntity
            {
                id = (int)p.id,
                ipaddress = p.ipaddress,
                date_added = p.date_added,
            }).ToList();
        }
        public static List<BlockIPEntity> LoadDropdownList(IQueryable<blockip> query)
        {
            return query.Select(p => new BlockIPEntity
            {
                id = (int)p.id,
                ipaddress = p.ipaddress
            }).ToList();
        }

        private static IQueryable<blockip> Where(IQueryable<blockip> query, BlockIPEntity entity)
        {
            if (entity.id > 0)
                query = query.Where(p => p.id == entity.id);
            if (entity.ipaddress != "")
                query = query.Where(p => p.ipaddress.Contains(entity.ipaddress));
            if (entity.term != "")
                query = query.Where(p => p.ipaddress.Contains(entity.term));

            return query;
        }
        private static IQueryable<blockip> OrderBy(IQueryable<blockip> query, BlockIPEntity entity)
        {
            if (entity.order != "")
            {
                var orderlist = entity.order.Split(char.Parse(","));
                foreach (var order in orderlist)
                {
                    if (order.Contains("id"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.id);
                        else
                            query = query.OrderBy(p => p.id);
                    }
                    else if (order.Contains("ipaddress"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.ipaddress);
                        else
                            query = query.OrderBy(p => p.ipaddress);
                    }
                    else if (order.Contains("date_added"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.date_added);
                        else
                            query = query.OrderBy(p => p.date_added);
                    }
                }
            }

            return query;
        }
        private static IQueryable<blockip> Skip(IQueryable<blockip> query, BlockIPEntity entity)
        {
            if (entity.pagenumber > 1)
                query = query.Skip(entity.pagesize * (entity.pagenumber - 1));

            return query;
        }
        private static IQueryable<blockip> Limit(IQueryable<blockip> query, BlockIPEntity entity)
        {
            if (!entity.loadall)
                query = query.Take(entity.pagesize);

            return query;
        }
        public static string ProcessAction(List<BlockIPEntity> list)
        {
            foreach (BlockIPEntity entity in list)
            {
                if (entity.id > 0)
                {
                    switch (entity.actionstatus)
                    {
                        case "delete":
                            BlockIPBLL.Delete_IP(entity.id);
                            break;
                    }
                }
            }
            return "Operation performed successfully";
        }
    }
}