﻿using System;
using Jugnoon.Utility;
using Jugnoon.Entity;
using System.Collections.Generic;
using System.Linq;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;


namespace Jugnoon.BLL
{

    public class ErrorLgBLL
    {
        // ErrorLog store error occurs when user browse website.
        public ErrorLgBLL()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        public static bool Add(string description, string url, string stack_trace)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                dbCtx.Entry(new error_log()
                {
                    description = UtilityBLL.processNull(description, 0),
                    url = UtilityBLL.processNull(url, 0),
                    stack_trace = UtilityBLL.processNull(stack_trace, 0),
                    added_date = DateTime.Now
                }).State = System.Data.Entity.EntityState.Added;

                DataBLL.SaveChanges(dbCtx);
            }
            return true;
        }

        public static bool Delete(int id)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new error_log { id = id };
                dbCtx.error_log.Attach(entity);
                dbCtx.error_log.Remove(entity);
                dbCtx.SaveChanges();
            }

            return true;
        }

        public static bool Delete()
        {
            using (var dbCtx = new mvcportalEntities())
            {
                dbCtx.Database.ExecuteSqlCommand("TRUNCATE TABLE error_log");
            }

            return true;
        }

        public static List<LogEntity> Load(LogEntity entity)
        {
            var _list = new List<LogEntity>();

            using (var ctx = new mvcportalEntities())
            {
                IQueryable<error_log> Query = ctx.error_log;
                Query = BuildQuery(Query, entity);
             
                if (entity.id > 0)
                {
                    entity.loadall = true;

                    _list = LoadCompleteList(Query);
                }
                else
                {
                    _list = LoadSummaryList(Query);
                }
            }

            return _list;
        }
                
        public static int Count(LogEntity entity)
        {
            int Records = 0;
            using (var entities = new mvcportalEntities())
            {
                IQueryable<error_log> Query = entities.error_log;
                entity.iscount = true;
                entity.id = 0;
                Query = BuildQuery(Query, entity);
                Records = Query.Count();
            }
            return Records;
        }
      
        private static IQueryable<error_log> BuildQuery(IQueryable<error_log> query, LogEntity entity)
        {
            query = Where(query, entity);
            if (entity.id == 0 && !entity.iscount)
            {
                query = OrderBy(query, entity);
                query = Skip(query, entity);
                query = Limit(query, entity);
            }
            return query;
        }

        public static List<LogEntity> LoadCompleteList(IQueryable<error_log> query)
        {
            return query.Select(p => new LogEntity
            {
                id = (int)p.id,
                description = p.description,
                url = p.url,
                stack_trace = p.stack_trace,
                added_date = (DateTime)p.added_date
            }).ToList();
        }
        public static List<LogEntity> LoadSummaryList(IQueryable<error_log> query)
        {
            return query.Select(p => new LogEntity
            {
                id = (int)p.id,
                url = p.url,
                description = p.description,
                stack_trace = p.stack_trace,
                added_date = (DateTime)p.added_date
            }).ToList();
        }
       
        private static IQueryable<error_log> Where(IQueryable<error_log> query, LogEntity entity)
        {
            if (entity.id > 0)
                query = query.Where(p => p.id == entity.id);
            if (entity.term != "")
                query = query.Where(p => p.description.Contains(entity.term) || p.stack_trace.Contains(entity.term) || p.url.Contains(entity.term));

            return query;
        }
        private static IQueryable<error_log> OrderBy(IQueryable<error_log> query, LogEntity entity)
        {
            if (entity.order != "")
            {
                var orderlist = entity.order.Split(char.Parse(","));
                foreach (var order in orderlist)
                {
                    if (order.Contains("id"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.id);
                        else
                            query = query.OrderBy(p => p.id);
                    }
                    else if (order.Contains("added_date"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.added_date);
                        else
                            query = query.OrderBy(p => p.added_date);
                    }
                }
            }

            return query;
        }
        private static IQueryable<error_log> Skip(IQueryable<error_log> query, LogEntity entity)
        {
            if (entity.pagenumber > 1)
                query = query.Skip(entity.pagesize * (entity.pagenumber - 1));

            return query;
        }
        private static IQueryable<error_log> Limit(IQueryable<error_log> query, LogEntity entity)
        {
            if (!entity.loadall)
                query = query.Take(entity.pagesize);

            return query;
        }

        public static string ProcessAction(List<LogEntity> list)
        {
            foreach (LogEntity entity in list)
            {
                if (entity.id > 0)
                {
                    switch (entity.actionstatus)
                    {
                        case "delete":
                            Delete(entity.id);
                            break;
                        case "deleteall":
                            Delete();
                            break;
                    }
                }
            }
            return "Operation performed successfully";
        }
    }
}