﻿using System;
using System.Linq;
using System.Collections.Generic;
using Jugnoon.Entity;
using Jugnoon.BLL;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;

/// <summary>
/// Summary description for FriendsBLL
/// </summary>
public class FriendsBLL
{
	public FriendsBLL()
	{
		//
		// TODO: Add constructor logic here
		//
	}

    // Note: Friend Section Important Terms

    // Status:
    // ........... 0: Allowed
    // ........... 1: Still not approved


    public static bool Add(string username, string friend_username, int status, string msg, string key, string reject_key)
    {
        using (var dbCtx = new mvcportalEntities())
        {
            dbCtx.Entry(new friend()
            {
                username = username,
                friend_username = friend_username,
                status = (byte)status,
                msg = msg,
                date_added = DateTime.Now,
                val_key = key,
                reject_key = reject_key
            }).State = System.Data.Entity.EntityState.Added;

            DataBLL.SaveChanges(dbCtx);
        }

        return true;
    }

   
    public static bool Update_Status(string username, string friend_username, int status)
    {
        using (var ctx = new mvcportalEntities())
        {
            var item = ctx.friends
                 .Where(p => p.username == username && p.friend_username == friend_username)
                 .FirstOrDefault<friend>();

            if(item != null)
            {
                item.status = (byte)status;

                ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                DataBLL.SaveChanges(ctx);
            }
            
        }

        if (status == 0)
        {
            // normal user
            int count = Convert.ToInt32(UserBLL.Return_Value(username, "stat_friends"));
            count++;
            UserBLL.Update_Field_V3(username, "stat_friends", count);
            // friend user
            count = Convert.ToInt32(UserBLL.Return_Value(friend_username, "stat_friends"));
            count++;
            UserBLL.Update_Field_V3(friend_username, "stat_friends", count);
        }
        return true;
    }


    public static bool Delete(string username, string friend_username)
    {
        using (var dbCtx = new mvcportalEntities())
        {
            var entity = new friend { username = username, friend_username = friend_username };
            dbCtx.friends.Attach(entity);
            dbCtx.friends.Remove(entity);
            dbCtx.SaveChanges();
        }
        
        int count = Convert.ToInt32(UserBLL.Return_Value(username, "stat_friends"));
        count--;
        if (count < 0)
            count = 0;
        UserBLL.Update_Field_V3(username, "stat_friends", count);
        return true;
    }
      
    
    
    public static bool isFriend(string username, string friendusername,int status)
    {
        bool flag = false;
        using (var ctx = new mvcportalEntities())
        {
            IQueryable<friend> query = ctx.friends.Where(p => p.username == username && p.friend_username == friendusername);
            if (status == 2)
                query = query.Where(p => p.status == 0);
            if (query.Count() > 0)
                flag = true;
        }
        
        return flag;
    }


    // Load Recent friends - top 6 friends
    public static List<FriendEntity> Fetch_User_Friends_SM(string username)
    {
        var _items = new List<FriendEntity>();
        using (var ctx = new mvcportalEntities())
        {
            _items = ctx.friends
                .Join(ctx.users,
                         m => m.friend_username,
                         v => v.username,
                         (m, v) => new { m, v })
                         .Where(x => x.m.username == username)
                         .Take(6)
                         .OrderByDescending(p => p.m.date_added)
                         .Select(p => new FriendEntity
                         {
                             friend_username = p.m.friend_username,
                             picturename = p.v.picturename
                         }).ToList();
        }
        return _items;
    }
                    /* 
                     
        // generate sql query
        StringBuilder str = new StringBuilder();
        string Query = "SELECT TOP 6 f.friend_username,u.picturename from friends as f inner join users as u on u.username=f.friend_username AND f.username=@username AND f.status=0 order by f.date_added desc";
        // QUERY PROCESSING LOGIC
        using (SqlConnection con = new SqlConnection(Config.ConnectionString))
        {            
            con.Open();
            using (SqlCommand cmd = new SqlCommand(Query, con))
            {
                cmd.CommandType = CommandType.Text;
                cmd.Parameters.Add(new SqlParameter("@username", username));
                using (SqlDataReader reader = cmd.ExecuteReader())
                {
                    while (reader.Read())
                    {
                        _item = new Friends_Struct();
                        _item.Friend_UserName = reader["friend_username"].ToString();
                        _item.PictureName = reader["picturename"].ToString();

                        _items.Add(_item);
                    }
                }
            }
        }

        return _items;
    }

    public static int RefreshStats(string username)
    {
        int Records = 0;
        using (SqlConnection con = new SqlConnection(Config.ConnectionString))
        {
            con.Open();

            string query = "SELECT count(*) from friends as f INNER JOIN Users as u ON u.username=f.friend_username where f.username=@username AND status=0";
            using (SqlCommand cmd = new SqlCommand(query.ToString(), con))
            {
                cmd.CommandType = CommandType.Text;
                cmd.Parameters.Add(new SqlParameter("@username", username));

                Records = Convert.ToInt32(cmd.ExecuteScalar());
            }
        }
             

        UserBLL.Update_Value(username, "stat_friends", Records.ToString());
        return Records;
    }

    
    // Load All User friends
    public static List<Friends_Struct> Fetch_User_Friends(string username,int PageNumber, int PageSize)
    {       
        List<Friends_Struct> _items = new List<Friends_Struct>();
        Friends_Struct _item;

        // generate sql query
        StringBuilder str = new StringBuilder();
        StringBuilder query = new StringBuilder();
        query.Append("f.friend_username,u.picturename from friends as f inner join users as u on u.username=f.friend_username AND f.username=@username AND f.status=0");
        // implement paging script for differnt data sources.
        switch (Config.Return_PaginationType())
        {
            case 0:
                // SQL SERVER 2005 or Later Supported Query
                str.Append(Pagination_Process.Prepare_SQLSERVER2005_Pagination(query.ToString(), "f.date_added desc", PageNumber, PageSize));
                break;
            case 1:
                // MySQL Supported Query
                string mysql_query = "SELECT " + query.ToString() + " ORDER BY f.date_added desc";
                str.Append(Pagination_Process.Prepare_MySQL_Pagination(mysql_query, PageNumber, PageSize));
                break;
        }
        // QUERY PROCESSING LOGIC
        using (SqlConnection con = new SqlConnection(Config.ConnectionString))
        {
            con.Open();
            using (SqlCommand cmd = new SqlCommand(str.ToString(), con))
            {
                cmd.CommandType = CommandType.Text;
                cmd.Parameters.Add(new SqlParameter("@username", username));
                using (SqlDataReader reader = cmd.ExecuteReader())
                {
                    while (reader.Read())
                    {
                        _item = new Friends_Struct();
                        _item.Friend_UserName = reader["friend_username"].ToString();
                        _item.PictureName = reader["picturename"].ToString();

                        _items.Add(_item);
                    }
                }
            }
        }
      
        return _items;
    }

    /* utility function check whether key and username is valid either with accept or reject key */
    public static bool is_Key_Valid(string username, string friendusername, string key, bool isAccept)
    {
        bool flag = false;
        using (var ctx = new mvcportalEntities())
        {
            var query = ctx.friends.Where(p => p.username == username && p.friend_username == friendusername);
            if (isAccept)
                query = query.Where(p => p.val_key == key);
            else
                query = query.Where(p => p.reject_key == key);

            if (query.Count() > 0)
                flag = true;

        }

        return flag;

    }

    public static List<FriendEntity> Fetch_Friend_Status(string profileusername, string loggedinusername)
    {
        var _items = new List<FriendEntity>();
        using (var ctx = new mvcportalEntities())
        {
            _items = ctx.friends.Where(p => p.username == profileusername && p.friend_username == loggedinusername)
                .Select(p => new FriendEntity()
                {
                    status = p.status
                }).ToList();
        }
        return _items;
    }


    public static bool Unfriend(string username, string friend_username)
    {
        using (var ctx = new mvcportalEntities())
        {
            ctx.friends.RemoveRange(ctx.friends.Where(x => x.username == username && x.friend_username == friend_username));
            ctx.SaveChanges();

            ctx.friends.RemoveRange(ctx.friends.Where(x => x.username == friend_username && x.friend_username == username));
            ctx.SaveChanges();
        }
                   
        /* username */
        int count = Convert.ToInt32(UserBLL.Return_Value(username, "stat_friends"));
        count--;
        if (count < 0)
            count = 0;
        UserBLL.Update_Field_V3(username, "stat_friends", count);
        /* friend username */
        count = Convert.ToInt32(UserBLL.Return_Value(friend_username, "stat_friends"));
        count--;
        if (count < 0)
            count = 0;
        UserBLL.Update_Field_V3(friend_username, "stat_friends", count);
        return true;
    }
}
