﻿using System;
using System.Web;
using System.Collections.Generic;
using System.Text;
using System.Web.Caching;
using System.Globalization;
/* Site Own Namespaces */
using Jugnoon.Entity;
using Jugnoon.Utility;
using Jugnoon.Settings;
using System.Linq;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;

namespace Jugnoon.BLL
{

    public class LanguageBLLC
    {
        public LanguageBLLC()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        // Note: Language Important Terms

        // isDefault:
        // ............ 0:- Normal Item
        // ............ 1:- Default Language (Only one can possible)

        // isSelected
        // ............ 0:- Normal Item
        // ............ 1:- Selected Language (Multiple options can be possible)
        public static string Return_Current_CultureName()
        {
            var culture = UtilityBLL.ReadCookie("CultureInfo");
            if (culture != null && culture != null)
                return culture;
            else
                return Site_Settings.Default_CultureInfo;
        }

        public static LanguageEntity Process(LanguageEntity lang)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                if (lang.languageid == 0)
                {
                    var entity = new language()
                    {
                        culturename = lang.culturename,
                        language1 = lang.language,
                        region = lang.region,
                        isselected = (byte)lang.isselected,
                        icon = lang.icon,
                        filename = lang.filename
                    };

                    dbCtx.Entry(entity).State = System.Data.Entity.EntityState.Added;

                    DataBLL.SaveChanges(dbCtx);

                    lang.languageid = entity.languageid;

                }
                else
                {
                    var item = dbCtx.languages
                    .Where(p => p.languageid == lang.languageid)
                    .FirstOrDefault<language>();

                    if(item != null)
                    {
                        item.culturename = lang.culturename;
                        item.language1 = lang.language;
                        item.region = lang.region;
                        item.isselected = (byte)lang.isselected;
                        item.icon = lang.icon;
                        item.filename = lang.filename;

                        DataBLL.SaveChanges(dbCtx);

                    }
                   
                }
            }
            return lang;
        }

        public static void Delete(int id)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new language { languageid = (short)id };
                dbCtx.languages.Attach(entity);
                dbCtx.languages.Remove(entity);
                dbCtx.SaveChanges();
            }
        }

        public static bool Update_Value(int id, string fieldname, dynamic Value)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.languages
                     .Where(p => p.languageid == id)
                     .FirstOrDefault<language>();

                if(item != null)
                {
                    foreach (var prop in item.GetType().GetProperties())
                    {
                        if (prop.Name.ToLower() == fieldname.ToLower())
                        {
                            prop.SetValue(item, Value);
                        }
                    }
                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
                
            }
            return true;
        }

        public static bool Reset_IsDefault()
        {
            using (var ctx = new mvcportalEntities())
            {
                var languages = ctx.languages.ToList();
                languages.ForEach(m => m.isdefault = 0);
                ctx.SaveChanges();
            }
            return true;
        }
        // load and cache culturename for fast retrieval
       /* public static string Return_CultureName(int languageid)
        {
            string _cache_key = "cach_culture_name" + languageid;
            if (HttpContext.Current.Cache[_cache_key] == null)
            {
                string culturename = Site_Settings.Default_CultureInfo; // retrieve default culture name in case of non exist
                if (Check(languageid))
                    culturename = rCultureName(languageid);

                HttpContext.Current.Cache.Add(_cache_key, culturename, null, DateTime.Now.AddMinutes(Config.GetCacheDuration()), TimeSpan.Zero, CacheItemPriority.High, null);
            }
            return HttpContext.Current.Cache[_cache_key].ToString();

        } */
        public static string Return_CultureName(string culture)
        {
            string _cache_key = "cach_culture_name" + culture;
            if (HttpContext.Current.Cache[_cache_key] == null)
            {
                string culturename = Site_Settings.Default_CultureInfo; // retrieve default culture name in case of non exist
                if (Check(culture))
                    culturename = rCultureName(culture);

                HttpContext.Current.Cache.Add(_cache_key, culturename, null, DateTime.Now.AddMinutes(Config.GetCacheDuration()), TimeSpan.Zero, CacheItemPriority.High, null);
            }
            return HttpContext.Current.Cache[_cache_key].ToString();
        }

       
        
        public static bool Check(string culture)
        {
            bool flag = false;
            using (var ctx = new mvcportalEntities())
            {
                if (ctx.languages.Where(p => p.culturename == culture).Count() > 0)
                    flag = true;
            }

            return flag;
        }

        public static string rCultureName(string culture)
        {
            string Value = "";
            using (var ctx = new mvcportalEntities())
            {
                Value = ctx.languages.Where(p => p.culturename == culture).Select(u => u.culturename).SingleOrDefault();
            }
            return Value;
        }
       
        // Return current user language from browser
        public static CultureInfo ResolveCulture()
        {
            string[] languages = HttpContext.Current.Request.UserLanguages;

            if (languages == null || languages.Length == 0)
                return null;

            try
            {
                string language = languages[0].ToLowerInvariant().Trim();
                return CultureInfo.CreateSpecificCulture(language);
            }
            catch (ArgumentException)
            {
                return null;
            }
        }

        // return current user country from browser
        public static RegionInfo ResolveCountry()
        {
            CultureInfo culture = ResolveCulture();
            if (culture != null)
                return new RegionInfo(culture.LCID);

            return null;
        }


        public static List<LanguageDataEntity> Load(LanguageEntity entity)
        {
            string cache_key = "lang_" + entity.languageid + "_" + entity.isselected + "_" + entity.isdefault;
            if (HttpContext.Current.Application[cache_key] == null)
                HttpContext.Current.Application.Add(cache_key, _Load(entity));

            return (List<LanguageDataEntity>)HttpContext.Current.Application[cache_key.ToString()];

        }
       
        private static List<LanguageDataEntity> _Load(LanguageEntity entity)
        {
            var _list = new List<LanguageDataEntity>();

            using (var ctx = new mvcportalEntities())
            {
                IQueryable<language> Query = ctx.languages;
                Query = BuildQuery(Query, entity);
           
                if (entity.languageid > 0)
                {
                    entity.loadall = true;

                    _list = LoadCompleteList(Query);
                }
                if (entity.issummary)
                {
                    _list = LoadSummaryList(Query);
                }
                else if (entity.isdropdown)
                {
                    _list = LoadDropdownList(Query);
                }
                else
                {
                    _list = LoadCompleteList(Query);
                }
            }

            return _list;
        }

        public static int Count(LanguageEntity entity)
        {
            if (!entity.iscache || Site_Settings.Cache_Duration == 0 || entity.pagenumber > 5)
            {
                return CountRecords(entity);
            }
            else
            {
                string key = GenerateKey("cnt_language", entity);
                if (HttpContext.Current.Cache[key] == null)
                {
                    int total_records = CountRecords(entity);

                    HttpContext.Current.Cache.Add(key, CountRecords(entity), null, DateTime.Now.AddMinutes(Site_Settings.Cache_Duration), TimeSpan.Zero, CacheItemPriority.High, null);
                }
                return Convert.ToInt32(HttpContext.Current.Cache[key]);
            }
        }

        private static int CountRecords(LanguageEntity entity)
        {
            int Records = 0;
            using (var entities = new mvcportalEntities())
            {
                IQueryable<language> Query = entities.languages;
                entity.iscount = true;
                entity.languageid = 0;
                Query = BuildQuery(Query, entity);
                Records = Query.Count();
            }
            return Records;
        }
        private static string GenerateKey(string key, LanguageEntity entity)
        {
            StringBuilder str = new StringBuilder();
            return key + entity.datefilter + "" + entity.filter + "" + UtilityBLL.ReplaceSpaceWithHyphin(entity.order.ToLower()) + "" + entity.pagenumber + "" + entity.term;
        }

        private static IQueryable<language> BuildQuery(IQueryable<language> query, LanguageEntity entity)
        {
            query = Where(query, entity);
            if (entity.languageid == 0 && !entity.iscount)
            {
                query = OrderBy(query, entity);
                query = Skip(query, entity);
                query = Limit(query, entity);
            }
            return query;
        }

        public static List<LanguageDataEntity> LoadCompleteList(IQueryable<language> query)
        {
            return query.Select(p => new LanguageDataEntity
            {
                languageid = (short)p.languageid,
                culturename = p.culturename,
                language1 = p.language1,
                region = p.region,
                isdefault = p.isdefault,
                isselected = p.isselected,
                icon = p.icon,
                filename = p.filename
            }).ToList();
        }
        public static List<LanguageDataEntity> LoadSummaryList(IQueryable<language> query)
        {
            return query.Select(p => new LanguageDataEntity
            {
                languageid = (short)p.languageid,
                culturename = p.culturename,
                language1 = p.language1,
                region = p.region,
                isdefault = p.isdefault,
                isselected = p.isselected,
                icon = p.icon,
                filename = p.filename
            }).ToList();
        }
        public static List<LanguageDataEntity> LoadDropdownList(IQueryable<language> query)
        {
            return query.Select(p => new LanguageDataEntity
            {
                languageid = (short)p.languageid,
                culturename = p.culturename,
                language1 = p.language1
               
            }).ToList();
        }

        private static IQueryable<language> Where(IQueryable<language> query, LanguageEntity entity)
        {
            if (entity.languageid > 0)
                query = query.Where(p => p.languageid == entity.languageid);
          
            if (entity.isselected != 2)
                query = query.Where(p => p.isselected == entity.isselected);
            if (entity.isdefault != 2)
                query = query.Where(p => p.isdefault == entity.isdefault);
            if (entity.term != "")
                query = query.Where(p => p.culturename.Contains(entity.term) || p.language1.Contains(entity.term) || p.region.Contains(entity.term));

            return query;
        }
        private static IQueryable<language> OrderBy(IQueryable<language> query, LanguageEntity entity)
        {
            if (entity.order != "")
            {
                var orderlist = entity.order.Split(char.Parse(","));
                foreach (var order in orderlist)
                {
                    if (order.Contains("languageid"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.languageid);
                        else
                            query = query.OrderBy(p => p.languageid);
                    }
                    else if (order.Contains("isselected"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.isselected);
                        else
                            query = query.OrderBy(p => p.isselected);
                    }
                    else if (order.Contains("culturename"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.culturename);
                        else
                            query = query.OrderBy(p => p.culturename);
                    }
                }
            }

            return query;
        }
        private static IQueryable<language> Skip(IQueryable<language> query, LanguageEntity entity)
        {
            if (entity.pagenumber > 1)
                query = query.Skip(entity.pagesize * (entity.pagenumber - 1));

            return query;
        }
        private static IQueryable<language> Limit(IQueryable<language> query, LanguageEntity entity)
        {
            if (!entity.loadall)
                query = query.Take(entity.pagesize);

            return query;
        }

        public static string ProcessAction(List<LanguageEntity> list)
        {
            foreach (var entity in list)
            {
                if (entity.languageid > 0)
                {
                    switch (entity.actionstatus)
                    {
                        case "default":
                            Reset_IsDefault();
                            Update_Value(entity.languageid, "isdefault", (byte)1);
                            break;
                        case "selected":
                            // toggle selection
                            if (entity.isselected == 1)
                                entity.isselected = 0;
                            else
                                entity.isselected = 1;

                            Update_Value(entity.languageid, "isselected", (byte)entity.isselected);
                            break;
                       
                        case "delete":
                            Delete(entity.languageid);

                            break;
                    }
                }
            }
            return "Operation performed successfully";
        }

        public static string returnFlagCss(string culturename, List<LanguageDataEntity> _lst)
        {
            var css = "flag-us";
            switch(culturename)
            {
                case "en":
                    css = "flag-us";
                    break;
                case "en-US":
                    css = "flag-us";
                    break;
                case "ar-SA":
                    css = "flag-sa";
                    break;
                case "de-DE":
                    css = "flag-de";
                    break;
                case "es-ES":
                    css = "flag-es";
                    break;
                case "fr-FR":
                    css = "flag-fr";
                    break;
                case "it-IT":
                    css = "flag-it";
                    break;
                case "ja-JP":
                    css = "flag-jp";
                    break;
                case "pt-BR":
                    css = "flag-br";
                    break;
                case "ru-RU":
                    css = "flag-ru";
                    break;
                case "tr-TR":
                    css = "flag-tr";
                    break;
                case "zh-CHS":
                    css = "flag-ch";
                    break;
            }
            return css;
        }

        public static string returnLanguage(string culturename, List<LanguageDataEntity> _lst)
        {
            var value = "English";
            if (culturename.Contains("en"))
                return value;

            foreach (var item in _lst)
            {
                if (item.culturename.Contains(culturename))
                {
                    value = item.language1;
                }
            }
            return value;
        }
    }

  
}