﻿using System;
using System.Web;
using System.Collections.Generic;
using System.Text;
/* Site Own Namespaces */
using Jugnoon.Entity;
using Jugnoon.Utility;
using Jugnoon.Settings;
using System.Linq;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;
using System.Web.Caching;

namespace Jugnoon.BLL
{
    public class MailTemplateBLL
    {
        public MailTemplateBLL()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        public static MailTemplateEntity Add(MailTemplateEntity entity) //string TemplateKey, string Description, string subjecttags, string tags, string subject, string contents, string type)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var _entity = new mailtemplate()
                {
                    templatekey = entity.templatekey,
                    description = entity.description,
                    subjecttags = entity.subjecttags,
                    tags = entity.tags,
                    subject = entity.subject,
                    contents = entity.contents,
                    type = entity.type
                };

                dbCtx.Entry(_entity).State = System.Data.Entity.EntityState.Added;

                DataBLL.SaveChanges(dbCtx);

                entity.id = _entity.id;
            }
           
            return entity;
        }

        public static bool Validate_TemplateKey(string templatekey)
        {

            bool flag = false;
            using (var ctx = new mvcportalEntities())
            {
                if (ctx.mailtemplates.Where(p => p.templatekey == templatekey).Count() > 0)
                    flag = true;
            }

            return flag;
        }
    
        public static List<MailTemplateEntity> Get_Record(string templatekey)
        {
            if (HttpContext.Current.Cache["ld_mailtemplates_" + templatekey] == null)
            {
                HttpContext.Current.Cache["ld_mailtemplates_" + templatekey] = Fetch_Record(templatekey);
            }
            return (List<MailTemplateEntity>)HttpContext.Current.Cache["ld_mailtemplates_" + templatekey];
        }

        public static List<MailTemplateEntity> Fetch_Record(string templatekey)
        {
            var items = new List<MailTemplateEntity>();
            using (var ctx = new mvcportalEntities())
            {
                items = ctx.mailtemplates
                     .Where(p => p.templatekey == templatekey)
                     .Select(p => new MailTemplateEntity()
                     {
                         subject = p.subject,
                         contents = p.contents
                     }).ToList();
            }
            
            return items;
        }

        public static bool Update_Record(int id, string subject, string description, string contents, string tags, string subjecttags)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.mailtemplates
                     .Where(p => p.id == id)
                     .FirstOrDefault<mailtemplate>();

                if(item != null)
                {
                    item.subject = UtilityBLL.processNull(subject, 0);
                    item.description = UtilityBLL.processNull(description, 100);
                    item.contents = UtilityBLL.processNull(contents, 0);
                    item.tags = tags;
                    item.subjecttags = subjecttags;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
                
            }
           
            return true;
        }


        public static List<MailTemplateDataEntity> Load(MailTemplateEntity entity)
        {
            var _list = new List<MailTemplateDataEntity>();

            using (var ctx = new mvcportalEntities())
            {
                IQueryable<mailtemplate> Query = ctx.mailtemplates;
                Query = BuildQuery(Query, entity);
            
                if (entity.id > 0)
                {
                    entity.loadall = true;

                    _list = LoadCompleteList(Query);
                }
                if (entity.issummary)
                {
                    _list = LoadSummaryList(Query);
                }
                else if (entity.isdropdown)
                {
                    _list = LoadDropdownList(Query);
                }
                else
                {
                    _list = LoadCompleteList(Query);
                }
            }

            return _list;
        }

        public static int Count(MailTemplateEntity entity)
        {
            if (!entity.iscache || Site_Settings.Cache_Duration == 0 || entity.pagenumber > 5)
            {
                return CountRecords(entity);
            }
            else
            {
                string key = GenerateKey("cnt_mailtemplate", entity);
                if (HttpContext.Current.Cache[key] == null)
                {
                    int total_records = CountRecords(entity);

                    HttpContext.Current.Cache.Add(key, CountRecords(entity), null, DateTime.Now.AddMinutes(Site_Settings.Cache_Duration), TimeSpan.Zero, CacheItemPriority.High, null);
                }
                return Convert.ToInt32(HttpContext.Current.Cache[key]);
            }
        }
        public static bool Delete(short id)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new mailtemplate { id = id };
                dbCtx.mailtemplates.Attach(entity);
                dbCtx.mailtemplates.Remove(entity);
                dbCtx.SaveChanges();
            }
          

            return true;
        }
        private static int CountRecords(MailTemplateEntity entity)
        {
            int Records = 0;
            using (var entities = new mvcportalEntities())
            {
               IQueryable<mailtemplate> Query = entities.mailtemplates;
                entity.iscount = true;
                entity.id = 0;
                Query = BuildQuery(Query, entity);
                Records = Query.Count();
            }
            return Records;
        }
        private static string GenerateKey(string key, MailTemplateEntity entity)
        {
            StringBuilder str = new StringBuilder();
            return key + entity.datefilter + "" + entity.filter + "" + UtilityBLL.ReplaceSpaceWithHyphin(entity.order.ToLower()) + "" + entity.pagenumber + "" + entity.term;
        }

        private static IQueryable<mailtemplate> BuildQuery(IQueryable<mailtemplate> query, MailTemplateEntity entity)
        {
            query = Where(query, entity);
            if (entity.id == 0 && !entity.iscount)
            {
                query = OrderBy(query, entity);
                query = Skip(query, entity);
                query = Limit(query, entity);
            }
            return query;
        }

        public static List<MailTemplateDataEntity> LoadCompleteList(IQueryable<mailtemplate> query)
        {
            return query.Select(p => new MailTemplateDataEntity
            {
                id = (int)p.id,
                templatekey = p.templatekey,
                description = p.description,
                tags = p.tags,
                subjecttags = p.subjecttags,
                subject = p.subject,
                contents = p.contents,
                type = p.type
            }).ToList();
        }

        public static List<MailTemplateDataEntity> LoadSummaryList(IQueryable<mailtemplate> query)
        {
            return query.Select(p => new MailTemplateDataEntity
            {
                id = (int)p.id,
                templatekey = p.templatekey,
                description = p.description,
                tags = p.tags,
                subjecttags = p.subjecttags,
                subject = p.subject,
                contents = p.contents,
                type = p.type
            }).ToList();
        }

        public static List<MailTemplateDataEntity> LoadDropdownList(IQueryable<mailtemplate> query)
        {
            return query.Select(p => new MailTemplateDataEntity
            {
                id = (int)p.id,
                subject = p.subject

            }).ToList();
        }

        private static IQueryable<mailtemplate> Where(IQueryable<mailtemplate> query, MailTemplateEntity entity)
        {
            if (entity.id > 0)
                query = query.Where(p => p.id == entity.id);
                        
            if (entity.term != "")
                query = query.Where(p => p.subject.Contains(entity.term) || p.templatekey.Contains(entity.term) || p.description.Contains(entity.term) || p.contents.Contains(entity.term));

            if (entity.templatekey != "")
                query = query.Where(p => p.templatekey == entity.templatekey);

            if (entity.type != "-1" && entity.id == 0)
                query = query.Where(p => p.type == entity.type);

            return query;
        }

        private static IQueryable<mailtemplate> OrderBy(IQueryable<mailtemplate> query, MailTemplateEntity entity)
        {
            if (entity.order != "")
            {
                var orderlist = entity.order.Split(char.Parse(","));
                foreach (var order in orderlist)
                {
                    if (order.Contains("id"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.id);
                        else
                            query = query.OrderBy(p => p.id);
                    }
                    else if (order.Contains("templatekey"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.templatekey);
                        else
                            query = query.OrderBy(p => p.templatekey);
                    }
                    else if (order.Contains("subject"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.subject);
                        else
                            query = query.OrderBy(p => p.subject);
                    }
                }
            }

            return query;
        }

        private static IQueryable<mailtemplate> Skip(IQueryable<mailtemplate> query, MailTemplateEntity entity)
        {
            if (entity.pagenumber > 1)
                query = query.Skip(entity.pagesize * (entity.pagenumber - 1));

            return query;
        }

        private static IQueryable<mailtemplate> Limit(IQueryable<mailtemplate> query, MailTemplateEntity entity)
        {
            if (!entity.loadall)
                query = query.Take(entity.pagesize);

            return query;
        }

        public static bool CheckTemplate(string key)
        {
            var Flag = false;
            using (var dbCtx = new mvcportalEntities())
            {
                var count = dbCtx.mailtemplates
                  .Where(o => o.templatekey == key)
                  .Count();

                if (count > 0)
                    Flag = true;
                else
                    Flag = false;

            }
            return Flag;

        }

        public static string ProcessAction(List<MailTemplateEntity> list)
        {
            foreach (MailTemplateEntity entity in list)
            {
                if (entity.id > 0)
                {
                    switch (entity.actionstatus)
                    {

                        case "delete":
                            Delete(Convert.ToInt16(entity.id));

                            break;
                    }
                }
            }
            return "Operation performed successfully";
        }
    }

   
}