﻿using System.Collections.Generic;
/* Site Own Namespaces */
using Jugnoon.Utility;
using System.Linq;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;
using Jugnoon.Entity;

namespace Jugnoon.BLL
{
    public class RoleObjectBLL
    {
        public static RoleDataObjectEntity Add(RoleDataObjectEntity entity)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var _entity = new vsk_roleobjects()
                {
                    objectname = entity.objectname,
                    description = entity.description,
                    uniqueid = entity.uniqueid
                };

                dbCtx.Entry(_entity).State = System.Data.Entity.EntityState.Added;
                entity.id = _entity.id;
                DataBLL.SaveChanges(dbCtx);

            }

            return entity;
        }
        public static bool Update_Record(RoleDataObjectEntity entity)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.vsk_roleobjects
                     .Where(p => p.id == entity.id)
                     .FirstOrDefault<vsk_roleobjects>();

                if (item != null)
                {
                    item.objectname = UtilityBLL.processNull(entity.objectname, 0);
                    item.description = UtilityBLL.processNull(entity.description, 0);
                   
                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }

            }

            return true;
        }
        public static bool Delete(short objectid)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new vsk_roleobjects { id = objectid };
                dbCtx.vsk_roleobjects.Attach(entity);
                dbCtx.vsk_roleobjects.Remove(entity);
                // dbCtx.SaveChanges();
            }

            return true;
        }

        public static string ProcessAction(List<RoleObject> list)
        {
            foreach (var entity in list)
            {
                if (entity.id > 0)
                {
                    switch (entity.actionstatus)
                    {
                        case "delete":
                            Delete((short)entity.id);
                            break;
                    }
                }
            }
            return "Operation performed successfully";
        }

        public static List<RoleDataObjectEntity> LoadItems(RoleObject entity)
        {
            var _list = new List<RoleDataObjectEntity>();

            using (var ctx = new mvcportalEntities())
            {
                IQueryable<vsk_roleobjects> Query = ctx.vsk_roleobjects;
                Query = BuildQuery(Query, entity);

                _list = LoadCompleteList(Query);
            }

            return _list;
        }


        public static int Count(RoleObject entity)
        {
            int Records = 0;
            using (var entities = new mvcportalEntities())
            {
                IQueryable<vsk_roleobjects> Query = entities.vsk_roleobjects;
                entity.iscount = true;
                entity.id = 0;
                Query = BuildQuery(Query, entity);
                Records = Query.Count();
            }
            return Records;
        }

        private static IQueryable<vsk_roleobjects> BuildQuery(IQueryable<vsk_roleobjects> query, RoleObject entity)
        {
            query = Where(query, entity);
            if (entity.id == 0 && !entity.iscount)
            {
                query = OrderBy(query, entity);
                query = Skip(query, entity);
                query = Limit(query, entity);
            }
            return query;
        }

        public static List<RoleDataObjectEntity> LoadCompleteList(IQueryable<vsk_roleobjects> query)
        {
            return query.Select(p => new RoleDataObjectEntity
            {
                id = p.id,
                objectname = p.objectname,
                description = p.description,
                uniqueid = p.uniqueid
            }).ToList();
        }

        private static IQueryable<vsk_roleobjects> Where(IQueryable<vsk_roleobjects> query, RoleObject entity)
        {
            if (entity.id > 0)
                query = query.Where(p => p.id == entity.id);

            if (entity.term != "")
                query = query.Where(p => p.objectname.Contains(entity.term) || p.objectname.Contains(entity.term));

            return query;
        }

        private static IQueryable<vsk_roleobjects> OrderBy(IQueryable<vsk_roleobjects> query, RoleObject entity)
        {
            if (entity.order != "")
            {
                var orderlist = entity.order.Split(char.Parse(","));
                foreach (var order in orderlist)
                {
                    if (order.Contains("id"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.id);
                        else
                            query = query.OrderBy(p => p.id);
                    }
                }
            }

            return query;
        }

        private static IQueryable<vsk_roleobjects> Skip(IQueryable<vsk_roleobjects> query, RoleObject entity)
        {
            if (entity.pagenumber > 1)
                query = query.Skip(entity.pagesize * (entity.pagenumber - 1));

            return query;
        }

        private static IQueryable<vsk_roleobjects> Limit(IQueryable<vsk_roleobjects> query, RoleObject entity)
        {
            if (!entity.loadall)
                query = query.Take(entity.pagesize);

            return query;
        }
    }
}