﻿using System.Collections.Generic;
using System.Linq;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;
using Jugnoon.Entity;

namespace Jugnoon.BLL
{
    public class RolePermission
    {
        public static bool Add(RoleDataPermissionEntity entity)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var _entity = new vsk_rolepermissions()
                {
                    roleid = entity.roleid,
                    objectid = entity.objectid
                };

                dbCtx.Entry(_entity).State = System.Data.Entity.EntityState.Added;

                DataBLL.SaveChanges(dbCtx);

            }

            return true;
        }

        public static bool Delete(short objectid)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new vsk_rolepermissions { id = objectid };
                dbCtx.vsk_rolepermissions.Attach(entity);
                dbCtx.vsk_rolepermissions.Remove(entity);
                dbCtx.SaveChanges();
            }

            return true;
        }
        public static void DeleteRole(short roleid)
        {            
            using (var dbCtx = new mvcportalEntities())
            {
                dbCtx.vsk_rolepermissions.RemoveRange(dbCtx.vsk_rolepermissions.Where(x => x.roleid == roleid));
                dbCtx.SaveChanges();
            }

        }
        public static string ProcessAction(List<RoleDPermissionEntity> list)
        {
            foreach (var entity in list)
            {
                if (entity.id > 0)
                {
                    switch (entity.actionstatus)
                    {
                        case "delete":
                            Delete((short)entity.id);
                            break;
                    }
                }
            }
            return "Operation performed successfully";
        }

        public static List<RoleDataPermissionEntity> LoadItems(RoleDPermissionEntity entity)
        {
            var _list = new List<RoleDataPermissionEntity>();

            using (var ctx = new mvcportalEntities())
            {
                IQueryable<RoleObjectPermission> Query = ctx.vsk_rolepermissions
                    .Join(ctx.vsk_roleobjects,
                         permission => permission.objectid,
                         robject => robject.id,
                         (permission, robject) => new RoleObjectPermission
                         {
                             permission = permission,
                             objects = robject
                         });
                         Query = BuildQuery(Query, entity);
                         _list = LoadCompleteList(Query);
            }
            return _list;
        }


        public static int Count(RoleDPermissionEntity entity)
        {
            int Records = 0;
            using (var ctx = new mvcportalEntities())
            {
                IQueryable<RoleObjectPermission> Query = ctx.vsk_rolepermissions
                    .Join(ctx.vsk_roleobjects,
                         permission => permission.objectid,
                         robject => robject.id,
                         (permission, robject) => new RoleObjectPermission
                         {
                             permission = permission,
                             objects = robject
                         });
                entity.iscount = true;
                entity.id = 0;
                Query = BuildQuery(Query, entity);
                Records = Query.Count();
            }
            return Records;
        }

        private static IQueryable<RoleObjectPermission> BuildQuery(IQueryable<RoleObjectPermission> query, RoleDPermissionEntity entity)
        {
            query = Where(query, entity);
            if (entity.id == 0 && !entity.iscount)
            {
                query = OrderBy(query, entity);
                query = Skip(query, entity);
                query = Limit(query, entity);
            }
            return query;
        }

        public static List<RoleDataPermissionEntity> LoadCompleteList(IQueryable<RoleObjectPermission> query)
        {
            return query.Select(p => new RoleDataPermissionEntity
            {
                id = p.permission.id,
                roleid = p.permission.roleid,
                objectid = p.permission.objectid,
                robject = new RoleDataObjectEntity
                {
                    id = p.objects.id,
                    objectname = p.objects.objectname,
                    description = p.objects.description,
                    uniqueid = p.objects.uniqueid
                }
            }).ToList();
        }

        private static IQueryable<RoleObjectPermission> Where(IQueryable<RoleObjectPermission> query, RoleDPermissionEntity entity)
        {
            if (entity.id > 0)
                query = query.Where(p => p.permission.id == entity.id);

            if (entity.roleid > 0)
                query = query.Where(p => p.permission.roleid == entity.roleid);

            return query;
        }

        private static IQueryable<RoleObjectPermission> OrderBy(IQueryable<RoleObjectPermission> query, RoleDPermissionEntity entity)
        {
            if (entity.order != "")
            {
                var orderlist = entity.order.Split(char.Parse(","));
                foreach (var order in orderlist)
                {
                    if (order.Contains("id"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.permission.id);
                        else
                            query = query.OrderBy(p => p.permission.id);
                    }
                }
            }

            return query;
        }

        private static IQueryable<RoleObjectPermission> Skip(IQueryable<RoleObjectPermission> query, RoleDPermissionEntity entity)
        {
            if (entity.pagenumber > 1)
                query = query.Skip(entity.pagesize * (entity.pagenumber - 1));

            return query;
        }

        private static IQueryable<RoleObjectPermission> Limit(IQueryable<RoleObjectPermission> query, RoleDPermissionEntity entity)
        {
            if (!entity.loadall)
                query = query.Take(entity.pagesize);

            return query;
        }
    }

    public class RoleObjectPermission
    {
        public vsk_rolepermissions permission { get; set; }
        public vsk_roleobjects objects { get; set; }
    }
}