using System;
using System.Web;
using System.Text;
using System.Collections.Generic;
using System.Web.Caching;
/* Site Own Namespaces */
using Jugnoon.Entity;
using Jugnoon.Utility;
using Jugnoon.Settings;
using System.Linq;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;

namespace Jugnoon.BLL
{
    public class TagsBLL
    {
        private UtilityBLL _utility = new UtilityBLL();


        // Note: tags Important Terms

        // Status:
        // ........... 0: Tag is inactive
        // ........... 1: Tag is active
        // ............2: All

        // Type:
        // ........... 0: Videos
        // ........... 1: Groups
        // ........... 2: Photos | Galleries
        // ........... 3: Blogs
        // ........... 4: Audio Files
        // ........... 5: Generalize
        // ........... 6: Galleries
        // .......... 7: Forums
        // .......... 8: Video Album
        // ......... 9: Audio Album
        // .......... 11: Q&A
        // ......... 100: All
        // Tag Type:
        //............. 0: normal tags
        // ............ 1: user searches
        // ............ 2: all
        // Tag Level
        // ............ 0: high
        // ............ 1: medium
        // ............ 2: low
        // ............ 100: all
        public enum Types
        {
            Videos = 0,
            Groups = 1,
            Photos = 2,
            Blogs = 4,
            Audio = 4,
            General = 5,
            Galleries = 6,
            Forums = 7,
            VideoAlbum = 8,
            AudioAlbum = 9,
            QA= 11,
            All = 100,
            Poll = 15,
            Artist = 16
        };
        public enum TagType
        {
            Normal = 0,
            UserSearches = 1,
            All = 2
        };
        public enum TagLevel
        {
            High = 0,
            Medium = 1,
            Low = 2,
            All = 100
        };
        public TagsBLL()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        public static bool Add(string tagname, int type, int records, string term)
        {
            if (tagname == null || tagname.Length < 3)
                return false;
            
            using (var dbCtx = new mvcportalEntities())
            {
                var _entity = new tag()
                {
                    tagname = tagname,
                    type = (byte)type,
                    records = records,
                    term = term
                };

                dbCtx.Entry(_entity).State = System.Data.Entity.EntityState.Added;

                DataBLL.SaveChanges(dbCtx);

            }
            
            return true;
        }

        public static bool Update(TagsDataEntity entity)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.tags
                     .Where(p => p.tagid == entity.tagid)
                     .FirstOrDefault<tag>();

                if(item != null)
                {
                    item.tagname = UtilityBLL.processNull(entity.tagname, 0);
                    item.status = entity.status;
                    item.tag_level = entity.tag_level;
                    item.tag_type = entity.tag_type;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }

            return true;
        }

        public static bool Update_Field_V3(long ID, dynamic Value, string FieldName)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.tags
                     .Where(p => p.tagid == ID)
                     .FirstOrDefault<tag>();

                if(item != null)
                {
                    foreach (var prop in item.GetType().GetProperties())
                    {
                        if (prop.Name.ToLower() == FieldName.ToLower())
                        {
                            prop.SetValue(item, Value);
                        }
                    }
                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
                
            }
            return true;
        }

        public static string ProcessAction(List<TagEntity> list)
        {
            foreach (var entity in list)
            {
                if (entity.tagid > 0)
                {
                    switch (entity.actionstatus)
                    {
                        case "disable":
                            Update_Field_V3(entity.tagid, (byte)0, "status");
                            break;
                        case "enable":
                            Update_Field_V3(entity.tagid, (byte)1, "status");
                            break;
                        case "togglestatus":
                            Update_Field_V3(entity.tagid, (byte)entity.status, "status");
                            break;
                        case "high":
                            Update_Field_V3(entity.tagid, (byte)0, "tag_level");
                            break;
                        case "medium":
                            Update_Field_V3(entity.tagid, (byte)1, "tag_level");
                            break;
                        case "low":
                            Update_Field_V3(entity.tagid, (byte)2, "tag_level");
                            break;
                        case "delete":
                            Delete(entity.tagid);
                            break;
                    }
                }
            }
            return "Operation performed successfully";
        }

        public static bool Add(string tagname, int type, int records, int tag_type, int status, string term)
        {
            if (tagname == null || tagname.Length < 3)
                return false;

            using (var dbCtx = new mvcportalEntities())
            {
                var _entity = new tag()
                {
                    tagname = tagname,
                    type = (byte)type,
                    records = records,
                    tag_type = (byte)tag_type,
                    status = (byte)status,
                    term = term
                };

                dbCtx.Entry(_entity).State = System.Data.Entity.EntityState.Added;

                DataBLL.SaveChanges(dbCtx);

            }

            return true;
        }


        public static bool Delete(int tagid)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new tag { tagid = tagid };
                dbCtx.tags.Attach(entity);
                dbCtx.tags.Remove(entity);
                dbCtx.SaveChanges();
            }
            
            return true;
        }

        public static bool Delete_Inactive()
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new tag { status = 0 };
                dbCtx.tags.Attach(entity);
                dbCtx.tags.Remove(entity);
                dbCtx.SaveChanges();
            }
            
            return true;
        }
 
        public static bool Update_Tag_Status(int tagid, string tagname, int status, int tag_level, int priority)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.tags
                     .Where(p => p.tagid == tagid)
                     .FirstOrDefault<tag>();

                if(item != null)
                {
                    item.tagname = UtilityBLL.processNull(tagname, 0);
                    item.status = (byte)status;
                    item.tag_level = (byte)tag_level;
                    item.periority = (byte)priority;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
                
            }

            return true;
        }

        public static bool Update_Tag_Level(int tagid, int status, int tag_level)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.tags
                     .Where(p => p.tagid == tagid)
                     .FirstOrDefault<tag>();
                            
                if(item != null)
                {
                    item.status = (byte)status;
                    item.tag_level = (byte)tag_level;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);

                }
                
            }
          
            return true;
        }

        public static bool Update_Records(int tagid, int records)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.tags
                     .Where(p => p.tagid == tagid)
                     .FirstOrDefault<tag>();

                if(item != null)
                {
                    item.records = records;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }
   
            return true;
        }
        
        // update total no of records available for tag.
        public static bool Process_Records(string tagname, int type)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.tags
                     .Where(p => p.tagname == tagname && p.type == type)
                     .FirstOrDefault<tag>();

                if(item != null)
                {
                    item.records = TagsBLL.Count_Tags(tagname, type);

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
                
            }

            return true;
        }
           
        public static bool Activate_All_Tags(int type)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.tags
                     .Where(p => p.status == 0 && p.type == type)
                     .FirstOrDefault<tag>();

                if(item != null)
                {
                    item.status = 1;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }
            
            return true;
        }
        
        public static int Count_Tags(string tagname, int type)
        {
            int Records = 0;
            using (var ctx = new mvcportalEntities())
            {
                switch (type)
                {
                    case 0:
                        // videos
                        Records = ctx.videos.Where(p => p.tags.Contains(tagname) && p.isprivate == 0 && p.isenabled == 1 && p.ispublished == 1 && p.isreviewed == 1).Count();
                        break;
                    case 3:
                        // blogs
                        Records = ctx.blogs.Where(p => p.tags.Contains(tagname) && p.isdisabled == 1 && p.isapproved == 1).Count();
                        break;
                    case 6:
                        // galleries
                        Records = ctx.galleries.Where(p => p.tags.Contains(tagname) && p.isdisabled == 1 && p.isapproved == 1).Count();
                        break;
                }
            }
            return Records;
        }

        // Core function to process content related tags
        public static void Process_Tags(string tags, int type, int tag_type)
        {
            if(tags != null)
            {
                // process tags
                var tagItems = tags.ToString().Split(char.Parse(","));
                foreach(var tag in tagItems)
                {
                    if(tag.Length > 3)
                    {
                        // if tag not already exist
                        string term = UtilityBLL.CleanSearchTerm(tag.Trim());
                        if (Count(new TagEntity()
                        {
                            tagname = term,
                            tag_level = 100, // skip filter
                            type = type,
                            status = 2, // skip filter
                            tag_type = tag_type
                        }) == 0)
                        {
                            // insert each tag into tags table
                            int records = 1; // show one record in start by selected tag
                            if (tag_type == 1)
                                Add(tag.Trim(), type, records, tag_type, 1, term);
                            else
                                Add(tag.Trim(), type, records, term);
                        }
                        else
                        {
                            // tag exist - update tag records
                            // Due to performance issue, don't enable it on front line. update tag statistics from control panel
                            // Process_Records(arr[j].ToString(), type);
                        }
                    }
                }
                
            }
            
        }
        
        // Script to check whether each tag user input has exceed maximum limit of 35 characters.
        public static bool Validate_Tags(string tags)
        {
            if (tags == null)
                return true; // skip validation

            bool flag = true;
            // process tags
            if (tags.Contains(","))
            {
                string[] arr;
                arr = tags.ToString().Split(char.Parse(","));
                int j = 0;
                for (j = 0; j <= arr.Length - 1; j++)
                {
                    // if tag length greater than 20 characters
                    if (arr[j].Length > 35)
                        flag = false;
                }
            }
            else
            {
                if (tags.Length > 35)
                    flag = false;
            }
            return flag;
        }

        // load tags for autocomplete
        /* public static string Load_Tags_AutoComplete(string term)
         {

             List<TagEntity> _items = new List<TagEntity>();
             StringBuilder suggestions = new StringBuilder();
             StringBuilder data = new StringBuilder();
             // generate sql query
             string query = "";
             // Mysql compatible
             //query = "SELECT Distinct tagname from tags where status=1 AND tagname like @term ORDER BY records DESC LIMIT 10";
             // SQL SERVER compatible
             query = "SELECT Distinct TOP 10 tagname,records from tags where status=1 AND tagname like @term ORDER BY records DESC";
             // execute sql query
             // QUERY PROCESSING LOGIC
             using (var con = new SqlConnection(Config.ConnectionString))
             {
                 con.Open();
                 using (SqlCommand cmd = new SqlCommand(query, con))
                 {
                     cmd.CommandType = CommandType.Text;
                     cmd.Parameters.Add(new SqlParameter("@term", term + '%'));
                     using (var reader = cmd.ExecuteReader())
                     {

                         int counter = 0;
                         suggestions.Append("suggestions:[");
                         data.Append("data:[");
                         while (reader.Read())
                         {
                             if (counter == 0)
                             {
                                 //suggestions.Append("'" + reader["tagname"] + " - " + reader["records"] + " records'");
                                 suggestions.Append("'" + reader["tagname"] + "'");
                                 data.Append("'" + reader["tagname"] + "'");
                             }
                             else
                             {
                                 // suggestions.Append(",'" + reader["tagname"] + " - " + reader["records"] + " records'");
                                 suggestions.Append(",'" + reader["tagname"] + "'");
                                 data.Append(",'" + reader["tagname"] + "'");
                             }
                             counter = counter + 1;
                         }
                         suggestions.Append("]");
                         data.Append("]");
                     }
                 }
             }
             StringBuilder jsonoutput = new StringBuilder();
             jsonoutput.Append("{query:'" + term + "'," + suggestions.ToString() + "," + data.ToString() + "}");

             return jsonoutput.ToString();
         } */

        // Fetch tags for internal content processing
        /*public static List<TagEntity> Fetch_Content_Tags(int type)
        {

            List<TagEntity> _items = new List<TagEntity>();
            TagEntity _item;

            // generate sql query
            var str = new StringBuilder();
            str.AppendLine("SELECT tagname from tags where status=1 AND tag_type=0 AND type=@type ORDER BY tagid desc");
            // QUERY PROCESSING LOGIC
            using (var con = new SqlConnection(Config.ConnectionString))
            {
                con.Open();
                using (var cmd = new SqlCommand(str.ToString(), con))
                {
                    cmd.CommandType = CommandType.Text;
                    cmd.Parameters.Add(new SqlParameter("@type", type));
                    using (var reader = cmd.ExecuteReader())
                    {
                        while (reader.Read())
                        {
                            _item = new TagEntity();
                            _item.tagname = reader["tagname"].ToString();
                            _items.Add(_item);
                        }
                    }
                }
            }

            return _items;
        }*/
        public static List<TagsDataEntity> LoadItems(TagEntity entity)
        {
            if (!entity.iscache || Config.GetCacheDuration() == 0 || entity.pagenumber > 5)
            {
                return FetchItems(entity);
            }
            else
            {
                string key = GenerateKey("lg_tag_", entity);
                if (HttpContext.Current.Cache[key] == null)
                    HttpContext.Current.Cache.Add(key, FetchItems(entity), null, DateTime.Now.AddMinutes(Config.GetCacheDuration()), TimeSpan.Zero, CacheItemPriority.High, null);

                return (List<TagsDataEntity>)(HttpContext.Current.Cache[key]);
            }
        }

        private static List<TagsDataEntity> FetchItems(TagEntity entity)
        {
            var _list = new List<TagsDataEntity>();

            using (var ctx = new mvcportalEntities())
            {
                IQueryable<tag> Query = ctx.tags;
                Query = BuildQuery(Query, entity);
              
                if (entity.tagid > 0)
                {
                    entity.loadall = true;

                    _list = LoadCompleteList(Query);
                }
                if (entity.issummary)
                {
                    _list = LoadSummaryList(Query);
                }
                else
                {
                    _list = LoadCompleteList(Query);
                }
            }

            return _list;
        }

        public static int Count(TagEntity entity)
        {
            if (!entity.iscache || Site_Settings.Cache_Duration == 0 || entity.pagenumber > 5)
            {
                return CountRecords(entity);
            }
            else
            {
                string key = GenerateKey("cnt_tag", entity);
                if (HttpContext.Current.Cache[key] == null)
                {
                    int total_records = CountRecords(entity);

                    HttpContext.Current.Cache.Add(key, total_records, null, DateTime.Now.AddMinutes(Site_Settings.Cache_Duration), TimeSpan.Zero, CacheItemPriority.High, null);
                }
                return Convert.ToInt32(HttpContext.Current.Cache[key]);
            }
        }

        private static int CountRecords(TagEntity entity)
        {
            int Records = 0;
            using (var entities = new mvcportalEntities())
            {
                IQueryable<tag> Query = entities.tags;
                entity.iscount = true;
                entity.tagid = 0;
                Query = BuildQuery(Query, entity);
                Records = Query.Count();
            }
            return Records;
        }
        private static string GenerateKey(string key, TagEntity entity)
        {
            var str = new StringBuilder();
            return key + entity.datefilter + "" + entity.filter + "" + UtilityBLL.ReplaceSpaceWithHyphin(entity.order.ToLower()) + "" + entity.tag_level + "" + entity.tag_type + "" + entity.type + "" + entity.pagenumber + "" + entity.term;
        }

        private static IQueryable<tag> BuildQuery(IQueryable<tag> query, TagEntity entity)
        {
            query = Where(query, entity);
            if (entity.tagid == 0 && !entity.iscount)
            {
                query = OrderBy(query, entity);
                query = Skip(query, entity);
                query = Limit(query, entity);
            }
            return query;
        }

        public static List<TagsDataEntity> LoadCompleteList(IQueryable<tag> query)
        {
            return query.Select(p => new TagsDataEntity
            {
                tagid = (int)p.tagid,
                tagname = p.tagname,
                status = p.status,
                tag_level = p.tag_level,
                periority = (byte)p.periority,
                type = p.type,
                records = p.records,
                tag_type = p.tag_type,
                term = p.term
            }).ToList();
        }

        public static List<TagsDataEntity> LoadSummaryList(IQueryable<tag> query)
        {
            return query.Select(p => new TagsDataEntity
            {
                tagid = (int)p.tagid,
                tagname = p.tagname,
                status = p.status,
                tag_level = p.tag_level,
                periority = (byte)p.periority,
                type = p.type,
                records = p.records,
                tag_type = p.tag_type,
                term = p.term
            }).ToList();
        }

        public static List<TagsDataEntity> LoadDropdownList(IQueryable<tag> query)
        {
            return query.Select(p => new TagsDataEntity
            {
                tagid = (int)p.tagid,
                tagname = p.tagname,
                term = p.term

            }).ToList();
        }

        private static IQueryable<tag> Where(IQueryable<tag> query, TagEntity entity)
        {
            if (entity.tagid > 0)
                query = query.Where(p => p.tagid == entity.tagid);

            if (entity.tagname != "")
                query = query.Where(p => p.tagname == entity.tagname);

            if (entity.term != "")
                query = query.Where(p => p.tagname.Contains(entity.term) || p.term.Contains(entity.term));
                       
            if (entity.type != 100)
                query = query.Where(p => p.type == entity.type);

            if (entity.status != 2)
                query = query.Where(p => p.status == entity.status);

            if (entity.tag_level != 100)
                query = query.Where(p => p.tag_level == entity.tag_level);

            if (entity.tag_type != 2)
                query = query.Where(p => p.tag_type == entity.tag_type);
            
            return query;
        }

        private static IQueryable<tag> OrderBy(IQueryable<tag> query, TagEntity entity)
        {
            if (entity.order != "")
            {
                var orderlist = entity.order.Split(char.Parse(","));
                foreach (var order in orderlist)
                {
                    if (order.Contains("tagid"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.tagid);
                        else
                            query = query.OrderBy(p => p.tagid);
                    }
                    else if (order.Contains("tagname"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.tagname);
                        else
                            query = query.OrderBy(p => p.tagname);
                    }
                    else if (order.Contains("type"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.type);
                        else
                            query = query.OrderBy(p => p.type);
                    }
                    else if (order.Contains("tag_level"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.tag_level);
                        else
                            query = query.OrderBy(p => p.tag_level);
                    }
                    else if (order.Contains("tag_type"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.tag_type);
                        else
                            query = query.OrderBy(p => p.tag_type);
                    }
                }
            }

            return query;
        }

        private static IQueryable<tag> Skip(IQueryable<tag> query, TagEntity entity)
        {
            if (entity.pagenumber > 1)
                query = query.Skip(entity.pagesize * (entity.pagenumber - 1));

            return query;
        }

        private static IQueryable<tag> Limit(IQueryable<tag> query, TagEntity entity)
        {
            if (!entity.loadall)
                query = query.Take(entity.pagesize);

            return query;
        }
    }
}
