﻿using System;
/* Site Own Namespaces */
using Jugnoon.Utility;
using System.Collections.Generic;
using System.Text;
using Jugnoon.Entity;
using System.Linq;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;
using Jugnoon.Settings;
using System.Web;
using System.Web.Caching;

namespace Jugnoon.BLL
{
    public class User_IPLogBLL
    {

        public User_IPLogBLL()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        public static bool Add_Ipaddress(string username, string ipaddress)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var _entity = new users_ipaddress()
                {
                    username = username,
                    ipaddress = ipaddress,
                    date_added = DateTime.Now
                };

                dbCtx.Entry(_entity).State = System.Data.Entity.EntityState.Added;

                DataBLL.SaveChanges(dbCtx);

            }

            return true;
        }

        public static bool Delete_IPAddress(string username)
        {
            long serialno = Get_Old_SerialID(username);
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new users_ipaddress { serialno = (int)serialno };
                dbCtx.users_ipaddress.Attach(entity);
                dbCtx.users_ipaddress.Remove(entity);
                dbCtx.SaveChanges();
            }
            
            return true;
        }

        public static long Get_Old_SerialID(string username)
        {
            long ID = 0;
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.users_ipaddress
                     .Where(p => p.username == username).Take(1).OrderBy(p => p.serialno).ToList();

                if(item != null)
                   ID = item[0].serialno;
            }
            return ID;
        }

        public static int Count_Ipaddress(string username)
        {
            int Records = 0;
            using (var ctx = new mvcportalEntities())
            {
                Records = ctx.users_ipaddress.Where(p => p.username == username).Count();
            }
            return Records;
        }

        public static bool Process_Ipaddress_Log(string username, string ipaddress)
        {
            int count = Count_Ipaddress(username);
            // keep top 5 login ip logs of each user
            if (count > 5)
            {
                // delete old ip address log
                Delete_IPAddress(username);
                // add ip address log
                Add_Ipaddress(username, ipaddress);
            }
            else
            {
                Add_Ipaddress(username, ipaddress);
            }
            return true;
        }

        public static List<UserIPEntity> LoadItems(UserIPEntity entity)
        {
            if (!entity.iscache || Config.GetCacheDuration() == 0 || entity.pagenumber > 5)
            {
                return FetchItems(entity);
            }
            else
            {
                string key = GenerateKey("lg_user_ipaddress_", entity);
                if (HttpContext.Current.Cache[key] == null)
                    HttpContext.Current.Cache.Add(key, FetchItems(entity), null, DateTime.Now.AddMinutes(Config.GetCacheDuration()), TimeSpan.Zero, CacheItemPriority.High, null);

                var _list = (List<UserIPEntity>)(HttpContext.Current.Cache[key]);
                return _list;
            }
        }

        private static List<UserIPEntity> FetchItems(UserIPEntity entity)
        {
            var _list = new List<UserIPEntity>();

            using (var ctx = new mvcportalEntities())
            {
                IQueryable<users_ipaddress> Query = ctx.users_ipaddress;
                Query = BuildQuery(Query, entity);
        
                if (entity.serialno > 0)
                {
                    entity.loadall = true;

                    _list = LoadCompleteList(Query);
                }
                if (entity.issummary)
                {
                    _list = LoadSummaryList(Query);
                }
                else
                {
                    _list = LoadCompleteList(Query);
                }
            }

            return _list;
        }

        public static int Count(UserIPEntity entity)
        {
            if (!entity.iscache || Site_Settings.Cache_Duration == 0 || entity.pagenumber > 5)
            {
                return CountRecords(entity);
            }
            else
            {
                string key = GenerateKey("cnt_user_ipaddress", entity);
                if (HttpContext.Current.Cache[key] == null)
                {
                    int total_records = CountRecords(entity);

                    HttpContext.Current.Cache.Add(key, CountRecords(entity), null, DateTime.Now.AddMinutes(Site_Settings.Cache_Duration), TimeSpan.Zero, CacheItemPriority.High, null);
                }
                return Convert.ToInt32(HttpContext.Current.Cache[key]);
            }
        }

        private static int CountRecords(UserIPEntity entity)
        {
            int Records = 0;
            using (var entities = new mvcportalEntities())
            {
                IQueryable<users_ipaddress> Query = entities.users_ipaddress;
                entity.iscount = true;
                entity.serialno = 0;
                Query = BuildQuery(Query, entity);
                Records = Query.Count();
            }
            return Records;
        }
        private static string GenerateKey(string key, UserIPEntity entity)
        {
            var str = new StringBuilder();
            return key + entity.datefilter + "" + entity.filter + "" + UtilityBLL.ReplaceSpaceWithHyphin(entity.order.ToLower()) + "" + entity.username + entity.pagenumber + "" + entity.term;
        }

        private static IQueryable<users_ipaddress> BuildQuery(IQueryable<users_ipaddress> query, UserIPEntity entity)
        {
            query = Where(query, entity);
            if (entity.serialno == 0 && !entity.iscount)
            {
                query = OrderBy(query, entity);
                query = Skip(query, entity);
                query = Limit(query, entity);
            }
            return query;
        }

        public static List<UserIPEntity> LoadCompleteList(IQueryable<users_ipaddress> query)
        {
            return query.Select(p => new UserIPEntity
            {
                serialno = (int)p.serialno,
                username = p.username,
                ipaddress = p.ipaddress,
                date_added = (DateTime)p.date_added
            }).ToList();
        }

        public static List<UserIPEntity> LoadSummaryList(IQueryable<users_ipaddress> query)
        {
            return query.Select(p => new UserIPEntity
            {
                serialno = (int)p.serialno,
                username = p.username,
                ipaddress = p.ipaddress,
                date_added = (DateTime)p.date_added
            }).ToList();
        }
              

        private static IQueryable<users_ipaddress> Where(IQueryable<users_ipaddress> query, UserIPEntity entity)
        {
            if (entity.serialno > 0)
                query = query.Where(p => p.serialno == entity.serialno);

            if (entity.term != "")
                query = query.Where(p => p.ipaddress.Contains(entity.term) || p.username.Contains(entity.term));
            
            if(entity.userid > 0)
            {
                var username = UserBLL.Return_Value(entity.userid, "username");
                query = query.Where(p => p.username == username);
            }
            if (entity.username != null && entity.username != "")
                query = query.Where(p => p.username == entity.username);

            return query;
        }

        private static IQueryable<users_ipaddress> OrderBy(IQueryable<users_ipaddress> query, UserIPEntity entity)
        {
            if (entity.order != "")
            {
                var orderlist = entity.order.Split(char.Parse(","));
                foreach (var order in orderlist)
                {
                    if (order.Contains("serialno"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.serialno);
                        else
                            query = query.OrderBy(p => p.serialno);
                    }
                    else if (order.Contains("username"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.username);
                        else
                            query = query.OrderBy(p => p.username);
                    }
                    else if (order.Contains("ipaddress"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.ipaddress);
                        else
                            query = query.OrderBy(p => p.ipaddress);
                    }
                   
                }
            }

            return query;
        }

        private static IQueryable<users_ipaddress> Skip(IQueryable<users_ipaddress> query, UserIPEntity entity)
        {
            if (entity.pagenumber > 1)
                query = query.Skip(entity.pagesize * (entity.pagenumber - 1));

            return query;
        }

        private static IQueryable<users_ipaddress> Limit(IQueryable<users_ipaddress> query, UserIPEntity entity)
        {
            if (!entity.loadall)
                query = query.Take(entity.pagesize);

            return query;
        }
    }
}