﻿using Jugnoon.Entity;
using System.Web;

namespace Jugnoon.Utility
{
    public class PhotoUrlConfig
    {
        public static string PrepareUrl(PhotoDataEntity entity)
        {
            return Config.GetUrl("photo/" + entity.imageid);
        }

        public static string GetDefaultPhoto()
        {
            return Config.GetUrl() + "assets/images/default.png";
        }

        public static string PreparePhotoUrl(string username, string filename, int type)
        {
            return PreparePhotoUrl(new PhotoDataEntity()
            {
                username = username,
                filename = filename,
            }, type);
        }

        public static string PreparePhotoUrl(PhotoDataEntity entity, int type)
        {
            if (entity.username == "")
                return "#";

            string URL = "";
            if (entity.filename == null || entity.filename == "" || entity.filename == "none")
                URL = GetDefaultPhoto();
            else if (entity.filename.StartsWith("http"))
                // photo save in cloud or in third party host
                URL = PrepareHttpPhotoUrl(entity, type);
            else
            {
                // type = 0: thumb, 1: mid thumb, 2: original
                string Imagetype = ""; // original
                switch (type)
                {
                    case 0:
                        Imagetype = "thumbs/";
                        break;
                    case 1:
                        Imagetype = "thumbs/"; // "midthumbs/" no long available
                        break;
                    case 2:
                        Imagetype = "midthumbs/"; // replaced with type = 2
                        break;
                }
                URL = Config.GetUrl() + "contents/member/" + entity.username + "/photos/" + Imagetype + "" + entity.filename;
            }
            return URL;
        }

        public static string PreparePhotoUrl(string username, string filename, int type, bool isresize, string size)
        {
            return PreparePhotoUrl(new PhotoDataEntity()
            {
                username = username,
                filename = filename,
            }, type, isresize, size);
        }
        public static string PreparePhotoUrl(PhotoDataEntity entity, int type, bool isresize, string size)
        {
            if (!isresize)
                return PreparePhotoUrl(entity, type);
            else
            {
                return Config.GetUrl("photos/resize?u=" + entity.username + "&file=" + entity.filename + "&cloud=0&size=" + size);
            }

        }
        public static string PreparePhotoPhysicalUrl(string username, string filename, int type)
        {
            return PreparePhotoPhysicalUrl(new PhotoDataEntity()
            {
                username = username,
                filename = filename
            }, type);
        }
        public static string PreparePhotoPhysicalUrl(PhotoDataEntity entity, int type)
        {
            if (entity.username == "")
                return "#";

            string URL = "";
            if (entity.filename == null || entity.filename == "" || entity.filename == "none")
                URL = GetDefaultPhoto();

            else if (entity.filename.StartsWith("http")) // media stored in cloud storage
                URL = PrepareHttpPhotoUrl(entity, type);
            else
            {
                // type = 0: thumb, 1: mid thumb, 2: original
                string Imagetype = ""; // original
                switch (type)
                {
                    case 0:
                        Imagetype = "thumbs/";
                        break;
                    case 1:
                        Imagetype = "midthumbs/";
                        break;
                }
                URL = HttpContext.Current.Server.MapPath(HttpContext.Current.Request.ApplicationPath) + "\\contents\\member\\" + entity.username + "\\photos\\" + Imagetype + "" + entity.filename;
            }

            return URL;
        }

        public static string PrepareHttpPhotoUrl(PhotoDataEntity entity, int type)
        {
            string URL = "";
            switch (type)
            {
                case 0:
                    // thumb path
                    URL = entity.filename;
                    break;
                case 1:
                    URL = entity.filename; // picturename.Replace("thumbs/", "midthumbs/"); midthumbs no more
                    break;
                case 2:
                    URL = entity.filename.Replace("thumbs/", "");
                    break;
                case 3:
                    URL = entity.filename.Replace("thumbs/", "midthumbs/");
                    break;
            }

            return URL;
        }

        public static string Return_Photo_DownloadUrl(long id)
        {
            return Config.GetUrl("get/" + id);
        }

    }
}