using System;
using System.Configuration;
using System.Web;
using System.Text;

using Jugnoon.Settings;

namespace Jugnoon.Utility
{
    public class Config
    {
        private static UtilityBLL _utility = new UtilityBLL();

       
        public Config()
        { }

        #region General Settings
              

        /// <summary>
        /// Toggle SQL SERVER / MySQL Server (0: SQL SERVER, 1: MySQL)
        /// </summary>
        public static int Load_DB_Type
        {
            get
            {
                return 0; // mysql not currently supported
            }
        }

        /// <summary>
        /// Set Site Theme
        /// </summary>
        public static string SiteTheme
        {
            get
            {
                if (HttpContext.Current.Request.Cookies["VSKTheme"] != null)
                    return HttpContext.Current.Request.Cookies["VSKTheme"].Value;
                else
                    return ConfigurationManager.AppSettings["SiteTheme"].ToString();
            }
        }
        
        public static string GetUrl()
        {
            return ConfigurationManager.AppSettings["url"];
        }

        /// <summary>
        /// Get current domain url
        /// </summary>
        /// 
        public static string GetUrl(string url)
        {
            string _url = GetUrl();
            if (_url.EndsWith("/") || _url.EndsWith(@"\"))
                return _url + "" + url;
            else
                return _url + "/" + url;
        }

        public static int Calculate_Ratings(double rating)
        {
            return (int)rating * 24;
        }
            
        /// <summary>
        /// UserBLLhip Account Upgrade Options
        /// </summary>
        public static int GetMembershipAccountUpgradeType()
        {
            int type = Convert.ToInt32(ConfigurationManager.AppSettings["Member_Account_Upgrade_Type"]);
            if (type > 1)
                type = 1;
            return type;
        }

        /// <summary>
        /// UserBLLhip Account Upgrade Redirect Option - 0: Redirect User To MyAccount After Registeration, 1: Redirect User To Upgrade Account Section After Registeration
        /// </summary>
        public static int GetMembershipAccountUpgradeRedirect()
        {
            int type = Convert.ToInt32(ConfigurationManager.AppSettings["Member_Account_Upgrade_Redirect"]);
            if (type > 1)
                type = 1;
            return type;
        }

        /// <summary>
        /// Validate administrator readonly status.
        /// </summary>
        public static bool isAdminActionAllowed()
        {
            if (HttpContext.Current.User.IsInRole("Admin"))
                return true;
            else
                return false;
        }

        /// <summary>
        /// Return current page url and append querystring
        /// </summary>
        public static string Return_Current_Page(string value)
        {
            string prot = "http";
            string https = HttpContext.Current.Request.ServerVariables["HTTPS"];
            if (https != "off")
                prot = "https";

            string domainname = HttpContext.Current.Request.ServerVariables["SERVER_NAME"];
            string filename = HttpContext.Current.Request.ServerVariables["SCRIPT_NAME"];
            //string querystring = HttpContext.Current.Request.ServerVariables["QUERY_STRING"];
            return prot + "://" + domainname + filename + "?" + value;
        }

        /// <summary>
        /// Return current page url.
        /// </summary>
        public static string Return_Current_Page()
        {
            string prot = "http";
            string https = HttpContext.Current.Request.ServerVariables["HTTPS"];
            if (https != "off")
                prot = "https";

            string domainname = HttpContext.Current.Request.ServerVariables["SERVER_NAME"];
            string filename = HttpContext.Current.Request.ServerVariables["SCRIPT_NAME"];
            //string querystring = HttpContext.Current.Request.ServerVariables["QUERY_STRING"];
            //if (querystring != "")
            //    return prot + "://" + domainname + filename + "?" + querystring;
            //else
            return prot + "://" + domainname + filename;
        }
        #endregion


        #region Website_Settings

        /// <summary>
        /// Get website title
        /// </summary>
        public static string GetWebsiteTitle()
        {
            return Site_Settings.Website_Title;
        }

        /// <summary>
        /// Get website description
        /// </summary>
        public static string GetWebsiteDescription()
        {
            return Site_Settings.Website_Description;
        }

        /// <summary>
        ///  Return Title Caption -/ Used at the end of page title -/ SEO Purpose only
        /// </summary>
        public static string GetUrlCaption()
        {
            return Site_Settings.Page_Caption;
        }

        /// <summary>
        /// Get Cache Duration
        /// </summary>
        public static int GetCacheDuration()
        {
            return Site_Settings.Cache_Duration;
        }

        /// <summary>
        /// Return Maximum Pagination Links for Main Content Listing -/ Videos, -/ Photos, -/Blogs
        /// </summary>
        public static int Get_Maximum_Pagination_Links()
        {
            return Site_Settings.Pagination_Links;
        }

        /// <summary>
        /// Return screen option 0-> screen use for highlight words, 1:-> screen and replace (e.g apple -> a***e)
        /// </summary>
        public static int Get_ScreeningOption()
        {
            return Site_Settings.Screen_Content;
        }

        /// <summary>
        /// Get approval type
        /// </summary>
        public static bool isAutomativeApproval()
        {
            int value = Site_Settings.Content_Approval;
            if (value == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Get pagination type of listing
        /// </summary>
        public static int Return_PaginationType()
        {
            return Site_Settings.Pagination_Type;
        }

        /// <summary>
        /// Get abuse | spam maximum count setting
        /// </summary>
        public static int Return_AbuseReport_Count()
        {
            return Site_Settings.Spam_Count;
        }

        /// <summary>
        /// Get admin email address for sending mails
        /// </summary>
        public static string Return_Email()
        {
            return Site_Settings.Admin_Mail;
        }

        /// <summary>
        /// Get admin email display name for sending mails
        /// </summary>
        public static string Return_EmailDisplayName()
        {
            return Site_Settings.Admin_Mail_DisplayName;
        }
        #endregion


        #region Feature_Settings

        /// <summary>
        /// Toggle On / Off Video Feature in Website
        /// </summary>
        public static bool isVideos()
        {

            int status = Site_Settings.Feature_Videos;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toggle On / Off Audio Feature in Website
        /// </summary>
        public static bool isAudios()
        {
            int status = Site_Settings.Feature_Audio;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toggle On / Off Photo Feature in Website
        /// </summary>
        public static bool isPhotos()
        {
            int status = Site_Settings.Feature_Photos;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toggle On / Off Blog Feature in Website
        /// </summary>
        public static bool isBlogs()
        {
            int status = Site_Settings.Feature_Blogs;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toggle On / Off QA Feature in Website
        /// </summary>
        public static bool isQA()
        {
            int status = Site_Settings.Feature_QA;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toggle On / Off Forum Feature in Website
        /// </summary>
        public static bool isForum()
        {
            int status = Site_Settings.Feature_Forums;
            if (status == 1)
                return true;
            else
                return false;
        }


        /// <summary>
        /// Toggle On / Off Video Feature in Website
        /// </summary>
        public static bool isSignUp()
        {
            int status = Site_Settings.Feature_MemberRegistration;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        ///  Toggle On / Off Community Feature in Website
        /// </summary>
        public static bool isGroups()
        {
            int status = Site_Settings.Feature_Groups;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        ///  Toggle On / Off Channel Feature in Website
        /// </summary>
        public static bool isChannels()
        {
            int status = Site_Settings.Feature_Channels;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off friend section
        /// </summary>
        public static bool isFriends()
        {
            int status = Site_Settings.Feature_Friends;
            if (status == 1)
                return true;
            else
                return false;
        }



        /// <summary>
        /// Toglle on | off internal messaging board
        /// </summary>
        public static bool isFeature_Messaging()
        {
            int status = Site_Settings.Feature_Messaging;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off playlist section
        /// </summary>
        public static bool isFeature_Playlist()
        {
            int status = Site_Settings.Feature_Playlist;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off upload section
        /// </summary>
        public static bool isFeature_Uploads()
        {
            int status = Site_Settings.Feature_Uploads;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off archive section
        /// </summary>
        public static bool isFeature_Archives()
        {
            int status = Site_Settings.Feature_Archives;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off filters, filter content by today viewed, this week viewed, this month viewed
        /// </summary>
        public static bool isFeature_Filter()
        {
            int status = Site_Settings.Feature_Filter;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off comment section
        /// </summary>
        public static bool isFeature_Comments()
        {
            int status = Site_Settings.Feature_Comments;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off video response section
        /// </summary>
        public static bool isFeature_VideoResponses()
        {
            int status = Site_Settings.Feature_Video_Responses;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off statistic section
        /// </summary>
        public static bool isFeature_Statistic()
        {
            int status = Site_Settings.Feature_Video_Statistics;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off related section
        /// </summary>
        public static bool isFeature_Related()
        {
            int status = Site_Settings.Feature_Related;
            if (status == 1)
                return true;
            else
                return false;
        }


        /// <summary>
        /// Toglle on | off views
        /// </summary>
        public static bool isFeature_Views()
        {
            int status = Site_Settings.Feature_Views;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off uploaded date
        /// </summary>
        public static bool isFeature_Date()
        {
            int status = Site_Settings.Feature_Date;
            if (status == 1)
                return true;
            else
                return false;
        }


        /// <summary>
        /// Toggle on | off advertisement
        /// </summary>
        public static bool isAdsEnabled()
        {
            int status = Site_Settings.Feature_Advertisement;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toggle on | off email processing
        /// </summary>
        public static bool isEmailEnabled()
        {
            int status = Site_Settings.Feature_Email;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toggle on | off adult validation
        /// </summary>
        public static bool isAdultValidation()
        {
            int status = Site_Settings.Feature_Adult_Verification;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Choose whether registration validation required at time of user registration
        /// </summary>
        public static bool isRegistrationValidation()
        {
            int status = Site_Settings.Feature_Email_Verification;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Post watermark on video at time of encoding
        /// </summary>
        public static bool isPostWaterMark()
        {
            int status = Site_Settings.Feature_Video_Watermark;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off username
        /// </summary>
        public static bool isFeature_UserName()
        {
            int status = Site_Settings.Feature_UserName;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off tags section
        /// </summary>
        public static bool isFeature_Tags()
        {
            int status = Site_Settings.Feature_Tags;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Toglle on | off comment section
        /// </summary>
        public static bool isFeature_Rating()
        {
            int status = Site_Settings.Feature_Rating;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Login required for rating content, by disabling rating no record tracking will store, mean single user can post multiple ratings
        /// </summary>
        public static bool isLoginRequired_Rating()
        {
            int status = Site_Settings.Feature_Login_Rating;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Login required for posting comment, by disabling rating user can post comment as guest user.
        /// </summary>
        public static bool isLoginRequired_Comment()
        {
            int status = Site_Settings.Feature_Login_Comment;
            if (status == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Login required for point good or bad points, by disabling this feature no tracking will store, single user can post multiple points.
        /// </summary>
        public static bool isLoginRequired_Points()
        {
            int status = Site_Settings.Feature_Login_Advice;
            if (status == 1)
                return true;
            else
                return false;
        }


        #endregion

        #region Media Settings


        /// <summary>
        /// Play video automatically on flash player
        /// </summary>
        public static string isAutoPlay()
        {
            int value = Site_Settings.Media_Play;
            if (value == 0)
                return "true";
            else
                return "false";
        }

        /// <summary>
        /// Get video publishing option
        /// </summary>
        public static int Return_EmbedVideo_Option()
        {
            return Site_Settings.Video_Upload_Option;
        }

        /// <summary>
        /// Get video publishing option
        /// </summary>
        public static bool isDirectPublishing()
        {
            int value = Site_Settings.Video_Publishing_Type;
            if (value == 1)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Get thumb type
        /// </summary>
        public static int Return_ThumbType()
        {
            return Site_Settings.Thumb_Preview_Type;
        }

        /// <summary>
        /// Get video output type
        /// </summary>
        public static int Return_OutputType()
        {
            return Site_Settings.Video_Output_Type;
        }

        /// <summary>
        /// Get video max sise -/ MB
        /// </summary>
        public static int Return_VideoMaxSize()
        {
            return Site_Settings.Video_Max_Size;
        }

        /// <summary>
        /// Get audio max sise -/ MB
        /// </summary>
        public static int Return_AudioMaxSize()
        {
            return Site_Settings.Audio_Max_Size;
        }

        /// <summary>
        /// Delete original video after publishing
        /// </summary>
        public static bool isDeleteOriginalVideo()
        {           
            if (Site_Settings.Video_Delete_Original)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Get thumbnail width
        /// </summary>
        public static int Return_ThumbWith()
        {
            return Site_Settings.Thumb_Width;
        }

        /// <summary>
        /// Get thumbnail height
        /// </summary>
        public static int Return_ThumbHeight()
        {
            return Site_Settings.Thumb_Height;
        }

        #endregion

        #region Url Settings

        /// <summary>
        /// Generate and return original video url
        /// </summary>
        public static string Return_Original_Video_Url(string url, string username)
        {
            if (url == "none" || url == "")
                return Config.GetUrl() + "contents/member/" + username + "/Default";
            else
                return url;
        }

        /// <summary>
        /// Generate and return flv video url
        /// </summary>
        public static string Return_FLV_Video_Url(string url, string username)
        {
            if (url == "none" || url == "")
                return Config.GetUrl() + "contents/member/" + username + "/FLV";
            else
            {
                return url;
            }
        }
        /// <summary>
        /// Generate and return mp3 audio url
        /// </summary>
        public static string Return_MP3_Audio_Url(string url, string username)
        {
            if (url == "none" || url == "")
                return Config.GetUrl() + "contents/member/" + username + "/MP3";
            else
            {
                return url;
            }
        }
        /// <summary>
        /// Generate and return video thumb url
        /// </summary>
        public static string Return_Video_Thumb_Url(string url, string username)
        {
            if (url == "none" || url == "")
                return Config.GetUrl() + "contents/member/" + username + "/Thumbs";
            else
                return url;
        }

        /// <summary>
        /// Generate and return encoded video physical path
        /// </summary>
        public static string Return_Video_physicalPath(string url, string username, string filename, int type)
        {
            string path = "flv\\";
            switch (type)
            {
                case 0:
                    // original video path
                    path = "default\\";
                    break;
                case 1:
                    // flv video path
                    path = "flv\\";
                    break;
                case 2:
                    // mp3 audio path
                    path = "mp3\\";
                    break;
                case 3:
                    // thumb path
                    path = "thumbs\\";
                    break;
            }
            if (url == "none" || url == "")
                return HttpContext.Current.Server.MapPath(HttpContext.Current.Request.ApplicationPath) + "\\contents\\member\\" + username + "\\" + path + "" + filename;
            else
                return url;
        }
        /// <summary>
        /// Return video player url.
        /// </summary>
        public static string Return_Player_Url()
        {
            return Config.GetUrl() + "player/player.swf";
        }
        #endregion

        #region Message Layouts

        /// <summary>
        /// Return loading message
        /// </summary>
        public static void SetLoadingMessage(System.Web.UI.HtmlControls.HtmlGenericControl pnl, string elementid, bool isvisible)
        {
            var str = new StringBuilder();
            string hidden = "";
            if (isvisible == false)
                hidden = "style=\"display:none;\"";
            str.AppendLine("<div id=\"" + elementid + "\" " + hidden + ">loading....</div>");
            pnl.InnerHtml = str.ToString();
        }

        /// <summary>
        /// Display message with close link, by default 
        /// </summary>
        public static void SetHiddenMessage_v2(System.Web.UI.HtmlControls.HtmlGenericControl pnl, string message, string elementid, bool isvisible, int messagetype)
        {
            pnl.InnerHtml = SetHiddenMessage_v2(message, elementid, isvisible, messagetype);
        }
        /// <summary>
        /// Display message with close link, by default 
        /// </summary>
        public static string SetHiddenMessage_v2(string message, string elementid, bool isvisible, int messagetype)
        {
            // type: 4 normal
            var str = new StringBuilder();
            if (messagetype == 4)
            {
                str.AppendLine("<div class=\"card m-b-10 panelsm\">\n<div class=\"card-body\">\n");
                str.AppendLine(message);
                str.AppendLine("</div></div>\n");
            }
            else
            {
                string css = "round m-b-10";
                switch (messagetype)
                {
                    case 0:
                        css = "alert alert-danger alert-dismissible";
                        break;
                    case 1:
                        css = "alert alert-success alert-dismissible";
                        break;
                    case 2:
                        css = "alert alert-info alert-dismissible";
                        break;
                    case 3:
                        css = "alert alert-block";
                        break;
                }
               
                string hidden = "";
                if (isvisible == false)
                    hidden = "style=\"display:none;\"";
                                
                str.AppendLine("<div id=\"" + elementid + "\" " + hidden + " class=\"" + css + "\" role =\"alert\">");
                str.AppendLine("<button type=\"button\" class=\"close\" data-dismiss=\"alert\" aria-label=\"Close\"><span aria-hidden=\"true\">&times;</span></button>");
                str.AppendLine(message);
                str.AppendLine("</div>");
            }
            return str.ToString();
        }

        #endregion

        // Twitter Bootstrap Message Structuring
        #region Twitter Bootstrap
        public static string SetStatus(int type, string gMessage, string bMessage)
        {
            string _css = "label-success";
            string _message = gMessage;
            if (type == 0)
            {
                _css = "label-danger";
                _message = bMessage;
            }
            return "<span class=\"label " + _css + "\">" + _message + "</span>\n";

        }

        // control three status like (admin, user, paid user) or (normal, featured, premium content)
        public static string SetStatus(int type, string gMessage, string bMessage, string cMessage)
        {
            string _css = "label-success";
            string _message = gMessage;
            if (type == 0)
            {
                _css = "label-danger";
                _message = bMessage;
            }
            else if(type == 1)
            {
                _message = gMessage;
            }
            else if (type == 2)
            {
                _css = "label-info";
                _message = cMessage;
            }
            return "<span class=\"label " + _css + "\">" + _message + "</span>\n";

        }


        #endregion

        public static string Prepare_Pagination(string query, string order, int pagenumber, int pagesize, string distinct = "")
        {
            // SQL SERVER 2012 OR NEWER
            int OFFSET = (pagenumber - 1) * pagesize;
            return "SELECT " + distinct + " " + query + " ORDER BY " + order + " OFFSET " + OFFSET + " ROWS FETCH NEXT " + pagesize + " ROWS ONLY";

            // SQL SERVER 2012 or LOWER

            // fetch and return only batch of data starting from star posting to lastbound
            // int startindex = (pagenumber - 1) * pagesize & 1;
            // int lastbound = (startindex - 1) & pagesize;
            //return "SELECT" & distinct & " TOP " & pagesize & " * FROM (SELECT ROW_NUMBER() OVER (ORDER BY " & order & ") AS RowNumber," & query & ") AS Results WHERE RowNumber >=" & startindex & " AND RowNumber <=" & lastbound;
        }
        
    }
}