﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;

using System.IO;
using Jugnoon.Settings;
using System.Linq;
using System.Net;

namespace Jugnoon.Utility
{
    public class Image
    {
        public Image()
        { }

        public static Bitmap CreateThumbnail(string lcFilename, int targetSize)
        {
            Bitmap loBMP = null;
            Bitmap bmpOut = null;
            try
            {
                loBMP = new Bitmap(lcFilename);
                ImageFormat loFormat = loBMP.RawFormat;
                Size newSize = CalculateDimensions(loBMP.Size, targetSize);
                bmpOut = new Bitmap(newSize.Width, newSize.Height);
                Graphics canvas = Graphics.FromImage(bmpOut);
                canvas.SmoothingMode = SmoothingMode.AntiAlias;
                canvas.InterpolationMode = InterpolationMode.HighQualityBicubic;
                canvas.PixelOffsetMode = PixelOffsetMode.HighQuality;
                canvas.DrawImage(loBMP, new Rectangle(new Point(0, 0), newSize));
            }
            finally
            {
                loBMP.Dispose();
            }

            return bmpOut;
        }

        public static Bitmap CreateThumbnail(string lcFilename, int width, int height)
        {
            if (height == 0)
            {
                return CreateThumbnail(lcFilename, width);
            }
            else
            {

                Bitmap loBMP = null;
                Bitmap bmpOut = null;
                try
                {
                    loBMP = new Bitmap(lcFilename);
                    ImageFormat loFormat = loBMP.RawFormat;
                    Size newSize = new Size();
                    newSize.Height = height;
                    newSize.Width = width;
                    bmpOut = new Bitmap(newSize.Width, newSize.Height);
                    Graphics canvas = Graphics.FromImage(bmpOut);
                    canvas.SmoothingMode = SmoothingMode.AntiAlias;
                    canvas.InterpolationMode = InterpolationMode.HighQualityBicubic;
                    canvas.PixelOffsetMode = PixelOffsetMode.HighQuality;
                    canvas.DrawImage(loBMP, new Rectangle(new Point(0, 0), newSize));
                }
                finally
                {
                    loBMP.Dispose();
                }

                return bmpOut;
            }

        }
        
        public static void CreateCropThumbnail(string lcFilename, int width, int height, string coverPath, bool targettop = false)
        {
            SaveCroppedImage(new Bitmap(lcFilename), width, height, coverPath, targettop);
        }

        private static Size CalculateDimensions(Size oldSize, int targetSize)
        {
            Size newSize = new Size();
            if (oldSize.Height > oldSize.Width)
            {
                newSize.Width = Convert.ToInt32(oldSize.Width * (Convert.ToSingle(targetSize) / Convert.ToSingle(oldSize.Height)));
                newSize.Height = targetSize;
            }
            else
            {
                newSize.Width = targetSize;

                newSize.Height = Convert.ToInt32(oldSize.Height * (Convert.ToSingle(targetSize) / Convert.ToSingle(oldSize.Width)));
            }
            return newSize;
        }

        public static void SaveJpeg(string path, Bitmap img, long quality)
        {
            // Encoder parameter for image quality
            EncoderParameter qualityParam = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, quality);

            // Jpeg image codec
            ImageCodecInfo jpegCodec = getEncoderInfo("image/jpeg");

            if (jpegCodec == null)
            {
                return;
            }

            EncoderParameters encoderParams = new EncoderParameters(1);
            encoderParams.Param[0] = qualityParam;

            img.Save(path, jpegCodec, encoderParams);
        }

        public static void SavePNG(string path, Bitmap img, long quality)
        {
            // Encoder parameter for image quality
            EncoderParameter qualityParam = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, quality);

            // Jpeg image codec
            ImageCodecInfo jpegCodec = getEncoderInfo("image/png");

            if (jpegCodec == null)
            {
                return;
            }

            EncoderParameters encoderParams = new EncoderParameters(1);
            encoderParams.Param[0] = qualityParam;

            img.Save(path, jpegCodec, encoderParams);
        }


        private static ImageCodecInfo getEncoderInfo(string mimeType)
        {
            // Get image codecs for all image formats
            ImageCodecInfo[] codecs = ImageCodecInfo.GetImageEncoders();

            // Find the correct image codec
            for (int i = 0; i <= codecs.Length - 1; i++)
            {
                if (codecs[i].MimeType == mimeType)
                {
                    return codecs[i];
                }
            }
            return null;
        }

        // Generate thumbs and midthumbs from photos
        public static string Generate_Thumbs(string path, string thumbpath, string midthumbpath, int thumbwidth, int thumbheight, int midthumbwidth)
        {
            if (!string.IsNullOrEmpty(thumbpath))
            {
                Bitmap mp = null;
                Copy_Photo(path, thumbpath);
                mp = CreateThumbnail(path, thumbwidth, thumbheight);
                if (mp == null)
                {
                    return "";
                }
                mp.Save(thumbpath);
                // Mid thumbs
                SaveJpeg(thumbpath, mp, Site_Settings.ThumbQuality);
                mp.Dispose();
            }
            if (!string.IsNullOrEmpty(midthumbpath))
            {
                Bitmap midmp = null;
                Copy_Photo(path, midthumbpath);
                midmp = CreateThumbnail(path, midthumbwidth);
                if (midmp == null)
                {
                    return "";
                }
                midmp.Save(midthumbpath);
                // Thumbs
                SaveJpeg(midthumbpath, midmp, Site_Settings.MediumThumbQuality);
                midmp.Dispose();
            }
            return "1";
        }

        public static string Generate_Thumbs(string path, string thumbpath, string midthumbpath, int thumbwidth, int thumbheight, int midthumbwidth, int midthumbheight)
        {
            if (!string.IsNullOrEmpty(thumbpath))
            {
                Bitmap mp = null;
                Copy_Photo(path, thumbpath);
                mp = CreateThumbnail(path, thumbwidth, thumbheight);
                if (mp == null)
                {
                    return "";
                }
                mp.Save(thumbpath);
                // Mid thumbs
                SaveJpeg(thumbpath, mp, Site_Settings.ThumbQuality);
                mp.Dispose();
            }
            if (!string.IsNullOrEmpty(midthumbpath))
            {
                Bitmap midmp = null;
                Copy_Photo(path, midthumbpath);
                midmp = CreateThumbnail(path, midthumbwidth, midthumbheight);
                if (midmp == null)
                {
                    return "";
                }
                midmp.Save(midthumbpath);
                // Thumbs
                SaveJpeg(midthumbpath, midmp, Site_Settings.MediumThumbQuality);
                midmp.Dispose();
            }
            return "1";
        }

        public static string Generate_Thumbs(string path, string thumbpath, string midthumbpath, string bannerPath, int thumbwidth, int thumbheight, int midthumbwidth, int midthumbheight, int bannerwidth, int bannerheight)
        {
            if (!string.IsNullOrEmpty(thumbpath))
            {
                Bitmap mp = null;
                Copy_Photo(path, thumbpath);
                mp = CreateThumbnail(path, thumbwidth, thumbheight);
                if (mp == null)
                {
                    return "";
                }
                mp.Save(thumbpath);
                // Mid thumbs
                SaveJpeg(thumbpath, mp, Site_Settings.ThumbQuality);
                mp.Dispose();
            }

            if (!string.IsNullOrEmpty(midthumbpath))
            {
                Jugnoon.Utility.Image.CreateCropThumbnail(path, Site_Settings.MediumThumbWidth, Site_Settings.MediumThumbHeight, midthumbpath);
                /* Bitmap midmp = null;
                 Copy_Photo(path, midthumbpath);
                 midmp = CreateThumbnail(path, midthumbwidth, midthumbheight);
                 if (midmp == null)
                 {
                     return "";
                 }
                 midmp.Save(midthumbpath);
                 // Thumbs
                 SaveJpeg(midthumbpath, midmp, Site_Settings.MediumThumbQuality);
                 midmp.Dispose();*/
            }

            if (!string.IsNullOrEmpty(bannerPath))
            {
                Bitmap bmp = null;
                Copy_Photo(path, bannerPath);
                bmp = CreateThumbnail(path, bannerwidth, bannerheight);
                if (bmp == null)
                {
                    return "";
                }
                bmp.Save(bannerPath);
                // Mid thumbs
                SaveJpeg(bannerPath, bmp, Site_Settings.MediumThumbQuality);
                bmp.Dispose();
            }

            return "1";
        }


        private static void Copy_Photo(string original_path, string new_path)
        {
            FileInfo TheFile = new FileInfo(original_path);
            if (TheFile.Exists)
            {
                File.Copy(original_path, new_path);
            }
            else
            {
                throw new FileNotFoundException();
            }
        }

        /* public void jCrop()
         {
             int X1 = Convert.ToInt32(Request.Form["x1"]);
             int Y1 = Convert.ToInt32(Request["y1"]);
             int X2 = Convert.ToInt32(Request.Form["x2"]);
             int Y2 = Convert.ToInt32(Request.Form["y2"]);
             int X = System.Math.Min(X1, X2);
             int Y = System.Math.Min(Y1, Y2);
             int w = Convert.ToInt32(Request.Form["w"]);
             int h = Convert.ToInt32(Request.Form["h"]);

             // That can be any image (jpg,jpeg,png,gif) from anywhere in the server
             string originalFile = Server.MapPath("~/images/miautito.jpg");


             using (System.Drawing.Image img = System.Drawing.Image.fromFile(originalFile))
             {
                 using (System.Drawing.Bitmap _bitmap = new System.Drawing.Bitmap(w, h))
                 {
                     _bitmap.SetResolution(img.HorizontalResolution, img.VerticalResolution);
                     using (Graphics _graphic = Graphics.fromImage(_bitmap))
                     {
                         _graphic.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                         _graphic.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                         _graphic.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
                         _graphic.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
                         _graphic.DrawImage(img, 0, 0, w, h);
                         _graphic.DrawImage(img, new Rectangle(0, 0, w, h), X, Y, w, h, GraphicsUnit.Pixel);

                         string extension = Path.GetExtension(originalFile);
                         string croppedFileName = Guid.NewGuid().ToString();
                         string path = Server.MapPath("~/cropped/");


                         // If the image is a gif file, change it into png
                         if (extension.EndsWith("gif", StringComparison.OrdinalIgnoreCase))
                         {
                             extension = ".png";
                         }

                         string newFullPathName = string.Concat(path, croppedFileName, extension);

                         using (EncoderParameters encoderParameters = new EncoderParameters(1))
                         {
                             encoderParameters.Param[0] = new EncoderParameter(Encoder.Quality, 100L);
                             _bitmap.Save(newFullPathName, GetImageCodec(extension), encoderParameters);
                         }
                     }
                 }
             }
         }

         public static ImageCodecInfo GetImageCodec(string extension)
         {
             extension = extension.ToUpperInvariant();
             ImageCodecInfo[] codecs = ImageCodecInfo.GetImageEncoders();
             foreach (ImageCodecInfo codec in codecs)
             {
                 if (codec.filenameExtension.Contains(extension))
                 {
                     return codec;
                 }
             }
             return codecs[1];
         } */
       
        public static bool SaveCroppedImage(System.Drawing.Image image, int maxWidth, int maxHeight, string filePath, bool targettop = false)
        {
            ImageCodecInfo jpgInfo = ImageCodecInfo.GetImageEncoders()
                                     .Where(codecInfo =>
                                     codecInfo.MimeType == "image/jpeg").First();
            System.Drawing.Image finalImage = image;
            System.Drawing.Bitmap bitmap = null;
            try
            {
                int left = 0;
                int top = 0;
                int srcWidth = maxWidth;
                int srcHeight = maxHeight;
                bitmap = new System.Drawing.Bitmap(maxWidth, maxHeight);
                double croppedHeightToWidth = (double)maxHeight / maxWidth;
                double croppedWidthToHeight = (double)maxWidth / maxHeight;

                if (image.Width > image.Height)
                {
                    srcWidth = (int)(Math.Round(image.Height * croppedWidthToHeight));
                    if (srcWidth < image.Width)
                    {
                        srcHeight = image.Height;
                        left = (image.Width - srcWidth) / 2;
                    }
                    else
                    {
                        srcHeight = (int)Math.Round(image.Height * ((double)image.Width / srcWidth));
                        srcWidth = image.Width;
                        top = (image.Height - srcHeight) / 2;
                    }
                }
                else
                {                    
                    srcHeight = (int)(Math.Round(image.Width * croppedHeightToWidth));
                    if (srcHeight < image.Height)
                    {
                        srcWidth = image.Width;
                        top = (image.Height - srcHeight) / 2;
                    }
                    else
                    {
                        srcWidth = (int)Math.Round(image.Width * ((double)image.Height / srcHeight));
                        srcHeight = image.Height;
                        left = (image.Width - srcWidth) / 2;
                    }

                    if (targettop)
                    {
                        // reset top / left = 0
                        top = 0;
                        left = 0;
                    }
                }
               
                using (Graphics g = Graphics.FromImage(bitmap))
                {
                    g.SmoothingMode = SmoothingMode.HighQuality;
                    g.PixelOffsetMode = PixelOffsetMode.HighQuality;
                    g.CompositingQuality = CompositingQuality.HighQuality;
                    g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                    g.DrawImage(image, new Rectangle(0, 0, bitmap.Width, bitmap.Height),
                    new Rectangle(left, top, srcWidth, srcHeight), GraphicsUnit.Pixel);
                }
                finalImage = bitmap;
            }
            catch {
            }
            try
            {
                using (EncoderParameters encParams = new EncoderParameters(1))
                {
                    encParams.Param[0] = new EncoderParameter(Encoder.Quality, (long)100);
                    //quality should be in the range 
                    //[0..100] .. 100 for max, 0 for min (0 best compression)
                    finalImage.Save(filePath, jpgInfo, encParams);
                    return true;
                }
            }
            catch { }
            if (bitmap != null)
            {
                bitmap.Dispose();
            }
            return false;
        }

        public static byte[] Generate_Online_Thumbnail(string lcFilename, int Width, int Height)
        {
            Bitmap loBMP = null;
            Bitmap bmpOut = null;
            try
            {
                loBMP = new Bitmap(lcFilename);
                ImageFormat loFormat = loBMP.RawFormat;
                Size newSize = new Size();
                newSize.Height = Height;
                newSize.Width = Width;
                bmpOut = new Bitmap(newSize.Width, newSize.Height);
                Graphics canvas = Graphics.FromImage(bmpOut);
                canvas.SmoothingMode = SmoothingMode.AntiAlias;
                canvas.InterpolationMode = InterpolationMode.HighQualityBicubic;
                canvas.PixelOffsetMode = PixelOffsetMode.HighQuality;
                canvas.DrawImage(loBMP, new Rectangle(new Point(0, 0), newSize));
            }
            catch (Exception ex)
            {

            }

            var imageStream = new MemoryStream();
            byte[] imageContent;
            if (lcFilename.StartsWith("http"))
            {
                string _fileName = lcFilename.Replace("thumbs/", "");
                // download and save image in memory stream
                var client = new WebClient();
                byte[] imageBytes = client.DownloadData(_fileName);
                imageContent = Jugnoon.Utility.Image.Generate_Online_Thumbnail(new MemoryStream(imageBytes), Width, Height);
                imageStream.Position = 0;
                imageStream.Read(imageContent, 0, (int)imageStream.Length);
            }
            else
            {
                bmpOut.Save(imageStream, System.Drawing.Imaging.ImageFormat.Jpeg);

                imageContent = new Byte[imageStream.Length];
                imageStream.Position = 0;
                imageStream.Read(imageContent, 0, (int)imageStream.Length);
            }
          
            return imageContent;
        }

        public static byte[] Generate_Online_Thumbnail(Stream lcFilename, int Width, int Height)
        {
            Bitmap loBMP = null;
            Bitmap bmpOut = null;
            try
            {
                loBMP = new Bitmap(lcFilename);
                ImageFormat loFormat = loBMP.RawFormat;
                Size newSize = new Size();
                newSize.Height = Height;
                newSize.Width = Width;
                bmpOut = new Bitmap(newSize.Width, newSize.Height);
                Graphics canvas = Graphics.FromImage(bmpOut);
                canvas.SmoothingMode = SmoothingMode.AntiAlias;
                canvas.InterpolationMode = InterpolationMode.HighQualityBicubic;
                canvas.PixelOffsetMode = PixelOffsetMode.HighQuality;
                canvas.DrawImage(loBMP, new Rectangle(new Point(0, 0), newSize));
            }
            catch (Exception ex)
            {

            }

            var imageStream = new MemoryStream();
            bmpOut.Save(imageStream, System.Drawing.Imaging.ImageFormat.Jpeg);

            byte[] imageContent = new Byte[imageStream.Length];
            imageStream.Position = 0;
            imageStream.Read(imageContent, 0, (int)imageStream.Length);
            return imageContent;
        }
    }
}