﻿using Jugnoon.Entity;
using Jugnoon.Settings;
using System.Web;

namespace Jugnoon.Utility
{
    public class CategoryUrlConfig
    {
        public static string PrepareUrl(string term, string path)
        {
            if (term == null || term == "")
                return "#";

            string query = UtilityBLL.ReplaceSpaceWithHyphin_v2(term.Trim().ToLower());

            return Config.GetUrl(path + "category/" + query);
        }

        public static string PrepareImageUrl(CategoryDataEntity entity)
        {
            if (entity.picturename == null || entity.picturename == "" || entity.picturename == "none")
                return getDefaultImageUrl();
            else if (entity.picturename.StartsWith("http"))
                return entity.picturename;
            else
                return Config.GetUrl("contents/category/" + entity.picturename);
        }

        public static string PrepareUrl(CategoryDataEntity entity, string path)
        {
            string _value = entity.term;
            if (_value == "")
                _value = entity.title;
            string query = UtilityBLL.ReplaceSpaceWithHyphin_v2(_value.Trim().ToLower());

            return Config.GetUrl(path + "category/" + query);
        }

        public static string BrowseAllUrl(string path)
        {
            return Config.GetUrl(path + "categories");
        }

        public static string getDefaultImageUrl()
        {
            return Config.GetUrl("assets/images/default.png");
        }
    }

    public class TagUrlConfig
    {
        public static string PrepareUrl(string term, string path)
        {
            if (term == null)
                return "#";

            string query = UtilityBLL.ReplaceSpaceWithHyphin_v2(term.Trim().ToLower());
            return Config.GetUrl(path + "tag/" + query);
        }

        public static string PrepareUrl(TagsDataEntity entity, string path)
        {
            var _tag = entity.term;
            if (_tag == "")
                _tag = entity.tagname;
            string query = UtilityBLL.ReplaceSpaceWithHyphin_v2(_tag.Trim().ToLower());

            return Config.GetUrl(path + "tag/" + query);
        }

        public static string BrowseAllUrl(string path)
        {           
            return Config.GetUrl(path + "taglist");
        }

    }

    public class ArchiveUrlConfig
    {
        public static string PrepareUrl(string monthname, int year, string path = "")
        {
            return Config.GetUrl(path + "archive/" + monthname + "/" + year);
        }

        public static string BrowseAllUrl(string path)
        {
            return Config.GetUrl(path + "archive");
        }

    }
    public class UrlConfig
    {
        public static string Upload_Path(string foldername)
        {
           return HttpContext.Current.Server.MapPath(HttpContext.Current.Request.ApplicationPath) + "contents/" + foldername;
        }
        public static string Upload_Path(string username, string foldername)
        {
            return HttpContext.Current.Server.MapPath(HttpContext.Current.Request.ApplicationPath) + "contents/member/" + username + "/" + foldername;
        }

        public static string Upload_URL(string username, string foldername)
        {
            return Config.GetUrl() + "contents/member/" + username + "/" + foldername;
        }
              
        // Return website logo url - url used for sending to Facebook (share
        public static string Return_Website_Logo_Url()
        {
            return Config.GetUrl("images/logo.gif");

        }

        public static string Prepare_Poll_Preview_Url(long id, string title, bool isadmin)
        {
            string _url = "";
            if (!isadmin)
                _url = Config.GetUrl("poll/" + id);
            else
                _url = Config.GetUrl("admin/poll/detail/" + id + "");
            return _url;
        }
        public static string Generate_Poll_Url(string id, string title, string month, string year)
        {
            string URLTITLE = "";
            int maxium_length = Site_Settings.Maxiumum_Dynamic_Url_Length;
            if (title.Length > maxium_length && maxium_length > 0)
                URLTITLE = title.Substring(0, maxium_length);
            else if (title.Length < 3)
                URLTITLE = "preview-post";
            else
                URLTITLE = title;

            URLTITLE = UtilityBLL.ReplaceSpaceWithHyphin(URLTITLE.Trim().ToLower());

            //return Config.GetUrl("event/" + id);
            return Config.GetUrl("poll/" + id + "/" + URLTITLE);
        }

        public static string Return_Gallery_Thumb_url(string username, string picturename, int type, int iscloud)
        {
            string URL = "";
            if (picturename.Contains("http"))
                URL = picturename;
            //else if (picturename != "none")
                //URL = PhotoUrlConfig.PreparePhotoUrl(username, picturename, type, iscloud);
            else
                URL = Config.GetUrl() + "assets/img/dmember.png";

            return URL;
        }

        /*************************************** */
        /************************** UPLOAD CONTENT PATH SETTINGS */
        // Photo Uploading Path
        public static string Photo_Upload_Path(string username, int type)
        {
            // type: 0 --/ Default
            // type: 1 --/ Mid thumb
            // type: 2 --/ Thumb
            string pathsettings = "";
            switch (type)
            {
                case 1:
                    pathsettings = "/midthumbs";
                    break;
                case 2:
                    pathsettings = "/thumbs";
                    break;
            }
            return HttpContext.Current.Server.MapPath("~/contents/member/" + username + "/photos" + pathsettings);
        }
        // Video Uploading Path
        public static string Video_Upload_Path(string username, string pathsettings)
        {
            //pathsettings = "/midthumbs";
            return HttpContext.Current.Server.MapPath("~/contents/member/" + username + "" + pathsettings);
        }

        public static string Return_Badge_Image(string Icon)
        {
            if(Icon == null)
                return Config.GetUrl("contents/images/default-badge.png");

            if (Icon.StartsWith("http"))
                return Icon;
            
            if (Icon == "" || Icon == "none")
                return Config.GetUrl("contents/images/default-badge.png");
            else
                return Config.GetUrl("contents/badges/" + Icon);
        }

         
       
    }
    
}