﻿using System;
using System.Collections.Generic;
using System.Text;
/* Site Own Namespaces */
using Jugnoon.Entity;
using Jugnoon.Utility;
using Jugnoon.Settings;
using System.Linq;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;

namespace Jugnoon.BLL
{
    public class PlayListBLL
    {
        // Note: PlayList Important Terms

        // isEnabled:
        // ........... 0: Disabled PlayList
        // ........... 1: Enable PlayList

        // isapproved:
        // ........... 0: Not approved
        // ........... 1: Approved

        // Privacy:
        // ........... 0: Private
        // ........... 1: Public


        // Videos:
        // Store Statistic for no of videos available in playlist

        public PlayListBLL()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        public static bool Add(string UserName, string Title)
        {
            if (Config.Get_ScreeningOption() == 1)
            {
                Title = DictionaryBLL.Process_Screening(Title);
            }
            
            using (var dbCtx = new mvcportalEntities())
            {
                var _entity = new user_playlists()
                {
                    username = UserName,
                    title = Title,
                    added_date = DateTime.Now
                };

                dbCtx.Entry(_entity).State = System.Data.Entity.EntityState.Added;

                DataBLL.SaveChanges(dbCtx);
                
            }
           
            return true;
        }

        public static bool Update(long PlayListID, string Title, string Description, string Tags, int Privacy)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.user_playlists
                     .Where(p => p.id == PlayListID)
                     .FirstOrDefault<user_playlists>();

                if(item != null)
                {
                    item.title = UtilityBLL.processNull(Title, 0);
                    item.description = UtilityBLL.processNull(Description, 0);
                    item.tags = Tags;
                    item.privacy = (byte)Privacy;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }
            
            return true;
        }

        public static bool Delete(long PlayListID)
        {
            // Delete all playlist videos
            PlayList_VideosBLL.Delete(PlayListID);
            // Delete playlist
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new user_playlists { id = PlayListID };
                dbCtx.user_playlists.Attach(entity);
                dbCtx.user_playlists.Remove(entity);
                dbCtx.SaveChanges();
            }
            
            return true;
        }

        public static bool Delete(long PlayListID, string UserName)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new user_playlists { id = PlayListID, username = UserName };
                dbCtx.user_playlists.Attach(entity);
                dbCtx.user_playlists.Remove(entity);
                dbCtx.SaveChanges();
            }
          
            return true;
        }

        public static string GetTitle(long PlayListID)
        {
            string Value = "";
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.user_playlists
                     .Where(p => p.id == PlayListID)
                     .FirstOrDefault<user_playlists>();

                Value = item.title;
            }

            return Value;

        }

        // update videos statistics, used to show no of videos available in current play list.
        public static bool Update_Video_Stat(long PlayListID, int Videos)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.user_playlists
                      .Where(p => p.id == PlayListID)
                      .FirstOrDefault<user_playlists>();

                if(item != null)
                {
                    item.videos = Videos;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }
          
            return true;
        }

        // enable, disable playlist
        public static bool Update_isEnabled(long PlayListID, int value)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.user_playlists
                      .Where(p => p.id == PlayListID)
                      .FirstOrDefault<user_playlists>();

                if(item != null)
                {
                    item.isenabled = (byte)value;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
              
            }
          
            return true;
        }

        // enable, disable playlist
        public static bool Update_isApproved(long PlayListID, int value)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.user_playlists
                      .Where(p => p.id == PlayListID)
                      .FirstOrDefault<user_playlists>();

                if(item != null)
                {
                    item.isapproved = (byte)value;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }
          
            return true;
        }


        // Load Playlist Information
        public static List<PlayListEntity> Fetch_PlayList(long PlayListID)
        {
            var items = new List<PlayListEntity>();
            using (var ctx = new mvcportalEntities())
            {
                items = ctx.user_playlists
                     .Where(p => p.id == PlayListID)
                     .Select(p => new PlayListEntity()
                     {
                         title = p.title,
                         description = p.description,
                         tags = p.tags,
                         privacy = p.privacy,
                         videos = p.videos,
                         username = p.username,
                         added_date = (DateTime)p.added_date,
                     }).ToList();
            }

            return items;
        }

        // Load user playlists
        public static List<PlayListEntity> Load_PlayLists(string UserName)
        {
            var items = new List<PlayListEntity>();
            using (var ctx = new mvcportalEntities())
            {
                items = ctx.user_playlists
                     .Where(p => p.username == UserName && p.isenabled == 1 && p.isapproved == 1)
                     .Select(p => new PlayListEntity()
                     {
                         playlistid = p.id,
                         title = p.title
                     }).ToList();
            }
            return items;
        }

        // Load user playlists // video preview section
        public static List<PlayListEntity> Load_Preview_PlayLists(string UserName)
        {
            var items = new List<PlayListEntity>();
            using (var ctx = new mvcportalEntities())
            {
                items = ctx.user_playlists
                     .Where(p => p.username == UserName && p.isenabled == 1 && p.isapproved == 1)
                     .Select(p => new PlayListEntity()
                     {
                         playlistid = p.id,
                         title = p.title,
                         videos = p.videos
                     }).ToList();
            }
            return items;
        }

        /*public static int Count_User_Playlist_Videos(long PlayListID)
        {
            var str = new StringBuilder();
            str.AppendLine("SELECT Count(v.videoid) FROM videos as v INNER JOIN playlist_videos as pl ON pl.videoid=v.videoid  WHERE pl.id=@playlistid AND v.isprivate=0 AND v.isenabled=1 AND v.ispublished=1 AND v.isreviewed=1");
            int Records = 0;
            using (var con = new SqlConnection(Config.ConnectionString))
            {
                con.Open();

                using (var cmd = new SqlCommand(str.ToString(), con))
                {
                    cmd.CommandType = CommandType.Text;
                    cmd.Parameters.Add(new SqlParameter("@playlistid", PlayListID));
                    Records = Convert.ToInt32(cmd.ExecuteScalar());
                }
            }

            return Records;
        }*/

    }
}