using System;
using System.Web;
using System.Collections.Generic;
using System.Text;
using System.Web.Caching;
using System.IO;
using System.Collections;
/* Site Own Namespaces */
using Jugnoon.Entity;
using Jugnoon.Utility;
using Jugnoon.Settings;
using Jugnoon.Scripts;
using System.Linq;
using JgnEnt.Framework;
using Jugnoon.EntityFramework;
using Google.Apis.YouTube.v3;
using Google.Apis.Services;
using System.Data.Entity.SqlServer;
using System.Text.RegularExpressions;

namespace Jugnoon.BLL
{
    public class VideoBLL
    {

        // Note: Important Video Terms

        // type:
        // .............. 0: Videos (Default)
        // .............. 1: Audio Files // If audio plugin enabled

        // isPrivate:
        // ........... 0: Public Video
        // ........... 1: Restricted Video
        // ........... 2: Unlisted Video

        // isEnabled:
        // ........... 0: Disabled Video
        // ........... 1: Enabled Video

        // isPublished:
        // ........... 0: Video not published or encoded yet
        // ........... 1: Video published or encoded properly

        // isReviewed:
        // ........... 0: Video not reviewed yet
        // ........... 1: Video reviewed by administrator

        // isComments:
        // ........... 0: Disabled comments
        // ........... 1: Enabled comments

        // isRatings:
        // ........... 0: Disabled ratings
        // ........... 1: Enabled ratings

        // isFeatured:
        // ........... 0: Normal Videos
        // ........... 1: Featured Videos   
        // ........... 3: Premium Videos

        // movieType:
        // ........... 0: Videos
        // ........... 1: Movies   
        // ........... 2: All

        // thumbPreview
        // if not empty it will act as thumb video rotator

        // isAdult:
        // ........... 0: Normal Video
        // ........... 1: Adult Video -> In case user redirected to adult content warning section. User must accept 18 year notification before continue.

        // For public video you must add the following filter action in your SQL query

        // isPriavate=0 AND isEnabled=1 AND isPublished=1 AND isReviewed=1


        // istagsreviewed (optional)
        // ............ 0: tags processed
        // ............ 1: not yet processed



        public enum MediaType
        {
            Videos = 0,
            Audio = 1
        };

        public VideoBLL()
        {

        }
        // Enable HTTP Streaming
        public static bool EnableHttpStreaming { get { return false; } }
        // Enable Html5 Player
        public static bool EnableHTML5Player { get { return false; } }
        // Enable Bandwidth Throttling
        public static bool EnableBandwidthTrottling { get { return true; } }
        // Set Bandwidth Rate e.g 100 -> 100kbps
        public static int MaxBandwidthSpeed { get { return 1500; } }
        // Set for Logged In User
        public static int MaxLogedInBandwidthSpeed { get { return 3000; } }

        // prepare bandwidth throtling parameter
        public static string GenerateBandwidthParam()
        {
            string maxParam = "";
            if (VideoBLL.EnableBandwidthTrottling)
            {
                if (HttpContext.Current.User.Identity.IsAuthenticated)
                {
                    if (VideoBLL.MaxLogedInBandwidthSpeed > 0) // For logged in Users -> may be higher bandwidth speed
                        maxParam = "&max=" + VideoBLL.MaxLogedInBandwidthSpeed;
                    else if (VideoBLL.MaxBandwidthSpeed > 0) // For normal users -> may be lower bandwidth speed
                        maxParam = "&max=" + VideoBLL.MaxBandwidthSpeed;
                }
                else
                {
                    if (VideoBLL.MaxBandwidthSpeed > 0)  // For normal users -> may be lower bandwidth speed
                        maxParam = "&max=" + VideoBLL.MaxBandwidthSpeed;
                }
            }
            return maxParam;
        }

        public static void Validation(VideoDataEntity gal)
        {
            // validation
            if (gal.title != null && gal.title.Length > 100)
                gal.title= gal.title.Substring(0, 99);

            if (gal.tags != null && gal.tags.Length > 200)
                gal.tags = gal.tags.Substring(0, 199);
        }

      
        #region Data Manipulation
        public static void Process_Video_File_Information(VideoDataEntity entity)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == entity.videoid)
                     .FirstOrDefault<video>();

                if(item != null)
                {
                    item.videofilename = UtilityBLL.processNull(entity.videofilename, 0);
                    item.thumbfilename = UtilityBLL.processNull(entity.thumbfilename, 0);
                    item.originalvideofilename = UtilityBLL.processNull(entity.originalvideofilename, 0);

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }
        }

        // Check video for authorization
        public static bool Check(long VideoID, string UserName)
        {
            if (VideoID == 0 | UserName == "")
                return false;

            bool flag = false;
            using (var ctx = new mvcportalEntities())
            {
                if (ctx.videos.Where(p => p.videoid == VideoID && p.username == UserName).Count() > 0)
                    flag = true;
            }

            return flag;
            
        }

        public static void Process_Video_Cloud_Information(VideoDataEntity entity)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == entity.videoid)
                     .FirstOrDefault<video>();

                if(item != null)
                {
                    item.flv_url = UtilityBLL.processNull(entity.flv_url, 0);
                    item.thumb_url = UtilityBLL.processNull(entity.thumb_url, 0);
                    item.org_url = UtilityBLL.processNull(entity.org_url, 0);

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
                
            }
        }

        // Process_Info() is used to post video information or update video information in database
        public static long Process_Info(VideoDataEntity entity)
        {
            if (Config.Get_ScreeningOption() == 1)
            {
                entity.tags = DictionaryBLL.Process_Screening(UtilityBLL.processNull(entity.tags, 0));
                entity.description = DictionaryBLL.Process_Screening(UtilityBLL.processNull(entity.description,0));
                entity.title= DictionaryBLL.Process_Screening(UtilityBLL.processNull(entity.title,0));
            }
           
            Validation(entity);

            entity.description = UtilityBLL.Process_Content(entity.description);

            if (entity.cloud_media_path == null)
                entity.cloud_media_path = "";
            if (entity.cloud_thumb_path == null)
                entity.cloud_thumb_path = "";
            if (entity.preview_url == null)
                entity.preview_url = "";
            if (entity.duration == null)
                entity.duration = "";
            if (entity.tags == null)
                entity.tags = "";
            if (entity.title == null)
                entity.title = "";

            long RecordID = 0;
            using (var dbCtx = new mvcportalEntities())
            {
                if(entity.videoid == 0)
                {
                    var _entity = new video()
                    {
                        categories = entity.categories,
                        username = entity.username,
                        title = entity.title,
                        description = entity.description,
                        tags = entity.tags,
                        date_added = DateTime.Now,
                        isprivate = (byte)entity.isprivate,
                        mode = (byte)entity.mode,
                        videofilename = entity.videofilename,
                        thumbfilename = entity.thumbfilename,
                        isenabled = (byte)entity.isenabled,
                        duration_sec = entity.duration_sec,
                        originalvideofilename = entity.originalvideofilename,
                        ispublished = (byte)entity.ispublished,
                        type = (byte)entity.type,
                        isratings = 1, // by default make rating available
                        iscomments = 1, // by default make comments available
                        isreviewed = (byte)entity.isreviewed,
                        flv_url = entity.flv_url,
                        org_url = entity.org_url,
                        thumb_url = entity.thumb_url,
                        embed_script = entity.embed_script,
                        isexternal = (byte)entity.isexternal,
                        ipaddress = entity.ipaddress,
                        youtubeid = entity.youtubeid,
                        istagsreviewed = (byte)entity.istagsreviewed,
                        authkey = entity.authkey,
                        galleryid = entity.galleryid,
                        preview_url = entity.preview_url,
                        language = "en"
                    };

                    dbCtx.Entry(_entity).State = System.Data.Entity.EntityState.Added;

                    DataBLL.SaveChanges(dbCtx);

                    RecordID = _entity.videoid;
                }
                else
                {
                    var item = dbCtx.videos
                        .Where(p => p.videoid == entity.videoid)
                        .FirstOrDefault<video>();

                    if(item != null)
                    {
                        item.categories = string.Join(",", entity.categories);
                        item.title = entity.title;
                        item.description = entity.description;
                        item.tags = entity.tags;
                        item.isprivate = (byte)entity.isprivate;
                        item.mode = (byte)entity.mode;

                        DataBLL.SaveChanges(dbCtx);
                    }
                   



                    RecordID = entity.videoid;
                }
               
            }
             
            if (RecordID > 0)
            {
                if (entity.isreviewed == 1 && entity.isprivate == 0 && entity.isenabled == 1 && entity.ispublished == 1)
                {
                    // update stats
                    Refresh_User_Stats(entity.username, entity.type);
                }
            }
            return RecordID;
        }

        // This update is only for pubplishing pending videos (unpublished videos only)
        public static bool Update(VideoDataEntity entity)
        {
            // validation
            Validation(entity);

            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == entity.videoid)
                     .FirstOrDefault<video>();

                if(item != null)
                {
                    item.duration = UtilityBLL.processNull(entity.duration, 0);
                    item.duration_sec = entity.duration_sec;
                    item.videofilename = UtilityBLL.processNull(entity.videofilename, 0);
                    item.thumbfilename = UtilityBLL.processNull(entity.thumbfilename, 0);
                    item.ispublished = (byte)entity.ispublished;
                    item.isenabled = (byte)entity.isenabled;
                    item.flv_url = entity.flv_url;
                    item.thumb_url = entity.thumb_url;
                    item.org_url = entity.org_url;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }
            if (entity.username != "")
            {
                // refresh user stats
                Refresh_User_Stats(entity.username, entity.type);
            }

            return true;
        }

        // Update video information - myaccount section - edit video
        public static bool Update_Video_Info(VideoDataEntity entity)
        {
            if (Config.Get_ScreeningOption() == 1)
            {
                entity.tags = DictionaryBLL.Process_Screening(entity.tags);
                entity.description = DictionaryBLL.Process_Screening(entity.description);
                entity.title= DictionaryBLL.Process_Screening(entity.title);
            }
            
            Validation(entity);
            
            if (entity.videoid == 0)
            {
                return false;
            }

            entity.description = UtilityBLL.Process_Content(entity.description);
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == entity.videoid)
                     .FirstOrDefault<video>();

                if(item != null)
                {
                    item.title = UtilityBLL.processNull(entity.title, 0);
                    item.description = UtilityBLL.processNull(entity.description, 0);
                    item.tags = UtilityBLL.processNull(entity.tags, 0);
                    item.isprivate = (byte)entity.isprivate;
                    item.iscomments = (byte)entity.iscomments;
                    item.isratings = (byte)entity.isratings;
                    if (entity.categories != null)
                        item.categories = entity.categories;
                    //item.mode = (byte)entity.mode;

                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
              
            }
            
            return true;
        }

        // Remove Video Data both from Database and folder (image, videos etc)
        public static void RemoveVideo(long VideoID, int type)
        {
            string flvvideoname = "";
            string originalvideoname = "";
            string thumbname = "";
            string username = "";

            var _lst = LoadItems(new VideoEntity()
            {
                videoid = VideoID
            });
            if (_lst.Count > 0)
            {
                flvvideoname = _lst[0].videofilename;
                originalvideoname = _lst[0].originalvideofilename;
                thumbname = _lst[0].thumbfilename;
                username =_lst[0].username;
            }

            // path of folder where video data (video, thumbs stored)
            string strPath = HttpContext.Current.Server.MapPath(HttpContext.Current.Request.ApplicationPath) + "/contents/member/" + username;

            // Delete Original Video | MP3 Audio
            if (originalvideoname != "")
            {
                if (File.Exists(strPath + "/Default/" + originalvideoname))
                    File.Delete(strPath + "/Default/" + originalvideoname);
            }

            // Delete FLV Video
            if (flvvideoname != "")
            {
                if (type == 0)
                {
                    // video -> flv
                    if (File.Exists(strPath + "/FLV/" + flvvideoname))
                        File.Delete(strPath + "/FLV/" + flvvideoname);
                }
                else
                {
                    // audio -> mp3
                    if (File.Exists(strPath + "/MP3/" + flvvideoname))
                        File.Delete(strPath + "/MP3/" + flvvideoname);
                }


            }

            // Delete Images
            string thumb_start_index = "";
            if (thumbname.Contains("_"))
            {
                thumb_start_index = thumbname.Remove(thumbname.LastIndexOf("_"));
                int total_thumbs = 15;
                int i = 0;
                for (i = 0; i <= total_thumbs; i++)
                {
                    string _thumbname = "";
                    if (i > 9)
                        _thumbname = thumb_start_index + "_0" + i + ".jpg";
                    else
                        _thumbname = thumb_start_index + "_00" + i + ".jpg";

                    if (File.Exists(strPath + "/Thumbs/" + _thumbname))
                        File.Delete(strPath + "/Thumbs/" + _thumbname);

                }
            }

            // Delete Record
            VideoBLL.Delete(VideoID, username, type);
            
        }

        public static void RemoveVideo(VideoDataEntity entity, long VideoID)
        {
            string flvvideoname = entity.videofilename;
            string originalvideoname = entity.originalvideofilename;
            string thumbname = entity.thumbfilename;
            string username = entity.username;
            
           
                // localstorage
                // path of folder where video data (video, thumbs stored)
                string strPath = HttpContext.Current.Server.MapPath(HttpContext.Current.Request.ApplicationPath) + "/contents/member/" + username;

                // Delete Original Video | MP3 Audio
                if (originalvideoname != "")
                {
                    if (File.Exists(strPath + "/Default/" + originalvideoname))
                        File.Delete(strPath + "/Default/" + originalvideoname);
                }

                // Delete FLV Video
                if (flvvideoname != "")
                {
                    if (entity.type == 0)
                    {
                        // video -> flv
                        if (File.Exists(strPath + "/FLV/" + flvvideoname))
                            File.Delete(strPath + "/FLV/" + flvvideoname);
                    }
                    else
                    {
                        // audio -> mp3
                        if (File.Exists(strPath + "/MP3/" + flvvideoname))
                            File.Delete(strPath + "/MP3/" + flvvideoname);
                    }


                }

                // Delete Images
                string thumb_start_index = "";
                if (thumbname.Contains("_"))
                {
                    thumb_start_index = thumbname.Remove(thumbname.LastIndexOf("_"));
                    int total_thumbs = 15;
                    int i = 0;
                    for (i = 0; i <= total_thumbs; i++)
                    {
                        string _thumbname = "";
                        if (i > 9)
                            _thumbname = thumb_start_index + "_0" + i + ".jpg";
                        else
                            _thumbname = thumb_start_index + "_00" + i + ".jpg";

                        if (File.Exists(strPath + "/Thumbs/" + _thumbname))
                            File.Delete(strPath + "/Thumbs/" + _thumbname);

                    }
                }
            
            // Delete Record
            VideoBLL.Delete(VideoID, username, entity.type);
 
        }

        // Rename published videos, original videos, and thumbs at once if required. mostly renaming required if you want to send files to cloud server in order to avoid file duplications.
        // Also update renamed files information in database.
        public static ArrayList RenameFileNames(long VideoID, string UserName, string FileName, string OrgFileName, string ThumbFileName)
        {
            // Rename published file path
            string dir_path = HttpContext.Current.Server.MapPath(HttpContext.Current.Request.ApplicationPath) + "\\contents\\member\\" + UserName + "\\";
            // apply userid in order to avoid duplication of content on cloud storage
            string UniqueGuidID = Guid.NewGuid().ToString().Substring(0, 6);
            string NewFileExtension = UserName + "-pub-" + UniqueGuidID;
            string NewFileName = NewFileExtension.ToLower() + "" + FileName.Remove(0, FileName.LastIndexOf("."));
            UtilityBLL.RenameFile(dir_path + "FLV\\" + FileName, dir_path + "FLV\\" + NewFileName);

            // Rename Original Video Path
            string NewOrgFileName = OrgFileName;
            if (Site_Settings.Video_Delete_Original)
            {
                // admin not want to delete original video
                NewFileExtension = UserName + "-org-" + UniqueGuidID;
                NewOrgFileName = NewFileExtension.ToLower() + "" + FileName.Remove(0, FileName.LastIndexOf("."));
                UtilityBLL.RenameFile(dir_path + "Default\\" + OrgFileName, dir_path + "Default\\" + NewOrgFileName);
            }

            // Rename ThumbFileNames
            // Thumbs available from 1 - 15
            // admin-uploaded-video-8e33c6_001
            string StartIndex = ThumbFileName.Remove(ThumbFileName.LastIndexOf("_"));
            string NewStartIndex = UserName + "-thumb-" + UniqueGuidID;
            string TFileName = StartIndex + "_";
            string NewThumbFileName = NewStartIndex.ToString() + "_";
            int i = 0; // 15 thumb need to be renamed
            int counter = 1;
            for (i = 0; i <= 14; i++)
            {
                string thumb = "";
                string newthumb = "";
                if (counter < 10)
                {
                    newthumb = NewThumbFileName + "00" + counter + ".jpg";
                    thumb = TFileName + "00" + counter + ".jpg";
                }
                else
                {
                    newthumb = NewThumbFileName + "0" + counter + ".jpg";
                    thumb = TFileName + "0" + counter + ".jpg";
                }
                counter++;
                UtilityBLL.RenameFile(dir_path + "Thumbs\\" + thumb, dir_path + "Thumbs\\" + newthumb);
            }

            // choose default thumb from 1 - 15 thumbs
            string DefaultThumbFileName = NewStartIndex.ToString() + "_008.jpg"; // eigth thumb (central one)
            if (!File.Exists(dir_path + "Thumbs\\" + DefaultThumbFileName))
                DefaultThumbFileName = NewStartIndex.ToString() + "_002.jpg"; // second thumb

            // update thumbs information in database
            var vd = new VideoDataEntity();
            vd.videofilename = NewFileName;
            if (NewOrgFileName.Length < 50)
            {
                vd.originalvideofilename = NewOrgFileName;
            }
            vd.thumbfilename = DefaultThumbFileName;
            vd.videoid = VideoID;
            VideoBLL.Process_Video_File_Information(vd);

            // add information in array for return
            ArrayList arr = new ArrayList();
            arr.Add(NewFileName);
            arr.Add(NewOrgFileName);
            arr.Add(DefaultThumbFileName);
            return arr;
        }

        // rename video filenames to avoid duplication and without storing in database
        public static ArrayList RenameFileNames(string UserName, string FileName, string OrgFileName, string ThumbFileName)
        {
            // Rename published file path
            string dir_path = HttpContext.Current.Server.MapPath(HttpContext.Current.Request.ApplicationPath) + "\\contents\\member\\" + UserName + "\\";
            // apply userid in order to avoid duplication of content on cloud storage
            string UniqueGuidID = Guid.NewGuid().ToString().Substring(0, 6);
            string NewFileExtension = UserName + "-pub-" + UniqueGuidID;
            string NewFileName = NewFileExtension.ToLower() + "" + FileName.Remove(0, FileName.LastIndexOf("."));
            UtilityBLL.RenameFile(dir_path + "FLV\\" + FileName, dir_path + "FLV\\" + NewFileName);

            // Rename Original Video Path
            string NewOrgFileName = OrgFileName;
            if (Site_Settings.Video_Delete_Original)
            {
                // admin not want to delete original video
                NewFileExtension = UserName + "-org-" + UniqueGuidID;
                NewOrgFileName = NewFileExtension.ToLower() + "" + FileName.Remove(0, FileName.LastIndexOf("."));
                UtilityBLL.RenameFile(dir_path + "Default\\" + OrgFileName, dir_path + "Default\\" + NewOrgFileName);
            }

            // Rename ThumbFileNames
            // Thumbs available from 1 - 15
            // admin-uploaded-video-8e33c6_001
            string StartIndex = ThumbFileName.Remove(ThumbFileName.LastIndexOf("_"));
            string NewStartIndex = UserName + "-thumb-" + UniqueGuidID;
            string TFileName = StartIndex + "_";
            string NewThumbFileName = NewStartIndex.ToString() + "_";
            int i = 0; // 15 thumb need to be renamed
            int counter = 1;
            for (i = 0; i <= 14; i++)
            {
                string thumb = "";
                string newthumb = "";
                if (counter < 10)
                {
                    newthumb = NewThumbFileName + "00" + counter + ".jpg";
                    thumb = TFileName + "00" + counter + ".jpg";
                }
                else
                {
                    newthumb = NewThumbFileName + "0" + counter + ".jpg";
                    thumb = TFileName + "0" + counter + ".jpg";
                }
                counter++;
                UtilityBLL.RenameFile(dir_path + "Thumbs\\" + thumb, dir_path + "Thumbs\\" + newthumb);
            }

            // choose default thumb from 1 - 15 thumbs
            string DefaultThumbFileName = NewStartIndex.ToString() + "_008.jpg"; // eigth thumb (central one)
            if (!File.Exists(dir_path + "Thumbs\\" + DefaultThumbFileName))
                DefaultThumbFileName = NewStartIndex.ToString() + "_002.jpg"; // second thumb

            // add information in array for return
            ArrayList arr = new ArrayList();
            arr.Add(NewFileName);
            arr.Add(NewOrgFileName);
            arr.Add(DefaultThumbFileName);
            return arr;
        }


        public static bool Update_VideoInfo_Adm(VideoDataEntity entity)
        {       
            if (Config.Get_ScreeningOption() == 1)
            {
                entity.tags = DictionaryBLL.Process_Screening(entity.tags);
                entity.description = DictionaryBLL.Process_Screening(entity.description);
                entity.title= DictionaryBLL.Process_Screening(entity.title);
            }
            
            Validation(entity);
            
            if (entity.videoid == 0)
            {
                return false;
            }

            entity.description = UtilityBLL.Process_Content(entity.description);

            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == entity.videoid)
                     .FirstOrDefault<video>();

                if(item != null)
                {
                    item.title = UtilityBLL.processNull(entity.title, 0);
                    item.description = UtilityBLL.processNull(entity.description, 0);
                    item.tags = UtilityBLL.processNull(entity.tags, 0);
                    item.isprivate = (byte)entity.isprivate;
                    item.iscomments = (byte)entity.iscomments;
                    item.isratings = (byte)entity.isratings;

                    if (entity.categories != null)
                        item.categories = entity.categories;
                    item.mode = (byte)entity.mode;
                    item.liked = entity.liked;
                    item.disliked = entity.disliked;
                    item.views = entity.views;
                    item.isadult = (byte)entity.isadult;
                    item.favorites = entity.favorites;
                    item.isreviewed = (byte)entity.isreviewed;
                    item.isenabled = (byte)entity.isenabled;
                    item.isfeatured = (byte)entity.isfeatured;
                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;

                    DataBLL.SaveChanges(ctx);
                }
                
            }
                        
            return true;
        }
        // for angular 5 app
        public static bool Update_VideoInfo_Adm_V2(VideoDataEntity entity)
        {
            if (Config.Get_ScreeningOption() == 1)
            {
                entity.tags = DictionaryBLL.Process_Screening(entity.tags);
                entity.description = DictionaryBLL.Process_Screening(entity.description);
                entity.title = DictionaryBLL.Process_Screening(entity.title);
            }

            Validation(entity);

            if (entity.videoid == 0)
            {
                return false;
            }

            entity.description = UtilityBLL.Process_Content(entity.description);

            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == entity.videoid)
                     .FirstOrDefault<video>();

                if (item != null)
                {
                    item.title = UtilityBLL.processNull(entity.title, 0);
                    item.description = UtilityBLL.processNull(entity.description, 0);
                    item.tags = UtilityBLL.processNull(entity.tags, 0);

                    if (entity.categories != null)
                        item.categories = entity.categories;
                  
                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;

                    DataBLL.SaveChanges(ctx);
                }

            }

            return true;
        }


        // approve or reject all gallery videos or audio
        public static void Update_isApproved_All(long GalleryID, string UserName, int Status, int Type)
        {
            var _approve = 1;
            if (Status == 1)
                _approve = 0;

            using (var dbCtx = new mvcportalEntities())
            {
                var videos = dbCtx.videos.Where(p => p.galleryid == GalleryID && p.isreviewed == _approve && p.type == Type).ToList();
                videos.ForEach(a => a.isreviewed = (byte)Status);
                dbCtx.SaveChanges();
            }
            
            // update user stats
            Refresh_User_Stats(UserName, Type);
        }

        // enable / disable all gallery videos or audio
        public static void Update_isEnabled_All(long GalleryID, string UserName, int Status, int Type)
        {
            var _approve = 1;
            if (Status == 1)
                _approve = 0;

            using (var dbCtx = new mvcportalEntities())
            {
                var videos = dbCtx.videos.Where(p => p.galleryid == GalleryID && p.isenabled == _approve && p.type == Type).ToList();
                videos.ForEach(a => a.isenabled = (byte)Status);
                dbCtx.SaveChanges();
            }

            // update user stats
            Refresh_User_Stats(UserName, Type);
        }

        // for cloud storage
        public static bool Update_Selected_Thumb(long VideoID, string ThumbFileName, string Thumb_Url)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == VideoID)
                     .FirstOrDefault<video>();

                item.thumbfilename = UtilityBLL.processNull(ThumbFileName, 0);
                item.thumb_url = UtilityBLL.processNull(Thumb_Url, 0);
               
                ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                DataBLL.SaveChanges(ctx);
            }
            return true;
        }

        // Delete video where statistic update is required
        public static bool Delete(long VideoID, string UserName, int type)
        {            
            // Delete Video
            Delete(VideoID);

            if (UserName != "")
            {
                // update user stats
                Refresh_User_Stats(UserName, type);
            }


            return true;
        }

        // Delete video where no statistic update is required
        // Mostly in case of youtube embed videos
        public static bool Delete(long VideoID)
        {
            using (var dbCtx = new mvcportalEntities())
            {
                var entity = new video { videoid = VideoID };
                dbCtx.videos.Attach(entity);
                dbCtx.videos.Remove(entity);
                dbCtx.SaveChanges();
            }
            
            return true;
        }

        public static bool Update_IsEnabled(long VideoID, int OldValue, int NewValue, string UserName, int type)
        {
            Update_Field_V3(VideoID, (byte)NewValue, "isEnabled");
            // update user stats
            Refresh_User_Stats(UserName, type);
            return true;
        }

        public static bool Update_IsPrivate(long VideoID, int OldValue, int NewValue, string UserName, int type)
        {
            Update_Field_V3(VideoID, (byte)NewValue, "isPrivate");
            // update user stats
            Refresh_User_Stats(UserName, type);
            return true;
        }

        public static bool Update_IsPublished(long VideoID, int Value, string UserName, int type)
        {
            Update_Field_V3(VideoID, (byte)Value, "ispublished");
            // update user stats
            Refresh_User_Stats(UserName, type);
            return true;
        }

        public static bool Update_IsReviewed(long VideoID, int OldValue, int NewValue, string UserName, int type)
        {
            Update_Field_V3(VideoID, (byte)NewValue, "isreviewed");
            // update user stats
            Refresh_User_Stats(UserName, type);

            if (NewValue == 1)
            {
                // send notification mail to user when record approved by admin
                string url = "<a href=\"" + Config.GetUrl("myaccount/videos/") + "\">Manage Videos</a>";
                MailProcess.User_Mail_Content_Action(UserName, "videos", url, "approved");
            }
            return true;
        }

        public bool Review_Video(long VideoID)
        {
            bool flg = false;
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == VideoID)
                     .FirstOrDefault<video>();

                if (item.isenabled == 1 && item.ispublished == 1 && item.isreviewed == 1)
                    flg = true;
            }
            return flg;
        }
        
        #endregion
        
        #region Perform Actions

        // count user enabled videos
        public static int Count(string username, int type)
        {
            int Records = 0;
            using (var ctx = new mvcportalEntities())
            {
                Records = ctx.videos.Where(p => p.username == username && p.type == type && p.isprivate == 0 && p.isenabled == 1 && p.ispublished == 1 && p.isreviewed == 1).Count();
            }
            return Records;
        }
        
        public static bool Update_Field_V3(long ID, dynamic Value, string FieldName)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == ID)
                     .FirstOrDefault<video>();

                if(item != null)
                {
                    foreach (var prop in item.GetType().GetProperties())
                    {
                        if (prop.Name.ToLower() == FieldName.ToLower())
                        {
                            prop.SetValue(item, Value);
                        }
                    }
                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }
            return true;
        }

        public static bool Update_Field_V3(long ID, DateTime Value, string FieldName)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == ID)
                     .FirstOrDefault<video>();

                if(item != null)
                {
                    foreach (var prop in item.GetType().GetProperties())
                    {
                        if (prop.Name.ToLower() == FieldName.ToLower())
                        {
                            prop.SetValue(item, Value);
                        }
                    }
                    ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                    DataBLL.SaveChanges(ctx);
                }
               
            }
            return true;

        }

        public static string Get_Field_Value(long ID, string FieldName)
        {
            string Value = "";
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == ID)
                     .FirstOrDefault<video>();

                if(item != null)
                {
                    foreach (var prop in item.GetType().GetProperties())
                    {
                        if (prop.Name.ToLower() == FieldName.ToLower())
                        {
                            if (prop.GetValue(item, null) != null)
                                Value = prop.GetValue(item, null).ToString();
                        }
                    }
                }
                
            }

            return Value;
        }

        public static long Get_Album_Media(long AlbumID, int type)
        {
            long Value = 0;
            using (var ctx = new mvcportalEntities())
            {
                var query = ctx.videos.Where(p => p.galleryid == AlbumID && p.type == type && p.isenabled == 1 && p.isreviewed == 1);
                if (query.Count() > 0)
                    Value =query.Max(p => p.videoid);
            }
            return Value;
        }

        // process video rating
        public static double Post_Rating(long VideoID, int current_rating, int total_ratings, double ratings)
        {
            // increment 
            total_ratings = total_ratings + 1;
            ratings = ratings + current_rating;
            double average_rating = (double)Math.Round(ratings / total_ratings, 2);
            // update rating
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == VideoID)
                     .FirstOrDefault<video>();

                item.ratings = (float)ratings;
                item.total_rating = total_ratings;
                item.avg_rating = (float)average_rating;

                ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                DataBLL.SaveChanges(ctx);
            }
          
            return average_rating;
        }

        public static bool Validate_VideoID(long VideoID)
        {
            bool flag = false;
            using (var ctx = new mvcportalEntities())
            {
                if (ctx.videos.Where(p => p.videoid == VideoID && p.isenabled == 1 && p.isprivate == 1 && p.ispublished == 1 && p.isreviewed == 1).Count() > 0)
                    flag = true;
            }
            return flag;
            
        }

        #endregion


        #region Core Loading Script

        public static List<VideoDataEntity> LoadItems(VideoEntity entity)
        {
            if (!entity.iscache || Config.GetCacheDuration() == 0 || entity.pagenumber > 5)
            {              
                return FetchItems(entity);
            }
            else
            {
                string key = GenerateKey("lg_video_1_", entity);
                if (CloudSettings.EnableRedisCache)
                    return _Fetch_Redis_Cache(entity, key);
                else
                    return _Fetch_Normal_Cache(entity, key);
                
            }
        }

        private static List<VideoDataEntity> FetchItems(VideoEntity entity)
        {
            var _list = new List<VideoDataEntity>();
            using (var entities = new mvcportalEntities())
            {
                entity.iscount = false; // load data
                if (entity.loadliked)
                {
                    _list = LoadLikedItems(entity);
                }
                else if (entity.loadfavorites)
                {
                    _list = LoadFavorited(entity);
                }
                else if (entity.loadPlayList)
                {
                    _list = LoadPlayListItems(entity);
                }
                else
                {
                    // No INNER JOIN
                    var Query = GetLoadQuery(entities, entity);
                    if (entity.videoid > 0)
                    {
                        entity.loadall = true;
                        _list = LoadCompleteList(Query);
                    }
                    else if (entity.issummary)
                    {
                        _list = LoadSummaryList(Query);
                    }
                    else if (entity.isdropdown)
                    {
                        _list = LoadDropdownList(Query);
                    }
                    else
                    {
                        _list = LoadCompleteList(Query);
                    }
                }
            }
               

            return _list;
        }

        public static int Count(VideoEntity entity)
        {
            if (!entity.iscache || Site_Settings.Cache_Duration == 0 || entity.pagenumber > 5)
            {
                return CountRecords(entity);
            }
            else
            {
                string key = GenerateKey("cnt_videos_1", entity);
                if (CloudSettings.EnableRedisCache)
                    return _Count_Redis_Cache(entity, key);
                else
                    return _Count_Normal_Cache(entity, key);
                            
            }
        }

        private static int CountRecords(VideoEntity entity)
        {
            int Records = 0;
            using (var entities = new mvcportalEntities())
            {
                if (entity.loadliked)
                {
                    Records = CountLikedItems(entity);
                }
                else if (entity.loadfavorites)
                {
                    Records = CountFavorited(entity);
                }
                else if (entity.loadPlayList)
                {
                    Records = CountPlaylist(entity);
                }
                else
                {
                    IQueryable<video> Query = entities.videos;
                    entity.iscount = true;
                    entity.videoid = 0;
                    Query = BuildQuery(Query, entity);
                    Records = Query.Count();
                }
                
            }
            return Records;
        }

        public static string GenerateKey(string key, VideoEntity vd)
        {
            var str = new StringBuilder();
            str.AppendLine(key + "_" + vd.categoryid + "" + vd.movietype + "" + vd.isfeatured + "" + vd.type + "" + vd.username.ToLower() + "" + vd.month + "" + vd.year + "" + vd.filter + "" + vd.isenabled + "" + vd.isreviewed + "" + vd.isprivate + "" + vd.isadult + "" + vd.isexternal + "" + vd.datefilter + "" + UtilityBLL.ReplaceSpaceWithHyphin(vd.order.ToLower()) + "" + vd.pagenumber + "" + vd.mode + "" + vd.pagesize + "" + vd.loadall);
            if (vd.term != "")
                str.AppendLine(UtilityBLL.ReplaceSpaceWithHyphin(vd.term.ToLower()));
            if (vd.tags != "")
                str.AppendLine(UtilityBLL.ReplaceSpaceWithHyphin(vd.tags.ToLower()));
            if (vd.categories.Length > 0)
                str.AppendLine(UtilityBLL.ReplaceSpaceWithHyphin(string.Join("_",vd.categories)));
            return str.ToString();
        }

        /* customizable select version*/
        public static IQueryable<video> GetLoadQuery(mvcportalEntities ctx, VideoEntity entity)
        {
            IQueryable<video> Query;
            Query = ctx.videos;
            Query = BuildQuery(Query, entity);
            return Query;
        }

        private static IQueryable<video> BuildQuery(IQueryable<video> query, VideoEntity entity)
        {
            query = Where(query, entity);
            if (entity.videoid == 0 && !entity.iscount)
            {
                query = OrderBy(query, entity);
                query = Skip(query, entity);
                query = Limit(query, entity);
            }
            return query;
        }

        private static int CountLikedItems(VideoEntity entity)
        {
            int Records = 0;

            using (var ctx = new mvcportalEntities())
            {
                var Query = ctx.videos
                .Join(ctx.user_ratings,
                 video => video.videoid,
                 rating => rating.itemid,
                 (video, rating) => new { video, rating });

                Query = Query.Where(p => p.rating.type == 0 && p.rating.rating == 0 && p.video.ispublished == 1 && p.video.isreviewed == 1 && p.video.isenabled == 1 && p.video.isprivate == 0);
                if (entity.username != "")
                    Query = Query.Where(x => x.rating.username == entity.username);

                Records = Query.Count();
            }

            return Records;
        }

        private static List<VideoDataEntity> LoadLikedItems(VideoEntity entity)
        {
            var _list = new List<VideoDataEntity>();

            using (var ctx = new mvcportalEntities())
            {
                // Load Liked -> Who Rated Posts ( r.type=9 AND r.rating=0)
                var Query = ctx.videos
                .Join(ctx.user_ratings,
                 video => video.videoid,
                 rating => rating.itemid,
                 (video, rating) => new { video, rating });

                Query = Query.Where(p => p.rating.type == 0 && p.rating.rating == 0 && p.video.ispublished == 1 && p.video.isreviewed == 1 && p.video.isenabled == 1 && p.video.isprivate == 0);
                if (entity.username != "")
                    Query = Query.Where(x => x.rating.username == entity.username);

                if (entity.order != "")
                {
                    var orderlist = entity.order.Split(char.Parse(","));
                    foreach (var order in orderlist)
                    {
                        if (order.Contains("galleryid"))
                        {
                            if (order.Contains("desc"))
                                Query = Query.OrderByDescending(p => p.video.galleryid);
                            else
                                Query = Query.OrderBy(p => p.video.galleryid);
                        }
                        else if (order.Contains("videoid"))
                        {
                            if (order.Contains("desc"))
                                Query = Query.OrderByDescending(p => p.video.videoid);
                            else
                                Query = Query.OrderBy(p => p.video.videoid);
                        }
                        else if (order.Contains("title"))
                        {
                            if (order.Contains("desc"))
                                Query = Query.OrderByDescending(p => p.video.title);
                            else
                                Query = Query.OrderBy(p => p.video.title);
                        }
                    }
                }
                if (entity.pagenumber > 1)
                    Query = Query.Skip(entity.pagesize * (entity.pagenumber - 1));

                if (!entity.loadall)
                    Query = Query.Take(entity.pagesize);

                _list = Query
                     .Select(p => new VideoDataEntity
                     {
                         videoid = p.video.videoid,
                         galleryid = p.video.galleryid,
                         username = p.video.username,
                         title = p.video.title,
                         isadult = (byte)p.video.isadult,
                         created_at = (DateTime)p.video.date_added,
                         isenabled = p.video.isenabled,
                         views = p.video.views,
                         isfeatured = (byte)p.video.isfeatured,
                         duration = p.video.duration,
                         iscomments = p.video.iscomments,
                         isreviewed = p.video.isreviewed,
                         liked = p.video.liked,
                         disliked = p.video.disliked,
                         duration_sec = p.video.duration_sec,
                         isprivate = p.video.isprivate,
                         mode = p.video.mode,
                         total_rating = p.video.total_rating,
                         ratings = p.video.ratings,
                         avg_rating = p.video.avg_rating,
                         favorites = p.video.favorites,
                         thumbfilename = p.video.thumbfilename,
                         isratings = (byte)p.video.isratings,
                         isexternal = (byte)p.video.isexternal,
                         ispublished = p.video.ispublished,
                         thumb_url = p.video.thumb_url,
                         errorcode = p.video.errorcode,
                         youtubeid = p.video.youtubeid,
                         price = p.video.price,
                         coverurl = p.video.coverurl,
                         thumb_preview = p.video.thumb_preview
                     }).ToList();
            }
            return _list;
        }

        private static int CountFavorited(VideoEntity entity)
        {
            int Records = 0;

            using (var ctx = new mvcportalEntities())
            {
                // Load Favorited ->
                var Query = ctx.videos
                .Join(ctx.favorites,
                 video => video.videoid,
                 favorite => favorite.videoid,
                 (video, favorite) => new { video, favorite });

                Query = Query.Where(p => p.favorite.type == 0 && p.video.ispublished == 1 && p.video.isreviewed == 1 && p.video.isenabled == 1 && p.video.isprivate == 0);
                if (entity.username != "")
                    Query = Query.Where(x => x.favorite.username == entity.username);

                Records = Query.Count();
            }

            return Records;
        }

        private static List<VideoDataEntity> LoadFavorited(VideoEntity entity)
        {
            var _list = new List<VideoDataEntity>();

            using (var ctx = new mvcportalEntities())
            {
                // Load Favorited ->
                var Query = ctx.videos
                .Join(ctx.favorites,
                 video => video.videoid,
                 favorite => favorite.videoid,
                 (video, favorite) => new { video, favorite });

                Query = Query.Where(p => p.favorite.type == 0 && p.video.ispublished == 1 && p.video.isreviewed == 1 && p.video.isenabled == 1 && p.video.isprivate == 0);
                if (entity.username != "")
                    Query = Query.Where(x => x.favorite.username == entity.username);

                if (entity.order != "")
                {
                    var orderlist = entity.order.Split(char.Parse(","));
                    foreach (var order in orderlist)
                    {
                        if (order.Contains("galleryid"))
                        {
                            if (order.Contains("desc"))
                                Query = Query.OrderByDescending(p => p.video.galleryid);
                            else
                                Query = Query.OrderBy(p => p.video.galleryid);
                        }
                        if (order.Contains("videoid"))
                        {
                            if (order.Contains("desc"))
                                Query = Query.OrderByDescending(p => p.video.videoid);
                            else
                                Query = Query.OrderBy(p => p.video.videoid);
                        }
                        else if (order.Contains("title"))
                        {
                            if (order.Contains("desc"))
                                Query = Query.OrderByDescending(p => p.video.title);
                            else
                                Query = Query.OrderBy(p => p.video.title);
                        }
                    }
                }
                if (entity.pagenumber > 1)
                    Query = Query.Skip(entity.pagesize * (entity.pagenumber - 1));

                if (!entity.loadall)
                    Query = Query.Take(entity.pagesize);

                _list = Query
                     .Select(p => new VideoDataEntity
                     {
                         videoid = p.video.videoid,
                         galleryid = p.video.galleryid,
                         username = p.video.username,
                         title = p.video.title,
                         isadult = (byte)p.video.isadult,
                         created_at = (DateTime)p.video.date_added,
                         isenabled = p.video.isenabled,
                         views = p.video.views,
                         isfeatured = (byte)p.video.isfeatured,
                         duration = p.video.duration,
                         iscomments = p.video.iscomments,
                         isreviewed = p.video.isreviewed,
                         liked = p.video.liked,
                         disliked = p.video.disliked,
                         duration_sec = p.video.duration_sec,
                         isprivate = p.video.isprivate,
                         mode = p.video.mode,
                         total_rating = p.video.total_rating,
                         ratings = p.video.ratings,
                         avg_rating = p.video.avg_rating,
                         favorites = p.video.favorites,
                         thumbfilename = p.video.thumbfilename,
                         isratings = (byte)p.video.isratings,
                         isexternal = (byte)p.video.isexternal,
                         ispublished = p.video.ispublished,
                         thumb_url = p.video.thumb_url,
                         errorcode = p.video.errorcode,
                         youtubeid = p.video.youtubeid,
                         coverurl = p.video.coverurl,
                         price = p.video.price,
                         thumb_preview = p.video.thumb_preview
                     }).ToList();
            }

            return _list;
        }

        private static int CountPlaylist(VideoEntity entity)
        {
            int Records = 0;

            using (var ctx = new mvcportalEntities())
            {
                // Load Favorited ->
                var Query = ctx.videos
                .Join(ctx.playlist_videos,
                 video => video.videoid,
                 playlist => playlist.videoid,
                 (video, playlist) => new { video, playlist });

                Query = Query.Where(p => p.video.isenabled == 1 && p.video.isreviewed == 1 && p.video.isprivate == 0 && p.video.ispublished == 1);
                if (entity.playlistid > 0)
                    Query = Query.Where(x => x.playlist.id == entity.playlistid);

                Records = Query.Count();
            }

            return Records;
        }

        private static List<VideoDataEntity> LoadPlayListItems(VideoEntity entity)
        {
            var _list = new List<VideoDataEntity>();

            using (var ctx = new mvcportalEntities())
            {
                // Load Favorited ->
                var Query = ctx.videos
                .Join(ctx.playlist_videos,
                 video => video.videoid,
                 playlist => playlist.videoid,
                 (video, playlist) => new { video, playlist });

                Query = Query.Where(p => p.video.isenabled == 1 && p.video.isreviewed == 1 && p.video.isprivate == 0 && p.video.ispublished == 1);
                if (entity.playlistid > 0)
                    Query = Query.Where(x => x.playlist.id == entity.playlistid);

                if (entity.order != "")
                {
                    var orderlist = entity.order.Split(char.Parse(","));
                    foreach (var order in orderlist)
                    {
                        if (order.Contains("galleryid"))
                        {
                            if (order.Contains("desc"))
                                Query = Query.OrderByDescending(p => p.video.galleryid);
                            else
                                Query = Query.OrderBy(p => p.video.galleryid);
                        }
                        if (order.Contains("videoid"))
                        {
                            if (order.Contains("desc"))
                                Query = Query.OrderByDescending(p => p.video.videoid);
                            else
                                Query = Query.OrderBy(p => p.video.videoid);
                        }
                        else if (order.Contains("title"))
                        {
                            if (order.Contains("desc"))
                                Query = Query.OrderByDescending(p => p.video.title);
                            else
                                Query = Query.OrderBy(p => p.video.title);
                        }
                    }
                }
                if (entity.pagenumber > 1)
                    Query = Query.Skip(entity.pagesize * (entity.pagenumber - 1));

                if (!entity.loadall)
                    Query = Query.Take(entity.pagesize);

                _list = Query
                     .Select(p => new VideoDataEntity
                     {
                         videoid = p.video.videoid,
                         galleryid = p.video.galleryid,
                         username = p.video.username,
                         title = p.video.title,
                         isadult = (byte)p.video.isadult,
                         created_at = (DateTime)p.video.date_added,
                         isenabled = p.video.isenabled,
                         views = p.video.views,
                         isfeatured = (byte)p.video.isfeatured,
                         duration = p.video.duration,
                         iscomments = p.video.iscomments,
                         isreviewed = p.video.isreviewed,
                         liked = p.video.liked,
                         disliked = p.video.disliked,
                         duration_sec = p.video.duration_sec,
                         isprivate = p.video.isprivate,
                         mode = p.video.mode,
                         total_rating = p.video.total_rating,
                         ratings = p.video.ratings,
                         avg_rating = p.video.avg_rating,
                         favorites = p.video.favorites,
                         thumbfilename = p.video.thumbfilename,
                         isratings = (byte)p.video.isratings,
                         isexternal = (byte)p.video.isexternal,
                         ispublished = p.video.ispublished,
                         thumb_url = p.video.thumb_url,
                         errorcode = p.video.errorcode,
                         youtubeid = p.video.youtubeid,
                         coverurl = p.video.coverurl,
                         price = p.video.price,
                         thumb_preview = p.video.thumb_preview
                     }).ToList();
            }

            return _list;
        }

        private static List<VideoDataEntity> LoadCompleteList(IQueryable<video> query)
        {
            //var list = UserBLL.TestLoad();
            return query.Select(p => new VideoDataEntity
            {
                videoid = p.videoid,
                galleryid = p.galleryid,
                username = p.username,
                title = p.title,
                description = p.description,
                tags = p.tags,
                categories = p.categories,
                isadult = (byte)p.isadult,
                created_at = (DateTime)p.date_added,
                isenabled = p.isenabled,
                views = p.views,
                comments = p.comments,
                isfeatured = (byte)p.isfeatured,
                duration = p.duration,
                iscomments = p.iscomments,
                isreviewed = p.isreviewed,
                liked = p.liked,
                disliked = p.disliked,
                duration_sec = p.duration_sec,
                isprivate = p.isprivate,
                mode = p.mode,
                total_rating = p.total_rating,
                ratings = p.ratings,
                avg_rating = p.avg_rating,
                favorites = p.favorites,
                videofilename = p.videofilename,
                thumbfilename = p.thumbfilename,
                originalvideofilename = p.originalvideofilename,
                embed_script = p.embed_script,
                isratings = (byte)p.isratings,
                isexternal = (byte)p.isexternal,
                ispublished = p.ispublished,
                flv_url = p.flv_url,
                org_url = p.org_url,
                thumb_url = p.thumb_url,
                errorcode = p.errorcode,
                youtubeid = p.youtubeid,
                istagsreviewed = p.istagsreviewed,
                downloads = p.downloads,
                coverurl = p.coverurl,
                preview_url = p.preview_url,
                price = p.price,
                actors = p.actors,
                actresses = p.actresses,
                movietype = p.movietype,
                streamoutputs = p.streamoutputs,
                thumb_preview = p.thumb_preview
            }).ToList();

        }

        private static List<VideoDataEntity> LoadSummaryList(IQueryable<video> query)
        {
            var _query = query.Select(p => new VideoDataEntity
            {
                videoid = p.videoid,
                galleryid = p.galleryid,
                username = p.username,
                title = p.title,
                isadult = (byte)p.isadult,
                created_at = (DateTime)p.date_added,
                isenabled = p.isenabled,
                views = p.views,
                isfeatured = (byte)p.isfeatured,
                duration = p.duration,
                iscomments = p.iscomments,
                isreviewed = p.isreviewed,
                liked = p.liked,
                disliked = p.disliked,
                duration_sec = p.duration_sec,
                isprivate = p.isprivate,
                mode = p.mode,
                total_rating = p.total_rating,
                ratings = p.ratings,
                avg_rating = p.avg_rating,
                favorites = p.favorites,
                thumbfilename = p.thumbfilename,
                isratings = (byte)p.isratings,
                isexternal = (byte)p.isexternal,
                ispublished = p.ispublished,
                thumb_url = p.thumb_url,
                errorcode = p.errorcode,
                youtubeid = p.youtubeid,
                price = p.price,
                coverurl = p.coverurl,
                thumb_preview = p.thumb_preview
            });

            return _query.ToList();
        }

        private static List<VideoDataEntity> LoadDropdownList(IQueryable<video> query)
        {
            return query.Select(p => new VideoDataEntity
            {
                videoid = (int)p.videoid,
                title = p.title
            }).ToList();
        }

        private static IQueryable<video> Where(IQueryable<video> query, VideoEntity entity)
        {
            if (entity.excludedid > 0)
                query = query.Where(p => p.videoid != entity.excludedid);

            if (entity.videoid > 0)
                query = query.Where(p => p.videoid == entity.videoid);

            if (entity.galleryid > 0)
                query = query.Where(p => p.galleryid == entity.galleryid);
            
            if (!entity.nofilter)
            {
                if (entity.type != 3)
                    query = query.Where(p => p.type == entity.type);

                if (entity.movietype != 2)
                    query = query.Where(p => p.movietype == entity.movietype);

                if (entity.categories != null)
                {
                    /* var predicate = PredicateBuilder.False<video>();

                    foreach (string category in entity.categories)
                    {
                        if (category != "")
                        {
                            var _filter = category.Trim();
                            predicate = predicate.Or(p => p.categories == _filter);
                            //query = query.Where(p => p.categories == _filter);
                        }
                    }
                    if(entity.categories.Length > 0 && entity.categories[0] != "")
                    {
                        query = (IQueryable<video>)query.AsEnumerable().Where(predicate.Compile());
                    } */

                    foreach (string category in entity.categories)
                    {
                        if (category != "")
                        {
                            var _filter = category.Trim();
                            query = query.Where(p => p.categories.Contains(_filter));
                        }
                    }
                }
                if (entity.tags != "" && entity.tags != null)
                {
                    foreach (var tag in entity.tags.Split(char.Parse(",")))
                    {
                        if (tag != "")
                        {
                            var filter = tag.Trim();
                            query = query.Where(p => p.tags.Contains(filter));
                        }
                    }
                }
                if (entity.actresses != "" && entity.actresses != null)
                {
                    foreach (var actress in entity.actresses.Split(char.Parse(",")))
                    {
                        if (actress != "")
                        {
                            var filter = actress.Trim();
                            query = query.Where(p => p.actresses == filter);
                        }
                    }
                }
                if (entity.actors != "" && entity.actors != null)
                {
                    foreach (var actor in entity.actresses.Split(char.Parse(",")))
                    {
                        if (actor != "")
                        {
                            var filter = actor.Trim();
                            query = query.Where(p => p.actors == filter);
                        }
                    }
                }
                if(entity.price > 0)
                {
                    query = query.Where(p => p.price > entity.price);
                }
                if (entity.username != "" && entity.username != null)
                {
                    query = query.Where(p => p.username == entity.username);
                }

                if (entity.mode > 0)
                    query = query.Where(p => p.mode == entity.mode);

                if (entity.isfeatured != 3)
                    query = query.Where(p => p.isfeatured == entity.isfeatured);

                if (entity.term != "" && entity.term != null)
                    query = query.Where(p => p.title.Contains(entity.term) || p.description.Contains(entity.term) || p.tags.Contains(entity.term));

                if (entity.month > 0 && entity.year > 0)
                    query = query.Where(p => p.date_added.Value.Month == entity.month && p.date_added.Value.Year == entity.year);
                else if (entity.year > 0)
                    query = query.Where(p => p.date_added.Value.Year == entity.year);
                else if (entity.month > 0)
                    query = query.Where(p => p.date_added.Value.Month == entity.month);
                if(entity.datefilter > 0)
                {
                    switch(entity.datefilter)
                    {
                        case 1:
                            // today record
                            query = query.Where(at => SqlFunctions.DateDiff("DAY", at.date_added, DateTime.Now) == 0);
                            break;
                        case 2:
                            // this week record
                            query = query.Where(at => SqlFunctions.DateDiff("DAY", at.date_added, DateTime.Now) >= 0 && SqlFunctions.DateDiff("DAY", at.date_added, DateTime.Now) < 7);
                            break;
                        case 3:
                            // this month record
                            query = query.Where(at => SqlFunctions.DateDiff("DAY", at.date_added, DateTime.Now) >= 0 && SqlFunctions.DateDiff("DAY", at.date_added, DateTime.Now) < 31);
                            break;
                    }
                }
                if (entity.ispublic)
                    query = query.Where(p => p.ispublished == 1 && p.isreviewed == 1 && p.isenabled == 1 && p.isprivate == 0);
                else
                {
                    // custom settings
                    if (entity.isenabled != 2)
                        query = query.Where(p => p.isenabled == entity.isenabled);

                    if (entity.isreviewed != 2)
                        query = query.Where(p => p.isreviewed == entity.isreviewed);

                    if (entity.isprivate != 3)
                        query = query.Where(p => p.isprivate == entity.isprivate);

                    if (entity.ispublished != 2)
                        query = query.Where(p => p.ispublished == entity.ispublished);
                }

                if (entity.isadult != 2)
                    query = query.Where(p => p.isadult == entity.isadult);


                if (entity.isexternal != 3)
                    query = query.Where(p => p.isexternal == entity.isexternal);

                if (entity.iscomments != 2)
                    query = query.Where(p => p.iscomments == entity.iscomments);

                if (entity.istagsreviewed != 2)
                    query = query.Where(p => p.istagsreviewed == entity.istagsreviewed);

                if (entity.maxid > 0)
                    query = query.Where(p => p.videoid > entity.maxid);

                if (entity.minid > 0)
                    query = query.Where(p => p.videoid < entity.minid);
            }
            

            return query;
        }

        private static IQueryable<video> OrderBy(IQueryable<video> query, VideoEntity entity)
        {
            if (entity.order != "")
            {
                var orderlist = entity.order.Split(char.Parse(","));
                foreach (var order in orderlist)
                {
                    if (order.Contains("galleryid"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.galleryid).OrderBy(p => p.date_added);
                        else
                            query = query.OrderBy(p => p.galleryid).OrderByDescending(p => p.date_added);

                        return query;
                    }
                    else if (order.Contains("videoid"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.videoid);
                        else
                            query = query.OrderBy(p => p.videoid);

                        return query;
                    }
                    else if (order.Contains("date_added"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.date_added);
                        else
                            query = query.OrderBy(p => p.date_added);

                        return query;
                    }
                    else if (order.Contains("title"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => p.title).OrderBy(p => p.date_added);
                        else
                            query = query.OrderBy(p => new { p.title }).OrderByDescending(p => p.date_added);

                        return query;
                    }
                    else if (order.Contains("views"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => new { p.views, p.date_added });
                        else
                            query = query.OrderBy(p => new { p.views, p.date_added });
                        return query;
                    }
                    else if (order.Contains("username"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => new { p.username, p.date_added });
                        else
                            query = query.OrderBy(p => p.username).OrderByDescending(p => p.date_added);
                        return query;
                    }
                    else if (order.Contains("liked"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => new { p.liked, p.views, p.date_added });
                        else
                            query = query.OrderBy(p => p.liked).OrderByDescending(p => p.date_added);
                        return query;
                    }
                    else if (order.Contains("isenabled"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => new { p.isenabled, p.date_added });
                        else
                            query = query.OrderBy(p => p.isenabled).OrderByDescending(p => p.date_added);
                        return query;
                    }
                    else if (order.Contains("isreviewed"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => new { p.isreviewed, p.date_added });
                        else
                            query = query.OrderBy(p => p.isreviewed).OrderByDescending(p => p.date_added);
                        return query;
                    }
                    else if (order.Contains("avg_rating"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => new { p.avg_rating, p.date_added });
                        else
                            query = query.OrderBy(p => p.avg_rating).OrderByDescending(p => p.date_added);
                        return query;
                    }
                    else if (order.Contains("total_ratings"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => new { p.total_rating, p.date_added });
                        else
                            query = query.OrderBy(p => p.total_rating).OrderByDescending(p => p.date_added);
                        return query;
                    }
                    else if (order.Contains("favorites"))
                    {
                        if (order.Contains("desc"))
                            query = query.OrderByDescending(p => new { p.favorites, p.date_added });
                        else
                            query = query.OrderBy(p => p.favorites).OrderByDescending(p => p.date_added);
                        return query;
                    }
                    
                }
            }

            return query;
        }
        private static IQueryable<video> Skip(IQueryable<video> query, VideoEntity entity)
        {
            if (entity.pagenumber > 1 && entity.pagesize > 0)
                query = query.Skip(entity.pagesize * (entity.pagenumber - 1));

            return query;
        }
        private static IQueryable<video> Limit(IQueryable<video> query, VideoEntity entity)
        {
            if (!entity.loadall && entity.pagesize > 0)
                query = query.Take(entity.pagesize);

            return query;
        }

        #endregion
         

        public static List<ArchiveEntity> Load_Arch_List(int type, int records, bool isall)
        {
            // cache implementation
            int cache_duration = Config.GetCacheDuration();
            if (cache_duration == 0) // no cache
                return Fetch_Archive_List(type, records, isall);
            else
            {
                string cache_key = "ld_vd_arc_lst_" + type + "" + records + "" + isall;
                if (HttpContext.Current.Cache[cache_key] == null)
                    HttpContext.Current.Cache.Add(cache_key, Fetch_Archive_List(type, records, isall), null, DateTime.Now.AddMinutes(cache_duration), TimeSpan.Zero, CacheItemPriority.High, null);

                List<ArchiveEntity> _list = (List<ArchiveEntity>)(HttpContext.Current.Cache[cache_key]);
                return _list;
            }

        }
        public static List<ArchiveEntity> Fetch_Archive_List(int type, int records, bool isall)
        {
            var context = new mvcportalEntities();
            var model = context.videos
                .GroupBy(o => new
                {
                    Month = o.date_added.Value.Month,
                    Year = o.date_added.Value.Year
                })
                .Select(g => new ArchiveEntity
                {
                    Month = g.Key.Month,
                    Year = g.Key.Year,
                    Total = g.Count()
                })
                .OrderByDescending(a => a.Year)
                .ThenByDescending(a => a.Month)
                .ToList();

            return model;
        }

        #region Youtube SDK
       
        public static bool Check_YOUTUBE_ID(string youtubeid)
        {
            bool flag = false;
            using (var ctx = new mvcportalEntities())
            {
                if (ctx.videos.Where(p => p.youtubeid == youtubeid).Count() > 0)
                    flag = true;
            }

            return flag;
        }

        public static bool Check_YOUTUBE_ID(string youtubeid, int filter)
        {
            bool flag = false;
            using (var ctx = new mvcportalEntities())
            {
                IQueryable<video> query = ctx.videos.Where(p => p.youtubeid == youtubeid);
                /*if (filter > 2 && filter < 0)
                    query.Append(" AND " + Config.Prepare_DateDiff("date_added", filter)); */ // to be adjust in linq
                if (query.Count() > 0)
                    flag = true;


            }
            return flag;

        }


        // Fetch un processed video tags
        public static long Count_InactiveTag_Videos()
        {
            int Records = 0;
            using (var ctx = new mvcportalEntities())
            {
                IQueryable<video> query = ctx.videos.Where(p => p.isenabled == 1 && p.ispublished == 1 && p.isreviewed == 1 && p.istagsreviewed == 1);
                Records = query.Count();


            }
            return Records;
            
        }

        public static List<VideoDataEntity> Fetch_Inactive_Tags(int pagenumber, int pagesize)
        {
            return LoadItems(new VideoEntity()
            {
                istagsreviewed = 1,
                pagenumber = pagenumber,
                pagesize = pagesize
            });
        }

        // update un process tag status to process
        public static bool Update_Unreviewed_Tag_Status(long videoid)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == videoid)
                     .FirstOrDefault<video>();

                item.istagsreviewed = 0; 
                ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                DataBLL.SaveChanges(ctx);
            }

            return true;
        }
        
        #endregion

      
        // Select Max Video ID within Gallery For Uploading new photos within gallery
        public static long MaxVideoID(long GalleryID)
        {
            long maxvidid = 0;
            using (var ctx = new mvcportalEntities())
            {
                maxvidid = ctx.videos.Where(p => p.galleryid == GalleryID).Max(p => p.videoid);
            }
            return maxvidid;
        }

        public static long MaxVideoID(string username)
        {
            long maxvidid = 0;
            using (var ctx = new mvcportalEntities())
            {
                maxvidid = ctx.videos.Where(p => p.username == username).Max(p => p.videoid);
            }
            return maxvidid;
        }


        // used to update photo info at time of final submission
        public static bool Update_Info(VideoDataEntity entity, bool updateCategories = false)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == entity.videoid)
                     .FirstOrDefault<video>();

                item.title = UtilityBLL.processNull(entity.title, 0);
                item.description = UtilityBLL.processNull(entity.description, 0);
                item.tags = UtilityBLL.processNull(entity.tags, 0);
                if (updateCategories)
                    item.categories = string.Join(",", entity.categories);
               
                ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                DataBLL.SaveChanges(ctx);
            }

            return true;
        }

        public static void Update_Unpublished_Video_Info(VideoDataEntity entity)
        {
            using (var ctx = new mvcportalEntities())
            {
                var item = ctx.videos
                     .Where(p => p.videoid == entity.videoid)
                     .FirstOrDefault<video>();

                item.videofilename = UtilityBLL.processNull(entity.videofilename, 0);
                item.thumbfilename = UtilityBLL.processNull(entity.thumbfilename, 0);
                item.ispublished = (byte)entity.ispublished;
                item.flv_url = entity.flv_url;
                item.thumb_url = entity.thumb_url;
                item.org_url = entity.org_url;
                item.errorcode = (byte)entity.errorcode;
                
                ctx.Entry(item).State = System.Data.Entity.EntityState.Modified;
                DataBLL.SaveChanges(ctx);
            }
        }
             
        public static void Refresh_User_Stats(string UserName, int type)
        {
            string field = "stat_videos";
            if (type == 1)
                field = "stat_audios";
            var count = CountRecords(new VideoEntity()
            {
                username = UserName,
                ispublic = true
            });
            UserBLL.Update_Field_V3(UserName, field, count);
        }
        
        #region Sitemaps


        public static string generateVideoSitemap(VideoEntity Entity)
        {
            var str = new StringBuilder();
            str.AppendLine("<urlset xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\" xmlns:video=\"http://www.google.com/schemas/sitemap-video/1.1\">");
            var _lst = LoadItems(Entity);
            foreach (var Item in _lst)
            {
                string mediaUrl = "";
                if (Item.type == 1)
                    mediaUrl = Config.Return_MP3_Audio_Url(Item.flv_url, Item.username) + "/" + Item.videofilename;
                else
                    mediaUrl = Config.Return_FLV_Video_Url(Item.flv_url, Item.username) + "/" + Item.videofilename;


                string thumburl = Config.Return_Video_Thumb_Url(Item.thumb_url, Item.username) + "/" + Item.thumbfilename;

                string videourl = VideoUrlConfig.PrepareUrl(Item);
                string albumurl = "";
                if (Entity.galleryid > 0)
                    albumurl = AlbumUrlConfig.PrepareUrl(new AlbumDataEntity()
                    {
                        galleryid = Entity.galleryid,
                        title = "preview-album"
                    }, 0);

                str.AppendLine("<url>");
                str.AppendLine("<loc>" + VideoUrlConfig.PrepareUrl(Item) + "</loc>");
                str.AppendLine("<video:video>");
                str.AppendLine("<video:thumbnail_loc>" + thumburl + "</video:thumbnail_loc>");
                str.AppendLine("<video:title>" + Item.title+ "</video:title>");
                if (Item.description != "")
                    str.AppendLine("<video:description>" + Item.description + "</video:description>");

                str.AppendLine("<video:content_loc>" + Item.flv_url + "</video:content_loc>");
                //str.AppendLine("<video:player_loc allow_embed=\"yes\" autoplay=\"ap=1\">");
                if (Item.duration_sec > 0)
                    str.AppendLine("<video:duration>" + Item.duration_sec + "</video:duration>");

                //str.AppendLine("<video:expiration_date>2009-11-05T19:20:30+08:00</video:expiration_date>");
                if (Item.avg_rating > 0)
                    str.AppendLine("<video:rating>" + Item.avg_rating + "</video:rating>");
                if (Item.views > 0)
                    str.AppendLine("<video:view_count>" + Item.views + "</video:view_count>");

                str.AppendLine("<video:publication_date>" + String.Format("{0:R}", Item.created_at) + "</video:publication_date>");
                string _friendly = "yes";
                if (Item.isadult == 1)
                    _friendly = "no";

                str.AppendLine("<video:family_friendly>" + _friendly + "</video:family_friendly>");
                if (albumurl != "")
                    str.AppendLine("<video:gallery_loc title=\"album preview\">" + albumurl + "</video:gallery_loc>");

                //str.AppendLine("<video:price currency="EUR">1.99</video:price>");
                string _issubscription = "no";
                if (Item.isfeatured == 3)
                    _issubscription = "yes";

                str.AppendLine("<video:requires_subscription>" + _issubscription + "</video:requires_subscription>");
                str.AppendLine("<video:uploader info=\"" + UserUrlConfig.ProfileUrl(Item.username) + "\">");
                str.AppendLine("</video:uploader>");
                //str.AppendLine("<video:live>no</video:live>");
                str.AppendLine("</video:video> ");
                str.AppendLine("</url> ");
            }
            str.AppendLine("</urlset>");

            return str.ToString();
        }

        public static string generateGoogleSitemap(VideoEntity Entity)
        {
            var str = new StringBuilder();
            str.AppendLine("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            str.AppendLine("<urlset xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\"");
            str.AppendLine(" xmlns:image=\"http://www.google.com/schemas/sitemap-image/1.1\"");
            str.AppendLine(" xmlns:video=\"http://www.google.com/schemas/sitemap-video/1.1\">");
            var _lst = LoadItems(Entity);
            foreach (var Item in _lst)
            {
                string mediaUrl = "";
                if (Item.type == 1)
                    mediaUrl = Config.Return_MP3_Audio_Url(Item.flv_url, Item.username) + "/" + Item.videofilename;
                else
                    mediaUrl = Config.Return_FLV_Video_Url(Item.flv_url, Item.username) + "/" + Item.videofilename;


                string thumburl = Config.Return_Video_Thumb_Url(Item.thumb_url, Item.username) + "/" + Item.thumbfilename;

                str.AppendLine("<url>");
                str.AppendLine("<loc>" + VideoUrlConfig.PrepareUrl(Item) + "</loc>");
                str.AppendLine("<image:image>");
                str.AppendLine("<image:loc>" + thumburl + "</image:loc>");
                str.AppendLine("<image:caption>" + Item.title+ "</image:caption>");
                str.AppendLine("</image:image>");
                str.AppendLine("<video:video>");
                str.AppendLine("<video:content_loc>");
                str.AppendLine(mediaUrl);
                str.AppendLine("</video:content_loc>");

                str.AppendLine("<video:thumbnail_loc>");
                str.AppendLine(thumburl);
                str.AppendLine("</video:thumbnail_loc>");
                str.AppendLine("<video:title>" + Item.title+ "</video:title>");

                if(Item.description != "")
                {
                    str.AppendLine("<video:description>");
                    str.AppendLine(Item.description);
                    str.AppendLine("</video:description>");
                }
                str.AppendLine("</video:video>");
              str.AppendLine("</url>");
            }
            str.AppendLine("</urlset>");
           
            return str.ToString();
        }

        public static string GenerateBingSitemap(VideoEntity Entity)
        {
            var str = new StringBuilder();
            str.AppendLine("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            str.AppendLine("<urlset xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\">");
            var _lst = LoadItems(Entity);
            foreach (var Item in _lst)
            {
                str.AppendLine("<url>");
                str.AppendLine("<loc>" + VideoUrlConfig.PrepareUrl(Item) + "</loc>");
                str.Append("</url>");
            }
            str.AppendLine("</urlset>");

            return str.ToString();
        }
        #endregion

        #region Feeds
        
        public static string generateRSS(VideoEntity Entity)
        {
            var str = new StringBuilder();
            str.AppendLine("<rss version=\"2.0\">");
            str.AppendLine("<channel>");
            str.AppendLine("<title>" + Config.GetWebsiteTitle() + "</title>");
            str.AppendLine("<description>" + Config.GetWebsiteDescription() + "</description>");
            str.AppendLine("<link>" + Config.GetUrl() + "</link>");
            str.AppendLine("<guid>" + Config.GetUrl() + "videos/" + "</guid>");
            var _lst = LoadItems(Entity);
            foreach (var Item in _lst)
            {
                string title_url = VideoUrlConfig.PrepareUrl(Item);
                string body = HttpContext.Current.Server.HtmlEncode(UtilityBLL.StripHTML_v2(Item.description));
                str.AppendLine("<item>");
                str.AppendLine("<title>" + UtilityBLL.CleanBlogHTML(UtilityBLL.StripHTML(Item.title)) + "</title>");
                str.AppendLine("<link>" + title_url + "</link>");
                str.AppendLine("<guid>" + title_url + "</guid>");
                str.AppendLine("<pubDate>" + String.Format("{0:R}", Item.created_at) + "</pubDate>");
                str.AppendLine("<description>" + body + "</description>");
                str.AppendLine("</item>");
            }
            str.AppendLine("</channel>");
            str.AppendLine("</rss>");

            return str.ToString();
        }

        public static string generateATOM(VideoEntity Entity, string url)
        {
            var str = new StringBuilder();
            str.AppendLine("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
            str.AppendLine("<feed xmlns=\"http://www.w3.org/2005/Atom\">\n");
            str.AppendLine("<title type=\"text\">" + Site_Settings.Website_Title + "</title>\n");
            str.AppendLine("<subtitle type=\"html\">" + Site_Settings.Website_Description + "</subtitle>");
            str.AppendLine("<id>tag:" + Config.GetUrl() + "," + DateTime.Now.Year + ":3</id>");
            str.AppendLine("<link rel=\"alternate\" type=\"text/html\" hreflang=\"en\" href=\"" + Config.GetUrl("videos/?type=atom") + "\"/>");
            str.AppendLine("<link rel=\"self\" type=\"application/atom+xml\" href=\"" + url + "\"/>");
            str.AppendLine("<rights>" + Site_Settings.Website_Title + "</rights>");
            str.AppendLine("<generator uri=\"" + Config.GetUrl("videos/") + "\" version=\"1.0\">");
            str.AppendLine("</generator>");
            var _lst = LoadItems(Entity);
            foreach (var Item in _lst)
            {
                string title_url = VideoUrlConfig.PrepareUrl(Item);
                string body = HttpContext.Current.Server.HtmlEncode(UtilityBLL.StripHTML_v2(Item.description));

                str.AppendLine("<entry>");
                str.AppendLine("<title type=\"text\">" + Item.title+ "</title>");
                str.AppendLine("<link rel=\"alternate\" type=\"text/html\" href=\"" + title_url + "\"/>");
                str.AppendLine("<id>tag:" + Config.GetUrl() + "," + Item.created_at.Value.Year + ":3." + Item.videoid + "</id>\n");
                str.AppendLine("<updated>" + String.Format("{0:R}", Item.created_at) + "</updated>\n");
                str.AppendLine("<published>" + String.Format("{0:R}", Item.created_at) + "</published>\n");
                str.AppendLine("<author>\n");
                str.AppendLine("<name>" + Item.username + "</name>\n");
                str.AppendLine("<uri>" + Config.GetUrl("videos/") + "</uri>\n");
                str.AppendLine("</author>\n");
                str.AppendLine("<content type=\"html\">" + body + "</content>\n");
                str.AppendLine("</entry>\n");
            }
            str.AppendLine("</feed>");

            return str.ToString();
        }
        #endregion

        public static bool ValidateTitle(List<VideoDataEntity> videos)
        {
            foreach (var video in videos)
            {
                if (video.title == null || video.title == "")
                {
                    return false;
                }
            }
            return true;
        }
        public static bool ValidateCategory(List<VideoDataEntity> videos)
        {
            foreach (var video in videos)
            {
                if (video.categories == null || video.categories == "")
                {
                    return false;
                }
            }
            return true;
        }

        public static object upload(string UserID)
        {
            if (HttpContext.Current.Request.Files.Count > 0)
            {
                int chunk = HttpContext.Current.Request["chunk"] != null ? int.Parse(HttpContext.Current.Request["chunk"]) : 0;
                string fileName = HttpContext.Current.Request["name"] != null ? HttpContext.Current.Request["name"] : string.Empty;

                string _userID = "";
                if (UserID != "")
                    _userID = UserID;
                else
                {
                    if (HttpContext.Current.Request.Headers["UName"] != null)
                        _userID = HttpContext.Current.Request.Headers["UName"].ToString();
                }
                
                string RootPath = HttpContext.Current.Server.MapPath(HttpContext.Current.Request.ApplicationPath);
                if (!Directory.Exists(RootPath + "/contents/member/" + _userID))
                {
                    Directory_Process.CreateRequiredDirectories(RootPath + "/contents/member/" + _userID);
                }

                string dirPath = UrlConfig.Upload_Path(_userID, "default");

                FileStream fs;
                using (fs = new FileStream(Path.Combine(dirPath, fileName), chunk == 0 ? FileMode.Create : FileMode.Append))
                {
                    byte[] buffer = new byte[HttpContext.Current.Request.Files[0].InputStream.Length];
                    HttpContext.Current.Request.Files[0].InputStream.Read(buffer, 0, buffer.Length);

                    fs.Write(buffer, 0, buffer.Length);
                }

                string url = VideoUrlConfig.Source_Video_Url(_userID) + "/" + fileName;
                string fileType = System.IO.Path.GetExtension(fileName);
                string fileIndex = fileName.Replace(fileType, "");
                string elementid = Guid.NewGuid().ToString().Substring(0, 6);

                return new { jsonrpc = "2.0", result = "OK", fname = fileName, url = url, filetype = fileType, filename = fileName, fileIndex = fileIndex, eleid = elementid };
            }
            else
            {
                return new { result = "Error" };
            }
        }

        public static object ffmpeg_proc(List<SaveVideoInfo> data)
        {
            foreach (var _rec in data)
            {
                _rec.isenabled = 1; // by default enabled
                if (Site_Settings.Content_Approval == 1)
                    _rec.isreviewed = 1;
                else
                    _rec.isreviewed = 0;

                foreach(var thumb in _rec.thumbs)
                {
                    if (thumb.selected)
                    {
                        _rec.thumbfilename = Path.GetFileName(thumb.filename);
                        _rec.thumb_url = thumb.filename;
                    }
                       
                }
                
                string _publish_path = "none";
                string _thumb_path = "none";
                string _preview_path = "none";
                string _source_path = "none";

                string _preview_filename = ""; // not yet included in new approach

                var Item = new VideoDataEntity()
                {
                    title = _rec.title,
                    description = _rec.description,
                    tags = _rec.tags,
                    galleryid = _rec.galleryid,
                    categoryid = 0,
                    categories = string.Join(",",_rec.categories),
                    duration = _rec.duration,
                    originalvideofilename = _rec.sf,
                    videofilename = _rec.pf,
                    thumbfilename = Path.GetFileName(_rec.tfile),
                    isenabled = _rec.isenabled,
                    isreviewed = _rec.isreviewed,
                    username = _rec.username,
                    type = _rec.type,
                    duration_sec = _rec.dursec,
                    isprivate = _rec.privacy,
                    authkey = "",
                    flv_url = _publish_path,
                    thumb_url = _thumb_path,
                    org_url = _source_path,
                    isexternal = 0,
                    youtubeid = "",
                    istagsreviewed = 1,
                    mode = 0,
                    ispublished = 1,
                    errorcode = _rec.errorcode,
                    ipaddress = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"].ToString(),
                    embed_script = "",
                    // submittype = 2 // ffmpeg target type
                };
                             

                Item.isreviewed = 1;
                // Approved by Default
                if (Site_Settings.Content_Approval == 0)
                {
                    // Moderator Review Required
                    Item.isreviewed = 0;
                }
                //XSS CLEANUP
                // not working on medium trust
                //string content = Sanitizer.GetSafeHtmlFragment(txt_content.Value);
                // Compress Code => remove \r\t etc which create unnecessary line breaking issues
                string content = UtilityBLL.CompressCode(Item.description);

                // Process Contents -> links, bbcodes etc
                content = UtilityBLL.Process_Content_Text(content);


                // Generate Gallery Preview
                //Content = GalleryBLLC.Generate_Blog_Gallery_Previews(Content)


                Item.videoid = VideoBLL.Process_Info(Item);
                
                              
                if (Item.tags != "")
                {
                    int tag_type = 0; // represent videos
                    if (Item.type == 1)
                        tag_type = 4; // represent audio file
                    TagsBLL.Process_Tags(Item.tags, tag_type, 0);
                }

                if (Item.galleryid > 0)
                {
                    var albumType = 1;
                    if (Item.type == 1)
                        albumType = 2;
                    int count = VideoBLL.Count(new VideoEntity()
                    {
                        type = albumType,
                        galleryid = Item.galleryid,
                        nofilter = true
                    });

                    GalleryBLLC.Update_Field_V3(Item.galleryid, (short)count, "total_photos");

                    // update cover
                    var _value = GalleryBLLC.Get_Field_Value(Item.galleryid, "customcover");
                    int customcover = 0;
                    if (_value != "")
                        customcover = Convert.ToInt32(_value);
                    
                    if (customcover == 0 && Item.type == 0) // only video, audio use default photo at this point
                    {
                        if (Item.thumb_url != "")
                        {
                            GalleryBLLC.Update_Field_V3(Item.galleryid, Item.thumb_url, "picturename");
                        }
                        else if (Item.thumbfilename != "none" || Item.thumbfilename != "")
                        {
                            GalleryBLLC.Update_Field_V3(Item.galleryid, Item.thumbfilename, "picturename");
                        }
                    }
                }
            }
            return new { status = "success", message = "Record processed successfully" };
        }

        public static object direct_proc(List<SaveVideoInfo> data)
        {
            foreach (var _rec in data)
            {
                _rec.isenabled = 1; // by default enabled
                if (Site_Settings.Content_Approval == 1)
                    _rec.isreviewed = 1;
                else
                    _rec.isreviewed = 0;

                var _counter = 1;
                foreach (var thumb in _rec.thumbs)
                {
                    var _prefix = "00";
                    if (_counter <= 9)
                        _prefix = "00";
                    else if (_counter <= 99)
                        _prefix = "0";
                    else
                        _prefix = "";

                  
                    /* save base64 image in physical path */
                    byte[] image = Convert.FromBase64String(thumb.filename.Replace("data:image/png;base64,",""));
                    string thumbFileName = _rec.pf.Remove(_rec.pf.LastIndexOf(".")) + "_" + _prefix + "" + _counter + ".png";
                    _counter++;
                    string path = VideoUrlConfig.Thumbs_Path(_rec.username) + "/" + thumbFileName;
                    if (System.IO.File.Exists(path))
                        System.IO.File.Delete(path);

                    System.IO.File.WriteAllBytes(path, image);

                    if (thumb.selected)
                        _rec.thumbfilename = thumbFileName;

                    _rec.tfile = _rec.thumbfilename;
                }
                try
                {
                    // in case of direct uploader, there is no published video, source video is published.
                    // shift source video to published directory
                    var SourcePath = VideoUrlConfig.Source_Video_Path(_rec.username); // no source video there
                    var publishedPath = VideoUrlConfig.Published_Video_Path(_rec.username);
                    var thumbsPath = VideoUrlConfig.Thumbs_Path(_rec.username);
                    if (File.Exists(publishedPath + "/" + _rec.pf))
                        File.Delete(publishedPath + "/" + _rec.pf);

                    File.Move(SourcePath + "/" + _rec.sf, publishedPath + "/" + _rec.pf);
                }
                catch (Exception ex)
                {
                    var err = ex.Message;

                }

                string _publish_path = "none";
                string _thumb_path = "none";
                
                var Item = new VideoDataEntity()
                {
                    title = _rec.title,
                    description = _rec.description,
                    tags = _rec.tags,
                    galleryid = _rec.galleryid,
                    categoryid = 0,
                    categories = string.Join(",", _rec.categories),
                    duration = _rec.duration,
                    originalvideofilename = _rec.sf,
                    videofilename = _rec.pf,
                    thumbfilename = _rec.tfile,
                    isenabled = _rec.isenabled,
                    isreviewed = _rec.isreviewed,
                    username = _rec.username,
                    type = _rec.type,
                    duration_sec = _rec.dursec,
                    isprivate = _rec.privacy,
                    authkey = "",
                    flv_url = _publish_path,
                    thumb_url = _thumb_path,
                    org_url = "",
                    isexternal = 0,
                    youtubeid = "",
                    istagsreviewed = 1,
                    mode = 0,
                    ispublished = 1,
                    errorcode = _rec.errorcode,
                    ipaddress = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"].ToString(),
                    embed_script = "",
                    // submittype = 2 // ffmpeg target type
                };


                Item.isreviewed = 1;
                // Approved by Default
                if (Site_Settings.Content_Approval == 0)
                {
                    // Moderator Review Required
                    Item.isreviewed = 0;
                }
                //XSS CLEANUP
                // not working on medium trust
                //string content = Sanitizer.GetSafeHtmlFragment(txt_content.Value);
                // Compress Code => remove \r\t etc which create unnecessary line breaking issues
                string content = UtilityBLL.CompressCode(Item.description);

                // Process Contents -> links, bbcodes etc
                content = UtilityBLL.Process_Content_Text(content);


                // Generate Gallery Preview
                //Content = GalleryBLLC.Generate_Blog_Gallery_Previews(Content)


                Item.videoid = VideoBLL.Process_Info(Item);
                
                if (Item.tags != "")
                {
                    int tag_type = 0; // represent videos
                    if (Item.type == 1)
                        tag_type = 4; // represent audio file
                    TagsBLL.Process_Tags(Item.tags, tag_type, 0);
                }

                if (Item.galleryid > 0)
                {
                    var albumType = 1;
                    if (Item.type == 1)
                        albumType = 2;
                    int count = VideoBLL.Count(new VideoEntity()
                    {
                        type = albumType,
                        galleryid = Item.galleryid,
                        nofilter = true
                    });

                    GalleryBLLC.Update_Field_V3(Item.galleryid, (short)count, "total_photos");

                    // update cover
                    var _value = GalleryBLLC.Get_Field_Value(Item.galleryid, "customcover");
                    int customcover = 0;
                    if (_value != "")
                        customcover = Convert.ToInt32(_value);

                    if (customcover == 0 && Item.type == 0) // only video, audio use default photo at this point
                    {
                        if (Item.thumb_url != "")
                        {
                            GalleryBLLC.Update_Field_V3(Item.galleryid, Item.thumb_url, "picturename");
                        }
                        else if (Item.thumbfilename != "none" || Item.thumbfilename != "")
                        {
                            GalleryBLLC.Update_Field_V3(Item.galleryid, Item.thumbfilename, "picturename");
                        }
                    }
                }
            }
            return new { status = "success", message = "Record processed successfully" };
        }

        public static object audio_proc(List<SaveVideoInfo> data)
        {
            foreach (var _rec in data)
            {
                _rec.isenabled = 1; // by default enabled
                if (Site_Settings.Content_Approval == 1)
                    _rec.isreviewed = 1;
                else
                    _rec.isreviewed = 0;

                try
                {
                    // in case of direct uploader, there is no published video, source video is published.
                    // shift source video to published directory
                    var SourcePath = VideoUrlConfig.Source_Video_Path(_rec.username); // no source video there
                    var publishedPath = VideoUrlConfig.Published_Audio_Path(_rec.username);
                    if (File.Exists(publishedPath + "/" + _rec.pf))
                        File.Delete(publishedPath + "/" + _rec.pf);

                    File.Move(SourcePath + "/" + _rec.sf, publishedPath + "/" + _rec.pf);
                }
                catch (Exception ex)
                {
                    var err = ex.Message;

                }

                string _publish_path = "none";
                string _source_path = "none";

                if (_rec.duration == null)
                    _rec.duration = "";

                var Item = new VideoDataEntity()
                {
                    title = _rec.title,
                    description = _rec.description,
                    tags = _rec.tags,
                    galleryid = _rec.galleryid,
                    categoryid = 0,
                    categories = string.Join(",", _rec.categories),
                    duration = _rec.duration,
                    originalvideofilename = _rec.sf,
                    videofilename = _rec.pf,
                    thumbfilename = "",
                    isenabled = _rec.isenabled,
                    isreviewed = _rec.isreviewed,
                    username = _rec.username,
                    type = 1, // _rec.type,
                    duration_sec = _rec.dursec,
                    isprivate = _rec.privacy,
                    authkey = "",
                    flv_url = _publish_path,
                    thumb_url = _rec.thumb_url,
                    org_url = "",
                    isexternal = 0,
                    youtubeid = "",
                    istagsreviewed = 1,
                    mode = 0,
                    ispublished = 1, 
                    errorcode = _rec.errorcode,
                    ipaddress = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"].ToString(),
                    embed_script = "",
                    // submittype = 2 // ffmpeg target type
                };


                Item.isreviewed = 1;
                // Approved by Default
                if (Site_Settings.Content_Approval == 0)
                {
                    // Moderator Review Required
                    Item.isreviewed = 0;
                }
                //XSS CLEANUP
                // not working on medium trust
                //string content = Sanitizer.GetSafeHtmlFragment(txt_content.Value);
                // Compress Code => remove \r\t etc which create unnecessary line breaking issues
                string content = UtilityBLL.CompressCode(Item.description);

                // Process Contents -> links, bbcodes etc
                content = UtilityBLL.Process_Content_Text(content);
                
                // Generate Gallery Preview
                //Content = GalleryBLLC.Generate_Blog_Gallery_Previews(Content)


                Item.videoid = VideoBLL.Process_Info(Item);
                if (Item.tags != "")
                {
                    int tag_type = 0; // represent videos
                    if (Item.type == 1)
                        tag_type = 4; // represent audio file
                    TagsBLL.Process_Tags(Item.tags, tag_type, 0);
                }

                if (Item.galleryid > 0)
                {
                    var albumType = 1;
                    if (Item.type == 1)
                        albumType = 2;
                    int count = VideoBLL.Count(new VideoEntity()
                    {
                        type = albumType,
                        galleryid = Item.galleryid,
                        nofilter = true
                    });

                    GalleryBLLC.Update_Field_V3(Item.galleryid, (short)count, "total_photos");

                    // update cover
                    var _value = GalleryBLLC.Get_Field_Value(Item.galleryid, "customcover");
                    int customcover = 0;
                    if (_value != "")
                        customcover = Convert.ToInt32(_value);

                    if (customcover == 0 && Item.type == 0) // only video, audio use default photo at this point
                    {
                        if (Item.thumb_url != "")
                        {
                            GalleryBLLC.Update_Field_V3(Item.galleryid, Item.thumb_url, "picturename");
                        }
                        else if (Item.thumbfilename != "none" || Item.thumbfilename != "")
                        {
                            GalleryBLLC.Update_Field_V3(Item.galleryid, Item.thumbfilename, "picturename");
                        }
                    }
                }
            }
            return new { status = "success", message = "Record processed successfully" };
        }
      
        public static object yt_proc(List<VideoDataEntity> data)
        {
            foreach (var _rec in data)
            {
                _rec.isenabled = 1; // by default enabled
                if (Site_Settings.Content_Approval == 1)
                    _rec.isreviewed = 1;
                else
                    _rec.isreviewed = 0;
                              

                var Item = new VideoDataEntity()
                {
                    title = _rec.title,
                    description = _rec.description,
                    tags = _rec.tags,
                    galleryid = _rec.galleryid,
                    categoryid = 0,
                    categories = string.Join(",", _rec.categories),
                    duration = _rec.duration,
                    originalvideofilename = "",
                    videofilename = "",
                    thumbfilename = "",
                    isenabled = _rec.isenabled,
                    isreviewed = _rec.isreviewed,
                    username = _rec.username,
                    type = _rec.type,
                    duration_sec = 0,
                    isprivate = 0,
                    authkey = "",
                    flv_url = "",
                    thumb_url = _rec.preview_url,
                    org_url = "",
                    isexternal = 1, // embed video
                    youtubeid = _rec.youtubeid,
                    istagsreviewed = 1,
                    mode = 0,
                    ispublished = 1, // not published video until ET send notification to website
                    errorcode = 0,
                    ipaddress = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"].ToString(),
                    embed_script = ""
                // submittype = 2 // ffmpeg target type
            };


                Item.isreviewed = 1;
                // Approved by Default
                if (Site_Settings.Content_Approval == 0)
                {
                    // Moderator Review Required
                    Item.isreviewed = 0;
                }
                //XSS CLEANUP
                // not working on medium trust
                //string content = Sanitizer.GetSafeHtmlFragment(txt_content.Value);
                // Compress Code => remove \r\t etc which create unnecessary line breaking issues
                string content = UtilityBLL.CompressCode(Item.description);

                // Process Contents -> links, bbcodes etc
                content = UtilityBLL.Process_Content_Text(content);


                // Generate Gallery Preview
                //Content = GalleryBLLC.Generate_Blog_Gallery_Previews(Content)


                Item.videoid = VideoBLL.Process_Info(Item);

            }
            return new { status = "success", message = "Record processed successfully" };
        }

        public static object embed_proc(List<VideoDataEntity> data)
        {
            foreach (var _rec in data)
            {
                _rec.isenabled = 1; // by default enabled
                if (Site_Settings.Content_Approval == 1)
                    _rec.isreviewed = 1;
                else
                    _rec.isreviewed = 0;


                var Item = new VideoDataEntity()
                {
                    title = _rec.title,
                    description = _rec.description,
                    tags = _rec.tags,
                    galleryid = _rec.galleryid,
                    categoryid = 0,
                    categories = string.Join(",", _rec.categories),
                    duration = _rec.duration,
                    originalvideofilename = "",
                    videofilename = "",
                    thumbfilename = "",
                    isenabled = _rec.isenabled,
                    isreviewed = _rec.isreviewed,
                    username = _rec.username,
                    type = _rec.type,
                    duration_sec = 0,
                    isprivate = 0,
                    authkey = "",
                    flv_url = "",
                    thumb_url = _rec.preview_url,
                    org_url = "",
                    isexternal = 1, // embed video
                    youtubeid = "",
                    istagsreviewed = 1,
                    mode = 0,
                    ispublished = 1, // not published video until ET send notification to website
                    isratings = 1,
                    errorcode = 0,
                    ipaddress = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"].ToString(),
                    embed_script = _rec.embed_script
                    // submittype = 2 // ffmpeg target type
                };


                Item.isreviewed = 1;
                // Approved by Default
                if (Site_Settings.Content_Approval == 0)
                {
                    // Moderator Review Required
                    Item.isreviewed = 0;
                }
                //XSS CLEANUP
                // not working on medium trust
                //string content = Sanitizer.GetSafeHtmlFragment(txt_content.Value);
                // Compress Code => remove \r\t etc which create unnecessary line breaking issues
                string content = UtilityBLL.CompressCode(Item.description);

                // Process Contents -> links, bbcodes etc
                content = UtilityBLL.Process_Content_Text(content);


                // Generate Gallery Preview
                //Content = GalleryBLLC.Generate_Blog_Gallery_Previews(Content)


                Item.videoid = VideoBLL.Process_Info(Item);

            }
            return new { status = "success", message = "Record processed successfully" };
        }
        /* single youtbue video fetcher */
        public object yt_single_proc(YoutubeEntity query)
        {
              var model = new VideoDataEntity();
              string ytid = Fetch_YoutubeID(query.term);
              if (ytid == "")
              {
                return new { status = "error", message = "Youtube video url failed to validate, please enter proper youtube id" };
              }

              string DeveloperID = Site_Settings.Yt_Developer_Key;

                try
                {
                    var youtubeService = new YouTubeService(new BaseClientService.Initializer()
                    {
                        ApiKey = DeveloperID,
                        ApplicationName = this.GetType().ToString()
                    });
                    var searchListRequest = youtubeService.Videos.List("snippet,contentDetails");
                    searchListRequest.Id = ytid;

                    var searchListResponse = searchListRequest.Execute();

                    List<string> videos = new List<string>();

                    foreach (var searchResult in searchListResponse.Items)
                    {

                        string videoid = searchResult.Id;
                        if (videoid.Contains(":"))
                            videoid = videoid.Remove(0, videoid.LastIndexOf(":") + 1);

                        string video_watch_id = "/watch?v=" + videoid;
                        model.title = searchResult.Snippet.Title;
                        model.description = searchResult.Snippet.Description;
                        model.duration = ""; // searchResult.ContentDetails.Duration.ToString();

                        ArrayList thumb_urls = new ArrayList();
                        /* foreach (MediaThumbnail thumbnail in video.Thumbnails)
                        {
                            thumb_urls.Add(thumbnail.Url);
                            // str.AppendLine("\tThumbnail URL: " + thumbnail.Url + "<br />");
                            // str.AppendLine("\tThumbnail time index: " + thumbnail.Time + "<br />");
                        }*/
                        if(searchResult.Snippet.Thumbnails.High != null)
                            thumb_urls.Add(searchResult.Snippet.Thumbnails.High.Url);
                        else if (searchResult.Snippet.Thumbnails.Medium != null)
                            thumb_urls.Add(searchResult.Snippet.Thumbnails.Medium.Url);
                        else if (searchResult.Snippet.Thumbnails.Standard != null)
                            thumb_urls.Add(searchResult.Snippet.Thumbnails.Standard.Url);
                        else if (searchResult.Snippet.Thumbnails.Default != null)
                            thumb_urls.Add(searchResult.Snippet.Thumbnails.Default.Url);

                        if (thumb_urls.Count > 1)
                        {
                            int count = Convert.ToInt32(thumb_urls.Count / 2);
                            model.preview_url = thumb_urls[count].ToString();
                        }
                        else
                        {
                            model.preview_url = thumb_urls[0].ToString();
                        }
                        
                        /*foreach (Google.GData.YouTube.MediaContent mediaContent in searchResult.Snippet.contents)
                        {
                           // str.AppendLine("\tMedia Location: " + mediaContent.Url + "<br />");
                           // str.AppendLine("\tMedia Type: " + mediaContent.Format + "<br />");
                           this.duration = mediaContent.duration;
                        }*/

                        model.youtubeid = video_watch_id;
                    }
                }
                catch (Exception ex)
                {
                    ErrorLgBLL.Add("Embed Video - Fetch Error", "", ex.Message);
                    return new { status = "error", message = ex.Message };
                }


                /*YouTubeQuery query = new YouTubeQuery("http://gdata.youtube.com/feeds/api/videos/" + ytid);
                //Feed<Video> videoFeed = req.Get<Video>(query);
                Video video = req.Retrieve<Video>(query);
                ProcessVideoEntry(video); */
                return new { status = "success", post = model };

        }
        private string Fetch_YoutubeID(string URL)
        {
            string reg = @"(?<=v(\=|\/))(?<yid>([-a-zA-Z0-9_]+))|(?<=youtu\.be\/)(?<yid2>([-a-zA-Z0-9_]+))";
            System.Text.RegularExpressions.Match VDMatch = System.Text.RegularExpressions.Regex.Match(URL, reg);
            if (VDMatch.Success)
            {
                if (VDMatch.Groups["yid"].Value != "")
                    return VDMatch.Groups["yid"].Value; // normal pattern
                else
                    return VDMatch.Groups["yid2"].Value; // pattern like http://youtu.be/[id]
            }
            else
                return "";
        }
        /* multiple youtube video fetcher */
        public object yt_multiple_proc(YoutubeEntity entity)
        {
            var _youtubeService = new YouTubeService(new BaseClientService.Initializer()
            {
                ApiKey = Site_Settings.Yt_Developer_Key,
                ApplicationName = this.GetType().ToString()
            });
            var model = new List<VideoDataEntity>();

            var searchListRequest = _youtubeService.Search.List("snippet");
            searchListRequest.Q = entity.term;
            // order date
            switch(entity.order)
            {
                case 0:
                    searchListRequest.Order = SearchResource.ListRequest.OrderEnum.Date;
                    break;
                case 1:
                    searchListRequest.Order = SearchResource.ListRequest.OrderEnum.Rating;
                    break;
                case 2:
                    searchListRequest.Order = SearchResource.ListRequest.OrderEnum.Relevance;
                    break;
                case 3:
                    searchListRequest.Order = SearchResource.ListRequest.OrderEnum.Title;
                    break;
                case 4:
                    searchListRequest.Order = SearchResource.ListRequest.OrderEnum.VideoCount;
                    break;
                case 5:
                    searchListRequest.Order = SearchResource.ListRequest.OrderEnum.ViewCount;
                    break;
            }

            if (entity.youtubecategory != null)
                searchListRequest.VideoCategoryId = entity.youtubecategory;
            // upload date
            switch (entity.uploaddate)
            {
                case 0:
                    searchListRequest.PublishedAfter = DateTime.Now.AddDays(-1);
                    break;
                case 1:
                    searchListRequest.PublishedAfter = DateTime.Now.AddDays(-7);
                    break;
                case 2:
                    searchListRequest.PublishedAfter = DateTime.Now.AddDays(-31);
                    break;
            }
            var ChannelID = "";
            var ChannelTitle = "";
            if (entity.username != null && entity.username != "")
            {
                var channeListRequest = _youtubeService.Channels.List("snippet");
                channeListRequest.ForUsername = entity.username;
                try
                {
                    var channeListResponse = searchListRequest.Execute();
                    foreach (var searchResult in channeListResponse.Items)
                    {
                        var data = searchResult;
                        ChannelID = searchResult.Snippet.ChannelId;
                        ChannelTitle = searchResult.Snippet.ChannelTitle;
                    }
                }
                catch (Exception ex)
                {
                    ErrorLgBLL.Add("Exception Youtube Service - Fetch Channels", "", ex.Message);
                }
            }

            // Replace with your search term.
            //searchListRequest.ChannelId = ""; // Search Channel ID
            //searchListRequest.Order = SearchResource.ListRequest.OrderEnum.Date; //
            // searchListRequest.RelatedToVideoId = "";
            // searchListRequest.VideoCategoryId = "";
            //searchListRequest.VideoDuration = SearchResource.ListRequest.VideoDurationEnum.Any;
            // searchListRequest.VideoLicense = SearchResource.ListRequest.VideoLicenseEnum.Any;
            searchListRequest.MaxResults = 50;
            searchListRequest.Type = "video";
            if (ChannelID != null && ChannelID != "")
                searchListRequest.ChannelId = ChannelID;

            //searchListRequest.ChannelId = "mediasoftpro1";
            try
            {
                var searchListResponse = searchListRequest.Execute();

                var videos = new List<string>();
                //List<string> channels = new List<string>();
                //List<string> playlists = new List<string>();

                // Add each result to the appropriate list, and then display the lists of
                // matching videos, channels, and playlists.
                foreach (var searchResult in searchListResponse.Items)
                {
                    switch (searchResult.Id.Kind)
                    {
                        case "youtube#video":
                            // add youtube video functionality

                            string videoid = searchResult.Id.VideoId;
                            if (videoid.Contains(":"))
                                videoid = videoid.Remove(0, videoid.LastIndexOf(":") + 1);

                            string video_watch_id = "/watch?v=" + videoid;
                            if (!VideoBLL.Check_YOUTUBE_ID(video_watch_id))
                            {
                                string title = searchResult.Snippet.Title;
                                string description = searchResult.Snippet.Description;

                                string thumburl = "";
                                ArrayList thumb_urls = new ArrayList();
                                if (searchResult.Snippet.Thumbnails.Maxres != null)
                                    thumb_urls.Add(searchResult.Snippet.Thumbnails.Maxres.Url);
                                else if (searchResult.Snippet.Thumbnails.High != null)
                                    thumb_urls.Add(searchResult.Snippet.Thumbnails.High.Url);
                                else if (searchResult.Snippet.Thumbnails.Standard != null)
                                    thumb_urls.Add(searchResult.Snippet.Thumbnails.Standard.Url);
                                else if (searchResult.Snippet.Thumbnails.Default != null)
                                    thumb_urls.Add(searchResult.Snippet.Thumbnails.Default.Url);
                               

                                if (thumb_urls.Count > 1)
                                {
                                    int count = Convert.ToInt32(thumb_urls.Count / 2);
                                    thumburl = thumb_urls[count].ToString();
                                }
                                else
                                {
                                    thumburl = thumb_urls[0].ToString();
                                }

                                // process and add video
                                string flv_filename = "";
                                string original_filename = "";
                                string thumb_filename = "";
                                string duration = "";
                                int isapproved = 1;
                                /*if (model.isapproved)
                                    isapproved = 1;*/
                                // set video actions : 1 -> on, 0 -> off
                                int isenabled = 1;
                                int ispublished = 1;
                                //int isreviewed = 1;
                                int isprivate = 0;

                                int isexternal = 1;
                                string flv_url = "none";
                                string _embed = "";

                                string ipaddress = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"].ToString();
                                //string username = "";
                                //username = Site_Settings.Yt_UserName;

                                // remove <a tags from description
                                if (description != null && description != "")
                                {
                                    Regex _href = new Regex(@"<a[^<]+?>", RegexOptions.Singleline); // remove <a href.....>
                                    Regex _hrefend = new Regex(@"</a>", RegexOptions.Singleline); // remove </a>
                                    description = _href.Replace(description, string.Empty);
                                    description = _hrefend.Replace(description, string.Empty);

                                    if (description.Length > 2000)
                                        description = description.Substring(0, 2000);

                                }


                                int istagsreviewed = 1; // tags not reviewed yet
                                var vd = new VideoDataEntity();
                                vd.categoryid = 0; // store categoryname or term instead of category id
                                vd.username = "";
                                vd.title = title;
                                if (description != null)
                                    vd.description = description;
                                vd.tags = "";
                                vd.duration = duration;
                                vd.duration_sec = 0; // Convert.ToInt32(duration_sec);
                                vd.originalvideofilename = original_filename;
                                vd.videofilename = flv_filename;
                                vd.thumbfilename = thumb_filename;
                                vd.isprivate = (byte)isprivate;
                                vd.isenabled = (byte)isenabled;
                               
                                vd.ispublished = (byte)ispublished;
                                vd.isreviewed = (byte)isapproved;
                                vd.flv_url = flv_url;
                                vd.preview_url = thumburl;
                                vd.embed_script = _embed;
                                vd.isexternal = (byte)isexternal;
                                vd.ipaddress = ipaddress;
                                vd.type = 0;
                                vd.youtubeid = video_watch_id;
                                vd.istagsreviewed = (byte)istagsreviewed;

                                model.Add(vd);
                            }

                            break;
                        case "youtube#channel":
                            //channels.Add(String.Format("{0} ({1})", searchResult.Snippet.Title, searchResult.Id.ChannelId));
                            break;

                        case "youtube#playlist":
                            //playlists.Add(String.Format("{0} ({1})", searchResult.Snippet.Title, searchResult.Id.PlaylistId));
                            break;
                    }
                }
            }
            catch (Exception ex)
            {
                ErrorLgBLL.Add("Exception Youtube Service - Process Videos", "", ex.Message);
                return new { status = "success", posts = ex.Message };
            }

            return new { status = "success", posts = model };
        }

        public static string ProcessAction(List<VideoEntity> list)
        {
            foreach (var entity in list)
            {
                if (entity.videoid > 0)
                {
                    switch (entity.actionstatus)
                    {
                        case "enable":
                            Update_Field_V3(entity.videoid, (byte)1, "isenabled");
                            break;

                        case "disable":
                            Update_Field_V3(entity.videoid, (byte)0, "isenabled");
                            break;
                        case "approve":
                            Update_Field_V3(entity.videoid, (byte)1, "isreviewed");
                            break;
                        case "featured":
                            Update_Field_V3(entity.videoid, (byte)1, "isfeatured");
                            break;
                        case "premium":
                            Update_Field_V3(entity.videoid, (byte)2, "isfeatured");
                            break;
                        case "normal":
                            Update_Field_V3(entity.videoid, (byte)0, "isfeatured");
                            break;
                        case "adult":
                            Update_Field_V3(entity.videoid, (byte)2, "isadult");
                            break;
                        case "nonadult":
                            Update_Field_V3(entity.videoid, (byte)0, "isadult");
                            break;
                        case "private":
                            Update_Field_V3(entity.videoid, (byte)1, "isprivate");
                            break;
                        case "public":
                            Update_Field_V3(entity.videoid, (byte)0, "isprivate");
                            break;
                        case "movie":
                            Update_Field_V3(entity.videoid, (byte)2, "type");
                            break;
                        case "clip":
                            Update_Field_V3(entity.videoid, (byte)0, "type");
                            break;
                        case "delete":
                            RemoveVideo(entity.videoid, entity.type);
                            break;
                        case "delete_fav":
                            Favorites.Delete(entity.videoid, entity.username, entity.type, 0);
                            break;
                        case "delete_like":
                            User_Ratings.Delete(entity.videoid, 0);
                            break;
                        case "delete_playlist":
                            //PlayListBLL.Delete(entity.videoid);
                            break;
                    }
                }
            }
            return "Operation performed successfully";
        }

        #region Cache
        private static List<VideoDataEntity> _Fetch_Normal_Cache(VideoEntity entity, string key)
        {
            /* ASP.NET Cache */
            if (HttpContext.Current.Cache[key] == null)
            {
                HttpContext.Current.Cache.Add(key, FetchItems(entity), null, DateTime.Now.AddMinutes(Site_Settings.Cache_Duration), TimeSpan.Zero, CacheItemPriority.High, null);
            }
            return (List<VideoDataEntity>)(HttpContext.Current.Cache[key]);
        }

        private static List<VideoDataEntity> _Fetch_Redis_Cache(VideoEntity entity, string key)
        {
            var conn = RedisHelper.RedisConnection.GetDatabase();
            try
            {
                if (Convert.ToString(conn.StringGet(key)) == null)
                {

                    conn.StringSet(key, Newtonsoft.Json.JsonConvert.SerializeObject(FetchItems(entity)), TimeSpan.FromMinutes(Site_Settings.Cache_Duration));
                }
                return Newtonsoft.Json.JsonConvert.DeserializeObject<List<VideoDataEntity>>(conn.StringGet(key));
            }
            catch (Exception ex)
            {
                return _Fetch_Normal_Cache(entity, key);
            }
        }
        private static int _Count_Normal_Cache(VideoEntity entity, string key)
        {
            /* ASP.NET Cache */
            if (HttpContext.Current.Cache[key] == null)
            {
                int total_records = CountRecords(entity);
                HttpContext.Current.Cache.Add(key, total_records, null, DateTime.Now.AddMinutes(Site_Settings.Cache_Duration), TimeSpan.Zero, CacheItemPriority.High, null);
            }
            return Convert.ToInt32(HttpContext.Current.Cache[key]);
        }

        private static int _Count_Redis_Cache(VideoEntity entity, string key)
        {
            var conn = RedisHelper.RedisConnection.GetDatabase();
            try
            {
                if (Convert.ToString(conn.StringGet(key)) == null)
                {
                    int total_records = CountRecords(entity);
                    conn.StringSet(key, Newtonsoft.Json.JsonConvert.SerializeObject(total_records), TimeSpan.FromMinutes(Site_Settings.Cache_Duration));
                }
                return Newtonsoft.Json.JsonConvert.DeserializeObject<int>(conn.StringGet(key));
            }
            catch (Exception ex)
            {
                return _Count_Normal_Cache(entity, key);
            }
        }
        #endregion
    }
}