﻿using Jugnoon.Utility;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;

namespace Jugnoon.models
{
    public class UpdatePasswordModelView
    {
        public long UserID { get; set; }
        
        [Display(Name = "username")]
        public string UserName { get; set; }

        [Required(ErrorMessage = "Password Required")]
        [DataType(DataType.Password)]
        [Display(Name = "Password")]
        [RegularExpression(@"^[\d\w]{6,15}$", ErrorMessage = "Invalid Password")]
        [StringLength(15, MinimumLength = 6, ErrorMessage = "Password must be inbetween 6 - 15 chars.")]
        public string Password { get; set; }

        [DataType(DataType.Password)]
        [Display(Name = "Repeat Password")]
        [Compare("Password", ErrorMessage = "The password and confirmation password do not match.")]
        public string ConfirmPassword { get; set; }

        public string Message { get; set; }

        public AlertTypes AlertType { get; set; }
    }

    public class ForgotViewModel
    {
        [Required(ErrorMessage = "UserName Required")]
        [Display(Name = "Email or Username")]
        public string UserName { get; set; }

        public string Message { get; set; }

        public AlertTypes AlertType { get; set; }
    }

    public class UserLoginViewModel
    {
        [Required(ErrorMessage = "UserName Required")]
        [Display(Name = "Email or Username")]
        public string UserName { get; set; }

        [Required(ErrorMessage = "Password Required")]
        [DataType(DataType.Password)]
        [Display(Name = "Password")]
        public string Password { get; set; }

        [Display(Name = "Remember Me")]
        public bool RememberMe { get; set; }

        public string Message { get; set; }

        public AlertTypes AlertType { get; set; }
    }

    public class UserSignupViewModel
    {

        [Required(ErrorMessage = "UserName Required")]
        [Display(Name = "Username")]
        [StringLength(15, MinimumLength = 6, ErrorMessage = "UserName must be inbetween 5 - 15 chars.")]
        [RegularExpression(@"^[a-z0-9_-]{5,15}$", ErrorMessage = "Invalid UserName")]
        public string UserName { get; set; }

        [Required(ErrorMessage = "Password Required")]
        [DataType(DataType.Password)]
        [Display(Name = "Password")]
        [RegularExpression(@"^[\d\w]{6,15}$", ErrorMessage = "Invalid Password")]
        [StringLength(15, MinimumLength = 6, ErrorMessage = "Password must be inbetween 6 - 15 chars.")]
        public string Password { get; set; }

        [DataType(DataType.Password)]
        [Display(Name = "Repeat Password")]
        [Compare("Password", ErrorMessage = "The password and confirmation password do not match.")]
        public string ConfirmPassword { get; set; }

        [Required(ErrorMessage = "Email Required")]
        [Display(Name = "Email Address")]
        [StringLength(50, ErrorMessage = "Email address must not be greater than 50 chars.")]
        [DataType(DataType.EmailAddress, ErrorMessage = "Invalid Email Address")]
        public string Email { get; set; }

        // first name
        [StringLength(100, ErrorMessage = "First name must not be longer than 50 chars.")]
        [Display(Name = "First Name")]
        public string FirstName { get; set; }

        // last name
        [StringLength(100, ErrorMessage = "Last name must not be longer than 50 chars.")]
        [Display(Name = "Last Name")]
        public string LastName { get; set; }

        public bool isBlocked { get; set; }

        public string Message { get; set; }

        public AlertTypes AlertType { get; set; }

        public bool isAdmin { get; set; }

        [Display(Name = "User Type")]
        public string Type { get; set; }

        public Dictionary<string, string> AccountTypes { get; set; }
        
    }

    public class ActivateAccountViewModel
    {
        public string Message { get; set; }

        public AlertTypes AlertType { get; set; }
    }

    public class ContactUsViewModel
    {
        [Required(ErrorMessage = "Sender Name Required")]
        [Display(Name = "Full Name")]
        public string SenderName { get; set; }

        [Required(ErrorMessage = "Phone Number Required")]
        [Display(Name = "Phone Number")]
        public string PhoneNumber { get; set; }

        [Required(ErrorMessage = "Email Required")]
        [Display(Name = "Email Address")]
        [StringLength(80, ErrorMessage = "Email Address Max Length 80.")]
        [DataType(DataType.EmailAddress, ErrorMessage = "Invalid Email Address")]
        public string EmailAddress { get; set; }

        [Required(ErrorMessage = "Message Required")]
        [Display(Name = "Message")]
        public string Body { get; set; }

        public string Message { get; set; }

        public AlertTypes AlertType { get; set; }
    }

}