﻿using System.Text;
using System.Web.UI.WebControls;
using Jugnoon.Entity;
using Jugnoon.Settings;
using Jugnoon.Utility;
using Ganss.XSS;
using System.Web;

namespace Jugnoon.Scripts
{
    public class UGeneral
    {
        public UGeneral()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        public static string SanitizeText(string text, bool isCompress = true)
        {
            var _sanitize = new HtmlSanitizer();
            string _text = _sanitize.Sanitize(text);
            if (isCompress)
                _text = UtilityBLL.CompressCodeBreak(_text);

            return _text;
        }
        
        //****************************************************
        // GENERAL UTILITY FUNCTIONS
        //****************************************************

        // Utility function to prepare description of any record
        public static string Prepare_Description(string description, int length)
        {
            string desc = UtilityBLL.CompressCodeBreak(BBCode.MakeHtml(description, true));

            // again replace coded word (~~ by bbcode to newline)
            desc = desc.Replace("~~", System.Environment.NewLine);

            if (desc == "")
                return desc;

            if (desc.Length > length && length > 0)
                desc = desc.Substring(0, length) + "..";

            return desc;
        }

        // utility function to prepare one or more category links for any record
        public static string Prepare_Categories(string categories, int length, string path)
        {
            if (categories == null)
                return "";

            string output = "";
            string cat = "";
            if (categories != "")
            {
                if (categories.Contains(","))
                {                                        
                    StringBuilder catstr = new StringBuilder();
                    string[] arr = categories.ToString().Split(char.Parse(","));
                    if (length == 0)
                        length = arr.Length;
                    else if (arr.Length < length)
                        length = arr.Length;
                    for (var i = 0; i <= length - 1; i++)
                    {
                        if (arr[i].ToString().Length > 3)
                        {
                            if (i > 0)
                                catstr.AppendLine(", ");
                            catstr.AppendLine("<a href=\"" + CategoryUrlConfig.PrepareUrl(arr[i].ToString(), path) + "\" title=\"" + arr[i].ToString() + "\">" + arr[i].ToString().Trim() + "</a>");
                        }
                    }
                    output = catstr.ToString();
                }
                else
                {
                    cat = categories;
                    if (cat.Length > length && length > 0)
                        cat = cat.Substring(0, length) + "..";
                    output = "<a href=\"" + CategoryUrlConfig.PrepareUrl(categories, path) + "\" title=\"" + categories + "\">" + cat.Trim() + "</a>";
                }
            }
            return output;
        }

        // utility function to prepare one or more tag links for any record
        public static string Prepare_Tags(string tags, int length, string path)
        {
            if(tags == null)
                return "";
            
            // type : type of category (e.g 0: video, 5: photo etc)
            // length: maximum characters allowed
            string output = "";
            string cat = "";
            if (tags != "")
            {
                if (tags.Contains(","))
                {
                    
                    StringBuilder catstr = new StringBuilder();
                    string[] arr = tags.ToString().Split(char.Parse(","));
                    if (length == 0)
                        length = arr.Length;
                    else if (arr.Length < length)
                        length = arr.Length;

                    for (var i = 0; i <= length - 1; i++)
                    {
                        if (arr[i].ToString().Length > 3)
                        {
                            if (i > 0)
                                catstr.AppendLine(", ");
                            catstr.AppendLine("<a href=\"" + TagUrlConfig.PrepareUrl(arr[i], path) + "\" title=\"" + arr[i].ToString() + "\">" + arr[i].ToString().Trim() + "</a>");
                        }
                    }
                    output = catstr.ToString();
                }
                else
                {
                    cat = tags;
                    if (cat.Length > length && length > 0)
                        cat = cat.Substring(0, length) + "..";
                    output = "<a href=\"" + TagUrlConfig.PrepareUrl(tags, path) + "\" title=\"" + tags + "\">" + cat.Trim() + "</a>";
                }
            }
            return output;
        }


        #region Video Thumb Processing Script
        // Main function used to process core video thumb stored on local server, remote server, cloud storage or third party storage e.g youtube video thumbs
        public static string ProcessVideoThumb(string url, VideoDataEntity vd, bool isaudio, ListItems attr)
        {
            // media thumb section
            var str = new StringBuilder();
            var img_str = new StringBuilder();
            // style=\"width:" + width + "px; height:" + height + "px;\"
            if (isaudio)
            {
                // audio thumb setup
                if (!vd.thumb_url.StartsWith("http"))
                {
                    if (vd.thumbfilename != "none" && vd.thumbfilename != "")
                        vd.thumb_url = PhotoUrlConfig.PreparePhotoUrl(vd.username, vd.thumbfilename, 0);
                    else
                        vd.thumb_url = Site_Settings.Audio_Path;
                }
                str.Append(Process_Embed_Video_Thumb(img_str, vd.thumb_url, attr));
            }
            else
            {

                // isExternal = 3; // Default Value for Query Processing. If 3 is set then use default option
                if (vd.isexternal == 0 || vd.isexternal == 3)
                {
                    if (vd.ispublished == 0 || vd.errorcode > 0 || vd.thumbfilename == "" || vd.thumbfilename == "none")
                    {
                        // video is not yet published
                        // or published failed - any level of error raised
                        // or thumbfilename is empty
                        vd.thumb_url = Config.GetUrl("/assets/images/video-thumbnail.jpg");
                        str.Append(Process_Embed_Video_Thumb(img_str, vd.thumb_url, attr));
                    }
                    else
                    {
                        // normal video upload
                        str.Append(Process_Normal_Video_Thumb(img_str, vd.username, vd.thumbfilename, vd.thumb_url, attr));
                    }
                }
                else if (vd.isexternal == 1)
                {
                    // embed video upload
                    str.Append(Process_Embed_Video_Thumb(img_str, vd.thumb_url, attr));
                }
                else if (vd.isexternal == 2)
                {
                    // process both embed and upload thumb
                    if (vd.embed_script != "")
                        str.Append(Process_Embed_Video_Thumb(img_str, vd.thumb_url, attr));
                    else
                    {
                        if (vd.ispublished == 0 || vd.errorcode > 0 || vd.thumbfilename == "" || vd.thumbfilename == "none")
                        {
                            // video is not yet published
                            // or published failed - any level of error raised
                            // or thumbfilename is empty
                            vd.thumb_url = Config.GetUrl("/assets/images/video-thumbnail.jpg");
                            str.Append(Process_Embed_Video_Thumb(img_str, vd.thumb_url, attr));
                        }
                        else
                        {
                            str.Append(Process_Normal_Video_Thumb(img_str, vd.username, vd.thumbfilename, vd.thumb_url, attr));
                        }
                    }
                }
            }
            return str.ToString().Replace("\n", "").Replace("\r","");
        }

        private static string Process_Embed_Video_Thumb(StringBuilder img_str, string thumburl, ListItems attr)
        {
            if(attr.isresize)
            {
                string _size = "800x600";
                if (attr.size != "")
                    _size = attr.size;

                img_str.Append(Config.GetUrl("handler/resize?file=" + HttpContext.Current.Server.UrlEncode(thumburl) + "&size=" + _size));
            }
            else
            {
                img_str.Append(thumburl);
            }
           
            return img_str.ToString();
        }

        private static string Process_Normal_Video_Thumb(StringBuilder img_str, string username, string mediapath, string thumburl, ListItems attr)
        {
            if(!attr.isresize)
            {
                // no resize needed
                string MediaName = mediapath;
                if (MediaName.Contains("_"))
                {
                    string thumb_start_index = MediaName.Remove(MediaName.LastIndexOf("_"));
                    string _thumb_url = Config.Return_Video_Thumb_Url(thumburl, username);
                    string thumb_start_sequence_path = "";
                    string imagepath = "";
                    if (thumburl == "" || thumburl == "none")
                    {
                        // default thumb fetching
                        thumb_start_sequence_path = _thumb_url + "/" + thumb_start_index + "_";
                        imagepath = _thumb_url + "/" + MediaName;
                    }
                    else
                    {
                        // cloud thumb fetching
                        if (_thumb_url.Contains("_"))
                            thumb_start_sequence_path = _thumb_url.Remove(_thumb_url.LastIndexOf("_")) + "_";
                        imagepath = _thumb_url;
                    }

                    img_str.Append(imagepath);

                }
            }
            else
            {
                // resizable thumbs
                string _filename = "";
                int _iscloud = 0;
                if (thumburl != "")
                {
                    _iscloud = 1;
                    _filename = thumburl;
                }
                else
                {
                    // generate filename
                    string _MediaName = mediapath;
                    string thumb_start_index = _MediaName.Remove(_MediaName.LastIndexOf("_"));
                    string _thumb_url = Config.Return_Video_Thumb_Url(thumburl, username);
                    string thumb_start_sequence_path = "";
                    if (_thumb_url.Contains("_"))
                        thumb_start_sequence_path = _thumb_url.Remove(_thumb_url.LastIndexOf("_")) + "_";
                    _filename = _thumb_url;
                }
                 
                string _size = "800x600";
                if (attr.size != "")
                    _size = attr.size;

                img_str.Append(Config.GetUrl("videos/resize?u=" + username + "&file=" + HttpContext.Current.Server.UrlEncode(thumburl) + "&cloud=" + _iscloud + "&size=" + _size));
            }
            
            return img_str.ToString();
        }

        #endregion
    }
}